#line 1 "./DecayIntegrator.cc"
// -*- C++ -*-
//
// DecayIntegrator.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the DecayIntegrator class.
//
// Author: Peter Richardson
// 

#include "DecayIntegrator.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/PDT/EnumParticles.h"
#include "ThePEG/EventRecord/Particle.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/Interface/Switch.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Interface/RefVector.h"
#include "ThePEG/Utilities/Debug.h"
#include "Herwig/Utilities/Kinematics.h"
#include "Herwig/PDT/GenericMassGenerator.h"
#include "DecayPhaseSpaceMode.h"
#include "Herwig/PDT/WidthCalculatorBase.h"
#include "ThePEG/Interface/Reference.h"

using namespace Herwig; 

ParticleVector DecayIntegrator::decay(const Particle & parent,
				      const tPDVector & children) const {
  // return empty vector if products heavier than parent
  Energy mout(ZERO);
  for(tPDVector::const_iterator it=children.begin();
      it!=children.end();++it) mout+=(**it).massMin();
  if(mout>parent.mass()) return ParticleVector();
  // generate the decay
  bool cc;
  _imode = modeNumber(cc,parent.dataPtr(),children);
  if(numberModes()==0) return ParticleVector();
  return _modes[_imode]->generate(_generateinter,cc,parent);
}
  
void DecayIntegrator::persistentOutput(PersistentOStream & os) const {
  os << _modes << _niter << _npoint << _ntry << _photongen << _generateinter << ounit(_eps,GeV);
}
  
void DecayIntegrator::persistentInput(PersistentIStream & is, int) {
  is >> _modes >> _niter >> _npoint >> _ntry >> _photongen >> _generateinter >> iunit(_eps,GeV);
}
  
// The following static variable is needed for the type
// description system in ThePEG.
DescribeAbstractClass<DecayIntegrator,HwDecayerBase>
describeHerwigDecayIntegrator("Herwig::DecayIntegrator", "Herwig.so");
  
void DecayIntegrator::Init() {
  
  static ClassDocumentation<DecayIntegrator> documentation
    ("The DecayIntegrator class is a base decayer class "
     "including a multi-channel integrator.");
  
  static Parameter<DecayIntegrator,int> interfaceIteration
    ("Iteration",
     "Number of iterations for the initialization of the phase space",
     &DecayIntegrator::_niter, 10, 0, 100,
     false, false, true);  
  
  static Parameter<DecayIntegrator,int> interfacePoints
    ("Points",
     "number of phase space points to generate in the initialisation.",
     &DecayIntegrator::_npoint, 10000, 1, 1000000000,
     false, false, true);
  
  static Parameter<DecayIntegrator,int> interfaceNtry
    ("Ntry",
     "Number of attempts to generate the decay",
     &DecayIntegrator::_ntry, 500, 0, 100000,
     false, false, true);

  static Reference<DecayIntegrator,DecayRadiationGenerator> interfacePhotonGenerator
    ("PhotonGenerator",
     "Object responsible for generating photons in the decay.",
     &DecayIntegrator::_photongen, false, false, true, true, false);
 
  static Switch<DecayIntegrator,bool> interfaceGenerateIntermediates
    ("GenerateIntermediates",
     "Whether or not to include intermediate particles in the output",
     &DecayIntegrator::_generateinter, false, false, false);
  static SwitchOption interfaceGenerateIntermediatesNoIntermediates
    (interfaceGenerateIntermediates,
     "No",
     "Don't include the intermediates",
     false);
  static SwitchOption interfaceGenerateIntermediatesIncludeIntermediates
    (interfaceGenerateIntermediates,
     "Yes",
     "include the intermediates",
     true);
}

// output info on the integrator
ostream & Herwig::operator<<(ostream & os, const DecayIntegrator & decay) {
  os << "The integrator has " << decay._modes.size() << " modes"  << endl;
  for(unsigned int ix=0;ix<decay._modes.size();++ix) {
    os << "Information on mode " << ix << endl;
    os << *(decay._modes[ix]);
  }
  return os;
}

// generate the momenta for the decay
ParticleVector DecayIntegrator::generate(bool inter,bool cc,
					 const unsigned int & imode,
					 const Particle & inpart) const {
  _imode=imode;
  return _modes[imode]->generate(inter,cc,inpart);
}  

// initialization for a run
void DecayIntegrator::doinitrun() {
  HwDecayerBase::doinitrun();
  if ( initialize() && Debug::level > 1 ) 
    CurrentGenerator::current().log() << "Start of the initialisation for " 
				      << this->name() << "\n";
  for(unsigned int ix=0;ix<_modes.size();++ix) {
    if(!_modes[ix]) continue;
    _modes[ix]->initrun();
    _imode=ix;
    _modes[ix]->initializePhaseSpace(initialize());
  }
  // CurrentGenerator::current().log() << *this << "\n";
}

// add a new mode
void DecayIntegrator::addMode(DecayPhaseSpaceModePtr in,double maxwgt,
				     const vector<double> inwgt) const {
  _modes.push_back(in);
  if(in) {
    in->setMaxWeight(maxwgt);
    in->setWeights(inwgt);
    in->setIntegrate(_niter,_npoint,_ntry);
    in->init();
  }
}

// reset the properities of all intermediates
void DecayIntegrator::resetIntermediate(tcPDPtr part, Energy mass, Energy width) {
  if(!part) return;
  for(unsigned int ix=0,N=_modes.size();ix<N;++ix) {
    _modes[ix]->resetIntermediate(part,mass,width);
  }
} 

WidthCalculatorBasePtr 
DecayIntegrator::threeBodyMEIntegrator(const DecayMode &) const {
  return WidthCalculatorBasePtr();
}

// set the code for the partial width
void DecayIntegrator::setPartialWidth(const DecayMode & dm, int imode) {
  vector<int> extid;
  tcPDPtr cc,cc2;
  int nfound(0),ifound,nmax(1),id;
  unsigned int ix(0),iy,N,iz,tmax,nmatched;
  if(dm.parent()->CC()) nmax=2;
  if(_modes.size()==0) return;
  do {
    if(!_modes[ix]) {
      ++ix;
      continue;
    }
    cc = _modes[ix]->externalParticles(0)->CC();
    tmax = cc ? 1 : 2;
    for(iz=0;iz<tmax;++iz) {
      ifound=-1;
      extid.clear();
      // check the parent
      if(dm.parent()->id()==_modes[ix]->externalParticles(0)->id()&&iz==0) {
	for(iy=0,N=_modes[ix]->numberofParticles();iy<N;++iy) {
	  extid.push_back(_modes[ix]->externalParticles(iy)->id());
	}
      }
      else if(dm.parent()->id()==_modes[ix]->externalParticles(0)->id()&&iz==1) {
	for(iy=0,N=_modes[ix]->numberofParticles();iy<N;++iy) {
	  cc2=_modes[ix]->externalParticles(iy)->CC();
	  extid.push_back( cc2 ? cc2->id() : _modes[ix]->externalParticles(iy)->id());
	}
      }
      else if(cc&&dm.parent()->id()==cc->id()) {
	for(iy=0,N=_modes[ix]->numberofParticles();iy<N;++iy) {
	  cc2 = _modes[ix]->externalParticles(iy)->CC();
	  extid.push_back( cc2 ? cc2->id() : _modes[ix]->externalParticles(iy)->id());
	}
      }
      // if the parents match
      if(!extid.empty()) {
	vector<bool> matched(extid.size(),false);
	bool done;
	nmatched=0;
	ParticleMSet::const_iterator pit = dm.products().begin();
	do {
	  id=(**pit).id();
	  done=false;
	  iy=1;
	  do {
	    if(id==extid[iy]&&!matched[iy]) {
	      matched[iy]=true;
	      ++nmatched;
	      done=true;
	    }
	    ++iy;
	  }
	  while(iy<extid.size()&&!done);
	  ++pit;
	}
	while(pit!=dm.products().end());
	if(nmatched==extid.size()-1) {
	  ifound=ix;
	  ++nfound;
	}
      }
      if(ifound>=0) _modes[ifound]->setPartialWidth(imode);
    }
    ++ix;
  }
  while(nfound<nmax&&ix<_modes.size());
}

int DecayIntegrator::findMode(const DecayMode & dm) {
  int imode(-1);
  vector<int> extid;
  tcPDPtr cc,cc2;
  bool found(false);
  int id;
  unsigned int ix(0),iy,N,iz,tmax,nmatched;
  if(_modes.size()==0) return -1;
  do {
    if(!_modes[ix]) {
      ++ix;
      continue;
    }
    cc = _modes[ix]->externalParticles(0)->CC();
    tmax=1;if(!cc){++tmax;}
    for(iz=0;iz<tmax;++iz) {
      extid.clear();
      // check the parent
      if(dm.parent()->id()==_modes[ix]->externalParticles(0)->id()&&iz==0) {
	for(iy=0,N=_modes[ix]->numberofParticles();iy<N;++iy) {
	  extid.push_back(_modes[ix]->externalParticles(iy)->id());
	}
      }
      else if(dm.parent()->id()==_modes[ix]->externalParticles(0)->id()&&iz==1) {
	for(iy=0,N=_modes[ix]->numberofParticles();iy<N;++iy) {
	  cc2=_modes[ix]->externalParticles(iy)->CC();
	  extid.push_back( cc2 ? cc2->id() : _modes[ix]->externalParticles(iy)->id());
	}
      }
      else if(cc&&dm.parent()->id()==cc->id()) {
	for(iy=0,N=_modes[ix]->numberofParticles();iy<N;++iy) {
	  cc = _modes[ix]->externalParticles(iy)->CC();
	  extid.push_back( cc ? cc->id() : _modes[ix]->externalParticles(iy)->id());
	}
      }
      // if the parents match
      if(!extid.empty()) {
	vector<bool> matched(extid.size(),false);
	bool done;
	nmatched=0;
	ParticleMSet::const_iterator pit = dm.products().begin();
	do {
	  id=(**pit).id();
	  done=false;
	  iy=1;
	  do {
	    if(id==extid[iy]&&!matched[iy]) {
	      matched[iy]=true;
	      ++nmatched;
	      done=true;
	    }
	    ++iy;
	  }
	  while(iy<extid.size()&&!done);
	  ++pit;
	}
	while(pit!=dm.products().end());
	if(nmatched==extid.size()-1) {
	  imode=ix;
	  found=true;
	}
      }
    }
    ++ix;
  }
  while(!found&&ix<_modes.size());
  return imode;
}

// output the information for the database
void DecayIntegrator::dataBaseOutput(ofstream & output,bool header) const {
  // header for MySQL
  if(header) output << "update decayers set parameters=\"";
  output << "newdef " << name() << ":Iteration " << _niter << "\n";
  output << "newdef " << name() << ":Ntry " << _ntry << "\n";
  output << "newdef " << name() << ":Points " << _npoint << "\n";
  //if(_photongen){;}
  output << "newdef " << name() << ":GenerateIntermediates " << _generateinter << " \n";
  // footer for MySQL
  if(header) {
    output << "\n\" where BINARY ThePEGName=\"" << fullName() << "\";\n";
  }
}
  
// pointer to a mode
tDecayPhaseSpaceModePtr DecayIntegrator::mode(unsigned int ix) {
  return _modes[ix];
}

tcDecayPhaseSpaceModePtr DecayIntegrator::mode(unsigned int ix) const {
  return _modes[ix];
}
 
Energy DecayIntegrator::initializePhaseSpaceMode(unsigned int imode,bool init, bool onShell) const{
  tcDecayPhaseSpaceModePtr cmodeptr=mode(imode);
  tDecayPhaseSpaceModePtr modeptr = const_ptr_cast<tDecayPhaseSpaceModePtr>(cmodeptr);
  modeptr->init();
  return modeptr->initializePhaseSpace(init,onShell);
}

// the matrix element to be integrated for the me
double DecayIntegrator::threeBodyMatrixElement(const int,const Energy2,
					       const Energy2,
					       const Energy2,const Energy2,
					       const Energy, const Energy, 
					       const Energy) const {
  throw DecayIntegratorError() 
    << "Calling the virtual DecayIntegrator::threeBodyMatrixElement"
    << "method. This must be overwritten in the classes "
    << "inheriting from DecayIntegrator where it is needed"
    << Exception::runerror;
}

// the differential three body decay rate with one integral performed
InvEnergy DecayIntegrator::threeBodydGammads(const int, const Energy2,
					     const Energy2,
					     const Energy, const Energy, 
					     const Energy) const {
  throw DecayIntegratorError() 
    << "Calling the virtual DecayIntegrator::threeBodydGammads()" 
    <<"method. This must be overwritten in the classes "
    << "inheriting from DecayIntegrator where it is needed"
    << Exception::runerror;
}

double DecayIntegrator::oneLoopVirtualME(unsigned int ,
					 const Particle &, 
					 const ParticleVector &) {
  throw DecayIntegratorError()
    << "DecayIntegrator::oneLoopVirtualME() called. This should"
    << " have been overidden in an inheriting class if it is used"
    << Exception::runerror;
}

InvEnergy2 DecayIntegrator::realEmissionME(unsigned int,
					   const Particle &, 
					   ParticleVector &,
					   unsigned int,
					   double, double, 
					   const LorentzRotation &,
					   const LorentzRotation &) {
  throw DecayIntegratorError()
    << "DecayIntegrator::realEmmisionME() called. This should"
    << " have been overidden in an inheriting class if it is used"
    << Exception::runerror;
}
#line 1 "./DecayPhaseSpaceChannel.cc"
// -*- C++ -*-
//
// DecayPhaseSpaceChannel.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the DecayPhaseSpaceChannel class.
//
// Author: Peter Richardson
// 

#include "DecayPhaseSpaceChannel.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "DecayPhaseSpaceMode.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Repository/CurrentGenerator.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Interface/RefVector.h"
#include "ThePEG/Interface/Switch.h"

#include <ThePEG/Repository/CurrentGenerator.h>

using namespace Herwig;

  
DecayPhaseSpaceChannel::DecayPhaseSpaceChannel(tcDecayPhaseSpaceModePtr in) 
  : _mode(in) {}

void DecayPhaseSpaceChannel::persistentOutput(PersistentOStream & os) const {
  os << _intpart << _jactype << ounit(_intmass,GeV) << ounit(_intwidth,GeV)
     << ounit(_intmass2,GeV2) << ounit(_intmwidth,GeV2) << _intpower 
     << _intdau1 << _intdau2 << _intext << _mode;
}
  
void DecayPhaseSpaceChannel::persistentInput(PersistentIStream & is, int) {
  is >> _intpart >> _jactype >> iunit(_intmass,GeV) >> iunit(_intwidth,GeV) 
     >> iunit(_intmass2,GeV2) >> iunit(_intmwidth,GeV2) >> _intpower
     >> _intdau1 >> _intdau2 >> _intext >> _mode;
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<DecayPhaseSpaceChannel,Base>
describeHerwigDecayPhaseSpaceChannel("Herwig::DecayPhaseSpaceChannel", "Herwig.so");

void DecayPhaseSpaceChannel::Init() {
    
  static RefVector<DecayPhaseSpaceChannel,ParticleData> interfaceIntermediateParticles
    ("IntermediateParticles",
     "The intermediate particles in the decay chain.",
     &DecayPhaseSpaceChannel::_intpart, 0, false, false, true, false);
}

// generate the momenta of the external particles
vector<Lorentz5Momentum> 
DecayPhaseSpaceChannel::generateMomenta(const Lorentz5Momentum & pin,
					const vector<Energy> & massext) {
  // integers for loops
  unsigned int ix,iy,idau[2],iz;
  // storage of the momenta of the external particles
  vector<Lorentz5Momentum> pexternal;
  // and the internal particles
  vector<Lorentz5Momentum> pinter; 
  // copy the momentum of the incoming particle
  pexternal.push_back(pin); pinter.push_back(pin);
  pexternal.resize(_mode->numberofParticles());
  pinter.resize(_intpart.size());
  // masses of the intermediate particles
  vector<Energy> massint(_intpart.size());
  massint[0]=pin.mass();
  // generate all the decays in the chain
  Energy lower,upper,lowerb[2];
  for(ix=0;ix<_intpart.size();++ix) {
    idau[0] = abs(_intdau1[ix]);
    idau[1] = abs(_intdau2[ix]);
    // if both decay products off-shell
    if(_intdau1[ix]<0&&_intdau2[ix]<0) {
      // lower limits on the masses of the two resonances
      for(iy=0;iy<2;++iy) {
	lowerb[iy]=ZERO;
	bool massless=true;
	for(iz=0;iz<_intext[idau[iy]].size();++iz) {
	  if(massext[_intext[idau[iy]][iz]]!=ZERO) massless = false;
	  lowerb[iy]+=massext[_intext[idau[iy]][iz]];
	}
	if(massless) lowerb[iy] = _mode->epsilonPS(); 
      }
      // randomize the order
      if(UseRandom::rnd()<0.5) {
	// mass of the first resonance
	upper = massint[ix]-lowerb[1];
	lower = lowerb[0];
	massint[idau[0]]=generateMass(idau[0],lower,upper);
	// mass of the second resonance
	upper = massint[ix]-massint[idau[0]];
	lower = lowerb[1];
	massint[idau[1]]=generateMass(idau[1],lower,upper);
      }
      else {
	// mass of the second resonance
	upper = massint[ix]-lowerb[0];
	lower = lowerb[1];
	massint[idau[1]]=generateMass(idau[1],lower,upper);
	// mass of the first resonance
	upper = massint[ix]-massint[idau[1]];
	lower = lowerb[0];
	massint[idau[0]]=generateMass(idau[0],lower,upper);
      }
      // generate the momenta of the decay products
      twoBodyDecay(pinter[ix],massint[idau[0]],massint[idau[1]],
		   pinter[idau[0]],pinter[idau[1]]);
    }
    // only first off-shell
    else if(_intdau1[ix]<0) {
      // compute the limits of integration
      upper = massint[ix]-massext[idau[1]];
      lower = ZERO;
      bool massless=true;
      for(iy=0;iy<_intext[idau[0]].size();++iy) {
	if(massext[_intext[idau[0]][iy]]!=ZERO) massless = false;
	lower+=massext[_intext[idau[0]][iy]];
      }
      if(massless) lower = _mode->epsilonPS();
      massint[idau[0]]=generateMass(idau[0],lower,upper);
      // generate the momenta of the decay products
      twoBodyDecay(pinter[ix],massint[idau[0]],massext[idau[1]], 
		   pinter[idau[0]],pexternal[idau[1]]);
    }
    // only second off-shell
    else if(_intdau2[ix]<0) {
      // compute the limits of integration
      upper = massint[ix]-massext[idau[0]];
      lower = ZERO;
      bool massless=true;
      for(iy=0;iy<_intext[idau[1]].size();++iy) {
	if(massext[_intext[idau[0]][iy]]!=ZERO) massless = false;
	lower+=massext[_intext[idau[1]][iy]];
      }
      if(massless) lower = _mode->epsilonPS();
      massint[idau[1]]=generateMass(idau[1],lower,upper);
      // generate the momenta of the decay products
      twoBodyDecay(pinter[ix],massext[idau[0]],massint[idau[1]], 
		   pexternal[idau[0]],pinter[idau[1]]);
    }
    // both on-shell
    else {
      // generate the momenta of the decay products
      twoBodyDecay(pinter[ix],massext[idau[0]],massext[idau[1]], 
		   pexternal[idau[0]],pexternal[idau[1]]);
    }
  }
  // return the external momenta
  return pexternal;
}

// generate the weight for this channel given a phase space configuration
double DecayPhaseSpaceChannel::generateWeight(const vector<Lorentz5Momentum> & output) {
  using Constants::pi;
  // integers for loops
  unsigned int ix,iy,idau[2],iz;
  // include the prefactor due to the weight of the channel
  double wgt=1.;
  // work out the masses of the intermediate particles
  static vector<Energy> intmass;
  intmass.clear();
  Lorentz5Momentum pinter;
  for(ix=0;ix<_intpart.size();++ix) {
    pinter=output[_intext[ix][0]];
    for(iz=1;iz<_intext[ix].size();++iz) pinter+=output[_intext[ix][iz]];
    pinter.rescaleMass();
    intmass.push_back( pinter.mass() );
  }
  Energy2 scale(sqr(intmass[0]));
  // calculate the terms for each of the decays
  Energy lower,upper,lowerb[2];
  for(ix=0;ix<_intpart.size();++ix) {
    idau[0] = abs(_intdau1[ix]);
    idau[1] = abs(_intdau2[ix]);
    // if both decay products off-shell
    Energy pcm;
    if(_intdau1[ix]<0&&_intdau2[ix]<0) {
      // lower limits on the masses of the two resonances
      for(iy=0;iy<2;++iy) {
	lowerb[iy]=ZERO;
	for(iz=0;iz<_intext[idau[iy]].size();++iz)
	  lowerb[iy]+=output[_intext[idau[iy]][iz]].mass();
      }
      // undo effect of randomising
      // weight for the first order
      // contribution of first resonance
      upper = intmass[ix]-lowerb[1];
      lower = lowerb[0];
      InvEnergy2 wgta=massWeight(idau[0],intmass[idau[0]],lower,upper);
      // contribution of second resonance
      upper = intmass[ix]-intmass[idau[0]];
      lower = lowerb[1];
      InvEnergy4 wgta2 = wgta*massWeight(idau[1],intmass[idau[1]],lower,upper);
      // weight for the second order
      upper = intmass[ix]-lowerb[0];
      lower = lowerb[1];
      InvEnergy2 wgtb=massWeight(idau[1],intmass[idau[1]],lower,upper);
      upper = intmass[ix]-intmass[idau[1]];
      lower = lowerb[0];
      InvEnergy4 wgtb2=wgtb*massWeight(idau[0],intmass[idau[0]],lower,upper);
      // weight factor
      wgt *=0.5*sqr(scale)*(wgta2+wgtb2);
      // factor for the kinematics
      pcm = Kinematics::pstarTwoBodyDecay(intmass[ix],intmass[idau[0]],
				   intmass[idau[1]]);
      if(pcm>ZERO)
	wgt *= intmass[ix]*8.*pi*pi/pcm;
      else
	wgt = 0.;
    }
    // only first off-shell
    else if(_intdau1[ix]<0) {
      // compute the limits of integration
      upper = intmass[ix]-output[idau[1]].mass();
      lower = ZERO;
      for(iy=0;iy<_intext[idau[0]].size();++iy)
	lower+=output[_intext[idau[0]][iy]].mass();
      wgt *=scale*massWeight(idau[0],intmass[idau[0]],lower,upper);
      pcm = Kinematics::pstarTwoBodyDecay(intmass[ix],intmass[idau[0]],
					  output[idau[1]].mass());
      if(pcm>ZERO)
	wgt *= intmass[ix]*8.*pi*pi/pcm;
      else
	wgt = 0.;
    }
    // only second off-shell
    else if(_intdau2[ix]<0) {
      // compute the limits of integration
      upper = intmass[ix]-output[idau[0]].mass(); 
      lower = ZERO;
      for(iy=0;iy<_intext[idau[1]].size();++iy)
	lower+=output[_intext[idau[1]][iy]].mass();
      wgt *=scale*massWeight(idau[1],intmass[idau[1]],lower,upper);
      pcm = Kinematics::pstarTwoBodyDecay(intmass[ix],intmass[idau[1]],
					  output[idau[0]].mass());
      if(pcm>ZERO)
	wgt *=intmass[ix]*8.*pi*pi/pcm;
      else
	wgt=0.;
    }
    // both on-shell
    else {
      pcm = Kinematics::pstarTwoBodyDecay(intmass[ix],output[idau[1]].mass(),
					  output[idau[0]].mass());
      if(pcm>ZERO)
	wgt *=intmass[ix]*8.*pi*pi/pcm;
      else
	wgt = 0.;
    }
  }
  // finally the overall factor
  wgt /= pi;
  // return the answer
  return wgt;
}

// output the information to a stream
ostream & Herwig::operator<<(ostream & os, const DecayPhaseSpaceChannel & channel) {
  // output of the external particles
  os << "Channel for the decay of " << channel._mode->externalParticles(0)->PDGName() 
     << " -> ";
  for(unsigned int ix=1;ix<channel._mode->numberofParticles();++ix)
    os << channel._mode->externalParticles(ix)->PDGName() << " ";
  os << endl;
  os << "Decay proceeds in following steps ";
  for(unsigned int ix=0;ix<channel._intpart.size();++ix) {
    os << channel._intpart[ix]->PDGName() << " -> ";
    if(channel._intdau1[ix]>0) {
      os << channel._mode->externalParticles(channel._intdau1[ix])->PDGName()  
	 << "(" << channel._intdau1[ix]<< ") ";
    }
    else {
      os << channel._intpart[-channel._intdau1[ix]]->PDGName() 
	 << "(" << channel._intdau1[ix]<< ") ";
    }
    if(channel._intdau2[ix]>0) {
      os << channel._mode->externalParticles(channel._intdau2[ix])->PDGName()  
	 << "(" <<channel._intdau2[ix] << ") ";
    }
    else{
      os << channel._intpart[-channel._intdau2[ix]]->PDGName() 
	 << "(" <<channel._intdau2[ix] << ") ";
    }
    os << endl;
  }
  return os;
}

// doinit method  
void DecayPhaseSpaceChannel::init() {
  // check if the mode pointer exists
  assert(_mode);
  // masses and widths of the intermediate particles
  for(unsigned int ix=0;ix<_intpart.size();++ix) {
    _intmass.push_back(_intpart[ix]->mass());
    _intwidth.push_back(_intpart[ix]->width());
    _intmass2.push_back(_intpart[ix]->mass()*_intpart[ix]->mass());
    _intmwidth.push_back(_intpart[ix]->mass()*_intpart[ix]->width());
  }
  // external particles for each intermediate particle
  vector<int> temp;
  _intext.resize(_intpart.size());
  // loop over the intermediate particles
  for(int ix=_intpart.size()-1;ix>=0;--ix) {
    temp.clear();
    // add the first daughter
    if(_intdau1[ix]>=0) {
      temp.push_back(_intdau1[ix]);
    }
    else {
      int iy = -_intdau1[ix];
      vector<int>::iterator istart=_intext[iy].begin();
      vector<int>::iterator iend=_intext[iy].end();
      for(;istart!=iend;++istart) temp.push_back(*istart);
    }
    // add the second daughter
    if(_intdau2[ix]>=0) {
      temp.push_back(_intdau2[ix]);
    }
    else {
      int iy = -_intdau2[ix];
      vector<int>::iterator istart=_intext[iy].begin();
      vector<int>::iterator iend=_intext[iy].end();
      for(;istart!=iend;++istart) temp.push_back(*istart);
    }
    _intext[ix]=temp;
  }
  // ensure intermediates either have the width set, or
  // can't possibly be on-shell
  Energy massmax;
  if(_mode->testOnShell()) {
    massmax = _mode->externalParticles(0)->mass();
    for(unsigned int ix=1;ix<_mode->numberofParticles();++ix) 
      massmax -= _mode->externalParticles(ix)->mass();
  }
  else { 
    massmax = _mode->externalParticles(0)->massMax();
    for(unsigned int ix=1;ix<_mode->numberofParticles();++ix) 
      massmax -= _mode->externalParticles(ix)->massMin();
  }
  for(unsigned int ix=0;ix<_intpart.size();++ix) {
    if(_intwidth[ix]==ZERO && ix>0 && _jactype[ix]==0 ) {
      Energy massmin(ZERO);
      for(unsigned int iy=0;iy<_intext[ix].size();++iy)
	massmin += _mode->testOnShell() ? 
	  _mode->externalParticles(_intext[ix][iy])->mass() :
	  _mode->externalParticles(_intext[ix][iy])->massMin();
      // check if can be on-shell
      if(_intmass[ix]>=massmin&&_intmass[ix]<=massmax+massmin) {
	string modeout;
	for(unsigned int iy=0;iy<_mode->numberofParticles();++iy) {
	  modeout += _mode->externalParticles(iy)->PDGName() + " ";
	}
	throw InitException() << "Width zero for " << _intpart[ix]->PDGName()
			      << " in DecayPhaseSpaceChannel::doinit() "
			      << modeout
			      << Exception::runerror;
      }
    }
  }
}

void DecayPhaseSpaceChannel::initrun() {
  if(!_mode->testOnShell()) return;
  _intmass.clear();
  _intwidth.clear();
  _intmass2.clear();
  _intmwidth.clear();
  // masses and widths of the intermediate particles
  for(unsigned int ix=0;ix<_intpart.size();++ix) {
    _intmass.push_back(_intpart[ix]->mass());
    _intwidth.push_back(_intpart[ix]->width());
    _intmass2.push_back(_intpart[ix]->mass()*_intpart[ix]->mass());
    _intmwidth.push_back(_intpart[ix]->mass()*_intpart[ix]->width());
  }
  // ensure intermediates either have the width set, or
  // can't possibly be on-shell
  Energy massmax = _mode->externalParticles(0)->massMax();
  for(unsigned int ix=1;ix<_mode->numberofParticles();++ix) 
    massmax -= _mode->externalParticles(ix)->massMin();
  for(unsigned int ix=0;ix<_intpart.size();++ix) {
    if(_intwidth[ix]==0.*MeV && ix>0 && _jactype[ix]==0 ) {
      Energy massmin(0.*GeV);
      for(unsigned int iy=0;iy<_intext[ix].size();++iy) {
	massmin += _mode->externalParticles(_intext[ix][iy])->massMin();
      }
      // check if can be on-shell
      if(_intmass[ix]>=massmin&&_intmass[ix]<=massmax+massmin) {
	string modeout;
	for(unsigned int iy=0;iy<_mode->numberofParticles();++iy) {
	  modeout += _mode->externalParticles(iy)->PDGName() + " ";
	}
	throw Exception() << "Width zero for " << _intpart[ix]->PDGName()
			  << " in DecayPhaseSpaceChannel::doinitrun() "
			  << modeout
			  << Exception::runerror;
      }
    }
  }
}

// generate the final-state particles including the intermediate resonances
void DecayPhaseSpaceChannel::generateIntermediates(bool cc, const Particle & in,
						   ParticleVector & out) {
  // integers for the loops
  unsigned int ix,iz;
  // create the particles
  // incoming particle
  ParticleVector external;
  external.push_back(const_ptr_cast<tPPtr>(&in));
  // outgoing
  for(ix=0;ix<out.size();++ix) external.push_back(out[ix]);
  out.clear();
  // now create the intermediates
  ParticleVector resonance;
  resonance.push_back(external[0]);
  PPtr respart;
  tcPDPtr parttemp;
  Lorentz5Momentum pinter;
  for(ix=1;ix<_intpart.size();++ix) {
    pinter=external[_intext[ix][0]]->momentum();
    for(iz=1;iz<_intext[ix].size();++iz) 
      pinter+=external[_intext[ix][iz]]->momentum();
    pinter.rescaleMass();
    respart = (cc&&_intpart[ix]->CC()) ? 
      _intpart[ix]->CC()->produceParticle(pinter) : 
      _intpart[ix]      ->produceParticle(pinter);
    resonance.push_back(respart);
  }
  // set up the mother daughter relations
  for(ix=1;ix<_intpart.size();++ix) {
    resonance[ix]->addChild( _intdau1[ix]<0 ? 
			     resonance[-_intdau1[ix]] : external[_intdau1[ix]]);
    resonance[ix]->addChild( _intdau2[ix]<0 ? 
			     resonance[-_intdau2[ix]] : external[_intdau2[ix]]);
    if(resonance[ix]->dataPtr()->stable())
      resonance[ix]->setLifeLength(Lorentz5Distance());
  }
  // construct the output with the particles in the first step
  out.push_back( _intdau1[0]>0 ? external[_intdau1[0]] : resonance[-_intdau1[0]]);
  out.push_back( _intdau2[0]>0 ? external[_intdau2[0]] : resonance[-_intdau2[0]]);
}
 
double DecayPhaseSpaceChannel::atanhelper_(int ires, Energy limit) {
  return atan2( limit*limit-_intmass2[ires], _intmwidth[ires] );
}

// return the weight for a given resonance
InvEnergy2 DecayPhaseSpaceChannel::massWeight(int ires, Energy moff,
					      Energy lower,Energy upper) {
  InvEnergy2 wgt = ZERO;
  if(lower>upper) {
    throw DecayPhaseSpaceError() << "DecayPhaseSpaceChannel::massWeight not allowed " 
				 << ires << "   " << _intpart[ires]->id() << "   " 
				 << moff/GeV << " " << lower/GeV << " " << upper/GeV << Exception::eventerror;
  } 
  // use a Breit-Wigner 
  if ( _jactype[ires] == 0 ) {
    double rhomin  = atanhelper_(ires,lower);
    double rhomax  = atanhelper_(ires,upper) - rhomin;
    if ( rhomax != 0.0 ) {
      Energy2 moff2=moff*moff-_intmass2[ires];
      wgt = _intmwidth[ires]/rhomax/(moff2*moff2+_intmwidth[ires]*_intmwidth[ires]);
    }
    else {
      wgt = 1./((sqr(upper)-sqr(lower))*sqr(sqr(moff)-_intmass2[ires])/
		(sqr(lower)-_intmass2[ires])/(sqr(upper)-_intmass2[ires]));
    }
  } 
  // power law
  else if(_jactype[ires]==1) {
    double rhomin = pow(sqr(lower/MeV),_intpower[ires]+1.);
    double rhomax = pow(sqr(upper/MeV),_intpower[ires]+1.)-rhomin;
    wgt = (_intpower[ires]+1.)/rhomax*pow(sqr(moff/MeV),_intpower[ires])
      /MeV/MeV;
  }
  else if(_jactype[ires]==2) {
    wgt = 1./Constants::pi/_intmwidth[ires];
  } 
  else {
    throw DecayPhaseSpaceError() << "Unknown type of Jacobian in " 
				 << "DecayPhaseSpaceChannel::massWeight"
				 << Exception::eventerror;
  } 
  return wgt;
}
Energy DecayPhaseSpaceChannel::generateMass(int ires,Energy lower,Energy upper) {
  static const Energy eps=1e-9*MeV;
  if(lower<eps) lower=eps;
  Energy mass=ZERO;
  if(lower>upper) throw DecayPhaseSpaceError() << "DecayPhaseSpaceChannel::generateMass"
					       << " not allowed" 
					       << Exception::eventerror;
  if(abs(lower-upper)/(lower+upper)>2e-10) {
    lower +=1e-10*(lower+upper);
    upper -=1e-10*(lower+upper);
  }
  else 
    return 0.5*(lower+upper);
  // use a Breit-Wigner
  if(_jactype[ires]==0) {
    if(_intmwidth[ires]!=ZERO) {
      Energy2 lower2 = sqr(lower);
      Energy2 upper2 = sqr(upper);

      double rhomin = atan2((lower2 - _intmass2[ires]),_intmwidth[ires]);
      double rhomax = atan2((upper2 - _intmass2[ires]),_intmwidth[ires])-rhomin;
      double rho = rhomin+rhomax*UseRandom::rnd();
      Energy2 mass2 = max(lower2,min(upper2,_intmass2[ires]+_intmwidth[ires]*tan(rho)));
      if(mass2<ZERO) mass2 = ZERO;
      mass = sqrt(mass2);
    }
    else {
      mass = sqrt(_intmass2[ires]+
		  (sqr(lower)-_intmass2[ires])*(sqr(upper)-_intmass2[ires])/
		  (sqr(lower)-_intmass2[ires]-UseRandom::rnd()*(sqr(lower)-sqr(upper))));
    }
  }
  // use a power-law
  else if(_jactype[ires]==1) {
    double rhomin = pow(sqr(lower/MeV),_intpower[ires]+1.);
    double rhomax = pow(sqr(upper/MeV),_intpower[ires]+1.)-rhomin;
    double rho = rhomin+rhomax*UseRandom::rnd();
    mass = pow(rho,0.5/(_intpower[ires]+1.))*MeV;
  }
  else if(_jactype[ires]==2) {
    mass = _intmass[ires];
  } 
  else {
    throw DecayPhaseSpaceError() << "Unknown type of Jacobian in " 
				 << "DecayPhaseSpaceChannel::generateMass" 
				 << Exception::eventerror;
  }
  if(mass<lower+1e-10*(lower+upper))      mass=lower+1e-10*(lower+upper);
  else if(mass>upper-1e-10*(lower+upper)) mass=upper-1e-10*(lower+upper);
  return mass;
}

void DecayPhaseSpaceChannel::twoBodyDecay(const Lorentz5Momentum & p,
					  const Energy m1, const Energy m2,
					  Lorentz5Momentum & p1,
					  Lorentz5Momentum & p2 ) {
  static const double eps=1e-6;
  double ctheta,phi;
  Kinematics::generateAngles(ctheta,phi);
  Axis unitDir1=Kinematics::unitDirection(ctheta,phi);
  Momentum3 pstarVector;
  Energy min=p.mass();
  if ( min >= m1 + m2  &&  m1 >= ZERO  &&  m2 >= ZERO  ) {
    pstarVector = unitDir1 * Kinematics::pstarTwoBodyDecay(min,m1,m2);
  }
  else if( m1 >= ZERO  &&  m2 >= ZERO && (m1+m2-min)/(min+m1+m2)<eps) {
    pstarVector = Momentum3();
  }
  else {
    throw DecayPhaseSpaceError() << "Two body decay cannot proceed "
				 << "p = " << p / GeV 
				 << " p.m() = " << min / GeV
				 << " -> " << m1/GeV 
				 << ' ' << m2/GeV << Exception::eventerror;
  }
  p1 = Lorentz5Momentum(m1, pstarVector);
  p2 = Lorentz5Momentum(m2,-pstarVector);
  // boost from CM to LAB
  Boost bv = p.boostVector();
  double gammarest = p.e()/p.mass();
  p1.boost( bv , gammarest );
  p2.boost( bv , gammarest );
}

bool DecayPhaseSpaceChannel::checkKinematics() {
  Energy massmax = _mode->externalParticles(0)->massMax();
  for(unsigned int ix=1;ix<_mode->numberofParticles();++ix) 
    massmax -= _mode->externalParticles(ix)->massMin();
  for(unsigned int ix=1;ix<_intpart.size();++ix) {
    Energy massmin(0.*GeV);
    for(unsigned int iy=0;iy<_intext[ix].size();++iy) {
      massmin += _mode->externalParticles(_intext[ix][iy])->massMin();
    }
    if(_intmass[ix]>=massmin&&_intmass[ix]<=massmax+massmin &&
       _intwidth[ix]==ZERO)
      return false;
  }
  return true;
}
#line 1 "./DecayPhaseSpaceMode.cc"
// -*- C++ -*-
//
// DecayPhaseSpaceMode.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the DecayPhaseSpaceMode class.
//

#include "DecayPhaseSpaceMode.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "Herwig/PDT/GenericWidthGenerator.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/Interface/RefVector.h"
#include "DecayIntegrator.h"
#include "Herwig/Utilities/Kinematics.h"
#include "ThePEG/EventRecord/HelicityVertex.h"
#include "Herwig/Decay/DecayVertex.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"

#include "ThePEG/EventRecord/Event.h"
#include "ThePEG/Utilities/Debug.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void DecayPhaseSpaceMode::persistentOutput(PersistentOStream & os) const {
  os << _integrator << _channels << _channelwgts << _maxweight << _niter 
     << _npoint << _ntry << _extpart << _partial << _widthgen << _massgen
     << _testOnShell << ounit(_eps,GeV);
}

void DecayPhaseSpaceMode::persistentInput(PersistentIStream & is, int) {
  is >> _integrator >> _channels >> _channelwgts >> _maxweight >> _niter 
     >> _npoint >> _ntry >> _extpart >> _partial >> _widthgen >> _massgen
     >> _testOnShell >> iunit(_eps,GeV);
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<DecayPhaseSpaceMode,Base>
describeHerwigDecayPhaseSpaceMode("Herwig::DecayPhaseSpaceMode", "Herwig.so");

void DecayPhaseSpaceMode::Init() {

  static ClassDocumentation<DecayPhaseSpaceMode> documentation
    ("The DecayPhaseSpaceMode class contains a number of phase space"
     " channels for the integration of a particular decay mode");
 
}

// flat phase space generation and weight
Energy DecayPhaseSpaceMode::flatPhaseSpace(bool cc, const Particle & inpart,
					   ParticleVector & outpart,
					   bool onShell) const {
  double wgt(1.);
  if(outpart.empty()) {
    outpart.reserve(_extpart.size()-1);
    for(unsigned int ix=1;ix<_extpart.size();++ix) {
      if(cc&&_extpart[ix]->CC()) {
	outpart.push_back((_extpart[ix]->CC())->produceParticle());
      }
      else {
	outpart.push_back(_extpart[ix]->produceParticle());
      }
    }
  }
  // masses of the particles
  Energy inmass(inpart.mass());
  vector<Energy> mass = externalMasses(inmass,wgt,onShell);
  // momenta of the particles
  vector<Lorentz5Momentum> part(outpart.size());
  // two body decay
  assert(outpart.size()==2);
  double ctheta,phi;
  Kinematics::generateAngles(ctheta,phi);
  if(! Kinematics::twoBodyDecay(inpart.momentum(), mass[1], mass[2],
				ctheta, phi,part[0],part[1])) 
    throw Exception() << "Incoming mass - Outgoing mass negative in "
		      << "DecayPhaseSpaceMode::flatPhaseSpace()"
		      << Exception::eventerror;
  wgt *= Kinematics::pstarTwoBodyDecay(inmass,mass[1],mass[2])/8./Constants::pi/inmass;
  outpart[0]->set5Momentum(part[0]);
  outpart[1]->set5Momentum(part[1]);
  return wgt*inmass;
}

// initialise the phase space
Energy DecayPhaseSpaceMode::initializePhaseSpace(bool init, bool onShell) {
  _integrator->ME(DecayMEPtr());
  Energy output(ZERO);
  // ensure that the weights add up to one
  if(!_channels.empty()) {
    double temp=0.;
    for(unsigned int ix=0;ix<_channels.size();++ix) temp+=_channelwgts[ix];
    for(unsigned int ix=0;ix<_channels.size();++ix) _channelwgts[ix]/=temp;
  }
  if(!init) return ZERO;
  // create a particle vector from the particle data one
  ThePEG::PPtr inpart=_extpart[0]->produceParticle();
  ParticleVector particles;
  // now if using flat phase space
  _maxweight=0.;
  double totsum(0.),totsq(0.);
  InvEnergy pre=1./MeV;
  Energy prewid;
  if(_channels.empty()) {
    double wsum=0.,wsqsum=0.;
    Energy m0,mmin(ZERO);
    for(unsigned int ix=1;ix<_extpart.size();++ix) {
      mmin+=_extpart[ix]->massMin();
    }
    for(int ix=0;ix<_npoint;++ix) {
      // set the mass of the decaying particle
      m0 = !onShell ? inpart->dataPtr()->generateMass() : inpart->dataPtr()->mass();
      double wgt=0.;
      if(m0>mmin) {
	inpart->set5Momentum(Lorentz5Momentum(m0));
	// compute the prefactor
	prewid = (_widthgen&&_partial>=0) ?
	  _widthgen->partialWidth(_partial,inpart->mass()) :
	  inpart->dataPtr()->width();
	pre = prewid>ZERO ? 1./prewid : 1./MeV;
	// generate the weight for this point
	try {
	  int dummy;
	  wgt = pre*weight(false,dummy,*inpart,particles,true,onShell);
	}
	catch (Veto) {
	  wgt=0.;
	}
      }
      if(wgt>_maxweight) _maxweight=wgt;
      wsum   += wgt;
      wsqsum += sqr(wgt);
    }
    wsum=wsum/_npoint;
    wsqsum=wsqsum/_npoint-sqr(wsum);
    if(wsqsum<0.) wsqsum=0.;
    wsqsum=sqrt(wsqsum/_npoint);
    Energy fact = (_widthgen&&_partial>=0) ? 
      _widthgen->partialWidth(_partial,inpart->nominalMass()) :
      inpart->dataPtr()->width();
    if(fact==ZERO) fact=MeV;
    // factor for the weight with spin correlations
    _maxweight *= inpart->dataPtr()->iSpin()==1 ? 1.1 : 1.6;
    if ( Debug::level > 1 ) {
      // ouptut the information on the initialisation
      CurrentGenerator::log() << "Initialized the phase space for the decay " 
			      << _extpart[0]->PDGName() << " -> ";
      for(unsigned int ix=1,N=_extpart.size();ix<N;++ix)
	CurrentGenerator::log() << _extpart[ix]->PDGName() << " ";
      CurrentGenerator::log() << "\n";
      if(fact!=MeV) CurrentGenerator::log() << "The branching ratio is " << wsum 
					    << " +/- " << wsqsum << "\n";
      CurrentGenerator::log() << "The partial width is " << wsum*fact/MeV 
			      << " +/- " << wsqsum*fact/MeV << " MeV\n";
      CurrentGenerator::log() << "The partial width is " 
			      << wsum*fact/6.58212E-22/MeV 
			      << " +/- " << wsqsum*fact/6.58212E-22/MeV<< " s-1\n";
      CurrentGenerator::log() << "The maximum weight is " 
			      << _maxweight << endl;
    }
    output=wsum*fact;
  }
  else {
    for(int iy=0;iy<_niter;++iy) {
      // zero the maximum weight
      _maxweight=0.;
      vector<double> wsum(_channels.size(),0.),wsqsum(_channels.size(),0.);
      vector<int> nchan(_channels.size(),0);
      totsum = 0.; totsq = 0.;
      Energy m0,mmin(ZERO);
      for(unsigned int ix=1;ix<_extpart.size();++ix) {
	mmin+=_extpart[ix]->massMin();
      }
      for(int ix=0;ix<_npoint;++ix) {
	m0 = !onShell ? inpart->dataPtr()->generateMass() : inpart->dataPtr()->mass();
	double wgt=0.; 
	int ichan(-1);
	if(m0>mmin) {
	  inpart->set5Momentum(Lorentz5Momentum(m0));
	  // compute the prefactor
	  prewid= (_widthgen&&_partial>=0) ? 
	    _widthgen->partialWidth(_partial,inpart->mass()) :
	    inpart->dataPtr()->width();
	  pre = prewid>ZERO ? 1./prewid : 1./MeV;
	  // generate the weight for this point
	  try {
	    wgt = pre*weight(false,ichan,*inpart,particles,true,onShell);
	  }
	  catch (Veto) {
	    wgt=0.;
	  }
	}
	if(wgt>_maxweight) _maxweight=wgt;
	if(ichan>=0) {
	  wsum[ichan]   += wgt;
	  wsqsum[ichan] += sqr(wgt);
	  ++nchan[ichan];
	}
	totsum+=wgt;
	totsq+=wgt*wgt;
      }
      totsum=totsum/_npoint;
      totsq=totsq/_npoint-sqr(totsum);
      if(totsq<0.) totsq=0.;
      totsq=sqrt(totsq/_npoint);
      if ( Debug::level > 1 )
	CurrentGenerator::log() << "The branching ratio is " << iy << " " 
				<< totsum << " +/- " << totsq 
				<< _maxweight << "\n";
      // compute the individual terms
      double total(0.);
      for(unsigned int ix=0;ix<_channels.size();++ix) {
	if(nchan[ix]!=0) {
	  wsum[ix]=wsum[ix]/nchan[ix];
	  wsqsum[ix]=wsqsum[ix]/nchan[ix];
	  if(wsqsum[ix]<0.) wsqsum[ix]=0.;
	  wsqsum[ix]=sqrt(wsqsum[ix]/nchan[ix]);
	}
	else {
	  wsum[ix]=0;
	  wsqsum[ix]=0;
	}
	total+=sqrt(wsqsum[ix])*_channelwgts[ix];
      }
      if(total>0.) {
	double temp;
	for(unsigned int ix=0;ix<_channels.size();++ix) {
	  temp=sqrt(wsqsum[ix])*_channelwgts[ix]/total;
	  _channelwgts[ix]=temp;
	}
      }
    }
    // factor for the weight with spin correlations
    _maxweight*= inpart->dataPtr()->iSpin()==1 ? 1.1 : 1.6;
    // ouptut the information on the initialisation
    Energy fact = (_widthgen&&_partial>=0) ? 
      _widthgen->partialWidth(_partial,inpart->nominalMass()) :
      inpart->dataPtr()->width();
    if(fact==ZERO) fact=MeV;
    if ( Debug::level > 1 ) {
      CurrentGenerator::log() << "Initialized the phase space for the decay " 
			      << _extpart[0]->PDGName() << " -> ";
      for(unsigned int ix=1,N=_extpart.size();ix<N;++ix)
	CurrentGenerator::log() << _extpart[ix]->PDGName() << " ";
      CurrentGenerator::log() << "\n";
      if(fact!=MeV) CurrentGenerator::log() << "The branching ratio is " << totsum 
					    << " +/- " << totsq << "\n";
      CurrentGenerator::log() << "The partial width is " << totsum*fact/MeV 
			      << " +/- " << totsq*fact/MeV << " MeV\n";
      CurrentGenerator::log() << "The partial width is " 
			      << totsum*fact/6.58212E-22/MeV 
			      << " +/- " << totsq*fact/6.58212E-22/MeV 
			      << " s-1\n";
      CurrentGenerator::log() << "The maximum weight is " 
			      << _maxweight << "\n";
      CurrentGenerator::log() << "The weights for the different phase" 
			      << " space channels are \n";
      for(unsigned int ix=0,N=_channels.size();ix<N;++ix) {
	CurrentGenerator::log() << "Channel " << ix 
				<< " had weight " << _channelwgts[ix] 
				<< "\n";
      }
      CurrentGenerator::log() << flush;
    }
    output=totsum*fact;
  }
  return output;
}
 
// generate a phase-space point using multichannel phase space
Energy DecayPhaseSpaceMode::channelPhaseSpace(bool cc,
					      int & ichan, const Particle & inpart,
					      ParticleVector & outpart,
					      bool onShell) const {
  // select the channel
  vector<Lorentz5Momentum> momenta;
  double wgt(UseRandom::rnd());
  // select a channel
  ichan=-1;
  do{++ichan;wgt-=_channelwgts[ichan];}
  while(ichan<int(_channels.size())&&wgt>0.);
  // generate the momenta
  if(ichan==int(_channels.size())) {
    throw DecayIntegratorError() << "DecayPhaseSpaceMode::channelPhaseSpace()"
				 << " failed to select a channel" 
				 << Exception::abortnow;
  }
  // generate the masses of the external particles
  double masswgt(1.);
  vector<Energy> mass(externalMasses(inpart.mass(),masswgt,onShell));
  momenta=_channels[ichan]->generateMomenta(inpart.momentum(),mass);
  // compute the denominator of the weight
  wgt=0.;
  unsigned int ix;
  for(ix=0;ix<_channels.size();++ix) {
    wgt+=_channelwgts[ix]*_channels[ix]->generateWeight(momenta);
  }
  // now we need to set the momenta of the particles
  // create the particles if they don't exist
  if(outpart.empty()) {
    for(ix=1;ix<_extpart.size();++ix) {
      if(cc&&_extpart[ix]->CC()) {
	outpart.push_back((_extpart[ix]->CC())->produceParticle());
      }
      else {
	outpart.push_back(_extpart[ix]->produceParticle());
      }
    }
  }
  // set up the momenta
  for(ix=0;ix<outpart.size();++ix) outpart[ix]->set5Momentum(momenta[ix+1]);
  // return the weight
  return wgt!=0. ? inpart.mass()*masswgt/wgt : ZERO;
}

// generate the decay
ParticleVector DecayPhaseSpaceMode::generate(bool intermediates,bool cc,
					     const Particle & inpart) const {
  _integrator->ME(DecayMEPtr());
  // compute the prefactor
  InvEnergy pre(1./MeV); 
  Energy prewid;
  if(_widthgen&&_partial>=0) prewid=_widthgen->partialWidth(_partial,inpart.mass());
  else                       prewid=(inpart.dataPtr()->width());
  pre = prewid>ZERO ? 1./prewid : 1./MeV;
  // Particle vector for the output
  ParticleVector particles;
  // boosts to/from rest
  Boost bv =-inpart.momentum().boostVector();
  double gammarest = inpart.momentum().e()/inpart.momentum().mass();
  LorentzRotation boostToRest( bv,gammarest);
  LorentzRotation boostFromRest(-bv,gammarest);
  // construct a new particle which is at rest
  Particle inrest(inpart);
  inrest.transform(boostToRest);
  int ncount(0),ichan; double wgt(0.);
  unsigned int ix;
  try {
    do {
      wgt=pre*weight(cc,ichan,inrest,particles,ncount==0);
      ++ncount;
      if(wgt>_maxweight) {
	CurrentGenerator::log() << "Resetting max weight for decay " 
				<< inrest.PDGName() << " -> ";
	for(ix=0;ix<particles.size();++ix)
	  CurrentGenerator::log() << "  " << particles[ix]->PDGName();
	CurrentGenerator::log() << "  " << _maxweight << "  " << wgt 
				<< "  " << inrest.mass()/MeV << "\n";
	_maxweight=wgt;
      }
    }
    while(_maxweight*UseRandom::rnd()>wgt&&ncount<_ntry);
    if(ncount>=_ntry) {
      CurrentGenerator::log() << "The decay " << inrest.PDGName() << " -> ";
      for(ix=0;ix<particles.size();++ix) 
	CurrentGenerator::log()  << "  " << particles[ix]->PDGName();
      CurrentGenerator::log() << "  " << _maxweight << " " << _ntry 
			      << " is too inefficient for the particle "
			      << inpart << "vetoing the decay \n";
      particles.clear();
      throw Veto();
    }
  }
  catch (Veto) {
    // restore the incoming particle to its original state
    inrest.transform(boostFromRest);
    throw Veto();
  }
  // set up the vertex for spin correlations
  me2(-1,inrest,particles,DecayIntegrator::Terminate);
  const_ptr_cast<tPPtr>(&inpart)->spinInfo(inrest.spinInfo());
  constructVertex(inpart,particles);
  // return if intermediate particles not required
  if(_channelwgts.empty()||!intermediates) {
    for(ix=0;ix<particles.size();++ix) particles[ix]->transform(boostFromRest);
  }
  // find the intermediate particles
  else {
    // select the channel
    _ichannel = selectChannel(inpart,particles);
    for(ix=0;ix<particles.size();++ix) particles[ix]->transform(boostFromRest);
    // generate the particle vector
    _channels[_ichannel]->generateIntermediates(cc,inpart,particles);
  }
  _integrator->ME(DecayMEPtr());
  return particles;
}

// construct the vertex for spin corrections
void DecayPhaseSpaceMode::constructVertex(const Particle & inpart, 
					  const ParticleVector & decay) const {
  // construct the decay vertex
  VertexPtr vertex(new_ptr(DecayVertex()));
  DVertexPtr Dvertex(dynamic_ptr_cast<DVertexPtr>(vertex));
  // set the incoming particle for the decay vertex
  (inpart.spinInfo())->decayVertex(vertex);
  for(unsigned int ix=0;ix<decay.size();++ix) {
    (decay[ix]->spinInfo())->productionVertex(vertex);
  }
  // set the matrix element
  Dvertex->ME(_integrator->ME());
  _integrator->ME(DecayMEPtr());
}

// output info on the mode
ostream & Herwig::operator<<(ostream & os, const DecayPhaseSpaceMode & decay) {
  os << "The mode has " << decay._channels.size() << " channels\n";
  os << "This is a mode for the decay of " << decay._extpart[0]->PDGName() 
     << " to ";
  for(unsigned int iz=1,N=decay._extpart.size();iz<N;++iz) {
    os << decay._extpart[iz]->PDGName() << " ";
  }
  os << "\n";
  for(unsigned int ix=0;ix<decay._channels.size();++ix) {
    os << "Information on channel " << ix << "\n";
    os << *(decay._channels[ix]);
  }
  return os;
}

void DecayPhaseSpaceMode::init() {
  // check the size of the weight vector is the same as the number of channels
  if(_channelwgts.size()!=numberChannels()) {
    throw Exception() << "Inconsistent number of channel weights and channels"
		      << " in DecayPhaseSpaceMode " << Exception::abortnow;
  }
  _massgen.resize(_extpart.size());
  _widthgen = dynamic_ptr_cast<cGenericWidthGeneratorPtr>(_extpart[0]->widthGenerator());
  for(unsigned int ix=0;ix<_extpart.size();++ix) {
    assert(_extpart[ix]);
    _massgen[ix]= dynamic_ptr_cast<cGenericMassGeneratorPtr>(_extpart[ix]->massGenerator());
  }
  for(unsigned int ix=0;ix<_channels.size();++ix) {
    _channels[ix]->init();
  }
  // set the phase-space cut off
  _eps = _integrator->epsilonPS();
}
  
void DecayPhaseSpaceMode::initrun() {
  // update the mass and width generators
  if(_extpart[0]->widthGenerator()!=_widthgen) {
    _widthgen= dynamic_ptr_cast<cGenericWidthGeneratorPtr>(_extpart[0]->widthGenerator());
  }
  for(unsigned int ix=0;ix<_extpart.size();++ix) {
    if(_massgen[ix]!=_extpart[ix]->massGenerator())
      _massgen[ix] = dynamic_ptr_cast<cGenericMassGeneratorPtr>(_extpart[ix]->massGenerator());
  }
  // check the size of the weight vector is the same as the number of channels
  if(_channelwgts.size()!=numberChannels()) {
    throw Exception() << "Inconsistent number of channel weights and channels"
		      << " in DecayPhaseSpaceMode " << Exception::abortnow;
  }
  for(unsigned int ix=0;ix<_channels.size();++ix) {
    _channels[ix]->initrun();
  }
  if(_widthgen) const_ptr_cast<GenericWidthGeneratorPtr>(_widthgen)->initrun();
  tcGenericWidthGeneratorPtr wtemp;
  for(unsigned int ix=1;ix<_extpart.size();++ix) {
    wtemp=
      dynamic_ptr_cast<tcGenericWidthGeneratorPtr>(_extpart[ix]->widthGenerator());
    if(wtemp) const_ptr_cast<tGenericWidthGeneratorPtr>(wtemp)->initrun();
  }
}

// generate the masses of the external particles
vector<Energy> DecayPhaseSpaceMode::externalMasses(Energy inmass,double & wgt,
						   bool onShell) const {
  vector<Energy> mass(1,inmass);
  mass.reserve(_extpart.size());
  vector<int> notdone;
  Energy mlow(ZERO);
  // set masses of stable particles and limits 
  for(unsigned int ix=1;ix<_extpart.size();++ix) {
    // get the mass of the particle if can't use weight
    if(onShell) {
      mass.push_back(_extpart[ix]->mass());
    }
    else if(!_massgen[ix] || _extpart[ix]->stable()) {
      mass.push_back(_extpart[ix]->generateMass());
      mlow+=mass[ix];
    }
    else {
      mass.push_back(ZERO);
      notdone.push_back(ix);
      mlow+=max(_extpart[ix]->mass()-_extpart[ix]->widthLoCut(),ZERO);
    }
  }
  if(mlow>inmass) {
    // if(_integrator->state()==runready) {
    //   CurrentGenerator::log() << "Decay mode " << _extpart[0]->PDGName() << " -> ";
    //   for(unsigned int ix=1;ix<_extpart.size();++ix) {
    // 	CurrentGenerator::log() << _extpart[ix]->PDGName() << " ";
    //   }
    //   CurrentGenerator::log() << "is below threshold in DecayPhaseMode::externalMasses()"
    // 			      << "the threshold is " << mlow/GeV
    // 			      << "GeV and the parent mass is " << inmass/GeV
    // 			      << " GeV\n";
    // }
    throw Veto();
  }
  // now we need to generate the masses for the particles we haven't
  unsigned int iloc;
  double wgttemp;
  Energy low=ZERO;
  for( ;!notdone.empty();) {
    iloc=long(UseRandom::rnd()*(notdone.size()-1));
    low=max(_extpart[notdone[iloc]]->mass()-_extpart[notdone[iloc]]->widthLoCut(),ZERO);
    mlow-=low;
    mass[notdone[iloc]]=
      _massgen[notdone[iloc]]->mass(wgttemp,*_extpart[notdone[iloc]],low,inmass-mlow);
    assert(mass[notdone[iloc]]>=low&&mass[notdone[iloc]]<=inmass-mlow);
    wgt   *= wgttemp;
    mlow  += mass[notdone[iloc]];
    notdone.erase(notdone.begin()+iloc);
  }
  return mass;
}
#line 1 "./HwDecayerBase.cc"
// -*- C++ -*-
//
// HwDecayerBase.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the HwDecayerBase class.
//

#include "HwDecayerBase.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Interface/Switch.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/Repository/CurrentGenerator.h"
#include "Herwig/Shower/RealEmissionProcess.h"
#include "Herwig/Shower/ShowerHandler.h"

using namespace Herwig;

bool HwDecayerBase::accept(const DecayMode & dm) const {
  // get the primary products
  tPDVector products=dm.orderedProducts();
  // add products for which the decay mode is all ready specified
  if(!dm.cascadeProducts().empty()) {
    for(ModeMSet::const_iterator mit=dm.cascadeProducts().begin();
	mit!=dm.cascadeProducts().end();++mit) {
      products.push_back(const_ptr_cast<PDPtr>((**mit).parent()));
    }
  }
  // can this mode be handled ?
  return accept(dm.parent(),products);
}

ParticleVector HwDecayerBase::decay(const DecayMode & dm,
				    const Particle & p) const {
  // handling of the decay including the special features of the
  // DecayMode  
  // get the primary products
  tPDVector products=dm.orderedProducts();
  // add products for which the decay mode is all ready specified
  if(!dm.cascadeProducts().empty()) {
    for(ModeMSet::const_iterator mit=dm.cascadeProducts().begin();
	mit!=dm.cascadeProducts().end();++mit) {
      products.push_back(const_ptr_cast<PDPtr>((**mit).parent()));
    }
  }
  // perform the primary decay
  ParticleVector output=decay(p,products);
  // perform the secondary decays
  if(!dm.cascadeProducts().empty()) {
    unsigned int iloc=dm.orderedProducts().size();
    for(ModeMSet::const_iterator mit=dm.cascadeProducts().begin();
	mit!=dm.cascadeProducts().end();++mit) {
      if(!(*mit)->decayer()) 
	throw Exception() << "Decay mode " << (**mit).tag() 
			  << "does not have a decayer, can't perform"
			  << "decay in  HwDecayerBase::decay()"
			  << Exception::eventerror;
      ParticleVector children=(*mit)->decayer()->decay(**mit,*output[iloc]);
      for(unsigned int ix=0;ix<children.size();++ix) {
	output[iloc]->addChild(children[ix]);
      }
      ++iloc;
    }
  }
  return output;
}

void HwDecayerBase::persistentOutput(PersistentOStream & os) const {
  os << _initialize << _dbOutput;
}

void HwDecayerBase::persistentInput(PersistentIStream & is, int) {
  is >> _initialize >> _dbOutput;
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeAbstractClass<HwDecayerBase,Decayer>
describeHerwigHwDecayerBase("Herwig::HwDecayerBase", "Herwig.so");

void HwDecayerBase::Init() {

  static ClassDocumentation<HwDecayerBase> documentation
    ("The HwDecayerBase class is the base class for Decayers in Hw++.");

  static Switch<HwDecayerBase,bool> interfaceInitialize
    ("Initialize",
     "Initialization of the phase space calculation",
     &HwDecayerBase::_initialize, false, false, false);
  static SwitchOption interfaceInitializeon
    (interfaceInitialize,
     "Yes",
     "At initialisation find max weight and optimise the integration",
     true);
  static SwitchOption interfaceInitializeoff
    (interfaceInitialize,
     "No",
     "Use the maximum weight and channel weights supplied for the integration",
     false);

  static Switch<HwDecayerBase,bool> interfaceDatabaseOutput
    ("DatabaseOutput",
     "Whether to print the database information",
     &HwDecayerBase::_dbOutput, false, false, false);
  static SwitchOption interfaceDatabaseOutputYes
    (interfaceDatabaseOutput,
     "Yes",
     "Output information on the decayer initialization",
     true);
  static SwitchOption interfaceDatabaseOutputNo
    (interfaceDatabaseOutput,
     "No",
     "Do not output information about the decayer initialization",
     false);
}

void HwDecayerBase::dofinish() {
  Decayer::dofinish();
  if(initialize() && databaseOutput()) {
    string fname = CurrentGenerator::current().filename() + 
      string("-") + name() + string(".output");
    ofstream output(fname.c_str());
    dataBaseOutput(output,true);
  }
}

bool HwDecayerBase::softMatrixElementVeto(PPtr , PPtr,
					  const bool &,
					  const Energy & ,
					  const vector<tcPDPtr> & ,
					  const double & ,
					  const Energy & ,
					  const Energy & ) {
  return false;
}

RealEmissionProcessPtr HwDecayerBase::generateHardest(RealEmissionProcessPtr) {
  return RealEmissionProcessPtr();
}

void HwDecayerBase::initializeMECorrection(RealEmissionProcessPtr , double & ,
			    double & ) {
  assert(false);
}

RealEmissionProcessPtr HwDecayerBase::applyHardMatrixElementCorrection(RealEmissionProcessPtr) {
  assert(false);
  return RealEmissionProcessPtr();
}

void HwDecayerBase::fixRho(RhoDMatrix & rho) const {
  if(ShowerHandler::currentHandlerIsSet() &&
     !ShowerHandler::currentHandler()->spinCorrelations())
    rho.reset();
}
#line 1 "./HwDecayHandler.cc"
// -*- C++ -*-
//
// HwDecayHandler.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the HwDecayHandler class.
//

#include "HwDecayHandler.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Handlers/EventHandler.h"
#include "ThePEG/Handlers/Hint.h"
#include "ThePEG/PDT/ParticleData.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/PDT/Decayer.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/Switch.h"
#include "ThePEG/Interface/RefVector.h"
#include "ThePEG/EventRecord/Step.h"
#include "ThePEG/EventRecord/Collision.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "DecayIntegrator.h"
#include "DecayPhaseSpaceMode.h"
#include "ThePEG/PDT/MixedParticleData.h"
#include "Herwig/Utilities/EnumParticles.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void HwDecayHandler::
handle(EventHandler &, const tPVector & tagged,
       const Hint &) {
  // First go through the tagged particles for unstable ones
  tPVector parents;
  for(int i = 0, N = tagged.size(); i<N; ++i) {
    if(tagged[i]) {
      // add to parents if not stable
      if(!tagged[i]->data().stable() &&
	 tagged[i]->data().id() != ParticleID::Remnant &&
	 _excluded.find( tagged[i]->dataPtr() ) == _excluded.end() ) {
	parents.push_back(tagged[i]);
      }
      // if stable and has spinInfo set the developed flag
      else {
	develop(tagged[i]);
      }
    }
  }
  // if nothing to be decayed return
  if(parents.empty()) return;
  useMe();
  // Create a new step, decay all particles and add their children to the step
  StepPtr newstep = _newstep ? newStep() : currentStep();
  for(int i = 0, N = parents.size(); i<N; ++i) {
    performDecay(newstep->find(parents[i]), *newstep);
  }
}

// perform decay method including modifications for spin correlations
// and for the decayer to specify intermediate decay products
void HwDecayHandler::performDecay(tPPtr parent, Step & s) const {
  long ntry = 0;
  tcSpinPtr hwspin;
  tcMixedParticleDataPtr 
    mixdata=dynamic_ptr_cast<tcMixedParticleDataPtr>(parent->dataPtr());
  if(mixdata) {
    pair<bool,Length> mixing = mixdata->generateLifeTime();
    develop(parent);
    parent->setLifeLength(Distance());
    PPtr newparent;
    if(mixing.first) {
      newparent = parent->dataPtr()->CC()->
	produceParticle(parent->momentum());
    }
    else {
      newparent = parent->dataPtr()      ->
	produceParticle(parent->momentum());
    }
    newparent->setLabVertex(parent->labDecayVertex());
    Lorentz5Distance lifeLength(mixing.second,
				parent->momentum().vect()*
				(mixing.second/parent->mass()));
    newparent->setLifeLength(lifeLength);
    s.addDecayProduct(parent, newparent);
    parent = newparent;
  }
  else if ( maxLifeTime() >= ZERO ) {
    if( ( lifeTimeOption() && parent->lifeLength().tau() > maxLifeTime())||
	(!lifeTimeOption() && parent->data().cTau()      > maxLifeTime()) ) {
      parent->setLifeLength(Distance());
      develop(parent);
      return;
    }
  }
  while ( true ) {
    // exit if fails
    if ( ++ntry >= maxLoop() ) 
      throw Exception() << "Too many tries " << maxLoop() << "to generate decay of "
			<< *parent << "in "
			<< "HwDecayHandler::performDecay" << Exception::eventerror;
    // select the decay mode
    tDMPtr   dm(parent->data().selectMode(*parent));
    // check we found a decay mode and it had a decayer
    if ( !dm ) {
      generator()->log() << *generator()->currentEvent() << "\n";
      generator()->log() << *parent << "\n";
      throw Exception() << "No DecayModes for " << parent->PDGName()
			<< " in HwDecayHandler::performDecay" 
			<< Exception::eventerror;
    }
    if ( !dm->decayer() ) throw Exception() << "No decayer for DecayMode of " 
					    << parent->PDGName()
					    << " in HwDecayHandler::performDecay" 
					    << Exception::eventerror;
    try {
      ParticleVector children = dm->decayer()->decay(*dm, *parent);
      if(children.empty()) continue;
      assert(parent->children().empty());
      // generate radiation in the decay
      tDecayIntegratorPtr hwdec=dynamic_ptr_cast<tDecayIntegratorPtr>(dm->decayer());
      if (hwdec && hwdec->canGeneratePhotons())
	children = hwdec->generatePhotons(*parent,children);
      // set up parent
      parent->decayMode(dm);
      // add children
      for ( int i = 0, N = children.size(); i < N; ++i ) {
	children[i]->setLabVertex(parent->labDecayVertex());
	if ( !s.addDecayProduct(parent, children[i]) ) 
	  throw Exception() << "Failed to add child " 
			    << children[i]->PDGName() 
			    << " in decay of " << parent->PDGName() 
			    << Exception::eventerror;
      }
      parent->scale(ZERO);
      // loop over the children
      for ( int i = 0, N = children.size(); i < N; ++i ) {
	// if the child has already been decayed add products to the record
	if(children[i]->decayed()) addDecayedParticle(children[i],s);
	// if not stable decay the child
	else if (!children[i]->data().stable() &&
		 _excluded.find( children[i]->dataPtr() ) == _excluded.end() ) {
	  performDecay(children[i], s);
	}
	// if stable and has spinInfo set up decay matrices etc.
	else {
	  develop(children[i]);
	}
      }
      // sort out the spinInfo for the parent after the decays
      if(parent->spinInfo()) parent->spinInfo()->develop();
      return;
    }
    catch (Veto) 
      {}
  }
}

// method to add an intermediate which has already been decayed to the event record
void HwDecayHandler::addDecayedParticle(tPPtr parent, Step & s) const {
  for ( int i = 0, N = parent->children().size(); i < N; ++i ) {
    parent->children()[i]->setLabVertex(parent->labDecayVertex());
    s.addDecayProduct(parent->children()[i]);
  }
  parent->scale(ZERO);
  for ( int i = 0, N = parent->children().size(); i < N; ++i ) {
    if((parent->children()[i])->decayed()) {
      for(unsigned int ix=0;ix<(parent->children()[i])->children().size();++ix)
	addDecayedParticle(parent->children()[i],s);
    }
    else if ( ! parent->children()[i]->data().stable() &&
	      _excluded.find( parent->children()[i]->dataPtr() ) == _excluded.end() ) {
      performDecay(parent->children()[i], s);
    }
    else {
      develop(parent->children()[i]);
    }
  }
  return;
}

void HwDecayHandler::persistentOutput(PersistentOStream & os) const {
  os << _newstep << _excluded << _excludedVector;
}

void HwDecayHandler::persistentInput(PersistentIStream & is, int)  {
  is >> _newstep >> _excluded >> _excludedVector;
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<HwDecayHandler,DecayHandler>
describeHerwigHwDecayHandler("Herwig::HwDecayHandler", "Herwig.so");

void HwDecayHandler::Init() {

  static ClassDocumentation<HwDecayHandler> documentation
    ("This is the handler for decays in Herwig.",
     "Decays in Herwig include full spin correlations, based on \\cite{Richardson:2001df}.",
     "%\\cite{Richardson:2001df}\n"
     "\\bibitem{Richardson:2001df}\n"
     "  P.~Richardson,\n"
     "  ``Spin correlations in Monte Carlo simulations,''\n"
     "  JHEP {\\bf 0111}, 029 (2001)\n"
     "  [arXiv:hep-ph/0110108].\n"
     "  %%CITATION = JHEPA,0111,029;%%\n"
     );

  static Switch<HwDecayHandler,bool> interfaceNewStep
    ("NewStep",
     "Add the particles in a new step",
     &HwDecayHandler::_newstep, true, false, false);
  static SwitchOption interfaceNewStepNew
    (interfaceNewStep,
     "Yes",
     "Add particles in a new step",
     true);
  static SwitchOption interfaceNewStepCurrent
    (interfaceNewStep,
     "No",
     "Add them in the current step",
     false);

  static RefVector<HwDecayHandler,ParticleData> interfaceExcluded
    ("Excluded",
     "Particles which should not be decayed",
     &HwDecayHandler::_excludedVector, -1, false, false, true, false, false);

}

void HwDecayHandler::doinit() {
  DecayHandler::doinit();
  _excluded = set<tcPDPtr>(_excludedVector.begin(),_excludedVector.end());
}
#line 1 "./DecayVertex.cc"
// -*- C++ -*-
//
// DecayVertex.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the DecayVertex class.
//
//  Author: Peter Richardson
//

#include <ThePEG/EventRecord/SpinInfo.h>
#include "DecayVertex.h"
#include "ThePEG/Utilities/DescribeClass.h"

using namespace Herwig;
using namespace ThePEG;

// Static variable needed for the type description system in ThePEG.
DescribeNoPIOClass<DecayVertex,HelicityVertex>
describeHerwigDecayVertex("Herwig::DecayVertex", "Herwig.so");
    
void DecayVertex::Init() {
  
  static ClassDocumentation<DecayVertex> documentation
    ("The DecayVertex class is the implementation of a "
     "vertex for a decay for the Herwig spin correlation algorithm");
  
}

// method to get the rho matrix for a given outgoing particle
RhoDMatrix DecayVertex::getRhoMatrix(int i,bool recursive) const {
  // get the rho matrix of the decaying particle
  RhoDMatrix input;
  tcSpinPtr inspin = incoming()[0];
  assert(inspin);
  if(recursive&&inspin->productionVertex()&&
     inspin->iSpin()!=PDT::Spin0) {
    input = inspin->productionVertex()->
      getRhoMatrix(inspin->productionLocation(),true);
    inspin->rhoMatrix() = input;
    inspin->needsUpdate();
  }
  else {
    input = inspin->rhoMatrix();
  }
  // get the D matrices for the outgoing particles
  vector<RhoDMatrix> rhoout(outgoing().size()-1);
  for(int ix=0,N=outgoing().size();ix<N;++ix) {
    if(ix<i)      
      rhoout[ix]   = outgoing()[ix]->DMatrix();
    else if(ix>i) 
      rhoout[ix-1] = outgoing()[ix]->DMatrix();
  }
  // calculate the spin density matrix
  return matrixElement_->calculateRhoMatrix(i,input,rhoout);
}

// method to get the D matrix for an incoming particle
RhoDMatrix DecayVertex::getDMatrix(int) const {
  tcSpinPtr inspin = incoming()[0];
  if(inspin->developed()==SpinInfo::Developed) 
    return inspin->DMatrix();
  // get the decay matrices for the outgoing particles
  vector<RhoDMatrix> Dout(outgoing().size());
  for(unsigned int ix=0,N=outgoing().size();ix<N;++ix) {
    tcSpinPtr hwspin = outgoing()[ix];
    if(hwspin->developed()!=SpinInfo::Developed) 
      hwspin->develop();
    Dout[ix] = hwspin->DMatrix();
  }
  // calculate the spin density matrix and return the answer
  return matrixElement_->calculateDMatrix(Dout);
}
#line 1 "./DecayMatrixElement.cc"
// -*- C++ -*-
//
// This is the implementation of the non-inlined, non-templated member
// functions of the DecayMatrixElement class.
//

#include "DecayMatrixElement.h"

using namespace Herwig;

DecayMatrixElement::~DecayMatrixElement() {}

#line 1 "./TwoBodyDecayMatrixElement.cc"
// -*- C++ -*-
//
// TwoBodyMatrixElement.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the TwoBodyMatrixElement class.
//
// Author: Peter Richardson
//

#include "TwoBodyDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG;
    
// calculate the decay matrix for this decay
RhoDMatrix TwoBodyDecayMatrixElement::
calculateDMatrix(const vector<RhoDMatrix> & rhoout) const {
  // rhomatrix to be returned
  RhoDMatrix output(inspin(), false);
  // loop over all helicity components of the matrix element
  for(int ihel1=0;ihel1<outspin()[0];++ihel1) {
    for(int jhel1=0;jhel1<outspin()[0];++jhel1) {
      if(rhoout[0](ihel1,jhel1)==0.)continue;
      for(int ihel2=0;ihel2<outspin()[1];++ihel2) {
        for(int jhel2=0;jhel2<outspin()[1];++jhel2) {
          if(rhoout[1](ihel2,jhel2)==0.)continue;
  	  for(int ihel0=0;ihel0<inspin();++ihel0) {
            for(int jhel0=0;jhel0<inspin();++jhel0) {
      
	      output(ihel0,jhel0) += 
		     matrixElement_[ihel0][ihel1][ihel2]*
		conj(matrixElement_[jhel0][jhel1][jhel2])*
		rhoout[0](ihel1,jhel1)*rhoout[1](ihel2,jhel2);
	    }
	  }
	}
      }
    }
  }
  // ensure unit trace for the matrix
  output.normalize();
  // return the answer
  return output;
}

// calculate the rho matrix for a given outgoing particle
RhoDMatrix TwoBodyDecayMatrixElement::
calculateRhoMatrix(int id,const RhoDMatrix & rhoin,
		   const vector<RhoDMatrix> & rhoout) const {
  // rhomatrix to be returned
  RhoDMatrix output(outspin()[id], false);
  // loop over all helicity components of the matrix element
  if(id==0) {
    for(int ihel0=0;ihel0<inspin();++ihel0) {
      for(int jhel0=0;jhel0<inspin();++jhel0) {
        if (rhoin(ihel0,jhel0)==0.)continue;
        for(int ihel2=0;ihel2<outspin()[1];++ihel2) {
          for(int jhel2=0;jhel2<outspin()[1];++jhel2) {
            if(rhoout[0](ihel2,jhel2)==0.)continue;
            for(int ihel1=0;ihel1<outspin()[0];++ihel1) {
              if(matrixElement_[ihel0][ihel1][ihel2]==0.)continue;
              for(int jhel1=0;jhel1<outspin()[0];++jhel1) {
                output(ihel1,jhel1) +=
                matrixElement_[ihel0][ihel1][ihel2]*
                conj(matrixElement_[jhel0][jhel1][jhel2])*
                rhoin(ihel0,jhel0)*
                rhoout[0](ihel2,jhel2);
              }
            }
          }
        }
      }
    }
  }
  else {
    for(int ihel0=0;ihel0<inspin();++ihel0) {
      for(int jhel0=0;jhel0<inspin();++jhel0) {
        if (rhoin(ihel0,jhel0)==0.)continue;
	for(int ihel1=0;ihel1<outspin()[0];++ihel1) {
	  for(int jhel1=0;jhel1<outspin()[0];++jhel1) {
            if (rhoout[0](ihel1,jhel1)==0.)continue;
	    for(int ihel2=0;ihel2<outspin()[1];++ihel2) {
	      for(int jhel2=0;jhel2<outspin()[1];++jhel2) {
		output(ihel2,jhel2) += 
		  matrixElement_[ihel0][ihel1][ihel2]*
		  conj(matrixElement_[jhel0][jhel1][jhel2])*
		  rhoin(ihel0,jhel0)*
		  rhoout[0](ihel1,jhel1);
	      }
	    }
	  }
	}
      }
    }
  }
  // return the answer
  output.normalize();
  return output;
}

// contract the matrix element with the rho matrix of the incoming particle
Complex TwoBodyDecayMatrixElement::contract(const RhoDMatrix & in) const {
  Complex me=0.;
  for(int ihel0=0;ihel0<inspin();++ihel0) {
    for(int jhel0=0;jhel0<inspin();++jhel0) {
      for(int ihel1=0;ihel1<outspin()[0];++ihel1) {
	  for(int ihel2=0;ihel2<outspin()[1];++ihel2) {
	      me +=  matrixElement_[ihel0][ihel1][ihel2]*
		conj(matrixElement_[jhel0][ihel1][ihel2])*in(ihel0,jhel0);
	  }
      }
    }
  }
  return me;
}

// contract the matrix element with the rho matrix of the incoming particle
Complex TwoBodyDecayMatrixElement::contract(const TwoBodyDecayMatrixElement & con, 
					    const RhoDMatrix & in) {
  Complex me=0.;
  for(int ihel0=0;ihel0<inspin();++ihel0) {
    for(int jhel0=0;jhel0<inspin();++jhel0) {
      for(int ihel1=0;ihel1<outspin()[0];++ihel1) {
	for(int ihel2=0;ihel2<outspin()[1];++ihel2) {
	  // compute the term
	  me += matrixElement_[ihel0][ihel1][ihel2]*
	    conj(con.matrixElement_[jhel0][ihel1][ihel2])*in(ihel0,jhel0);
	}
      }
    }
  }
  return me;
}
#line 1 "./GeneralDecayMatrixElement.cc"
// -*- C++ -*-
//
// GeneralDecayMatrixElement.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the GeneralDecayMatrixElement class.
//
// Author: Peter Richardson
//

#include "GeneralDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG;
    
// calculate the decay matrix for this decay
RhoDMatrix GeneralDecayMatrixElement::calculateDMatrix(const vector<RhoDMatrix> & rhoout) const {
  // vectors for the helicities
  vector<int> ihel1(outspin().size()+1),ihel2(outspin().size()+1);
  // rhomatrix to be returned
  RhoDMatrix output(inspin(), false);
  // loop over all helicity components of the matrix element
  // outer loop
  Complex temp;
  unsigned int ix,iy,iz;
  int ixa,iya;
  for(ix=0;ix<matrixElement_.size();++ix) {
    // map the vector index to the helicities
    for(ixa=outspin().size();ixa>=0;--ixa) 
      ihel1[ixa]=(ix%constants_[ixa])/constants_[ixa+1];
    // inner loop
    for(iy=0;iy<matrixElement_.size();++iy) {
      // map the vector index to the helicities	   
      for(iya=outspin().size();iya>=0;--iya)
	ihel2[iya]=(iy%constants_[iya])/constants_[iya+1];
      // matrix element piece
      temp=matrixElement_[ix]*conj(matrixElement_[iy]);
      // spin density matrices for the outgoing particles
      for(iz=0;iz<outspin().size();++iz)
	temp*=rhoout[iz](ihel1[iz+1],ihel2[iz+1]);
      output(ihel1[0],ihel2[0])+=temp;
    }
  }
  // ensure unit trace for the matrix
  output.normalize();
  // return the answer
  return output;
}

// calculate the rho matrix for a given outgoing particle
RhoDMatrix GeneralDecayMatrixElement::
calculateRhoMatrix(int id,const RhoDMatrix & rhoin,
		   const vector<RhoDMatrix> & rhoout) const {
  // vectors for the helicities
  vector<int> ihel1(outspin().size()+1),ihel2(outspin().size()+1);
  // rhomatrix to be returned
  RhoDMatrix output(outspin()[id], false);
  // loop over all helicity components of the matrix element
  // outer loop
  Complex temp;
  unsigned int ix,iy,iz;
  int ixa,iya;
  for(ix=0;ix<matrixElement_.size();++ix) {
    // map the vector index to the helicities
    for(ixa=outspin().size();ixa>=0;--ixa)
      ihel1[ixa]=(ix%constants_[ixa])/constants_[ixa+1];
    // inner loop
    for(iy=0;iy<matrixElement_.size();++iy) {
      // map the vector index to the helicities	   
      for(iya=outspin().size();iya>=0;--iya)
	ihel2[iya]=(iy%constants_[iya])/constants_[iya+1];
      // matrix element piece
      temp=matrixElement_[ix]*conj(matrixElement_[iy]);
      // spin denisty matrix for the incoming particle
      temp *= rhoin(ihel1[0],ihel2[0]);
      // spin density matrix for the outgoing particles
      for(iz=0;iz<outspin().size()-1;++iz) {
	if(int(iz)<id) temp*=rhoout[iz](ihel1[iz+1],ihel2[iz+1]);
	else           temp*=rhoout[iz](ihel1[iz+2],ihel2[iz+2]);
      }
      // add to the rho matrix
      output(ihel1[id+1],ihel2[id+1])+=temp;
    }
  }
  // return the answer
  output.normalize();
  return output;
}

// contract the matrix element with the rho matrix of the incoming particle
Complex GeneralDecayMatrixElement::contract(const RhoDMatrix & in) const {
  unsigned int ispin(abs(int(inspin())));
  Complex me=0.;
  for(unsigned int ix=0;ix<constants_[1];++ix) {
    for(unsigned int inhel1=0;inhel1<ispin;++inhel1) {
      for(unsigned int inhel2=0;inhel2<ispin;++inhel2) {
	// compute the term
	me+=matrixElement_[inhel1*constants_[1]+ix]*
	  conj(matrixElement_[inhel2*constants_[1]+ix])*in(inhel1,inhel2);
      }
    }
  }
  return me;
}

// contract the matrix element with the rho matrix of the incoming particle
Complex GeneralDecayMatrixElement::contract(const GeneralDecayMatrixElement & con, 
					    const RhoDMatrix & in) {
  unsigned int ispin(abs(int(inspin())));
  Complex me=0.;
  unsigned int ix,inhel1,inhel2;
  for(ix=0;ix<constants_[1];++ix) {
    for(inhel1=0;inhel1<ispin;++inhel1) {
      for(inhel2=0;inhel2<ispin;++inhel2) {
	// compute the term
	me+=matrixElement_[inhel1*constants_[1]+ix]*
	  conj(con.matrixElement_[inhel2*constants_[1]+ix])*in(inhel1,inhel2);
      }
    }
  }
  return me;
}
#line 1 "./BranchingRatioReweighter.cc"
// -*- C++ -*-
//
// This is the implementation of the non-inlined, non-templated member
// functions of the BranchingRatioReweighter class.
//

#include "BranchingRatioReweighter.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/EventRecord/Particle.h"
#include "ThePEG/EventRecord/Event.h"
#include "ThePEG/Handlers/EventHandler.h"
#include "ThePEG/Repository/UseRandom.h"
#include "ThePEG/Repository/EventGenerator.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/EventRecord/StandardSelectors.h"
#include "ThePEG/Handlers/StandardEventHandler.h"
#include "Herwig/Utilities/EnumParticles.h"

using namespace Herwig;

BranchingRatioReweighter::BranchingRatioReweighter() {}

BranchingRatioReweighter::~BranchingRatioReweighter() {}

void BranchingRatioReweighter::
handle(EventHandler & eh,const tPVector & ,const Hint & ) {
  tEventPtr event = eh.currentEvent();
  // weight
  double weight = 1.;
  // get all the particles
  set<tcPPtr> particles;
  event->select(inserter(particles),ThePEG::AllSelector());
  for(set<tcPPtr>::const_iterator it=particles.begin();it!=particles.end();++it) {
    // skip stable
    if((**it).dataPtr()->stable()) continue;
    // skip remnant and clusters
    if((**it).id()==ParticleID::Remnant || 
       (**it).id()==ParticleID::Cluster) continue;
    // if spacelike skip
    if((**it).mass()<ZERO) continue;
    if(*it == event->incoming().first || 
       *it == event->incoming().second ) continue;
    // find unique particles
    bool unique = true;
    for(unsigned int ix=0;ix<(**it).children().size();++ix) {
      if((**it).children()[ix]->id()==(**it).id()) {
	unique = false;
	break;
      }
    }
    if(!unique) continue;
    weight *= (**it).dataPtr()->decaySelector().sum();
  }
  // do the reweighting
  if ( dynamic_cast<StandardEventHandler*>(&eh) ) {
    StandardEventHandler& seh = 
      dynamic_cast<StandardEventHandler&>(eh);
    seh.reweight(weight);
  }
}

IBPtr BranchingRatioReweighter::clone() const {
  return new_ptr(*this);
}

IBPtr BranchingRatioReweighter::fullclone() const {
  return new_ptr(*this);
}

// The following static variable is needed for the type description system in ThePEG.
DescribeNoPIOClass<BranchingRatioReweighter,StepHandler>
describeHerwigBranchingRatioReweighter("Herwig::BranchingRatioReweighter", "Herwig.so");

void BranchingRatioReweighter::Init() {

  static ClassDocumentation<BranchingRatioReweighter> documentation
    ("The BranchingRatioReweighter class reweights events if some"
     " decay modes are switched off");

}

#line 1 "./PerturbativeDecayer.cc"
// -*- C++ -*-
//
// This is the implementation of the non-inlined, non-templated member
// functions of the PerturbativeDecayer class.
//

#include "PerturbativeDecayer.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/EventRecord/Particle.h"
#include "ThePEG/Repository/UseRandom.h"
#include "ThePEG/Repository/EventGenerator.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/Interface/Reference.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/Interface/Switch.h"
#include "ThePEG/Utilities/EnumIO.h"

using namespace Herwig;

void PerturbativeDecayer::persistentOutput(PersistentOStream & os) const {
  os << ounit(pTmin_,GeV) << oenum(inter_) << alphaS_ << alphaEM_
     << useMEforT2_ << C_ << ymax_ << phaseOpt_;
}

void PerturbativeDecayer::persistentInput(PersistentIStream & is, int) {
  is >> iunit(pTmin_,GeV) >> ienum(inter_) >> alphaS_ >> alphaEM_
     >> useMEforT2_ >> C_ >> ymax_ >> phaseOpt_;
}


// The following static variable is needed for the type
// description system in ThePEG.
DescribeAbstractClass<PerturbativeDecayer,DecayIntegrator>
describeHerwigPerturbativeDecayer("Herwig::PerturbativeDecayer",
				  "Herwig.so HwPerturbativeDecay.so");

void PerturbativeDecayer::Init() {

  static ClassDocumentation<PerturbativeDecayer> documentation
    ("The PerturbativeDecayer class is the mase class for "
     "perturbative decays in Herwig");

  static Parameter<PerturbativeDecayer,Energy> interfacepTmin
    ("pTmin",
     "Minimum transverse momentum from gluon radiation",
     &PerturbativeDecayer::pTmin_, GeV, 1.0*GeV, 0.0*GeV, 10.0*GeV,
     false, false, Interface::limited);

  static Switch<PerturbativeDecayer,ShowerInteraction> interfaceInteractions
    ("Interactions",
     "which interactions to include for the hard corrections",
     &PerturbativeDecayer::inter_, ShowerInteraction::QCD, false, false);
  static SwitchOption interfaceInteractionsQCD
    (interfaceInteractions,
     "QCD",
     "QCD Only",
     ShowerInteraction::QCD);
  static SwitchOption interfaceInteractionsQED
    (interfaceInteractions,
     "QED",
     "QED only",
     ShowerInteraction::QED);
  static SwitchOption interfaceInteractionsQCDandQED
    (interfaceInteractions,
     "QCDandQED",
     "Both QCD and QED",
     ShowerInteraction::Both);

  static Reference<PerturbativeDecayer,ShowerAlpha> interfaceAlphaS
    ("AlphaS",
     "Object for the coupling in the generation of hard QCD radiation",
     &PerturbativeDecayer::alphaS_, false, false, true, true, false);

  static Reference<PerturbativeDecayer,ShowerAlpha> interfaceAlphaEM
    ("AlphaEM",
     "Object for the coupling in the generation of hard QED radiation",
     &PerturbativeDecayer::alphaEM_, false, false, true, true, false);

  static Switch<PerturbativeDecayer,bool> interfaceUseMEForT2
    ("UseMEForT2",
     "Use the matrix element correction, if available to fill the T2"
     " region for the decay shower and don't fill using the shower",
     &PerturbativeDecayer::useMEforT2_, true, false, false);
  static SwitchOption interfaceUseMEForT2Shower
    (interfaceUseMEForT2,
     "Shower",
     "Use the shower to fill the T2 region",
     false);
  static SwitchOption interfaceUseMEForT2ME
    (interfaceUseMEForT2,
     "ME",
     "Use the Matrix element to fill the T2 region",
     true);

    static Parameter<PerturbativeDecayer,double> interfacePrefactor
    ("Prefactor",
     "The prefactor for the sampling of the powheg Sudakov",
     &PerturbativeDecayer::C_, 6.3, 0.0, 1e10,
     false, false, Interface::limited);

  static Parameter<PerturbativeDecayer,double> interfaceYMax
    ("YMax",
     "The maximum value for the rapidity",
     &PerturbativeDecayer::ymax_, 10., 0.0, 100.,
     false, false, Interface::limited);

  static Switch<PerturbativeDecayer,unsigned int> interfacePhaseSpaceOption
    ("PhaseSpaceOption",
     "Option for the phase-space sampling",
     &PerturbativeDecayer::phaseOpt_, 0, false, false);
  static SwitchOption interfacePhaseSpaceOptionFixedYLimits
    (interfacePhaseSpaceOption,
     "FixedYLimits",
     "Use a fixed limit for the rapidity",
     0);
  static SwitchOption interfacePhaseSpaceOptionVariableYLimits
    (interfacePhaseSpaceOption,
     "VariableYLimits",
     "Change limit for the rapidity with pT",
     1);

}

double PerturbativeDecayer::matrixElementRatio(const Particle & , 
					       const ParticleVector & ,
					       const ParticleVector & , 
					       MEOption ,
					       ShowerInteraction ) {
  throw Exception() << "Base class PerturbativeDecayer::matrixElementRatio() "
		    << "called, should have an implementation in the inheriting class"
		    << Exception::runerror;
  return 0.;
}

RealEmissionProcessPtr PerturbativeDecayer::generateHardest(RealEmissionProcessPtr born) {
  return getHardEvent(born,false,inter_);
}

RealEmissionProcessPtr PerturbativeDecayer::applyHardMatrixElementCorrection(RealEmissionProcessPtr born) {
  return getHardEvent(born,true,ShowerInteraction::QCD);
}

RealEmissionProcessPtr PerturbativeDecayer::getHardEvent(RealEmissionProcessPtr born,
							 bool inDeadZone,
							 ShowerInteraction inter) {
  // check one incoming
  assert(born->bornIncoming().size()==1);
  // search for coloured/charged particles
  bool colouredParticles=born->bornIncoming()[0]->dataPtr()->coloured();
  bool chargedParticles=born->bornIncoming()[0]->dataPtr()->charged();
  for(unsigned int ix=0;ix<born->bornOutgoing().size();++ix) {
    if(born->bornOutgoing()[ix]->dataPtr()->coloured())
      colouredParticles=true;
    if(born->bornOutgoing()[ix]->dataPtr()->charged())
      chargedParticles=true;
  }
  // if no coloured/charged particles return
  if ( !colouredParticles && !chargedParticles ) return RealEmissionProcessPtr();
  if ( !colouredParticles && inter==ShowerInteraction::QCD ) return RealEmissionProcessPtr();
  if ( ! chargedParticles && inter==ShowerInteraction::QED ) return RealEmissionProcessPtr();
  // check exactly two outgoing particles
  if(born->bornOutgoing().size()==2) return RealEmissionProcessPtr();
  // for decay b -> a c 
  // set progenitors
  PPtr cProgenitor = born->bornOutgoing()[0];
  PPtr aProgenitor = born->bornOutgoing()[1];
  // get the decaying particle
  PPtr bProgenitor = born->bornIncoming()[0];
  // identify which dipoles are required
  vector<DipoleType> dipoles;
  if(!identifyDipoles(dipoles,aProgenitor,bProgenitor,cProgenitor,inter)) {
    return RealEmissionProcessPtr();
  }
  Energy trialpT = pTmin_;
  LorentzRotation eventFrame;
  vector<Lorentz5Momentum> momenta;
  vector<Lorentz5Momentum> trialMomenta(4);
  PPtr finalEmitter, finalSpectator;
  PPtr trialEmitter, trialSpectator;
  DipoleType finalType(FFa,ShowerInteraction::QCD);
  for (int i=0; i<int(dipoles.size()); ++i) {
    if(dipoles[i].type==FFg) continue;
    // assign emitter and spectator based on current dipole
    if (dipoles[i].type==FFc || dipoles[i].type==IFc || dipoles[i].type==IFbc) {
      trialEmitter   = cProgenitor;
      trialSpectator = aProgenitor;
    }
    else if (dipoles[i].type==FFa || dipoles[i].type==IFa || dipoles[i].type==IFba) {
      trialEmitter   = aProgenitor;
      trialSpectator = cProgenitor;
    }
    
    // find rotation from lab to frame with the spectator along -z
    LorentzRotation trialEventFrame(bProgenitor->momentum().findBoostToCM());
    Lorentz5Momentum pspectator = (trialEventFrame*trialSpectator->momentum());
    trialEventFrame.rotateZ( -pspectator.phi() );
    trialEventFrame.rotateY( -pspectator.theta() - Constants::pi );
    // invert it
    trialEventFrame.invert();
    // try to generate an emission
    pT_ = pTmin_;
    vector<Lorentz5Momentum> trialMomenta 
      = hardMomenta(bProgenitor, trialEmitter, trialSpectator,
		    dipoles, i, inDeadZone);
    // select dipole which gives highest pT emission
    if(pT_>trialpT) {
      trialpT        = pT_;
      momenta        = trialMomenta;
      eventFrame     = trialEventFrame;
      finalEmitter   = trialEmitter;
      finalSpectator = trialSpectator;
      finalType      = dipoles[i];
      if (dipoles[i].type==FFc || dipoles[i].type==FFa ) {
      	if((momenta[3]+momenta[1]).m2()-momenta[1].m2()>
	   (momenta[3]+momenta[2]).m2()-momenta[2].m2()) {
      	  swap(finalEmitter,finalSpectator);
      	  swap(momenta[1],momenta[2]);
      	}
      }
    }
  }
  pT_ = trialpT;
  // if no emission return
  if(momenta.empty()) {
    if(inter==ShowerInteraction::Both || inter==ShowerInteraction::QCD)
      born->pT()[ShowerInteraction::QCD] = pTmin_;
    if(inter==ShowerInteraction::Both || inter==ShowerInteraction::QED)
      born->pT()[ShowerInteraction::QED] = pTmin_;
    return born;
  }
  // rotate momenta back to the lab
  for(unsigned int ix=0;ix<momenta.size();++ix) {
    momenta[ix] *= eventFrame;
  }
 
  // set maximum pT for subsequent branchings
  if(inter==ShowerInteraction::Both || inter==ShowerInteraction::QCD)
    born->pT()[ShowerInteraction::QCD] = pT_;
  if(inter==ShowerInteraction::Both || inter==ShowerInteraction::QED)
    born->pT()[ShowerInteraction::QED] = pT_;

  // get ParticleData objects
  tcPDPtr b = bProgenitor   ->dataPtr();
  tcPDPtr e = finalEmitter  ->dataPtr();
  tcPDPtr s = finalSpectator->dataPtr();
  
  tcPDPtr boson  = getParticleData(finalType.interaction==ShowerInteraction::QCD ?
				   ParticleID::g : ParticleID::gamma);

  // create new ShowerParticles
  PPtr emitter   = e    ->produceParticle(momenta[1]);
  PPtr spectator = s    ->produceParticle(momenta[2]);
  PPtr gauge     = boson->produceParticle(momenta[3]);
  PPtr incoming  = b    ->produceParticle(bProgenitor->momentum());

  // insert the particles
  born->incoming().push_back(incoming);
  unsigned int iemit(0),ispect(0);
  for(unsigned int ix=0;ix<born->bornOutgoing().size();++ix) {
    if(born->bornOutgoing()[ix]==finalEmitter) {
      born->outgoing().push_back(emitter);
      iemit = born->outgoing().size();
    }
    else if(born->bornOutgoing()[ix]==finalSpectator) {
      born->outgoing().push_back(spectator);
      ispect = born->outgoing().size();
    }
  }
  born->outgoing().push_back(gauge);
  if(!spectator->dataPtr()->coloured() ||
     (finalType.type != FFa && finalType.type!=FFc) ) ispect = 0;
  born->emitter(iemit);
  born->spectator(ispect);
  born->emitted(3);
  // boost if being use as ME correction
  if(inDeadZone) {
    if(finalType.type==IFa || finalType.type==IFba) {
      LorentzRotation trans(cProgenitor->momentum().findBoostToCM());
      trans.boost(spectator->momentum().boostVector());
      born->transformation(trans);
    }
    else if(finalType.type==IFc || finalType.type==IFbc) {
      LorentzRotation trans(bProgenitor->momentum().findBoostToCM());
      trans.boost(spectator->momentum().boostVector());
      born->transformation(trans);
    }
  }
  // set the interaction
  born->interaction(finalType.interaction);
  // set up colour lines
  getColourLines(born);
  // return the tree
  return born;
}

bool PerturbativeDecayer::identifyDipoles(vector<DipoleType>  & dipoles,
					  PPtr & aProgenitor,
					  PPtr & bProgenitor,
					  PPtr & cProgenitor,
					  ShowerInteraction inter) const {
  enhance_ = 1.;
  // identify any QCD dipoles
  if(inter==ShowerInteraction::QCD ||
     inter==ShowerInteraction::Both) {
    PDT::Colour bColour = bProgenitor->dataPtr()->iColour();
    PDT::Colour cColour = cProgenitor->dataPtr()->iColour();
    PDT::Colour aColour = aProgenitor->dataPtr()->iColour();
    
    // decaying colour singlet
    if    (bColour==PDT::Colour0 ) {
      if ((cColour==PDT::Colour3    && aColour==PDT::Colour3bar) ||
	  (cColour==PDT::Colour3bar && aColour==PDT::Colour3)    ||
	  (cColour==PDT::Colour8    && aColour==PDT::Colour8)){
	dipoles.push_back(DipoleType(FFa,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(FFc,ShowerInteraction::QCD));
	if(aProgenitor->id()==ParticleID::g &&
	   cProgenitor->id()==ParticleID::g ) {
	  enhance_ = 1.5;
	  dipoles.push_back(DipoleType(FFg,ShowerInteraction::QCD));
	}
      }
    }
    // decaying colour triplet
    else if (bColour==PDT::Colour3 ) {
      if (cColour==PDT::Colour3 && aColour==PDT::Colour0){
	dipoles.push_back(DipoleType(IFbc,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFc ,ShowerInteraction::QCD));
      }
      else if (cColour==PDT::Colour0 && aColour==PDT::Colour3){
	dipoles.push_back(DipoleType(IFba,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFa ,ShowerInteraction::QCD));
      }
      else if (cColour==PDT::Colour8 && aColour==PDT::Colour3){
	dipoles.push_back(DipoleType(IFbc,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFc ,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(FFc ,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(FFa ,ShowerInteraction::QCD));
      }
      else if (cColour==PDT::Colour3 && aColour==PDT::Colour8){
	dipoles.push_back(DipoleType(IFba,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFa ,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(FFc ,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(FFa ,ShowerInteraction::QCD));
      }
      else if(cColour==PDT::Colour3bar && aColour==PDT::Colour3bar) {
	dipoles.push_back(DipoleType(IFba,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFbc,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFa,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFc,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(FFc ,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(FFa ,ShowerInteraction::QCD));
      }
    }
    // decaying colour anti-triplet 
    else if (bColour==PDT::Colour3bar) {
      if ((cColour==PDT::Colour3bar && aColour==PDT::Colour0)){
	dipoles.push_back(DipoleType(IFbc,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFc ,ShowerInteraction::QCD));
      }
      else if ((cColour==PDT::Colour0 && aColour==PDT::Colour3bar)){
	dipoles.push_back(DipoleType(IFba,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFa ,ShowerInteraction::QCD));      
      }
      else if (cColour==PDT::Colour8 && aColour==PDT::Colour3bar){
	dipoles.push_back(DipoleType(IFbc,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFc ,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(FFc ,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(FFa ,ShowerInteraction::QCD));
      }
      else if (cColour==PDT::Colour3bar && aColour==PDT::Colour8){
	dipoles.push_back(DipoleType(IFba,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFa ,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(FFc ,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(FFa ,ShowerInteraction::QCD));
      }
      else if(cColour==PDT::Colour3 && aColour==PDT::Colour3) {
	dipoles.push_back(DipoleType(IFba,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFbc,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFa,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFc,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(FFc ,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(FFa ,ShowerInteraction::QCD));
      }
    }
    // decaying colour octet
    else if (bColour==PDT::Colour8){
      if ((cColour==PDT::Colour3    && aColour==PDT::Colour3bar) ||
	  (cColour==PDT::Colour3bar && aColour==PDT::Colour3)){
	dipoles.push_back(DipoleType(IFba,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFbc,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFa,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFc,ShowerInteraction::QCD));
      }
      else if (cColour==PDT::Colour8 && aColour==PDT::Colour0){
	dipoles.push_back(DipoleType(IFbc,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFc,ShowerInteraction::QCD));
      }
      else if (cColour==PDT::Colour0 && aColour==PDT::Colour8){
	dipoles.push_back(DipoleType(IFba,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFa,ShowerInteraction::QCD));
      }
    }
    // decaying colour sextet
    else if(bColour==PDT::Colour6) {
      if (cColour==PDT::Colour3 && aColour==PDT::Colour3) {
	dipoles.push_back(DipoleType(IFba,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFbc,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFa,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFc,ShowerInteraction::QCD));
      }
    }
    // decaying colour antisextet
    else if(bColour==PDT::Colour6bar) {
      if (cColour==PDT::Colour3bar && aColour==PDT::Colour3bar) {
	dipoles.push_back(DipoleType(IFba,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFbc,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFa,ShowerInteraction::QCD));
	dipoles.push_back(DipoleType(IFc,ShowerInteraction::QCD));
      }
    }
  }
  // QED dipoles
  if(inter==ShowerInteraction::Both ||
     inter==ShowerInteraction::QED) {
    const bool & bCharged = bProgenitor->dataPtr()->charged();
    const bool & cCharged = cProgenitor->dataPtr()->charged();
    const bool & aCharged = aProgenitor->dataPtr()->charged();
    // initial-final
    if(bCharged && aCharged) {
      dipoles.push_back(DipoleType(IFba,ShowerInteraction::QED));
      dipoles.push_back(DipoleType(IFa ,ShowerInteraction::QED));
    }
    if(bCharged && cCharged) {
      dipoles.push_back(DipoleType(IFbc,ShowerInteraction::QED));
      dipoles.push_back(DipoleType(IFc ,ShowerInteraction::QED));
    }
    // final-state
    if(aCharged && cCharged) {
      dipoles.push_back(DipoleType(FFa,ShowerInteraction::QED));
      dipoles.push_back(DipoleType(FFc,ShowerInteraction::QED));
    }
  }
  // check colour structure is allowed
  return !dipoles.empty();
}

vector<Lorentz5Momentum>  PerturbativeDecayer::hardMomenta(PPtr in, PPtr emitter, 
							   PPtr spectator, 
							   const vector<DipoleType>  &dipoles, 
							   int i, bool inDeadZone) {
  // get masses of the particles
  mb_  = in       ->momentum().mass();
  e_   = emitter  ->momentum().mass()/mb_;
  s_   = spectator->momentum().mass()/mb_;
  e2_  = sqr(e_);
  s2_  = sqr(s_);

  vector<Lorentz5Momentum> particleMomenta;
  Energy2 lambda = sqr(mb_)*sqrt(1.+sqr(s2_)+sqr(e2_)-2.*s2_-2.*e2_-2.*s2_*e2_);    

  // calculate A
  double pre = C_;
  // multiply by the colour factor of the dipole
  // ISR
  if (dipoles[i].type==IFba || dipoles[i].type==IFbc) {
    pre *= colourCoeff(in->dataPtr(),emitter->dataPtr(),spectator->dataPtr(),dipoles[i]);
  }
  // radiation from a/c with initial-final connection
  else if (dipoles[i].type==IFa || dipoles[i].type==IFc) {
    pre *= colourCoeff(emitter->dataPtr(),in->dataPtr(),spectator->dataPtr(),dipoles[i]);
  }
  // radiation from a/c with final-final connection
  else if (dipoles[i].type==FFa || dipoles[i].type==FFc) {
    pre *= colourCoeff(emitter->dataPtr(),spectator->dataPtr(),in->dataPtr(),dipoles[i]);
  }
  double A = 2.*abs(pre)/Constants::twopi;
  // factor due sampling choice
  if(phaseOpt_==0) A *= ymax_;
  // coupling factor
  if(dipoles[i].interaction==ShowerInteraction::QCD)
    A *= alphaS() ->overestimateValue();
  else
    A *= alphaEM()->overestimateValue();

  Energy pTmax = 0.5*mb_*(1.-sqr(s_+e_));

  // if no possible branching return
  if ( pTmax < pTmin_ ) return particleMomenta;
  // loop over the two regions
  for(unsigned int j=0;j<2;++j) {
    Energy pT=pTmax;
    vector<Lorentz5Momentum> momenta(4);
    while (pT >= pTmin_) {
      double ymax;
      // overestimate with flat y limit
      if(phaseOpt_==0) {
	pT *= pow(UseRandom::rnd(),(1./A));
	ymax=ymax_;
      }
      // pT sampling including tighter pT dependent y limit
      else {
	pT = 2.*pTmax*exp(-sqrt(-2.*log(UseRandom::rnd())/A+sqr(log(2.*pTmax/pT))));
	// choice of limit overestimate ln(2*pTmax/pT) (true limit acosh(pTmax/pT))
	ymax = log(2.*pTmax/pT);
      }
      if (pT < pTmin_) break;
      double phi = UseRandom::rnd()*Constants::twopi;
      double y   = ymax*(2.*UseRandom::rnd()-1.);
      double xs, xe, xe_z, xg;
      // check if the momenta are physical
      if (!calcMomenta(j, pT, y, phi, xg, xs, xe,
		       xe_z, momenta)) 
	continue;
      // check if point lies within phase space
      if (!psCheck(xg, xs)) continue;
      // check if point lies within the dead-zone (if required)
      if(inDeadZone && !inTotalDeadZone(xg,xs,dipoles,i)) continue;
      // decay products for 3 body decay
      PPtr inpart   = in        ->dataPtr()->produceParticle(momenta[0]);
      ParticleVector decay3;
      decay3.push_back(emitter  ->dataPtr()->produceParticle(momenta[1]));
      decay3.push_back(spectator->dataPtr()->produceParticle(momenta[2]));
      if(dipoles[i].interaction==ShowerInteraction::QCD)
	decay3.push_back(getParticleData(ParticleID::g    )->produceParticle(momenta[3]));
      else
	decay3.push_back(getParticleData(ParticleID::gamma)->produceParticle(momenta[3]));
      // decay products for 2 body decay
      Lorentz5Momentum p1(ZERO,ZERO, lambda/2./mb_,(mb_/2.)*(1.+e2_-s2_),mb_*e_);
      Lorentz5Momentum p2(ZERO,ZERO,-lambda/2./mb_,(mb_/2.)*(1.+s2_-e2_),mb_*s_);
      ParticleVector decay2;
      decay2.push_back(emitter  ->dataPtr()->produceParticle(p1));
      decay2.push_back(spectator->dataPtr()->produceParticle(p2));
      if (dipoles[i].type==FFa || dipoles[i].type==IFa || dipoles[i].type==IFba) {
	swap(decay2[0],decay2[1]);
	swap(decay3[0],decay3[1]);
      }
      // calculate matrix element ratio R/B
      double meRatio = matrixElementRatio(*inpart,decay2,decay3,Initialize,dipoles[i].interaction);
      // calculate dipole factor
      double dipoleSum(0.),numerator(0.);
      for (int k=0; k<int(dipoles.size()); ++k) {
	// skip dipoles which are not of the interaction being considered
	if(dipoles[k].interaction!=dipoles[i].interaction) continue;
	pair<double,double> dipole = calculateDipole(dipoles[k],*inpart,decay3);
	dipoleSum += abs(dipole.first);
	if (k==i) numerator = abs(dipole.second);
      }
      meRatio *= numerator/dipoleSum;
      // calculate jacobian
      Energy2 denom = (mb_-momenta[3].e())*momenta[2].vect().mag() -
	momenta[2].e()*momenta[3].z(); 
      InvEnergy2  J  = (momenta[2].vect().mag2())/(lambda*denom);
      // calculate weight
      double weight = enhance_*meRatio*fabs(sqr(pT)*J)/pre/Constants::twopi; 
      if(dipoles[i].interaction==ShowerInteraction::QCD)
	weight *= alphaS() ->ratio(pT*pT);
      else
	weight *= alphaEM()->ratio(pT*pT);
      // accept point if weight > R
      if (pT > pT_ && weight > UseRandom::rnd()) {
	particleMomenta=momenta;
	if (weight > 1.) {
	  generator()->log() << "WEIGHT PROBLEM " << fullName() << " " << weight << "\n";
	  generator()->log() << xe << " " << xs << " " << xg << "\n";
	  for(unsigned int ix=0;ix<particleMomenta.size();++ix)
	    generator()->log() << particleMomenta[ix]/GeV << "\n";
	}
	pT_ = pT;
	break;
      }
    }
  }
  return particleMomenta;
}

bool PerturbativeDecayer::calcMomenta(int j, Energy pT, double y, double phi,
					double& xg, double& xs, double& xe, double& xe_z,
					vector<Lorentz5Momentum>& particleMomenta) {
  // calculate xg
  xg = 2.*pT*cosh(y) / mb_;
  if (xg>(1. - sqr(e_ + s_)) || xg<0.) return false;
  // calculate the two values of zs
  double xT  = 2.*pT / mb_;
  double zg = 2.*pT*sinh(y) / mb_;
  double A = (sqr(xT) - 4. * xg + 4.);
  double B = 2. * zg * (s2_ - e2_ - xg + 1.);
  double det = -4. * (-sqr(s2_) + (2. * e2_ + sqr(xT) - 2. * xg + 2.) * s2_ - sqr(e2_ + xg - 1.)) * sqr(xg - 2.);
  if (det<0.) return false;
  double zs= j==0 ? (-B+sqrt(det))/A : (-B-sqrt(det))/A;
  // zs must be negative
  if(zs>0.) return false;
  xs = sqrt(sqr(zs)+4.*s2_);
  // check value of xs is physical
  if (xs>(1.+s2_-e2_) || xs<2.*s_) return false;
  // calculate xe
  xe = 2.-xs-xg;     
  // check value of xe is physical
  if (xe>(1.+e2_-s2_) || xe<2.*e_) return false;       
  // calculate xe_z
  xe_z = -zg-zs;
  // calculate 4 momenta
  particleMomenta[0].setE   ( mb_);
  particleMomenta[0].setX   ( ZERO);
  particleMomenta[0].setY   ( ZERO);
  particleMomenta[0].setZ   ( ZERO);
  particleMomenta[0].setMass( mb_);

  particleMomenta[1].setE   ( mb_*xe/2.);
  particleMomenta[1].setX   (-pT*cos(phi));
  particleMomenta[1].setY   (-pT*sin(phi));
  particleMomenta[1].setZ   ( mb_*xe_z/2.);
  particleMomenta[1].setMass( mb_*e_);

  particleMomenta[2].setE   ( mb_*xs/2.);
  particleMomenta[2].setX   ( ZERO);
  particleMomenta[2].setY   ( ZERO);
  particleMomenta[2].setZ   ( mb_*zs/2.);
  particleMomenta[2].setMass( mb_*s_);

  particleMomenta[3].setE   ( pT*cosh(y));
  particleMomenta[3].setX   ( pT*cos(phi));
  particleMomenta[3].setY   ( pT*sin(phi));
  particleMomenta[3].setZ   ( pT*sinh(y));
  particleMomenta[3].setMass( ZERO);

  return true;
}

bool PerturbativeDecayer::psCheck(const double xg, const double xs) {

  // check is point is in allowed region of phase space
  double xe_star = (1.-s2_+e2_-xg)/sqrt(1.-xg);
  double xg_star = xg/sqrt(1.-xg);

  if ((sqr(xe_star)-4.*e2_) < 1e-10) return false;
  double xs_max = (4.+4.*s2_-sqr(xe_star+xg_star)+ 
		   sqr(sqrt(sqr(xe_star)-4.*e2_)+xg_star))/ 4.;
  double xs_min = (4.+4.*s2_-sqr(xe_star+xg_star)+ 
		   sqr(sqrt(sqr(xe_star)-4.*e2_)-xg_star))/ 4.;

  if (xs < xs_min || xs > xs_max) return false;

  return true;
}

pair<double,double> PerturbativeDecayer::calculateDipole(const DipoleType & dipoleId,
							 const Particle & inpart,
							 const ParticleVector & decay3) {
  // calculate dipole for decay b->ac
  pair<double,double> dipole = make_pair(0.,0.);
  double x1 = 2.*decay3[0]->momentum().e()/mb_;
  double x2 = 2.*decay3[1]->momentum().e()/mb_;
  double xg = 2.*decay3[2]->momentum().e()/mb_;
  double mu12 = sqr(decay3[0]->mass()/mb_);
  double mu22 = sqr(decay3[1]->mass()/mb_);
  tcPDPtr part[3] = {inpart.dataPtr(),decay3[0]->dataPtr(),decay3[1]->dataPtr()};
  if(dipoleId.type==FFa || dipoleId.type == IFa || dipoleId.type == IFba) {
    swap(part[1],part[2]);
    swap(x1,x2);
    swap(mu12,mu22);
  }
  // radiation from b with initial-final connection 
  if (dipoleId.type==IFba || dipoleId.type==IFbc) {
    dipole.first  = -2./sqr(xg);
    dipole.first *= colourCoeff(part[0],part[1],part[2],dipoleId);
  }
  // radiation from a/c with initial-final connection
  else if (dipoleId.type==IFa || dipoleId.type==IFc) {
    double z  = 1. - xg/(1.-mu22+mu12);
    dipole.first = (-2.*mu12/sqr(1.-x2+mu22-mu12) + (1./(1.-x2+mu22-mu12))*
	      (2./(1.-z)-dipoleSpinFactor(part[1],z))); 
    dipole.first *= colourCoeff(part[1],part[0],part[2],dipoleId);
  }
  // radiation from a/c with final-final connection
  else if (dipoleId.type==FFa || dipoleId.type==FFc) {
    double z = 1. + ((x1-1.+mu22-mu12)/(x2-2.*mu22));
    double y = (1.-x2-mu12+mu22)/(1.-mu12-mu22);
    double vt = sqrt((1.-sqr(e_+s_))*(1.-sqr(e_-s_)))/(1.-mu12-mu22);
    double v  = sqrt(sqr(2.*mu22+(1.-mu12-mu22)*(1.-y))-4.*mu22)
      /(1.-y)/(1.-mu12-mu22);
    if(part[1]->iSpin()!=PDT::Spin1) {
      dipole.first = (1./(1.-x2+mu22-mu12))*
	((2./(1.-z*(1.-y)))-vt/v*(dipoleSpinFactor(part[1],z)+(2.*mu12/(1.+mu22-mu12-x2))));
    }
    else {
      dipole.first  = (1./(1.-x2+mu22-mu12))*
	(1./(1.-z*(1.-y))+1./(1.-(1.-z)*(1.-y))+(z*(1.-z)-2.)/v-vt/v*(2.*mu12/(1.+mu22-mu12-x2)));
      dipole.second = (1./(1.-x2+mu22-mu12))*
	(2./(1.-z*(1.-y))+(z*(1.-z)-2.)/v-vt/v*(2.*mu12/(1.+mu22-mu12-x2)));
    dipole.second   *= colourCoeff(part[1],part[2],part[0],dipoleId);
    }
    dipole.first *= colourCoeff(part[1],part[2],part[0],dipoleId);
  }
  // special for the case that all particles are gluons
  else if(dipoleId.type==FFg) {
    double z = (1.-x2)/xg;
    double y = 1.-xg;
    dipole.first = 1./(1.-xg)*(1./(1.-z*(1.-y))+1./(1.-(1.-z)*(1.-y))+(z*(1.-z)-2.));
    dipole.first *= colourCoeff(part[1],part[2],part[0],dipoleId);
  }
  else
    assert(false);
  // coupling prefactors
  if(dipole.second==0.) dipole.second=dipole.first;
  dipole.first  *= 8.*Constants::pi;
  dipole.second *= 8.*Constants::pi;
  // return the answer
  return dipole;
}

double PerturbativeDecayer::dipoleSpinFactor(tcPDPtr part, double z){
  // calculate the spin dependent component of the dipole  
  if      (part->iSpin()==PDT::Spin0)
    return 2.;
  else if (part->iSpin()==PDT::Spin1Half)
    return (1. + z);
  else if (part->iSpin()==PDT::Spin1)
    return -(z*(1.-z) - 1./(1.-z) + 1./z -2.);
  return 0.;
}

namespace {

double colourCharge(PDT::Colour icol) {
  switch(icol) {
  case PDT::Colour0 :
    return 0.;
  case PDT::Colour3 : case PDT::Colour3bar :
    return 4./3.;
  case PDT::Colour8:
    return 3.;
  case PDT::Colour6 : case PDT::Colour6bar :
    return 10./3.;
  default :
    assert(false);
    return 0.;
  }
}
}

double PerturbativeDecayer::colourCoeff(tcPDPtr emitter,
					tcPDPtr spectator,
					tcPDPtr other,
					DipoleType dipole) {
  if(dipole.interaction==ShowerInteraction::QCD) {
    double emitterColour   = colourCharge(emitter  ->iColour());
    double spectatorColour = colourCharge(spectator->iColour());
    double otherColour     = colourCharge(other    ->iColour());
    double val = 0.5*(sqr(emitterColour)+sqr(spectatorColour)-sqr(otherColour))/emitterColour;
    return val;
  }
  else {
    double val = double(emitter->iCharge()*spectator->iCharge())/9.;
    // FF dipoles
    if(dipole.type==FFa || dipole.type == FFc) return -val;
    // IF dipoles
    else                                       return  val;
  }
}

void PerturbativeDecayer::getColourLines(RealEmissionProcessPtr real) {
  // extract the particles
  vector<tPPtr> branchingPart;
  branchingPart.push_back(real->incoming()[0]);
  for(unsigned int ix=0;ix<real->outgoing().size();++ix) {
    branchingPart.push_back(real->outgoing()[ix]);
  }
  vector<unsigned int> sing,trip,atrip,oct,sex,asex;
  for (size_t ib=0;ib<branchingPart.size()-1;++ib) {
    if     (branchingPart[ib]->dataPtr()->iColour()==PDT::Colour0   ) sing. push_back(ib);
    else if(branchingPart[ib]->dataPtr()->iColour()==PDT::Colour3   ) trip. push_back(ib);
    else if(branchingPart[ib]->dataPtr()->iColour()==PDT::Colour3bar) atrip.push_back(ib);
    else if(branchingPart[ib]->dataPtr()->iColour()==PDT::Colour8   ) oct.  push_back(ib);
    else if(branchingPart[ib]->dataPtr()->iColour()==PDT::Colour6   ) sex.  push_back(ib);
    else if(branchingPart[ib]->dataPtr()->iColour()==PDT::Colour6bar) asex. push_back(ib);
  }
  // decaying colour singlet
  if (branchingPart[0]->dataPtr()->iColour()==PDT::Colour0) {
    // 0 -> 3 3bar
    if (trip.size()==1 && atrip.size()==1) {
      if(real->interaction()==ShowerInteraction::QCD) {
	branchingPart[atrip[0]]->colourConnect(branchingPart[   3   ]);
	branchingPart[    3   ]->colourConnect(branchingPart[trip[0]]);
      }
      else {
	branchingPart[atrip[0]]->colourConnect(branchingPart[trip[0]]);
      }
    }
    // 0 -> 8 8
    else if (oct.size()==2 ) {
      if(real->interaction()==ShowerInteraction::QCD) {
	bool col = UseRandom::rndbool();
	branchingPart[oct[0]]->colourConnect(branchingPart[   3  ],col);
	branchingPart[   3  ]->colourConnect(branchingPart[oct[1]],col);
	branchingPart[oct[1]]->colourConnect(branchingPart[oct[0]],col);
      }
      else {
	branchingPart[oct[0]]->colourConnect(branchingPart[oct[1]]);
	branchingPart[oct[1]]->colourConnect(branchingPart[oct[0]]);
      }
    }
    else 
      assert(real->interaction()==ShowerInteraction::QED);
  }
  // decaying colour triplet
  else if (branchingPart[0]->dataPtr()->iColour()==PDT::Colour3 ) {
    // 3 -> 3 0
    if (trip.size()==2 && sing.size()==1) {
      if(real->interaction()==ShowerInteraction::QCD) {
	branchingPart[3]->incomingColour(branchingPart[trip[0]]);
	branchingPart[3]-> colourConnect(branchingPart[trip[1]]);
      }
      else {
	branchingPart[trip[1]]->incomingColour(branchingPart[trip[0]]);
      }
    }
    // 3 -> 3 8
    else if (trip.size()==2 && oct.size()==1) {
      if(real->interaction()==ShowerInteraction::QCD) {
	// 8 emit incoming partner
	if(real->emitter()==oct[0]&&real->spectator()==0) {
	  branchingPart[  3   ]->incomingColour(branchingPart[trip[0]]);
	  branchingPart[  3   ]-> colourConnect(branchingPart[oct[0] ]);
	  branchingPart[oct[0]]-> colourConnect(branchingPart[trip[1]]);
	}
	// 8 emit final spectator or vice veras
	else {
	  branchingPart[oct[0]]->incomingColour(branchingPart[trip[0]]);
	  branchingPart[oct[0]]-> colourConnect(branchingPart[   3   ]);
	  branchingPart[   3  ]-> colourConnect(branchingPart[trip[1]]);
	}
      }
      else {
	branchingPart[oct[0]]->incomingColour(branchingPart[trip[0]]);
	branchingPart[oct[0]]-> colourConnect(branchingPart[trip[1]]);
      }
    }
    // 3  -> 3bar 3bar
    else if(trip.size() ==1 && atrip.size()==2) {
      if(real->interaction()==ShowerInteraction::QCD) {
	if(real->emitter()==atrip[0]) {
	  branchingPart[3]->colourConnect(branchingPart[atrip[0]],true);
	  tColinePtr col[3] = {ColourLine::create(branchingPart[ trip[0]],false),
			       ColourLine::create(branchingPart[       3],true ),
			       ColourLine::create(branchingPart[atrip[1]],true)};
	  col[0]->setSinkNeighbours(col[1],col[2]);
	}
	else {
	  branchingPart[3]->colourConnect(branchingPart[atrip[1]],true);
	  tColinePtr col[3] = {ColourLine::create(branchingPart[ trip[0]],false),
			       ColourLine::create(branchingPart[atrip[0]],true ),
			       ColourLine::create(branchingPart[       3],true)};
	  col[0]->setSinkNeighbours(col[1],col[2]);
	}
      }
      else {
	tColinePtr col[3] = {ColourLine::create(branchingPart[ trip[0]],false),
			     ColourLine::create(branchingPart[atrip[0]],true ),
			     ColourLine::create(branchingPart[atrip[1]],true)};
	col[0]->setSinkNeighbours(col[1],col[2]);
      }
    }
    else
      assert(false);
  }
  // decaying colour anti-triplet
  else if (branchingPart[0]->dataPtr()->iColour()==PDT::Colour3bar) {
    // 3bar -> 3bar 0
    if (atrip.size()==2 && sing.size()==1) {
      if(real->interaction()==ShowerInteraction::QCD) {
	branchingPart[3]->incomingColour(branchingPart[atrip[0]],true);
	branchingPart[3]-> colourConnect(branchingPart[atrip[1]],true);
      }
      else {
	branchingPart[atrip[1]]->incomingColour(branchingPart[atrip[0]],true);
      }
    }
    // 3 -> 3 8
    else if (atrip.size()==2 && oct.size()==1){
      if(real->interaction()==ShowerInteraction::QCD) {
	// 8 emit incoming partner
	if(real->emitter()==oct[0]&&real->spectator()==0) {
	  branchingPart[   3  ]->incomingColour(branchingPart[atrip[0]],true);
	  branchingPart[   3  ]-> colourConnect(branchingPart[oct[0]  ],true);
	  branchingPart[oct[0]]-> colourConnect(branchingPart[atrip[1]],true);
	}
	// 8 emit final spectator or vice veras
	else {
	  if(real->interaction()==ShowerInteraction::QCD) {
	    branchingPart[oct[0]]->incomingColour(branchingPart[atrip[0]],true);
	    branchingPart[oct[0]]-> colourConnect(branchingPart[   3    ],true);
	    branchingPart[3]-> colourConnect(branchingPart[atrip[1]]     ,true);
	  }
	}
      }
      else {
	branchingPart[oct[0]]->incomingColour(branchingPart[atrip[0]],true);
	branchingPart[oct[0]]-> colourConnect(branchingPart[atrip[1]],true);
      }
    }
    // 3bar  -> 3 3 
    else if(atrip.size() ==1 && trip.size()==2) {
      if(real->interaction()==ShowerInteraction::QCD) {
	if(real->emitter()==trip[0]) {
	  branchingPart[3]->colourConnect(branchingPart[trip[0]],false);
	  tColinePtr col[3] = {ColourLine::create(branchingPart[atrip[0]],true ),
			       ColourLine::create(branchingPart[       3],false),
			       ColourLine::create(branchingPart[ trip[1]],false)};
	  col[0]->setSourceNeighbours(col[1],col[2]);
	}
	else {
	  branchingPart[3]->colourConnect(branchingPart[trip[1]],false);
	  tColinePtr col[3] = {ColourLine::create(branchingPart[atrip[0]],true ),
			       ColourLine::create(branchingPart[ trip[0]],false),
			       ColourLine::create(branchingPart[       3],false)};
	  col[0]->setSourceNeighbours(col[1],col[2]);
	}
      }
      else {
	tColinePtr col[3] = {ColourLine::create(branchingPart[atrip[0]],true ),
			     ColourLine::create(branchingPart[ trip[0]],false),
			     ColourLine::create(branchingPart[ trip[1]],false)};
	col[0]->setSourceNeighbours(col[1],col[2]);
      }
    }
    else
      assert(false);
  }
  // decaying colour octet
  else if(branchingPart[0]->dataPtr()->iColour()==PDT::Colour8 ) {
    // 8 -> 3 3bar
    if (trip.size()==1 && atrip.size()==1) {
      if(real->interaction()==ShowerInteraction::QCD) {
	// 3 emits
	if(trip[0]==real->emitter()) {
	  branchingPart[3]       ->incomingColour(branchingPart[oct[0]] );
	  branchingPart[3]       -> colourConnect(branchingPart[trip[0]]);
	  branchingPart[atrip[0]]->incomingColour(branchingPart[oct[0]],true);
	}
	// 3bar emits
	else {
	  branchingPart[3]       ->incomingColour(branchingPart[oct[0]]  ,true);
	  branchingPart[3]       -> colourConnect(branchingPart[atrip[0]],true);
	  branchingPart[trip[0]]->incomingColour(branchingPart[oct[0]]  );
	}
      }
      else {
	branchingPart[trip[0]]->incomingColour(branchingPart[oct[0]] );
	branchingPart[atrip[0]]->incomingColour(branchingPart[oct[0]],true);
      }
    }
    // 8 -> 8 0 
    else if (sing.size()==1 && oct.size()==2) {
      if(real->interaction()==ShowerInteraction::QCD) {
	bool col = UseRandom::rndbool();
	branchingPart[   3  ]->colourConnect (branchingPart[oct[1]], col);
	branchingPart[   3  ]->incomingColour(branchingPart[oct[0]], col);
	branchingPart[oct[1]]->incomingColour(branchingPart[oct[0]],!col);
      }
      else {
	branchingPart[oct[1]]->incomingColour(branchingPart[oct[0]]);
	branchingPart[oct[1]]->incomingColour(branchingPart[oct[0]],true);
      }
    }
    else
      assert(false);
  }
  // sextet
  else if(branchingPart[0]->dataPtr()->iColour() == PDT::Colour6) {
    if(trip.size()==2) {
      if(real->interaction()==ShowerInteraction::QCD) {
	Ptr<MultiColour>::pointer parentColour = 
	  dynamic_ptr_cast<Ptr<MultiColour>::pointer>
	  (branchingPart[0]->colourInfo());
	if(trip[0]==real->emitter()) {
	  ColinePtr cline = new_ptr(ColourLine());
	  parentColour->colourLine(cline);
	  cline->addColoured(branchingPart[3]);
	  branchingPart[3]       -> colourConnect(branchingPart[trip[0]]);
	  cline = new_ptr(ColourLine());
	  parentColour->colourLine(cline);
	  cline->addColoured(branchingPart[trip[1]]);
	}
	else {
	  ColinePtr cline = new_ptr(ColourLine());
	  parentColour->colourLine(cline);
	  cline->addColoured(branchingPart[3]);
	  branchingPart[3]       -> colourConnect(branchingPart[trip[1]]);
	  cline = new_ptr(ColourLine());
	  parentColour->colourLine(cline);
	  cline->addColoured(branchingPart[trip[0]]);
	}
      }
      else {
	Ptr<MultiColour>::pointer parentColour = 
	  dynamic_ptr_cast<Ptr<MultiColour>::pointer>
	  (branchingPart[0]->colourInfo());
	for(unsigned int ix=0;ix<2;++ix) {
	  ColinePtr cline = new_ptr(ColourLine());
	  parentColour->colourLine(cline);
	  cline->addColoured(branchingPart[trip[ix]]);
	}
      }
    }
    else
      assert(false);
  }
  // antisextet
  else if(branchingPart[0]->dataPtr()->iColour() == PDT::Colour6bar) {
    if(atrip.size()==2) {
      if(real->interaction()==ShowerInteraction::QCD) {
	Ptr<MultiColour>::pointer parentColour = 
	  dynamic_ptr_cast<Ptr<MultiColour>::pointer>
	  (branchingPart[0]->colourInfo());
	if(atrip[0]==real->emitter()) {
	  ColinePtr cline = new_ptr(ColourLine());
	  parentColour->antiColourLine(cline);
	  cline->addAntiColoured(branchingPart[3]);
	  branchingPart[3]->antiColourConnect(branchingPart[atrip[0]]);
	  cline = new_ptr(ColourLine());
	  parentColour->antiColourLine(cline);
	  cline->addAntiColoured(branchingPart[atrip[1]]);
	}
	else {
	  ColinePtr cline = new_ptr(ColourLine());
	  parentColour->antiColourLine(cline);
	  cline->addAntiColoured(branchingPart[3]);
	  branchingPart[3]->antiColourConnect(branchingPart[atrip[1]]);
	  cline = new_ptr(ColourLine());
	  parentColour->antiColourLine(cline);
	  cline->addAntiColoured(branchingPart[trip[0]]);
	}
      }
      else {
	Ptr<MultiColour>::pointer parentColour = 
	  dynamic_ptr_cast<Ptr<MultiColour>::pointer>
	  (branchingPart[0]->colourInfo());
	for(unsigned int ix=0;ix<2;++ix) {
	  ColinePtr cline = new_ptr(ColourLine());
	  parentColour->antiColourLine(cline);
	  cline->addColoured(branchingPart[atrip[ix]],true);
	}
      }
    }
    else
      assert(false);
  }
  else
    assert(false);
}

PerturbativeDecayer::phaseSpaceRegion
PerturbativeDecayer::inInitialFinalDeadZone(double xg, double xa,
					    double a, double c) const {
  double lam    = sqrt(1.+a*a+c*c-2.*a-2.*c-2.*a*c);
  double kappab = 1.+0.5*(1.-a+c+lam);
  double kappac = kappab-1.+c;
  double kappa(0.);
  // check whether or not in the region for emission from c
  double r = 0.5;
  if(c!=0.) r += 0.5*c/(1.+a-xa);
  double pa = sqrt(sqr(xa)-4.*a);
  double z = ((2.-xa)*(1.-r)+r*pa-xg)/pa;
  if(z<1. && z>0.) {
    kappa = (1.+a-c-xa)/(z*(1.-z));
    if(kappa<kappac)
      return emissionFromC;
  }
  // check in region for emission from b (T1)
  double cq = sqr(1.+a-c)-4*a;
  double bq = -2.*kappab*(1.-a-c);
  double aq = sqr(kappab)-4.*a*(kappab-1);
  double dis = sqr(bq)-4.*aq*cq;
  z=1.-(-bq-sqrt(dis))/2./aq;
  double w = 1.-(1.-z)*(kappab-1.);
  double xgmax = (1.-z)*kappab;
  // possibly in T1 region
  if(xg<xgmax) {
    z = 1.-xg/kappab;
    kappa=kappab;
  }
  // possibly in T2 region
  else {
    aq = 4.*a;
    bq = -4.*a*(2.-xg);
    cq = sqr(1.+a-c-xg);
    dis = sqr(bq)-4.*aq*cq;
    z = (-bq-sqrt(dis))/2./aq;
    kappa = xg/(1.-z);
  }
  // compute limit on xa
  double u = 1.+a-c-(1.-z)*kappa;
  w = 1.-(1.-z)*(kappa-1.);
  double v = sqr(u)-4.*z*a*w;
  if(v<0. && v>-1e-10) v= 0.;
  v = sqrt(v);
  if(xa<0.5*((u+v)/w+(u-v)/z)) {
    if(xg<xgmax)
      return emissionFromA1;
    else if(useMEforT2_)
      return deadZone;
    else
      return emissionFromA2;
  }
  else
    return deadZone;
}

PerturbativeDecayer::phaseSpaceRegion
PerturbativeDecayer::inFinalFinalDeadZone(double xb, double xc,
					  double b, double c) const {
  // basic kinematics
  double lam = sqrt(1.+b*b+c*c-2.*b-2.*c-2.*b*c);
  // check whether or not in the region for emission from b
  double r = 0.5;
  if(b!=0.) r+=0.5*b/(1.+c-xc);
  double pc = sqrt(sqr(xc)-4.*c);
  double z = -((2.-xc)*r-r*pc-xb)/pc;
  if(z<1. and z>0.) {
    if((1.-b+c-xc)/(z*(1.-z))<0.5*(1.+b-c+lam)) return emissionFromB;
  }
  // check whether or not in the region for emission from c
  r = 0.5;
  if(c!=0.) r+=0.5*c/(1.+b-xb);
  double pb = sqrt(sqr(xb)-4.*b);
  z = -((2.-xb)*r-r*pb-xc)/pb;
  if(z<1. and z>0.) {
    if((1.-c+b-xb)/(z*(1.-z))<0.5*(1.-b+c+lam)) return emissionFromC;
  }
  return deadZone;
}

bool PerturbativeDecayer::inTotalDeadZone(double xg, double xs,
					  const vector<DipoleType>  & dipoles,
					  int i) {
  double xb,xc,b,c;
  if(dipoles[i].type==FFa || dipoles[i].type == IFa || dipoles[i].type == IFba) {
    xc = xs;
    xb = 2.-xg-xs;
    b = e2_;
    c = s2_;
  }
  else {
    xb = xs;
    xc = 2.-xg-xs;
    b = s2_;
    c = e2_;
  }
  for(unsigned int ix=0;ix<dipoles.size();++ix) {
    if(dipoles[ix].interaction!=dipoles[i].interaction)
      continue;
    // should also remove negative QED dipoles but shouldn't be an issue unless we
    // support QED ME corrections
    switch (dipoles[ix].type) {
    case FFa :
      if(inFinalFinalDeadZone(xb,xc,b,c)!=deadZone) return false;
      break;
    case FFc :
      if(inFinalFinalDeadZone(xc,xb,c,b)!=deadZone) return false;
      break;
    case IFa : case IFba:
      if(inInitialFinalDeadZone(xg,xc,c,b)!=deadZone) return false;
      break;
    case IFc : case IFbc:
      if(inInitialFinalDeadZone(xg,xb,b,c)!=deadZone) return false;
      break;
    case FFg:
      break;
    }
  }
  return true;
}
