#line 1 "./EtaPiGammaGammaDecayer.cc"
// -*- C++ -*-
//
// EtaPiGammaGammaDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the EtaPiGammaGammaDecayer class.
//
#include "EtaPiGammaGammaDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/Switch.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/PDT/DecayMode.h"
#include "Herwig/PDT/ThreeBodyAllOnCalculator.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "Herwig/Decay/GeneralDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void EtaPiGammaGammaDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()) {
    _etamax  = mode(0)->maxWeight();
    _etapmax = mode(1)->maxWeight();
  }
}

EtaPiGammaGammaDecayer::EtaPiGammaGammaDecayer()
  : _grhoomega(12.924/GeV), _fpi(130.7*MeV),_rhomass(771.1*MeV),
    _rhowidth(149.2*MeV),_grho(_rhomass/_fpi),_mpi(ZERO),_rhoconst(0.),
    _localparameters(true),_ratiofpif8(1./1.3),_ratiofpif0(1./1.04),
    _theta(-Constants::pi/9.),_etamax(2.36858),_etapmax(0.006),
    _dconst(2), _econst(2) {
  // intermediates
  generateIntermediates(false);
}

void EtaPiGammaGammaDecayer::doinit() {
  DecayIntegrator::doinit();
  // set rho parameters if needed
  tPDPtr rho(getParticleData(ParticleID::rho0));
  if(!_localparameters) {
    _rhomass  = rho->mass();
    _rhowidth = rho->width();
  }
  // constant for the running rho width
  _mpi=getParticleData(ParticleID::pi0)->mass();
  Energy pcm =Kinematics::pstarTwoBodyDecay(_rhomass,_mpi,_mpi);
  _rhoconst=_rhomass*_rhomass*_rhowidth/(pcm*pcm*pcm);
  // set the prefactors
  double conv(sqrt(4.*Constants::pi*SM().alphaEM()));
  conv *=_fpi*_fpi*_grho/_rhomass/_rhomass;
  InvEnergy2 pre(2.*sqrt(3.)/9.*sqr(_grhoomega*conv));
  double fact[2];
  // constants for eta
  fact[0] = _ratiofpif8*cos(_theta)-sqrt(2.)*_ratiofpif0*sin(_theta);
  // constants for eta'
  fact[1] = _ratiofpif8*sin(_theta)+sqrt(2.)*_ratiofpif0*cos(_theta);
  for(unsigned int ix=0;ix<2;++ix) {
    _dconst[ix]=fact[ix]*pre;
    _econst[ix]=fact[ix]*pre;
  }
  // set up the phsae space for the decays
  tPDPtr eta[2]={getParticleData(ParticleID::eta),getParticleData(ParticleID::etaprime)};
  tPDVector extpart;extpart.resize(4);
  extpart[1] = getParticleData(ParticleID::pi0);
  extpart[2] = getParticleData(ParticleID::gamma);
  extpart[3] = getParticleData(ParticleID::gamma);
  vector<double> dummyweights(2,0.5);
  DecayPhaseSpaceChannelPtr newchannel;
  DecayPhaseSpaceModePtr mode;
  for(unsigned int ix=0;ix<2;++ix) {
    extpart[0] = eta[ix];
    mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,2);
    newchannel->addIntermediate(rho,0,0.0, 1,3);
    mode->addChannel(newchannel);
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,3);
    newchannel->addIntermediate(rho,0,0.0, 1,2);
    mode->addChannel(newchannel);
    addMode(mode, (ix==0 ? _etamax : _etapmax),dummyweights);
  }
}

int EtaPiGammaGammaDecayer::modeNumber(bool & cc,tcPDPtr parent,
				       const tPDVector & children) const {
  cc=false;
  int id;
  if(children.size()!=3) return -1;
  tPDVector::const_iterator pit = children.begin();
  unsigned int npi0(0),ngamma(0);
  for( ;pit!=children.end();++pit) {
    id=(**pit).id();
    if(id==ParticleID::pi0)         ++npi0;
    else if(id==ParticleID::gamma)  ++ngamma;
  }
  if(!(npi0==1&&ngamma==2)) return -1;
  // number of the mode
  switch (parent->id()) {
  case ParticleID::eta     : return 0;
  case ParticleID::etaprime: return 1;
  default: return -1;
  }
}

void EtaPiGammaGammaDecayer::persistentOutput(PersistentOStream & os) const {
  os << ounit(_grhoomega,1/GeV)<< ounit(_fpi,GeV)<< _grho 
     << ounit(_rhomass,GeV)<< ounit(_rhowidth,GeV)<< _localparameters 
     << _ratiofpif8 << _ratiofpif0 << _theta << _etamax << _etapmax 
     << _rhoconst << ounit(_mpi,GeV) << ounit(_dconst,1/GeV2) 
     << ounit(_econst,1/GeV2);
}

void EtaPiGammaGammaDecayer::persistentInput(PersistentIStream & is, int) {
  is >> iunit(_grhoomega,1/GeV) >> iunit(_fpi,GeV)>> _grho 
     >> iunit(_rhomass,GeV)>> iunit(_rhowidth,GeV)>> _localparameters 
     >> _ratiofpif8 >> _ratiofpif0 >> _theta >> _etamax >> _etapmax 
     >> _rhoconst >> iunit(_mpi,GeV) >> iunit(_dconst,1/GeV2) 
     >> iunit(_econst,1/GeV2);
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<EtaPiGammaGammaDecayer,DecayIntegrator>
describeHerwigEtaPiGammaGammaDecayer("Herwig::EtaPiGammaGammaDecayer", "HwSMDecay.so");

void EtaPiGammaGammaDecayer::Init() {

  static ClassDocumentation<EtaPiGammaGammaDecayer> documentation
    ("The EtaPiGammaGammaDecayer class implements a VMD model for the"
     " decay of the eta or etaprime to a pion and two photons.",
     "The decays of $\\eta,\\eta'\\to\\pi^0\\gamma\\gamma$ were simulated using"
     " the matrix elements of \\cite{Holstein:2001bt}",
     "\\bibitem{Holstein:2001bt} B.~R.~Holstein,\n"
     " Phys.\\ Scripta {\\bf T99} (2002) 55 [arXiv:hep-ph/0112150].\n"
     "%%CITATION = PHSTB,T99,55;%%\n");

  static Parameter<EtaPiGammaGammaDecayer,InvEnergy> interfacegrhoomega
    ("grhoomega",
     "The couping of the rho, omega and a pion",
     &EtaPiGammaGammaDecayer::_grhoomega, 1./GeV, 12.924/GeV, ZERO, 100./GeV,
     false, false, true);

  static Parameter<EtaPiGammaGammaDecayer,Energy> interfaceFpi
    ("Fpi",
     "The pion decay constant",
     &EtaPiGammaGammaDecayer::_fpi, MeV, 130.7*MeV, ZERO, 200.0*MeV,
     false, false, true);

  static Parameter<EtaPiGammaGammaDecayer,double> interfacegrho
    ("grho",
     "Rho decay constant",
     &EtaPiGammaGammaDecayer::_grho, 5.9, 0.0, 10.0,
     false, false, true);

  static Parameter<EtaPiGammaGammaDecayer,Energy> interfaceRhoMass
    ("RhoMass",
     "The mass of the rho meson",
     &EtaPiGammaGammaDecayer::_rhomass, MeV, 771.1*MeV, 500.0*MeV, 1000.0*MeV,
     false, false, true);

  static Parameter<EtaPiGammaGammaDecayer,Energy> interfaceRhoWidth
    ("RhoWidth",
     "The width of the rho meson",
     &EtaPiGammaGammaDecayer::_rhowidth, MeV, 149.2*MeV, 100.0*MeV, 200.0*MeV,
     false, false, true);

  static Parameter<EtaPiGammaGammaDecayer,double> interfaceRatioFpiF8
    ("RatioFpiF8",
     "The ratio of the decay constant Fpi to F8",
     &EtaPiGammaGammaDecayer::_ratiofpif8, 1./1.3, 0.0, 10.0,
     false, false, true);

  static Parameter<EtaPiGammaGammaDecayer,double> interfaceRatioFpiF0
    ("RatioFpiF0",
     "The ratio of the decay constant Fpi to F0",
     &EtaPiGammaGammaDecayer::_ratiofpif0, 1./1.04, 0.0, 10.0,
     false, false, true);

  static Parameter<EtaPiGammaGammaDecayer,double> interfaceTheta
    ("Theta",
     "The eta etaprime mixing angle",
     &EtaPiGammaGammaDecayer::_theta, -Constants::pi/9., -Constants::pi, Constants::pi,
     false, false, true);

  static Parameter<EtaPiGammaGammaDecayer,double> interfaceEtaMax
    ("EtaMax",
     "THe maximum weight for the eta decay",
     &EtaPiGammaGammaDecayer::_etamax, 1.35, -1.0e12, 1.0e12,
     false, false, false);

  static Parameter<EtaPiGammaGammaDecayer,double> interfaceEtaPrimeMax
    ("EtaPrimeMax",
     "THe maximum weight for the eta prime decay",
     &EtaPiGammaGammaDecayer::_etapmax, 0.006, -1.0e12, 1.0e12,
     false, false, false);

  static Switch<EtaPiGammaGammaDecayer,bool> interfaceLocalParameters
    ("LocalParameters",
     "Use local values of the parameters",
     &EtaPiGammaGammaDecayer::_localparameters, true, false, false);
  static SwitchOption interfaceLocalParametersLocal
    (interfaceLocalParameters,
     "Local",
     "Use local values",
     true);
  static SwitchOption interfaceLocalParametersParticleData
    (interfaceLocalParameters,
     "ParticleData",
     "Use values from the particle data objects",
     false);
}

double EtaPiGammaGammaDecayer::me2(const int,const Particle & inpart,
				   const ParticleVector& decay,
				   MEOption meopt) const {
  if(!ME())
    ME(new_ptr(GeneralDecayMatrixElement(PDT::Spin0,PDT::Spin0,PDT::Spin1,PDT::Spin1)));
  useMe();
  if(meopt==Initialize) {
    ScalarWaveFunction::
      calculateWaveFunctions(_rho,const_ptr_cast<tPPtr>(&inpart),incoming);
  }
  if(meopt==Terminate) {
    // set up the spin information for the decay products
    ScalarWaveFunction::constructSpinInfo(const_ptr_cast<tPPtr>(&inpart),
					  incoming,true);
    ScalarWaveFunction::constructSpinInfo(decay[0],outgoing,true);
    for(unsigned int ix=0;ix<2;++ix)
      VectorWaveFunction::constructSpinInfo(_vectors[ix],decay[ix+1],
					    outgoing,true,true);
    return 0.;
  }
  for(unsigned int ix=0;ix<2;++ix)
    VectorWaveFunction::calculateWaveFunctions(_vectors[ix],decay[ix+1],
					       outgoing,true);
  // dot products we need
  Energy2 q1dotq2(decay[1]->momentum()*decay[2]->momentum()),
    pdotq1(inpart.momentum()*decay[1]->momentum()),
    pdotq2(inpart.momentum()*decay[2]->momentum());
  complex<Energy> e1dotq2[3],e1dotp[3],e2dotq1[3],e2dotp[3];
  for(unsigned int ix=0;ix<3;++ix) {
    if(ix==1) {
      e1dotq2[ix]=ZERO;
      e1dotp[ix] =ZERO;
      e2dotq1[ix]=ZERO;
      e2dotp[ix] =ZERO;
    }
    else {
      e1dotq2[ix] =_vectors[0][ix]*decay[2]->momentum();
      e1dotp[ix]  =_vectors[0][ix]*inpart.momentum();
      e2dotq1[ix] =_vectors[1][ix]*decay[1]->momentum();
      e2dotp[ix]  =_vectors[1][ix]*inpart.momentum();
    }
  }
  // the momentum dependent pieces of the matrix element
  Complex ii(0.,1.);
  Energy2 mpi2(sqr(decay[0]->mass())),meta2(sqr(inpart.mass())),
    mrho2(sqr(_rhomass)),
    t(mpi2+2.*((decay[0]->momentum())*(decay[1]->momentum()))),
    u(mpi2+2.*((decay[0]->momentum())*(decay[2]->momentum())));
  Energy q(sqrt(t)),pcm(Kinematics::pstarTwoBodyDecay(q,_mpi,_mpi));
  complex<Energy2> tgamma(ii*pcm*pcm*pcm*_rhoconst/q);
  q=sqrt(u);pcm = Kinematics::pstarTwoBodyDecay(q,_mpi,_mpi);
  complex<Energy2> ugamma(ii*pcm*pcm*pcm*_rhoconst/q);
  complex<InvEnergy2> prop1(1./(mrho2-t-tgamma)),prop2(1./(mrho2-u-ugamma));
  complex<InvEnergy2> Dfact(_dconst[imode()]*(prop1*(pdotq2-meta2)
					      +prop2*(pdotq1-meta2)));
  complex<InvEnergy4> Efact(_econst[imode()]*(prop1+prop2));
  Complex e1dote2;
  for(unsigned int ix=0;ix<3;++ix) {
    for(unsigned int iy=0;iy<3;++iy) {
      if(ix==1||iy==1) (*ME())(0,0,ix,iy)=0.;
      else {
	e1dote2=_vectors[0][ix].dot(_vectors[1][iy]);
	(*ME())(0,0,ix,iy) = 
	  Complex(Dfact*complex<Energy2>(e1dote2*q1dotq2-
					 e1dotq2[ix]*e2dotq1[iy])
		  -Efact*complex<Energy4>(-e1dote2*pdotq1*pdotq2
					  -e1dotp[ix]*e2dotp[iy]*q1dotq2
					  +e1dotq2[ix]*e2dotp[iy]*pdotq1
					  +e1dotp[ix]*e2dotq1[iy]*pdotq2));
      }
    }
  }
  /*
  double me(ME()->contract(rhoin).real());
  Energy M(inpart.mass()),M2(M*M);
  Energy2 s1(2.*(decay[1]->momentum()*decay[2]->momentum()));
  Energy2 s2(M2-2.*(inpart.momentum()*decay[1]->momentum()));
  Energy2 s3(M2-2.*(inpart.momentum()*decay[2]->momentum()));
  cout << "testing the matrix element " << (
   2*(2*(Dfact*conj(Dfact)).real() + 2*(Dfact*conj(Efact)).real()*M2 
      + (Efact*conj(Efact)).real()*M2*M2)*
      s1*s1 - 2*(Efact*conj(Efact)).real()*M2*s1*(M2 - s2)*
   (M2 - s3) +(Efact*conj(Efact)).real()*(M2 - s2)*(M2 - s2)*
   (M2-s3)*(M2-s3))/8. - me << endl;
  return me;
  */
  return ME()->contract(_rho).real();
}
 
double EtaPiGammaGammaDecayer::
threeBodyMatrixElement(const int imodeb, const Energy2 q2,const  Energy2 s3,
		       const Energy2 s2,const Energy2 s1,const Energy ,
		       const Energy ,const Energy ) const {
  // compute the prefactors
  Energy2 mrho2 = sqr(_rhomass);
  Energy q = sqrt(s3);
  Energy pcm = Kinematics::pstarTwoBodyDecay(q,_mpi,_mpi);
  Complex ii(0.,1.);
  complex<Energy2> tgamma(ii*pcm*pcm*pcm*_rhoconst/q);
  q = sqrt(s2);
  pcm = Kinematics::pstarTwoBodyDecay(q,_mpi,_mpi);
  complex<Energy2> ugamma(ii*pcm*pcm*pcm*_rhoconst/q);
  complex<InvEnergy2> prop1(1./(mrho2-s3-tgamma)), prop2(1./(mrho2-s2-ugamma));
  Energy2 pdotq2(0.5*(q2-s3)), pdotq1(0.5*(q2-s2));
  complex<InvEnergy2> Dfact(_dconst[imodeb]*(prop1*(pdotq2-q2)+prop2*(pdotq1-q2)));
  complex<InvEnergy4> Efact(_econst[imodeb]*(prop1+prop2));
  InvEnergy4 D2 = (Dfact*conj(Dfact)).real();
  InvEnergy8 E2((Efact*conj(Efact)).real());
  InvEnergy6 ED((Efact*conj(Dfact)).real());
  return (2 * (2*D2 + 2*ED*q2 + E2*sqr(q2)) * sqr(s1)
	  - double(2*E2*q2*s1*(q2-s2)*(q2-s3))
	  + double(E2*sqr(q2-s2)*sqr(q2-s3))
	  )/8.;
}

WidthCalculatorBasePtr 
EtaPiGammaGammaDecayer::threeBodyMEIntegrator(const DecayMode & dm) const {
  // workout which mode we are doing
  int id(dm.parent()->id()),imode(1);
  if(id==ParticleID::eta){imode=0;}
  // construct the integrator
  vector<double> inweights; inweights.push_back(0.5); inweights.push_back(0.5);
  Energy mrho(getParticleData(ParticleID::rhoplus)->mass());
  Energy wrho(getParticleData(ParticleID::rhoplus)->width());
  vector<Energy> inmass;  inmass.push_back(mrho);  inmass.push_back(mrho);
  vector<Energy> inwidth; inwidth.push_back(wrho); inwidth.push_back(wrho);
  vector<int> intype; intype.push_back(1); intype.push_back(2);
  vector<double> inpow(2,0.0);
  return new_ptr(ThreeBodyAllOnCalculator<EtaPiGammaGammaDecayer>
		 (inweights,intype,inmass,inwidth,inpow,*this,
		  imode,_mpi,ZERO,ZERO));
}

void EtaPiGammaGammaDecayer::dataBaseOutput(ofstream & output, 
					    bool header) const {
  if(header) output << "update decayers set parameters=\"";
  DecayIntegrator::dataBaseOutput(output,false);
  output << "newdef " << name() << ":grhoomega " << _grhoomega*GeV << "\n";
  output << "newdef " << name() << ":Fpi " << _fpi/MeV  << "\n";
  output << "newdef " << name() << ":grho " << _grho << "\n";
  output << "newdef " << name() << ":RhoMass " << _rhomass/MeV << "\n";
  output << "newdef " << name() << ":RhoWidth " << _rhowidth/MeV << "\n";
  output << "newdef " << name() << ":RatioFpiF8 " << _ratiofpif8 << "\n";
  output << "newdef " << name() << ":RatioFpiF0 " << _ratiofpif0 << "\n";
  output << "newdef " << name() << ":Theta " << _theta  << "\n";
  output << "newdef " << name() << ":EtaMax " << _etamax << "\n";
  output << "newdef " << name() << ":EtaPrimeMax " << _etapmax << "\n";
  output << "newdef " << name() << ":LocalParameters " << _localparameters << "\n";
  if(header) output << "\n\" where BINARY ThePEGName=\"" << fullName() << "\";" << endl;
}
#line 1 "./EtaPiPiGammaDecayer.cc"
// -*- C++ -*-
//
// EtaPiPiGammaDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the EtaPiPiGammaDecayer class.
//

#include "EtaPiPiGammaDecayer.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/Switch.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "ThePEG/Helicity/epsilon.h"
#include "Herwig/PDT/ThreeBodyAllOnCalculator.h"
#include "Herwig/Utilities/GaussianIntegrator.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "Herwig/Decay/GeneralDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;


DescribeClass<EtaPiPiGammaDecayer,DecayIntegrator>
describeHerwigEtaPiPiGammaDecayer("Herwig::EtaPiPiGammaDecayer",
				  "HwSMDecay.so");
HERWIG_INTERPOLATOR_CLASSDESC(EtaPiPiGammaDecayer,double,Energy)



void EtaPiPiGammaDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()) {
    for(unsigned int ix=0;ix<_maxweight.size();++ix)
      _maxweight[ix]=mode(ix)->maxWeight();
  }
}

EtaPiPiGammaDecayer::EtaPiPiGammaDecayer() 
  : _incoming(2), _coupling(2), _maxweight(2), _option(2) {
  // the pion decay constant
  _fpi=130.7*MeV;
  // the rho mass
  _mrho=0.7711*GeV;
  _rhowidth=0.1492*GeV;
  // the constants for the omnes function form
  _aconst=0.5/_mrho/_mrho;
  _cconst=1.0;
  // use local values of the parameters
  _localparameters=true;
  // the modes
  // eta decay
  _incoming[0] = 221; 
  _option[0] = 3; 
  _coupling[0] = 5.060e-3; 
  _maxweight[0] = 3.95072; 
  // eta' decay
  _incoming[1] = 331; 
  _option[1] = 3; 
  _coupling[1] = 4.278e-3; 
  _maxweight[1] = 3.53141; 
  _rhoconst=0.;
  _mpi=ZERO;
  // initialization of the experimental function
  _initialize =false;
  _npoints=100;
  Energy en[35]={300*MeV, 320*MeV, 340*MeV, 360*MeV, 380*MeV,
		 400*MeV, 420*MeV, 440*MeV, 460*MeV, 480*MeV,
		 500*MeV, 520*MeV, 540*MeV, 560*MeV, 580*MeV,
		 600*MeV, 620*MeV, 640*MeV, 660*MeV, 680*MeV,
		 700*MeV, 720*MeV, 740*MeV, 760*MeV, 780*MeV,
		 800*MeV, 820*MeV, 840*MeV, 860*MeV, 880*MeV,
		 900*MeV, 920*MeV, 940*MeV, 960*MeV, 980*MeV};
  _energy=vector<Energy>(en,en+35);
  double phs[35]={  00.1,     0.4,     0.7,     1.0,     1.5,
		    02.0,     2.5,     3.2,     4.0,     4.9,
		    05.9,     7.1,     8.5,    10.1,    12.1,
		    14.4,    17.3,    20.9,    25.4,    31.2,
		    38.7,    48.4,    60.6,    74.9,    90.0, 
		   103.8,   115.3,   124.3,   131.3,   136.7,
		   141.0,   144.5,   147.3,   149.7,   151.8};
  _phase=vector<double>(phs,phs+35);
  // experimental omnes function 
  Energy omnesen[100]
    ={ 282.534*MeV, 289.32 *MeV, 296.106*MeV, 302.893*MeV, 309.679*MeV,
       316.466*MeV, 323.252*MeV, 330.038*MeV, 336.825*MeV, 343.611*MeV,
       350.398*MeV, 357.184*MeV, 363.97 *MeV, 370.757*MeV, 377.543*MeV,
       384.33 *MeV, 391.116*MeV, 397.902*MeV, 404.689*MeV, 411.475*MeV,
       418.261*MeV, 425.048*MeV, 431.834*MeV, 438.621*MeV, 445.407*MeV, 
       452.193*MeV, 458.98 *MeV, 465.766*MeV, 472.553*MeV, 479.339*MeV,
       486.125*MeV, 492.912*MeV, 499.698*MeV, 506.485*MeV, 513.271*MeV,
       520.057*MeV, 526.844*MeV, 533.63 *MeV, 540.417*MeV, 547.203*MeV,
       553.989*MeV, 560.776*MeV, 567.562*MeV, 574.349*MeV, 581.135*MeV,
       587.921*MeV, 594.708*MeV, 601.494*MeV, 608.281*MeV, 615.067*MeV,
       621.853*MeV, 628.64 *MeV, 635.426*MeV, 642.213*MeV, 648.999*MeV,
       655.785*MeV, 662.572*MeV, 669.358*MeV, 676.145*MeV, 682.931*MeV,
       689.717*MeV, 696.504*MeV, 703.29 *MeV, 710.077*MeV, 716.863*MeV,
       723.649*MeV, 730.436*MeV, 737.222*MeV, 744.009*MeV, 750.795*MeV,
       757.581*MeV, 764.368*MeV, 771.154*MeV, 777.94 *MeV, 784.727*MeV, 
       791.513*MeV, 798.3  *MeV, 805.086*MeV, 811.872*MeV, 818.659*MeV, 
       825.445*MeV, 832.232*MeV, 839.018*MeV, 845.804*MeV, 852.591*MeV, 
       859.377*MeV, 866.164*MeV, 872.95 *MeV, 879.736*MeV, 886.523*MeV,
       893.309*MeV, 900.096*MeV, 906.882*MeV, 913.668*MeV, 920.455*MeV, 
       927.241*MeV, 934.028*MeV, 940.814*MeV, 947.6  *MeV, 954.387*MeV};
  _omnesenergy.assign(omnesen,omnesen+100);
  double omnesre[100]
    ={ 0.860676  , 0.851786  , 0.843688  , 0.835827  , 0.828031  ,
       0.820229  , 0.812370  , 0.804424  , 0.796354  , 0.788143  ,
       0.779698  , 0.770939  , 0.761692  , 0.752707  , 0.743823  ,
       0.735004  , 0.726091  , 0.717047  , 0.707862  , 0.698439  ,
       0.688685  , 0.678510  , 0.668518  , 0.658481  , 0.648344  ,
       0.638219  , 0.627989  , 0.617603  , 0.607222  , 0.596711  ,
       0.586026  , 0.575280  , 0.564282  , 0.553067  , 0.541923  ,
       0.530574  , 0.519112  , 0.507690  , 0.496055  , 0.484313  ,
       0.472496  , 0.460245  , 0.447943  , 0.435766  , 0.423390  ,
       0.410997  , 0.398510  , 0.385479  , 0.372458  , 0.359520  ,
       0.346129  , 0.332837  , 0.319623  , 0.305858  , 0.292238  ,
       0.278690  , 0.264391  , 0.250316  , 0.236400  , 0.221655  ,
       0.207196  , 0.192956  , 0.177745  , 0.162833  , 0.148209  ,
       0.132603  , 0.117202  , 0.102090  , 0.0862283 , 0.0703392 ,     
       0.0545317 , 0.0383762 , 0.0219486 , 0.00518648,-0.0113217 ,    
      -0.0280201 ,-0.045445  ,-0.0625479 ,-0.079748  ,-0.0978819 ,    
      -0.11569   ,-0.133447  ,-0.152117  ,-0.170608  ,-0.189137  ,     
      -0.208597  ,-0.227864  ,-0.247185  ,-0.267306  ,-0.287382  ,
      -0.307707  ,-0.328882  ,-0.350103  ,-0.37178   ,-0.394464  ,
      -0.417228  ,-0.440561  ,-0.464976  ,-0.490278  ,-0.517527};
  _omnesfunctionreal.assign(omnesre,omnesre+100);
  
  double omnesim[100]
    ={ 0.00243346, 0.000894972,-0.000612496,-0.00209178,-0.00354344,
      -0.00496737,-0.00636316 ,-0.00773022 ,-0.00906769,-0.0103569 ,
      -0.0116108 ,-0.0128658  ,-0.0145424  ,-0.0165746 ,-0.0186438 ,
      -0.0206363 ,-0.0225379  ,-0.0243827  ,-0.0261488 ,-0.0278572 ,
      -0.0295317 ,-0.0316349  ,-0.0339321  ,-0.0362345 ,-0.0386555 ,
      -0.0410799 ,-0.0434534  ,-0.0459509  ,-0.0484302 ,-0.0508376 ,
      -0.0533398 ,-0.0557937  ,-0.0581587  ,-0.0608612 ,-0.0635382 ,
      -0.0661231 ,-0.068983   ,-0.0717604  ,-0.0744215 ,-0.0772635 ,
      -0.0799845 ,-0.0825991  ,-0.0857537  ,-0.0888139 ,-0.0917441 ,
      -0.0948263 ,-0.0977055  ,-0.100462   ,-0.103773  ,-0.106912  ,
      -0.109931  ,-0.113413   ,-0.116647   ,-0.119722  ,-0.123282  ,
      -0.126521  ,-0.129593   ,-0.133324   ,-0.136691  ,-0.139854  ,
      -0.143729  ,-0.14718    ,-0.150356   ,-0.154353  ,-0.157926  ,
      -0.161133  ,-0.165174   ,-0.168899   ,-0.172212  ,-0.176116  ,
      -0.179892  ,-0.183445   ,-0.187134   ,-0.190947  ,-0.195144  ,
      -0.198771  ,-0.202443   ,-0.206906   ,-0.210561  ,-0.214207  ,
      -0.218943  ,-0.222806   ,-0.226551   ,-0.231273  ,-0.235267  ,
      -0.239178  ,-0.244082   ,-0.24836    ,-0.252492  ,-0.257394  ,
      -0.261812  ,-0.266156   ,-0.271161   ,-0.275849  ,-0.280675  ,
      -0.286275  ,-0.291716   ,-0.297353   ,-0.303621  ,-0.310452  };
  _omnesfunctionimag.assign(omnesim,omnesim+100);
  // integration cut parameter
  _epscut=0.4*MeV;
  // size of the arrays
  _nsizea = _energy.size();_nsizeb = _omnesenergy.size();
  // intermediates
  generateIntermediates(false);
}

void EtaPiPiGammaDecayer::doinit() {
  DecayIntegrator::doinit();
  // check the consistence of the parameters
  unsigned int isize=_incoming.size();
  if(isize!=_coupling.size()||isize!=_option.size()||isize!=_maxweight.size()||
     _energy.size()!=_phase.size()||_omnesenergy.size()!=_omnesfunctionreal.size()||
     _omnesenergy.size()!=_omnesfunctionimag.size())
    throw InitException() << "Inconsistent parameters in " 
			  << "EtaPiPiGammaDecayer::doinit()" << Exception::abortnow;
  // set the parameters
  tPDPtr rho(getParticleData(ParticleID::rho0));
  if(!_localparameters) {
    _mrho=rho->mass();
    _rhowidth=rho->width();
  }
  _mpi=getParticleData(ParticleID::piplus)->mass();
  Energy pcm(Kinematics::pstarTwoBodyDecay(_mrho,_mpi,_mpi));
  _rhoconst=sqr(_mrho)*_rhowidth/pow<3,1>(pcm);
  // set up the experimental omnes function if needed
  if(_initialize) {
    // convert the phase shift into radians
    vector<double> radphase;
    for(unsigned int ix=0;ix<_phase.size();++ix) {
      radphase.push_back(_phase[ix]/180.*Constants::pi);
    }
    // set up an interpolator for this
    Interpolator<double,Energy>::Ptr intphase=make_InterpolatorPtr(radphase,_energy,3);
    // limits and step sizes
    Energy moff(2.*_mpi),meta(getParticleData(ParticleID::etaprime)->mass()),
      upp(meta),step((meta-moff)/_npoints);
    // intergrator
    GaussianIntegrator integrator;
    // integrand
    OmnesIntegrand D1(intphase,sqr(_epscut));
    // loop for integrals
    double D1real,D1imag;
    Complex ii(0.,1.),answer;
    moff+=0.5*step;
    _omnesfunctionreal.clear();
    _omnesfunctionimag.clear();
    _omnesenergy.clear();
    for( ;moff<upp;moff+=step) {
      D1.setScale(moff*moff);
      // piece between 0 and 1 GeV
      using Constants::pi;
      Energy2 moff2(sqr(moff)),eps2(sqr(_epscut));
      D1real=-moff2*(integrator.value(D1,4.*_mpi*_mpi,moff2-eps2)+
		     integrator.value(D1,moff2+eps2,upp*upp))/pi;
      D1imag=-(*intphase)(moff);
      // piece above 1 GeV
      D1real+=-(*intphase)(upp)/pi*log(upp*upp/(upp*upp-moff*moff));
      // calculate the answer
      answer = exp(D1real+ii*D1imag);
      // put into the arrays
      _omnesfunctionreal.push_back(answer.real());
      _omnesfunctionimag.push_back(answer.imag());
      _omnesenergy.push_back(moff);
    }
  }
  // set up the modes
  tPDVector extpart(4);
  extpart[1] = getParticleData(ParticleID::piplus);
  extpart[2] = getParticleData(ParticleID::piminus);
  extpart[3] = getParticleData(ParticleID::gamma);
  vector<double> dummyweights(1,1.);
  DecayPhaseSpaceChannelPtr newchannel;
  DecayPhaseSpaceModePtr mode;
  for(unsigned int ix=0;ix<_coupling.size();++ix) {
    extpart[0] = getParticleData(_incoming[ix]);
    mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,3);
    newchannel->addIntermediate(rho,0,0.0, 1,2);
    mode->addChannel(newchannel);
    addMode(mode,_maxweight[ix],dummyweights);
  }
}

int EtaPiPiGammaDecayer::modeNumber(bool & cc,tcPDPtr parent,
				    const tPDVector & children) const {
  int imode(-1);
  // check number of external particles
  if(children.size()!=3){return imode;}
  // check the outgoing particles
  unsigned int npip(0),npim(0),ngamma(0);
  tPDVector::const_iterator pit = children.begin();
  int id;
  for(;pit!=children.end();++pit) {
    id=(**pit).id();
    if(id==ParticleID::piplus)       ++npip;
    else if(id==ParticleID::piminus) ++npim;
    else if(id==ParticleID::gamma)   ++ngamma;
  }
  if(!(npip==1&&npim==1&&ngamma==1)) return imode;
  unsigned int ix(0);
  id=parent->id();
  do{if(id==_incoming[ix]){imode=ix;}++ix;}
  while(imode<0&&ix<_incoming.size());
  cc=false;
  return imode;
}

void EtaPiPiGammaDecayer::persistentOutput(PersistentOStream & os) const {
  os << ounit(_fpi,MeV) << _incoming << _coupling << _maxweight << _option 
     << ounit(_aconst,1/MeV2) << _cconst <<ounit(_mrho,MeV) << ounit(_rhowidth,MeV) 
     << _rhoconst << ounit(_mpi,MeV) << _localparameters
     << ounit(_energy,MeV) << ounit(_omnesenergy,MeV) 
     << _phase << _omnesfunctionreal << _omnesfunctionimag << _initialize
     << _npoints << ounit(_epscut,MeV);
}

void EtaPiPiGammaDecayer::persistentInput(PersistentIStream & is, int) {
  is >> iunit(_fpi,MeV) >> _incoming >> _coupling >> _maxweight >> _option 
     >> iunit(_aconst,1/MeV2) >> _cconst >>iunit(_mrho,MeV) >> iunit(_rhowidth,MeV) 
     >> _rhoconst >> iunit(_mpi,MeV) >> _localparameters
     >> iunit(_energy,MeV) >> iunit(_omnesenergy,MeV) 
     >> _phase >>_omnesfunctionreal >> _omnesfunctionimag >> _initialize
     >> _npoints >> iunit(_epscut,MeV);
}

void EtaPiPiGammaDecayer::Init() {

  static ClassDocumentation<EtaPiPiGammaDecayer> documentation
    ("The EtaPiPiGammaDecayer class is design for the decay of"
     " the eta and eta prime to pi+pi-gamma",
     "The decays of $\\eta,\\eta'\\to\\pi^+\\pi^-\\gamma$ were simulated"
     " using the matrix elements from \\cite{Venugopal:1998fq,Holstein:2001bt}",
     "\\bibitem{Venugopal:1998fq} E.~P.~Venugopal and B.~R.~Holstein,\n"
     "Phys.\\ Rev.\\  D {\\bf 57} (1998) 4397 [arXiv:hep-ph/9710382].\n"
     "%%CITATION = PHRVA,D57,4397;%%\n"
     "\\bibitem{Holstein:2001bt} B.~R.~Holstein,\n"
     " Phys.\\ Scripta {\\bf T99} (2002) 55 [arXiv:hep-ph/0112150].\n"
     "%%CITATION = PHSTB,T99,55;%%\n");

  static Parameter<EtaPiPiGammaDecayer,Energy> interfacefpi
    ("fpi",
     "The pion decay constant",
     &EtaPiPiGammaDecayer::_fpi, MeV, 130.7*MeV, ZERO, 200.*MeV,
     false, false, false); 

  static ParVector<EtaPiPiGammaDecayer,int> interfaceIncoming
    ("Incoming",
     "The PDG code for the incoming particle",
     &EtaPiPiGammaDecayer::_incoming,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<EtaPiPiGammaDecayer,double> interfaceCoupling
    ("Coupling",
     "The coupling for the decay mode",
     &EtaPiPiGammaDecayer::_coupling,
     0, 0, 0, 0., 100., false, false, true);

  static ParVector<EtaPiPiGammaDecayer,double> interfaceMaxWeight
    ("MaxWeight",
     "The maximum weight for the decay mode",
     &EtaPiPiGammaDecayer::_maxweight,
     0, 0, 0, 0., 100., false, false, true);

  static Parameter<EtaPiPiGammaDecayer,Energy> interfaceRhoMass
    ("RhoMass",
     "The mass of the rho",
     &EtaPiPiGammaDecayer::_mrho, MeV, 771.1*MeV, 400.*MeV, 1000.*MeV,
     false, false, false);

  static Parameter<EtaPiPiGammaDecayer,Energy> interfaceRhoWidth
    ("RhoWidth",
     "The width of the rho",
     &EtaPiPiGammaDecayer::_rhowidth, MeV, 149.2*MeV, 100.*MeV, 300.*MeV,
     false, false, false);

  static Switch<EtaPiPiGammaDecayer,bool> interfaceLocalParameters
    ("LocalParameters",
     "Use local values of the rho mass and width",
     &EtaPiPiGammaDecayer::_localparameters, true, false, false);
  static SwitchOption interfaceLocalParametersLocal
    (interfaceLocalParameters,
     "Local",
     "Use local parameters",
     true);
  static SwitchOption interfaceLocalParametersParticleData
    (interfaceLocalParameters,
     "ParticleData",
     "Use values from the particle data objects",
     false);

  static Parameter<EtaPiPiGammaDecayer,double> interfaceOmnesC
    ("OmnesC",
     "The constant c for the Omnes form of the prefactor",
     &EtaPiPiGammaDecayer::_cconst, 1.0, -10., 10.,
     false, false, false);

  static Parameter<EtaPiPiGammaDecayer,InvEnergy2> interfaceOmnesA
    ("OmnesA",
     "The constant a for the Omnes form of the prefactor",
     &EtaPiPiGammaDecayer::_aconst, 1./GeV2, 0.8409082/GeV2, ZERO,
     10./GeV2,
     false, false, false);

  static ParVector<EtaPiPiGammaDecayer,int> interfaceOption
    ("Option",
     "The form of the prefactor 0 is a VMD model using M Gamma for the width term,"
     "1 is a VMD model using q Gamma for the width term,"
     "2. analytic form of the Omnes function,"
     "3. experimental form of the Omnes function.",
     &EtaPiPiGammaDecayer::_option,
     0, 0, 0, 0, 4, false, false, true);

  static ParVector<EtaPiPiGammaDecayer,Energy> interfacePhase_Energy
    ("Phase_Energy",
     "The energy values for the phase shift for the experimental version of the"
     " Omnes function",
     &EtaPiPiGammaDecayer::_energy, MeV, -1, 1.0*MeV, 300.0*MeV, 2000.0*MeV,
     false, false, true);

  static ParVector<EtaPiPiGammaDecayer,double> interfacePhase_Shift
    ("Phase_Shift",
     "The experimental values of the phase shift for the experimental version"
     " of the Omnes function",
     &EtaPiPiGammaDecayer::_phase, 1.0, -1, 0.0, 0.0, 1000.0,
     false, false, true);

  static ParVector<EtaPiPiGammaDecayer,Energy> interfaceOmnesEnergy
    ("OmnesEnergy",
     "The energy values for the interpolation of the experimental Omnes function",
     &EtaPiPiGammaDecayer::_omnesenergy, MeV, -1, 1.*MeV, 250.0*MeV, 2000.*MeV,
     false, false, true);

  static ParVector<EtaPiPiGammaDecayer,double> interfaceOmnesReal
    ("OmnesReal",
     "The real part of the experimental Omnes function for the interpolation.",
     &EtaPiPiGammaDecayer::_omnesfunctionreal, 1., -1, 1., -100.,
     100.,
     false, false, true);

  static ParVector<EtaPiPiGammaDecayer,double> interfaceOmnesImag
    ("OmnesImag",
     "The imaginary part of the experimental Omnes function for the interpolation.",
     &EtaPiPiGammaDecayer::_omnesfunctionimag, 1., -1, 1., -100.,
     100.,
     false, false, true);

  static Switch<EtaPiPiGammaDecayer,bool> interfaceInitializeOmnes
    ("InitializeOmnes",
     "Initialize the experimental version of the Omnes function.",
     &EtaPiPiGammaDecayer::_initialize, false, false, false);
  static SwitchOption interfaceInitializeOmnesInitialize
    (interfaceInitializeOmnes,
     "Yes",
     "Perform the initialization",
     true);
  static SwitchOption interfaceInitializeOmnesNoInitialization
    (interfaceInitializeOmnes,
     "No",
     "No initialization",
     false);

  static Parameter<EtaPiPiGammaDecayer,unsigned int> interfaceOmnesPoints
    ("OmnesPoints",
     "The number of points for the interpolation table for the experimental"
     " Omnes function.",
     &EtaPiPiGammaDecayer::_npoints, 100, 50, 200,
     false, false, true);

  static Parameter<EtaPiPiGammaDecayer,Energy> interfaceOmnesCut
    ("OmnesCut",
     "The cut parameter for the integral in the experimental Omnes function.",
     &EtaPiPiGammaDecayer::_epscut, MeV, 0.1*MeV, 0.001*MeV, 1.0*MeV,
     false, false, true);

}

double EtaPiPiGammaDecayer::me2(const int,const Particle & inpart,
				const ParticleVector & decay,
				MEOption meopt) const {
  if(!ME())
    ME(new_ptr(GeneralDecayMatrixElement(PDT::Spin0,PDT::Spin0,PDT::Spin0,PDT::Spin1)));
  useMe();
  if(meopt==Initialize) {
    ScalarWaveFunction::
      calculateWaveFunctions(_rho,const_ptr_cast<tPPtr>(&inpart),incoming);
  }
  if(meopt==Terminate) {
    // set up the spin information for the decay products
    ScalarWaveFunction::constructSpinInfo(const_ptr_cast<tPPtr>(&inpart),
					  incoming,true);
    for(unsigned int ix=0;ix<2;++ix)
    ScalarWaveFunction::constructSpinInfo(decay[ix],outgoing,true);
    VectorWaveFunction::constructSpinInfo(_vectors,decay[2],
					  outgoing,true,true);
    return 0.;
  }
  VectorWaveFunction::calculateWaveFunctions(_vectors,decay[2],
					     outgoing,true);
  // prefactor for the matrix element
  complex<InvEnergy3> pre(_coupling[imode()]*2.*sqrt(2.)/(_fpi*_fpi*_fpi));
  Lorentz5Momentum ppipi(decay[0]->momentum()+decay[1]->momentum());
  ppipi.rescaleMass();
  Energy q(ppipi.mass());
  Energy2 q2(q*q);
  Complex ii(0.,1.);
  // first VMD option
  Complex fact;
  if(_option[imode()]==0) {
    Energy pcm(Kinematics::pstarTwoBodyDecay(q,_mpi,_mpi));
    Complex resfact(q2/(_mrho*_mrho-q2-ii*_mrho*pcm*pcm*pcm*_rhoconst/q2));
    fact=(1.+1.5*resfact);
  }
  // second VMD option
  else if(_option[imode()]==1) {
    Energy pcm(Kinematics::pstarTwoBodyDecay(q,_mpi,_mpi));
    Complex resfact(q2/(_mrho*_mrho-q2-ii*pcm*pcm*pcm*_rhoconst/q));
    fact=(1.+1.5*resfact);
  }
  // analytic omne function
  else if(_option[imode()]==2) {
    fact=(1.-_cconst+_cconst*(1.+_aconst*q2)/analyticOmnes(q2));
  }
  // experimental omnes function
  else if(_option[imode()]==3) {
    fact=(1.-_cconst+_cconst*(1.+_aconst*q2)/experimentalOmnes(q2));
  }
  pre = pre*fact;
  LorentzPolarizationVector epstemp(pre*Helicity::epsilon(decay[0]->momentum(),
							  decay[1]->momentum(),
							  decay[2]->momentum()));
  // compute the matrix element
  vector<unsigned int> ispin(4,0);
  for(ispin[3]=0;ispin[3]<3;++ispin[3]) {
    if(ispin[3]==1) (*ME())(ispin)=0.;
    else            (*ME())(ispin)=epstemp.dot(_vectors[ispin[3]]);
  }
  // contract the whole thing
  return ME()->contract(_rho).real();
}

double EtaPiPiGammaDecayer::
threeBodyMatrixElement(const int imodeb,const Energy2 ,const  Energy2 s3,const 
		       Energy2 s2,const Energy2 s1,const Energy ,
		       const Energy ,const Energy ) const {
  complex<InvEnergy3> pre(_coupling[imodeb]*2.*sqrt(2.)/pow<3,1>(_fpi));
  Energy q(sqrt(s3));
  Complex ii(0.,1.);
  // first VMD option
  Complex fact;
  if(_option[imodeb]==0) {
    Energy pcm(Kinematics::pstarTwoBodyDecay(q,_mpi,_mpi));
    Complex resfact(s3/(_mrho*_mrho-s3-ii*_mrho*pcm*pcm*pcm*_rhoconst/s3));
    fact=(1.+1.5*resfact);
  }
  // second VMD option
  else if(_option[imodeb]==1) {
    Energy pcm(Kinematics::pstarTwoBodyDecay(q,_mpi,_mpi));
    Complex resfact(s3/(_mrho*_mrho-s3-ii*pcm*pcm*pcm*_rhoconst/q));
    fact=(1.+1.5*resfact);
  }
  // analytic omne function
  else if(_option[imodeb]==2) {
    fact=(1.-_cconst+_cconst*(1.+_aconst*s3)/analyticOmnes(s3));
  }
  // experimental omnes function
  else if(_option[imodeb]==3) {
    fact=(1.-_cconst+_cconst*(1.+_aconst*s3)/experimentalOmnes(s3));
  }
  pre =pre*fact;
  InvEnergy6 factor((pre*conj(pre)).real());
  Energy2 mpi2(_mpi*_mpi);
  return factor*((-mpi2*(-2*mpi2+s1+s2)*(-2*mpi2+s1+s2)+(mpi2-s1)*(mpi2-s2)*s3)/4.);
}

WidthCalculatorBasePtr 
EtaPiPiGammaDecayer::threeBodyMEIntegrator(const DecayMode & dm) const {
  // workout which mode we are doing
  int id(dm.parent()->id()),imode(1);
  if(id==ParticleID::eta){imode=0;}
  // construct the integrator
  vector<double> inweights(1,1.);
  vector<Energy> inmass(1,getParticleData(ParticleID::rho0)->mass());
  vector<Energy> inwidth(1,getParticleData(ParticleID::rho0)->width());
  vector<int> intype(1,1);
  vector<double> inpow(1,0.0);
  WidthCalculatorBasePtr 
    output(new_ptr(ThreeBodyAllOnCalculator<EtaPiPiGammaDecayer>
		   (inweights,intype,inmass,inwidth,inpow,*this,imode,_mpi,_mpi,ZERO)));
  return output;
}

void EtaPiPiGammaDecayer::dataBaseOutput(ofstream & output,
					 bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  output << "newdef " << name() << ":fpi             " << _fpi/MeV         << "\n";
  output << "newdef " << name() << ":RhoMass         " << _mrho/MeV        << "\n";
  output << "newdef " << name() << ":RhoWidth        " << _rhowidth/MeV    << "\n";
  output << "newdef " << name() << ":LocalParameters " << _localparameters << "\n";
  output << "newdef " << name() << ":OmnesC          " << _cconst          << "\n";
  output << "newdef " << name() << ":OmnesA          " << _aconst*GeV2     << "\n";
  output << "newdef " << name() << ":InitializeOmnes " << _initialize      << "\n";
  output << "newdef " << name() << ":OmnesPoints     " << _npoints         << "\n";
  output << "newdef " << name() << ":OmnesCut        " << _epscut/MeV      << "\n";
  for(unsigned int ix=0;ix<2;++ix) {
    output << "newdef " << name() << ":Incoming    " << ix << "  " 
	   << _incoming[ix]    << "\n";
    output << "newdef " << name() << ":Coupling    " << ix << "  " 
	   << _coupling[ix]    << "\n";
    output << "newdef " << name() << ":MaxWeight   " << ix << "  " 
	   << _maxweight[ix]   << "\n";
    output << "newdef " << name() << ":Option      " << ix << "  " 
	   << _option[ix]      << "\n";
  }
  for(unsigned int ix=0;ix<_energy.size();++ix) {
    if(ix<_nsizea) {
      output << "newdef " << name() << ":Phase_Energy " << ix << "  " 
	     << _energy[ix]/MeV << "\n";
      output << "newdef " << name() << ":Phase_Shift  " << ix << "  " 
	     << _phase[ix]  << "\n";
    }
    else {
      output << "insert " << name() << ":Phase_Energy " << ix << "  " 
	     << _energy[ix]/MeV << "\n";
      output << "insert " << name() << ":Phase_Shift  " << ix << "  " 
	     << _phase[ix]  << "\n";
    }
  }
  for(unsigned int ix=0;ix<_omnesenergy.size();++ix) {
      if(ix<_nsizeb) {
	output << "newdef " << name() << ":OmnesEnergy " << ix << "  " 
	       << _omnesenergy[ix]/MeV << "\n";
	output << "newdef " << name() << ":OmnesReal " << ix << "  " 
	       << _omnesfunctionreal[ix] << "\n";
	output << "newdef " << name() << ":OmnesImag " << ix << "  " 
	       << _omnesfunctionimag [ix] << "\n";
      }
      else {
	output << "insert " << name() << ":OmnesEnergy " << ix << "  " 
	       << _omnesenergy[ix]/MeV << "\n";
	output << "insert " << name() << ":OmnesReal " << ix << "  " 
	       << _omnesfunctionreal[ix] << "\n";
	output << "insert " << name() << ":OmnesImag " << ix << "  " 
	       << _omnesfunctionimag [ix] << "\n";
      }
  }
  if(header) output << "\n\" where BINARY ThePEGName=\"" << fullName() << "\";" << endl;
}
#line 1 "./EtaPiPiPiDecayer.cc"
// -*- C++ -*-
//
// EtaPiPiPiDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the EtaPiPiPiDecayer class.
//
#include "EtaPiPiPiDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "Herwig/PDT/ThreeBodyAllOn1IntegralCalculator.h"
#include "Herwig/PDT/OneOffShellCalculator.h"
#include "Herwig/Decay/GeneralDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void EtaPiPiPiDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()) {
    for(unsigned int ix=0;ix<_incoming.size();++ix)
      if(mode(ix)) _maxweight[ix] = mode(ix)->maxWeight();
  }
}

EtaPiPiPiDecayer::EtaPiPiPiDecayer() 
  : _incoming(6), _outgoing(6), _charged(6), _prefactor(6),
    _a(6), _b(6), _c(6), _maxweight(6) {
  // eta to pi+pi-pi0
  _incoming[0] = 221; _outgoing[0] = 111; _charged[0] = true; 
  _prefactor[0] = 0.06477; _maxweight[0] = 1.72861;
  _a[0] = -1.17; _b[0] = 0.21; _c[0] = 0.06; 
  // eta to pi0pi0pi0
  _incoming[1] = 221; _outgoing[1] = 111; _charged[1] = false; 
  _prefactor[1] = 0.0883547; _maxweight[1] = 1.45813; 
  _a[1] = 0.; _b[1] = -0.062; _c[1] = -0.062; 
  // eta' to pi+pi-pi0
  _incoming[2] = 331; _outgoing[2] = 111; _charged[2] = true; 
  _prefactor[2] = 0.037165; _maxweight[2] = 0.0153201;
  _a[2] = -3.08; _b[2] = 0.13; _c[2] = 0.62; 
  // eta' to pi0pi0pi0
  _incoming[3] = 331; _outgoing[3] = 111; _charged[3] = false; 
  _prefactor[3] = 0.016203; _maxweight[3] = 2.52411; 
  _a[3] = 0.0; _b[3] = -0.86; _c[3] = -0.86; 
  // eta' to pi+pi-eta
  _incoming[4] = 331; _outgoing[4] = 221; _charged[4] = true; 
  _prefactor[4] = 49.42; _maxweight[4] = 1.421;
  _a[4] = -0.093; _b[4] = -0.059; _c[4] = -0.003; 
  // eta' to pi0pi0eta
  _incoming[5] = 331; _outgoing[5] = 221; _charged[5] = false; 
  _prefactor[5] = 20.62; _maxweight[5] = 1.42649;
  _a[5] = -0.105; _b[5] = -0.065; _c[5] = -0.004; 
  // initial size of the arrays
  _initsize=_maxweight.size();
  // intermediates
  generateIntermediates(false);
}
 
void EtaPiPiPiDecayer::doinit() {
  DecayIntegrator::doinit();
  // check consistency of the parameters
  unsigned int isize(_incoming.size());
  if(isize!=_outgoing.size()||isize!=_prefactor.size()||
     isize!=_charged.size()||isize!=_a.size()||
     isize!=_b.size()||isize!=_c.size()||isize!=_maxweight.size())
    throw InitException() << "Inconsistent parameters in EtaPiPiPiDecayer::doinit()"
			  << Exception::runerror;
  // external particles for the modes
  tPDVector extneut(4),extcharged(4);
  extneut[1]    = getParticleData(ParticleID::pi0);
  extneut[2]    = getParticleData(ParticleID::pi0);
  extcharged[1] = getParticleData(ParticleID::piplus);
  extcharged[2] = getParticleData(ParticleID::piminus);
  tPDPtr rho(getParticleData(113));
  DecayPhaseSpaceModePtr mode;
  DecayPhaseSpaceChannelPtr newchannel;
  vector<double> dummyweights(1,1.);
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    extneut[0]    = getParticleData(_incoming[ix]);
    extcharged[0] = getParticleData(_incoming[ix]);
    extneut[3]    = getParticleData(_outgoing[ix]);
    extcharged[3] = getParticleData(_outgoing[ix]);
    if(_charged[ix]) {
      // the pi+pi- mode
      mode = new_ptr(DecayPhaseSpaceMode(extcharged,this));
      newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
      newchannel->addIntermediate(extcharged[0],0, 0.0,-1,3);
      newchannel->addIntermediate(rho,1,0.0, 1,2);
      mode->addChannel(newchannel);
    }
    else {
      // the pi0pi0 mode
      mode = new_ptr(DecayPhaseSpaceMode(extneut,this));
      newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
      newchannel->addIntermediate(extneut[0],0, 0.0,-1,3);
      newchannel->addIntermediate(rho,1,0.0, 1,2);
      mode->addChannel(newchannel);
    }
    addMode(mode,_maxweight[ix],dummyweights);
  }
  resetIntermediate(rho,600.*MeV,600.*MeV);
}

int EtaPiPiPiDecayer::modeNumber(bool & cc,tcPDPtr parent,
				 const tPDVector & children) const {
  if(children.size()!=3) return -1;
  unsigned int npi0(0),npip(0),npim(0); int id,iother(0);
  tPDVector::const_iterator pit = children.begin();
  for( ;pit!=children.end();++pit) {
    id=(**pit).id();
    if(id==ParticleID::piplus)           ++npip;
    else if(id==ParticleID::piminus)     ++npim;
    else if(id==ParticleID::pi0&&npi0<2) ++npi0;
    else iother=id;
  }
  bool charged;
  if(npim==1&&npip==1) {
    charged=true;
    if(npi0==1) iother=ParticleID::pi0;
  }
  else if(npi0==2) charged=false;
  else return -1;
  // find the mode
  id=parent->id();
  unsigned int ix(0);
  int imode(-1);
  do {
    if(id==_incoming[ix]&&iother==_outgoing[ix]&&_charged[ix]==charged) 
      imode=ix;
    ++ix;
  }
  while(imode<0&&ix<_incoming.size());
  cc=false;
  return imode;
}

void EtaPiPiPiDecayer::persistentOutput(PersistentOStream & os) const {
  os << _incoming << _outgoing << _charged << _prefactor << _a << _b << _c  
     << _maxweight;
}

void EtaPiPiPiDecayer::persistentInput(PersistentIStream & is, int) {
  is >> _incoming >> _outgoing >> _charged >> _prefactor >> _a >> _b >> _c 
     >> _maxweight;
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<EtaPiPiPiDecayer,DecayIntegrator>
describeHerwigEtaPiPiPiDecayer("Herwig::EtaPiPiPiDecayer", "HwSMDecay.so");

void EtaPiPiPiDecayer::Init() {

  static ClassDocumentation<EtaPiPiPiDecayer> documentation
    ("The EtaPiPiPiDecayer class performs the decay of a scalar meson to"
     " two pions and another meson using a simple paramterisation of the dalitz plot.",
     "The decay of eta to two pions follows \\cite{Beisert:2003zs,Gormley:1970qz,Tippens:2001fm}.",
     "%\\cite{Beisert:2003zs}\n"
     "\\bibitem{Beisert:2003zs}\n"
     "  N.~Beisert and B.~Borasoy,\n"
     "  %``Hadronic decays of eta and eta' with coupled channels,''\n"
     "  Nucl.\\ Phys.\\  A {\\bf 716}, 186 (2003)\n"
     "  [arXiv:hep-ph/0301058].\n"
     "  %%CITATION = NUPHA,A716,186;%%\n"
     "%\\cite{Gormley:1970qz}\n"
     "\\bibitem{Gormley:1970qz}\n"
     "  M.~Gormley, E.~Hyman, W.~Y.~Lee, T.~Nash, J.~Peoples, C.~Schultz and S.~Stein,\n"
     "   ``Experimental determination of the dalitz-plot distribution of the decays\n"
     "   eta $\\to$ pi+ pi- pi0 and eta $\\to$ pi+ pi- gamma, and the branching ratio\n"
     "  %eta $\\to$ pi+ pi- gamma/eta $\\to$ pi+,''\n"
     "  Phys.\\ Rev.\\  D {\\bf 2}, 501 (1970).\n"
     "  %%CITATION = PHRVA,D2,501;%%\n"
     "%\\cite{Tippens:2001fm}\n"
     "\\bibitem{Tippens:2001fm}\n"
     "  W.~B.~Tippens {\\it et al.}  [Crystal Ball Collaboration],\n"
     "  %``Determination of the quadratic slope parameter in eta $\\to$ 3pi0 decay,''\n"
     "  Phys.\\ Rev.\\ Lett.\\  {\\bf 87}, 192001 (2001).\n"
     "  %%CITATION = PRLTA,87,192001;%%\n"
     );

  static ParVector<EtaPiPiPiDecayer,int> interfaceIncoming
    ("Incoming",
     "The PDG code of the incoming particle",
     &EtaPiPiPiDecayer::_incoming, -1, 0,  0, 1000000,
     false, false, true);

  static ParVector<EtaPiPiPiDecayer,int> interfaceOutgoing
    ("Outgoing",
     "The PDG code of the outgoing particle",
     &EtaPiPiPiDecayer::_outgoing, -1, 0,  0, 1000000,
     false, false, true);

  static ParVector<EtaPiPiPiDecayer,bool> interfaceCharged
    ("Charged",
     "Whether the pions or charged or neutral",
     &EtaPiPiPiDecayer::_charged,  -1,false, 0, 0,
     false, false, false);

  static ParVector<EtaPiPiPiDecayer,double> interfacePrefactor
    ("Prefactor",
     "The prefactor for the decay to get the correct partial width",
     &EtaPiPiPiDecayer::_prefactor, -1,1.0,  0, 0,
     false, false, false);

  static ParVector<EtaPiPiPiDecayer,double> interfacea
    ("a",
     "The a parameter for the dalitz plot",
     &EtaPiPiPiDecayer::_a, -1, 0.0,  -10.0, 10.0,
     false, false, true);

  static ParVector<EtaPiPiPiDecayer,double> interfaceb
    ("b",
     "The b parameter for the dalitz plot",
     &EtaPiPiPiDecayer::_b, -1, 0.0,  -10.0, 10.0,
     false, false, true);

  static ParVector<EtaPiPiPiDecayer,double> interfacec
    ("c",
     "The c parameter for the dalitz plot",
     &EtaPiPiPiDecayer::_c, -1, 0.0,  -10.0, 10.0,
     false, false, true);

  static ParVector<EtaPiPiPiDecayer,double> interfaceMaxWeight
    ("MaxWeight",
     "The maximum weight for the decay mode",
     &EtaPiPiPiDecayer::_maxweight,
     0, 0, 0, 0., 200., false, false, true);

}

double EtaPiPiPiDecayer::me2(const int,const Particle & inpart,
			     const ParticleVector & decay,
			     MEOption meopt) const {
  if(!ME())
    ME(new_ptr(GeneralDecayMatrixElement(PDT::Spin0,PDT::Spin0,PDT::Spin0,PDT::Spin0)));
  useMe();
  if(meopt==Initialize) {
    ScalarWaveFunction::
      calculateWaveFunctions(_rho,const_ptr_cast<tPPtr>(&inpart),incoming);
  }
  if(meopt==Terminate) {
    // set up the spin information for the decay products
    ScalarWaveFunction::constructSpinInfo(const_ptr_cast<tPPtr>(&inpart),
					  incoming,true);
    for(unsigned int ix=0;ix<3;++ix)
      ScalarWaveFunction::constructSpinInfo(decay[ix],outgoing,true);
    return 0.;
  }
  // calculate the matrix element
  // compute the variables we need
  Lorentz5Momentum ps(inpart.momentum()-decay[2]->momentum());ps.rescaleMass();
  Lorentz5Momentum pu(inpart.momentum()-decay[0]->momentum());pu.rescaleMass();
  Lorentz5Momentum pt(inpart.momentum()-decay[1]->momentum());pt.rescaleMass();
  Energy2 s(ps.mass2()),u(pu.mass2()),t(pt.mass2());
  Energy m34(0.5*(decay[0]->mass()+decay[1]->mass()));
  Energy msum(decay[2]->mass()+2.*m34);
  Energy Q(inpart.mass()-msum);
  Energy2 Mmm2((inpart.mass()-decay[2]->mass())*(inpart.mass()-decay[2]->mass()));
  // compute the variables
  double x(0.5*sqrt(3.)*(u-t)/inpart.mass()/Q),x2(x*x);
  double y(0.5*msum/inpart.mass()*(Mmm2-s)/m34/Q-1),y2(y*y);
  double me(_prefactor[imode()]*(1+_a[imode()]*y+_b[imode()]*y2+_c[imode()]*x2));
  (*ME())(0,0,0,0)=sqrt(me);
  return me;
}

InvEnergy EtaPiPiPiDecayer::threeBodydGammads(const int imodeb, const Energy2 q2,
					   const  Energy2 s, const Energy m1,
					   const Energy m2, const Energy m3) const {
  Energy q(sqrt(q2)),m34(m1+m2),msum(m34+m3),Q(q-msum);
  Energy2 Mmm2((q-m3)*(q-m3)),m12(m1*m1),m22(m2*m2),m32(m3*m3);
  double y(0.5*msum/q*(Mmm2-s)/m34/Q-1),y2(y*y);
  InvEnergy2 xfact=0.5*sqrt(3.)/q/Q;
  Energy2 xc(q2+m12+m22+m32-s);
  Energy rs(sqrt(s)),e2star(0.5*(s-m12+m22)/rs),e3star(0.5*(q2-s-m32)/rs);
  Energy e2sm(sqrt(e2star*e2star-m22)),e3sm(sqrt(e3star*e3star-m32));
  Energy2 a(2*e2star*e3star+m22+m32),b(2*e2sm*e3sm);
  Energy2 output=2*b*(1+_a[imodeb]*y+_b[imodeb]*y2+_c[imodeb]*xfact*xfact*(xc*xc))
    +_c[imodeb]*(-8.*xfact*xfact*xc*a*b
		 +4.*2*b*(3.*a*a+b*b)/3.*xfact*xfact);
  using Constants::pi;
  return output*_prefactor[imodeb]/256./pi/pi/pi/q2/q;
}


WidthCalculatorBasePtr 
EtaPiPiPiDecayer::threeBodyMEIntegrator(const DecayMode & dm) const {
  int idout(0),id,imode(-1);
  unsigned int npi0(0),ix(0);
  ParticleMSet::const_iterator pit(dm.products().begin());
  for( ;pit!=dm.products().end();++pit) {
    id=(**pit).id();
    if(id==ParticleID::pi0&&npi0<2)                            ++npi0;
    else if(id!=ParticleID::piplus&&id!=ParticleID::piminus) idout=id;
  }
  if(npi0==1) idout=ParticleID::pi0;
  bool charged(npi0<2);
  id=dm.parent()->id();
  do {
    if(id==_incoming[ix]&&idout==_outgoing[ix]&&_charged[ix]==charged) 
      imode=ix;
    ++ix;
  }
  while(imode<0&&ix<_incoming.size());
  Energy mpi;
  if(charged){mpi=getParticleData(ParticleID::piplus)->mass();}
  else{mpi=getParticleData(ParticleID::pi0)->mass();}
  Energy m[3]={mpi,mpi,getParticleData(_outgoing[imode])->mass()};
  WidthCalculatorBasePtr 
    temp(new_ptr(ThreeBodyAllOn1IntegralCalculator<EtaPiPiPiDecayer>
		 (1,-1000.*MeV,ZERO,0.0,*this,imode,m[0],m[1],m[2])));
  if(_outgoing[imode]==ParticleID::eta) {
    tcGenericMassGeneratorPtr test;
    tGenericMassGeneratorPtr massptr;
    if(getParticleData(_outgoing[imode])->massGenerator()) {
      test=dynamic_ptr_cast<tcGenericMassGeneratorPtr>
	(getParticleData(_outgoing[imode])->massGenerator());
      massptr=const_ptr_cast<tGenericMassGeneratorPtr>(test);
    }
    if(massptr) {
      massptr->init();
      return new_ptr(OneOffShellCalculator(3,temp,massptr,ZERO));
    }
  }
  return temp;
} 
  
void EtaPiPiPiDecayer::dataBaseOutput(ofstream & output,
				      bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    if(ix<_initsize) {
      output << "newdef " << name() << ":Incoming   " << ix << " "
	     << _incoming[ix]   << "\n";
      output << "newdef " << name() << ":Outgoing  " << ix << " "
	     << _outgoing[ix]  << "\n";
      output << "newdef " << name() << ":Charged " << ix << " "
		 << _charged[ix]  << "\n";
      output << "newdef " << name() << ":Prefactor " << ix << " "
	     << _prefactor[ix]  << "\n";
      output << "newdef " << name() << ":a " << ix << " "
	     << _a[ix]  << "\n";
      output << "newdef " << name() << ":b " << ix << " "
	     << _b[ix]  << "\n";
      output << "newdef " << name() << ":c " << ix << " "
	     << _c[ix]  << "\n";
      output << "newdef " << name() << ":MaxWeight  " << ix << " "
	     << _maxweight[ix]  << "\n";
    }
    else {
      output << "insert " << name() << ":Incoming   " << ix << " "
	     << _incoming[ix]   << "\n";
      output << "insert " << name() << ":Outgoing  " << ix << " "
	     << _outgoing[ix]  << "\n";
      output << "insert " << name() << ":Charged " << ix << " "
	     << _charged[ix]  << "\n";
      output << "insert " << name() << ":Prefactor " << ix << " "
	     << _prefactor[ix]  << "\n";
      output << "insert " << name() << ":a " << ix << " "
	     << _a[ix]  << "\n";
      output << "insert " << name() << ":b " << ix << " "
	     << _b[ix]  << "\n";
      output << "insert " << name() << ":c " << ix << " "
	     << _c[ix]  << "\n";
      output << "insert " << name() << ":MaxWeight  " << ix << " "
	     << _maxweight[ix]  << "\n";
    }
  }
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}
#line 1 "./PScalar4FermionsDecayer.cc"
// -*- C++ -*-
//
// PScalar4FermionsDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the PScalar4FermionsDecayer class.
//

#include "PScalar4FermionsDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/StandardModel/StandardModelBase.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Helicity/ScalarSpinInfo.h"
#include "ThePEG/Helicity/FermionSpinInfo.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/SpinorWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/SpinorBarWaveFunction.h"
#include "ThePEG/Helicity/epsilon.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "Herwig/Decay/GeneralDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void PScalar4FermionsDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()) {
    for(unsigned int ix=0;ix<_incoming.size();++ix)
      _maxweight[ix] = mode(ix)->maxWeight();
  }
}

PScalar4FermionsDecayer::PScalar4FermionsDecayer() 
  : _coupling(1,0.025159062/GeV), _incoming(1,111), _outgoing1(1,11), 
    _outgoing2(1,11), _maxweight(1,0.000234211), 
    _includeVMD(1,2),_VMDid(1,113), _VMDmass(1,0.7758*GeV), 
    _VMDwidth(1,0.1503*GeV), _initsize(1) {
  // intermediates
  generateIntermediates(false);
}

void PScalar4FermionsDecayer::doinit() {
  DecayIntegrator::doinit();
  // check the parameters are consistent
  unsigned int isize=_coupling.size();
  if(isize!=_incoming.size()  || isize!=_outgoing1.size() || isize!=_outgoing2.size()||
     isize!=_maxweight.size() || isize!=_includeVMD.size()|| isize!=_VMDid.size()    ||
     isize!=_VMDmass.size()  || isize!=_VMDwidth.size())
    throw InitException() << "Inconsistent parameters in PScalar4FermionsDecayer"
			  << Exception::abortnow;
  // create the integration channels for each mode 
  tPDVector extpart(5);
  tPDPtr gamma=getParticleData(ParticleID::gamma);
  DecayPhaseSpaceChannelPtr newchannel;
  DecayPhaseSpaceModePtr mode;
  vector<double> wgt;
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    wgt.resize(1);wgt[0]=1.;
    extpart[0] = getParticleData(_incoming[ix]);
    extpart[1] = getParticleData( _outgoing1[ix]);
    extpart[2] = getParticleData(-_outgoing1[ix]);
    extpart[3] = getParticleData( _outgoing2[ix]);
    extpart[4] = getParticleData(-_outgoing2[ix]);
    mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
    // first channel always need this
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,-2);
    newchannel->addIntermediate(gamma     ,1,-1.1, 1,2);
    newchannel->addIntermediate(gamma     ,1,-1.1, 3,4);
    mode->addChannel(newchannel);
    if(_outgoing1[ix]==_outgoing2[ix]) {
      newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
      newchannel->addIntermediate(extpart[0],0, 0.0,-1,-2);
      newchannel->addIntermediate(gamma     ,1,-1.1, 3,2);
      newchannel->addIntermediate(gamma     ,1,-1.1, 1,4);
      mode->addChannel(newchannel);
      wgt.resize(2);wgt[0]=0.5;wgt[1]=0.5;
    }
    else{wgt.resize(1);wgt[0]=1.;}
    addMode(mode,_maxweight[ix],wgt);
  }
  // set up the values for the VMD factor if needed (copy the default mass and width 
  //                                                 into the array)
  for(unsigned ix=0;ix<isize;++ix) {
    if(_includeVMD[ix]==1) {
      _VMDmass[ix]=getParticleData(_VMDid[ix])->mass();
      _VMDwidth[ix]=getParticleData(_VMDid[ix])->width();
    }
  }
}

int PScalar4FermionsDecayer::modeNumber(bool & cc,tcPDPtr parent,
					const tPDVector & children) const {
  // must be four outgoing particles
  if(children.size()!=4) return -1;
  // get the id's of the outgoing particles
  int id[4]={0,0,0,0}; 
  bool done[4]={false,false,false,false}; 
  unsigned int ix(0),iy(0);
  // ids of the particles
  int id0(parent->id()),idtemp(-1),idl1(-1),idl2(-1),idt[2];
  tPDVector::const_iterator pit = children.begin();
  for ( ;pit!=children.end();++pit) {
    id[ix]=(**pit).id();
    done[ix]=false;
    ++ix;
  }
  // find the two lepton pairs
  // find the first fermion
  ix=0;
  do {
    if( id[ix]>0 && !done[ix] ) {
      done[ix]=true;
      idtemp=id[ix];
    }
    ++ix;
  }
  while(ix<4&&idtemp<0);
  if(idtemp<0) return -1;
  // find its antiparticle
  ix=0;
  do {
    if( id[ix]==-idtemp && !done[ix] ) {
      done[ix]=true;
      idl1=idtemp;
    }
    ++ix;
  } while( ix<4 && idl1<0 );
  if(idl1<0) return -1;
  // find the second particle antiparticle pair
  for(ix=0;ix<4;++ix) {
    if(!done[ix]) {
      idt[iy]=id[ix];
      ++iy;
    }
  }
  if(idt[0]==-idt[1]) idl2=abs(idt[0]);
  if(idl2<0) return -1;
  // loop over the modes and see if this is one of them
  ix=0;
  int imode(-1);
  do {
    if(_incoming[ix]==id0) {
      if((idl1==_outgoing1[ix]&&idl2==_outgoing2[ix])||
	 (idl2==_outgoing1[ix]&&idl1==_outgoing2[ix])) imode=ix;
    }
    ++ix;
  }
  while(imode<0&&ix<_incoming.size());
  cc=false;
  return imode;
}

void PScalar4FermionsDecayer::persistentOutput(PersistentOStream & os) const {
  os << ounit(_coupling,1/MeV) 
     << _incoming << _outgoing1 << _outgoing2 << _maxweight 
     << _includeVMD << _VMDid 
     << ounit(_VMDmass,MeV) << ounit(_VMDwidth,MeV);
}

void PScalar4FermionsDecayer::persistentInput(PersistentIStream & is, int) {
  is >> iunit(_coupling,1/MeV) 
     >> _incoming >> _outgoing1 >> _outgoing2 >> _maxweight 
     >> _includeVMD >> _VMDid 
     >> iunit(_VMDmass,MeV) >> iunit(_VMDwidth,MeV);
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<PScalar4FermionsDecayer,DecayIntegrator>
describeHerwigPScalar4FermionsDecayer("Herwig::PScalar4FermionsDecayer", "HwSMDecay.so");

void PScalar4FermionsDecayer::Init() {

  static ClassDocumentation<PScalar4FermionsDecayer> documentation
    ("The PScalar4FermionsDecayer class is designed for the decay"
     " of a pseudosclar meson to four fermions. It is intended for the decay of"
     "the pion to two electron-positron pairs.");

  static ParVector<PScalar4FermionsDecayer,int> interfaceIncoming
    ("Incoming",
     "The PDG code for the incoming particle",
     &PScalar4FermionsDecayer::_incoming,
     0, 0, 0, -10000, 10000, false, false, true);

  static ParVector<PScalar4FermionsDecayer,int> interfaceOutcoming1
    ("Outgoing1",
     "The PDG code for the first outgoing fermion",
     &PScalar4FermionsDecayer::_outgoing1,
     0, 0, 0, -10000, 10000, false, false, true);

  static ParVector<PScalar4FermionsDecayer,int> interfaceOutcoming2
    ("Outgoing2",
     "The PDG code for the second outgoing fermion",
     &PScalar4FermionsDecayer::_outgoing2,
     0, 0, 0, -10000, 10000, false, false, true);

  static ParVector<PScalar4FermionsDecayer,InvEnergy> interfaceCoupling
    ("Coupling",
     "The coupling for the decay mode",
     &PScalar4FermionsDecayer::_coupling,
     1/MeV, 0, ZERO, -10000/MeV, 10000/MeV, false, false, true);

  static ParVector<PScalar4FermionsDecayer,double> interfaceMaxWeight
    ("MaxWeight",
     "The maximum weight for the decay mode",
     &PScalar4FermionsDecayer::_maxweight,
     0, 0, 0, -10000, 10000, false, false, true);

  static ParVector<PScalar4FermionsDecayer,int> interfaceIncludeVMD
    ("IncludeVMD",
     "There are three options for 0 the VMD factor is not included, for 1 the factor "
     "is included using the default mass and width of the particle specified by"
     " VMDID, and for 2 the factor is included using the mass and width specified"
     " by VMDwidth and VMDmass.",
     &PScalar4FermionsDecayer::_includeVMD,
     0, 0, 0, 0, 2, false, false, true);

  static ParVector<PScalar4FermionsDecayer,int> interfaceVMDID
    ("VMDID",
     "The PDG code for the particle to be used for the VMD factor.",
     &PScalar4FermionsDecayer::_VMDid,
     0, 0, 0, -10000, 10000, false, false, true);

  static ParVector<PScalar4FermionsDecayer,Energy> interfaceVMDmass
    ("VMDmass",
     "The mass to use for the particle in the VMD factor",
     &PScalar4FermionsDecayer::_VMDmass,
     1.*MeV, -1, ZERO, -10000*MeV, 10000*MeV, false, false, true);

  static ParVector<PScalar4FermionsDecayer,Energy> interfaceVMDwidth
    ("VMDwidth",
     "The width to use for the particle in the VMD factor",
     &PScalar4FermionsDecayer::_VMDwidth,
     1.*MeV, -1, ZERO, -10000*MeV, 10000*MeV, false, false, true);

}

double PScalar4FermionsDecayer::me2(const int,
				    const Particle & inpart,
				    const ParticleVector & decay,
				    MEOption meopt) const {
  if(!ME())
    ME(new_ptr(GeneralDecayMatrixElement(PDT::Spin0,PDT::Spin1Half,PDT::Spin1Half,
					 PDT::Spin1Half,PDT::Spin1Half)));
  bool identical((_outgoing1[imode()]==_outgoing2[imode()]));
  if(meopt==Initialize) {
    ScalarWaveFunction::
      calculateWaveFunctions(_rho,const_ptr_cast<tPPtr>(&inpart),incoming);
  }
  if(meopt==Terminate) {
    // set up the spin information for the decay products
    ScalarWaveFunction::constructSpinInfo(const_ptr_cast<tPPtr>(&inpart),
					  incoming,true);
    // set up the spin information for the decay products
    for(unsigned int ix=0;ix<2;++ix) {
      SpinorBarWaveFunction::
	constructSpinInfo(_wavebar[ix],decay[2*ix  ],outgoing,true);
      SpinorWaveFunction::
	constructSpinInfo(_wave[ix]   ,decay[2*ix+1],outgoing,true);
    }
    return 0.;
  }
  // calculate the spinors
  for(unsigned int ix=0;ix<2;++ix) {
    SpinorBarWaveFunction::
      calculateWaveFunctions(_wavebar[ix],decay[2*ix  ],outgoing);
    SpinorWaveFunction::
      calculateWaveFunctions(_wave[ix]   ,decay[2*ix+1],outgoing);
  }
  // momenta of the outgoing photons
  Lorentz5Momentum momentum[4];
  momentum[0]=decay[0]->momentum()+decay[1]->momentum();momentum[0].rescaleMass();
  momentum[1]=decay[2]->momentum()+decay[3]->momentum();momentum[1].rescaleMass();
  if(identical) {
    momentum[2]=decay[2]->momentum()+decay[1]->momentum();momentum[2].rescaleMass();
    momentum[3]=decay[0]->momentum()+decay[3]->momentum();momentum[3].rescaleMass();
  }
  // compute the currents for the two leptonic decays
  LorentzPolarizationVectorE current[4][2][2];
  unsigned int it,ix,iy,iz;
  for(iz=0;iz<2;++iz) {
    it = iz==0 ? 1 : 0;
    for(ix=0;ix<2;++ix) {
      for(iy=0;iy<2;++iy) {
	current[iz][iy][ix] = _wave[iz][ix].vectorCurrent(_wavebar[iz][iy]);
	// the second two currents      
	if(identical)
	  current[iz+2][iy][ix] = _wave[it][ix].vectorCurrent(_wavebar[iz][iy]);
      }
    }
  }
  // invariants
  Energy2 m12(sqr(momentum[0].mass()));
  Energy2 m34(sqr(momentum[1].mass()));
  Energy2 m14(ZERO), m23(ZERO);
  complex<InvEnergy4> prop1(1./m12/m34),prop2(0./sqr(MeV2));
  Complex ii(0.,1.);
  if(identical) {
    m14=momentum[2].mass()*momentum[2].mass();
    m23=momentum[3].mass()*momentum[3].mass();
    prop2=1./m14/m23;
  }
  // the VMD factor if needed
  if(_includeVMD[imode()]>0) {
    Energy2 mrho2(_VMDmass[imode()]*_VMDmass[imode()]);
    Energy2 mwrho(_VMDmass[imode()]*_VMDwidth[imode()]);
    prop1 = prop1*(-mrho2+ii*mwrho)/(m12-mrho2+ii*mwrho)*
                  (-mrho2+ii*mwrho)/(m34-mrho2+ii*mwrho);
    if(identical) {
      prop2 = prop2*(-mrho2+ii*mwrho)/(m14-mrho2+ii*mwrho)*
	            (-mrho2+ii*mwrho)/(m23-mrho2+ii*mwrho);
    }
  }
  // prefactor
  Complex pre(_coupling[imode()]*4.*Constants::pi
	      *SM().alphaEM()*inpart.mass());
  Complex diag;
  // now compute the matrix element
  LorentzVector<complex<Energy3> > eps;
  vector<unsigned int> ispin(5,0);
  for(ispin[1]=0; ispin[1]<2;++ispin[1]) {
    for(ispin[2]=0;ispin[2]<2;++ispin[2]) {
      for(ispin[3]=0;ispin[3]<2;++ispin[3]) {
	for(ispin[4]=0;ispin[4]<2;++ispin[4]) {
	  // the first diagram
	  eps = epsilon(current[0][ispin[1]][ispin[2]],momentum[1],
			current[1][ispin[3]][ispin[4]]);
	  diag = Complex(prop1*(eps*momentum[0]));
	  // exchanged diagram if identical particles
	  //  (sign due normal ordering) 
	  if(identical) {
	    eps = epsilon(current[2][ispin[1]][ispin[4]],momentum[3],
			  current[3][ispin[3]][ispin[2]]);
	    diag-= Complex(prop2*(eps*momentum[2]));
	  }
	  (*ME())(ispin)=pre*diag;
	}
      }
    }
  }
  double me=ME()->contract(_rho).real();
  if(identical) me *= 0.25;
  return me;
}

// output the setup info for the particle database
void PScalar4FermionsDecayer::dataBaseOutput(ofstream & output,
					     bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    if(ix<_initsize) {
      output << "newdef " << name() << ":Incoming   " << ix << " " 
	     << _incoming[ix]   << "\n";
      output << "newdef " << name() << ":Outgoing1  " << ix << " " 
	     << _outgoing1[ix]  << "\n";
      output << "newdef " << name() << ":Outgoing2  " << ix << " " 
	     << _outgoing2[ix]  << "\n";
      output << "newdef " << name() << ":Coupling   " << ix << " " 
	     << _coupling[ix]*MeV   << "\n";
      output << "newdef " << name() << ":MaxWeight  " << ix << " " 
	     << _maxweight[ix]  << "\n";
      output << "newdef " << name() << ":IncludeVMD " << ix << " " 
	     << _includeVMD[ix] << "\n";
      output << "newdef " << name() << ":VMDID      " << ix << " " 
	     << _VMDid[ix]      << "\n";
      output << "newdef " << name() << ":VMDmass    " << ix << " " 
	     << _VMDmass[ix]/MeV    << "\n";
      output << "newdef " << name() << ":VMDwidth   " << ix << " " 
	     << _VMDwidth[ix]/MeV   << "\n";
    }
    else {
      output << "insert " << name() << ":Incoming   " << ix << " " 
	     << _incoming[ix]   << "\n";
      output << "insert " << name() << ":Outgoing1  " << ix << " " 
	     << _outgoing1[ix]  << "\n";
      output << "insert " << name() << ":Outgoing2  " << ix << " " 
	     << _outgoing2[ix]  << "\n";
      output << "insert " << name() << ":Coupling   " << ix << " " 
		 << _coupling[ix]*MeV   << "\n";
      output << "insert " << name() << ":MaxWeight  " << ix << " " 
	     << _maxweight[ix]  << "\n";
      output << "insert " << name() << ":IncludeVMD " << ix << " " 
	     << _includeVMD[ix] << "\n";
      output << "insert " << name() << ":VMDID      " << ix << " " 
	     << _VMDid[ix]      << "\n";
      output << "insert " << name() << ":VMDmass    " << ix << " " 
	     << _VMDmass[ix]/MeV    << "\n";
      output << "insert " << name() << ":VMDwidth   " << ix << " " 
	     << _VMDwidth[ix]/MeV   << "\n";
    }
  }
  if(header) output << "\n\" where BINARY ThePEGName=\"" << fullName() << "\";" << endl;
}
#line 1 "./PScalarLeptonNeutrinoDecayer.cc"
// -*- C++ -*-
//
// PScalarLeptonNeutrinoDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the PScalarLeptonNeutrinoDecayer class.
//

#include "PScalarLeptonNeutrinoDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/SpinorWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/SpinorBarWaveFunction.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/StandardModel/StandardModelBase.h"
#include "Herwig/Decay/TwoBodyDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void PScalarLeptonNeutrinoDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  unsigned int iz(0),ix,iy;
  if(initialize()) {
    for(ix=0;ix<_incoming.size();++ix) {
      for(iy=0;iy<_leptons[ix];++iy) {
	if(iy==0)      _maxweighte  [ix] = mode(iz)->maxWeight();
	else if(iy==1) _maxweightmu [ix] = mode(iz)->maxWeight();
	else if(iy==2) _maxweighttau[ix] = mode(iz)->maxWeight();
	++iz;
      }
    }
  }
}

PScalarLeptonNeutrinoDecayer::PScalarLeptonNeutrinoDecayer() 
  : _incoming(6), _decayconstant(6), _leptons(6), _maxweighte(6), 
    _maxweightmu(6), _maxweighttau(6) {
  // pion decay
  _incoming[0] = 211; _decayconstant[0] = 127.27*MeV; _leptons[0] = 2; 
  _maxweighte[0] = 0.00014; _maxweightmu[0] = 1.09; 
  _maxweighttau[0] = 0.0; 
  // kaon decay
  _incoming[1] = 321; _decayconstant[1] = 36.06*MeV; _leptons[1] = 2; 
  _maxweighte[1] = 1.87e-05; _maxweightmu[1] = 0.74; 
  _maxweighttau[1] = 0.0; 
  // D_s decay
  _incoming[2] = 431; _decayconstant[2] = 281.36*MeV; _leptons[2] = 3; 
  _maxweighte[2] = 1.83428e-07; _maxweightmu[2] = 0.00782778; 
  _maxweighttau[2] = 0.00782778; 
  // D decay
  _incoming[3] = 411; _decayconstant[3] = 51.20*MeV; _leptons[3] = 3; 
  _maxweighte[3] = 1.19915e-08; _maxweightmu[3] = 0.000511418; 
  _maxweighttau[3] = 0.00135052; 
  // B_c decay
  _incoming[4] = 541; _decayconstant[4] = 16.0*MeV; _leptons[4] = 3; 
  _maxweighte[4] = 1.7429e-09; _maxweightmu[4] = 7.47667e-05; 
  _maxweighttau[4] = 0.0179128; 
  // B_u decays
  _incoming[5] = 521; _decayconstant[5] = 0.759*MeV; _leptons[5] = 3; 
  _maxweighte[5] = 1.19578e-11; _maxweightmu[5] = 5.12844e-07; 
  _maxweighttau[5] = 0.000114138; 
  // initial size of the vectors
  _initsize=_incoming.size();
  // intermediates
  generateIntermediates(false);
}

void PScalarLeptonNeutrinoDecayer::doinit() {
  DecayIntegrator::doinit();
  // check the parameters are consistent
  unsigned int isize(_incoming.size());
  if(isize!=_decayconstant.size()||isize!=_leptons.size()||isize!=_maxweighte.size()||
     isize!=_maxweightmu.size()||isize!=_maxweighttau.size())
    throw InitException() << "Inconsistent parameters in PScalarLeptonNeutrinoDecayer"
			  << Exception::abortnow;
  // create the integration channels
  tPDVector extpart(3);  
  tPDPtr nu[3]={getParticleData(ParticleID::nu_e),
		getParticleData(ParticleID::nu_mu),
		getParticleData(ParticleID::nu_tau)};
  tPDPtr nubar[3]={getParticleData(ParticleID::nu_ebar),
		   getParticleData(ParticleID::nu_mubar),
		   getParticleData(ParticleID::nu_taubar)};
  tPDPtr lep[3]={getParticleData(ParticleID::eminus),
		 getParticleData(ParticleID::muminus),
		 getParticleData(ParticleID::tauminus)};
  tPDPtr lepbar[3]={getParticleData(ParticleID::eplus),
		    getParticleData(ParticleID::muplus),
		    getParticleData(ParticleID::tauplus)};
  int charge;
  vector<double> dummyweights;
  double wgt;
  DecayPhaseSpaceModePtr mode;
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    extpart[0]=getParticleData(_incoming[ix]);
    charge=extpart[0]->iCharge();
    for(unsigned int iy=0;iy<_leptons[ix];++iy) {
      if(charge>0) {
	extpart[1]=lepbar[iy];
	extpart[2]=nu[iy];
      }
      else {
	extpart[1]=lep[iy];
	extpart[2]=nubar[iy];
      }
      mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
      if(iy==0)      wgt = _maxweighte[ix];
      else if(iy==1) wgt = _maxweightmu[ix];
      else           wgt = _maxweighttau[ix];
      addMode(mode,wgt,dummyweights);
    }
  }
}

int PScalarLeptonNeutrinoDecayer::modeNumber(bool & cc,tcPDPtr parent,
					     const tPDVector & children) const {
  int imode(-1);
  if(children.size()!=2) return imode;
  // ids of the particles
  int id0(parent->id()),id0bar(id0);
  if(parent->CC()){id0bar=-id0;}
  tPDVector::const_iterator pit = children.begin();
  int id;
  unsigned ilep(4);
  for(;pit!=children.end();++pit) {
    id=abs((**pit).id());
    if(id>=11&&id<=16&&id%2==0) ilep=(id-10)/2;
  }
  // find the channel we need
  bool found(false);
  int ichan(-1);
  unsigned int ix(0);
  do {
    if(id0   ==_incoming[ix]||id0bar==_incoming[ix]) {
      found=true;ichan+=ilep;
      cc=id0bar==_incoming[ix];
    }
    else {
      ichan+=_leptons[ix];
    }
    ++ix;
  }
  while (!found&&ix<_incoming.size());
  if(found) imode=ichan;
  return imode;
}


void PScalarLeptonNeutrinoDecayer::persistentOutput(PersistentOStream & os) const {
  os << _incoming << ounit(_decayconstant,GeV)
     << _leptons << _maxweighte << _maxweightmu 
     << _maxweighttau;
}

void PScalarLeptonNeutrinoDecayer::persistentInput(PersistentIStream & is, int) {
  is >> _incoming >> iunit(_decayconstant,GeV) 
     >> _leptons >> _maxweighte >> _maxweightmu 
     >> _maxweighttau;
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<PScalarLeptonNeutrinoDecayer,DecayIntegrator>
describeHerwigPScalarLeptonNeutrinoDecayer("Herwig::PScalarLeptonNeutrinoDecayer", "HwSMDecay.so");

void PScalarLeptonNeutrinoDecayer::Init() {

  static ClassDocumentation<PScalarLeptonNeutrinoDecayer> documentation
    ("The PScalarLeptonNeutrinoDecayer class is the base class for"
     " the implementation of leptonic decay of a pseudoscalar meson to a lepton"
     "and a neutrino.");

  static ParVector<PScalarLeptonNeutrinoDecayer,int> interfaceIncoming
    ("Incoming",
     "The PDG code for the incoming particle",
     &PScalarLeptonNeutrinoDecayer::_incoming,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<PScalarLeptonNeutrinoDecayer,unsigned int> interfaceLepton
    ("Leptons",
     "The allowed outgoing leptons, 1 is only electrons, 2 is electrons and muons and "
     "3 is all lepton flavours",
     &PScalarLeptonNeutrinoDecayer::_leptons,
     0, 0, 0, 1, 3, false, false, true);

  static ParVector<PScalarLeptonNeutrinoDecayer,double> interfaceMaxWeighte
    ("MaxWeightElectron",
     "The maximum weight for the electron decay mode",
     &PScalarLeptonNeutrinoDecayer::_maxweighte,
     0, 0, 0, 0., 100., false, false, true);

  static ParVector<PScalarLeptonNeutrinoDecayer,double> interfaceMaxWeightmu
    ("MaxWeightMuon",
     "The maximum weight for the muon decay mode",
     &PScalarLeptonNeutrinoDecayer::_maxweightmu,
     0, 0, 0, 0., 100., false, false, true);

  static ParVector<PScalarLeptonNeutrinoDecayer,double> interfaceMaxWeighttau
    ("MaxWeightTau",
     "The maximum weight for the tau decay mode",
     &PScalarLeptonNeutrinoDecayer::_maxweighttau,
     0, 0, 0, 0., 100., false, false, true);

  static ParVector<PScalarLeptonNeutrinoDecayer,Energy> interfaceDecayConstant
    ("DecayConstant",
     "The decay constant for the incoming pseudoscaalr meson.",
     &PScalarLeptonNeutrinoDecayer::_decayconstant,
     MeV, 0, ZERO, ZERO, 1000.*MeV, false, false, true);
}

double PScalarLeptonNeutrinoDecayer::me2(const int,const Particle & inpart,
					 const ParticleVector & decay,
					 MEOption meopt) const {
  if(!ME())
    ME(new_ptr(TwoBodyDecayMatrixElement(PDT::Spin0,PDT::Spin1Half,PDT::Spin1Half)));
  // work out which decay constant to use
  int icoup(0),id(abs(inpart.id()));
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    if(id==abs(_incoming[ix])) icoup=ix;
  }
  // find the particles
  unsigned int iferm(0),ianti(0);
  for(unsigned ix=0;ix<decay.size();++ix) {
    id=decay[ix]->id();
    if(id<=-11&&id>=-16)    ianti=ix;
    else if(id>=11&&id<=16) iferm=ix;
  }
  int idferm = decay[iferm]->id();
  // initialization
  if(meopt==Initialize) {
    ScalarWaveFunction::
      calculateWaveFunctions(_rho,const_ptr_cast<tPPtr>(&inpart),incoming);
  }
  if(meopt==Terminate) {
    // set up the spin information for the decay products
    ScalarWaveFunction::constructSpinInfo(const_ptr_cast<tPPtr>(&inpart),
					  incoming,true);
    // set up the spin information for the decay products
    SpinorBarWaveFunction::
      constructSpinInfo(_wavebar,decay[iferm],outgoing,true);
    SpinorWaveFunction::
      constructSpinInfo(_wave   ,decay[ianti],outgoing,true);
  }
  // calculate the spinors
  SpinorBarWaveFunction::
    calculateWaveFunctions(_wavebar,decay[iferm],outgoing);
  SpinorWaveFunction::
    calculateWaveFunctions(_wave   ,decay[ianti],outgoing);
  // the prefactor
  Energy premass =  idferm%2==0 ? decay[ianti]->mass() : decay[iferm]->mass();
  InvEnergy pre = premass * 2.*_decayconstant[icoup]*SM().fermiConstant()/inpart.mass();
  // compute the matrix element
  vector<unsigned int> ispin(3,0);
  for(ispin[ianti+1]=0;ispin[ianti+1]<2;++ispin[ianti+1]) {
    for(ispin[iferm+1]=0;ispin[iferm+1]<2;++ispin[iferm+1]) {
      (*ME())(ispin)= idferm%2==0 ? 
	Complex(pre*_wave[ispin[ianti+1]].rightScalar(_wavebar[ispin[iferm+1]])) :
	Complex(pre*_wave[ispin[ianti+1]].leftScalar( _wavebar[ispin[iferm+1]])) ;
    }
  }
//   // test of the matrix element
//   double me=newME.contract(rhoin).real();
//   Energy mass = idferm%2==0 ? decay[ianti]->mass() : decay[iferm]->mass();
//   double test = sqr(_decayconstant[icoup]*SM().fermiConstant()*2.*mass/inpart.mass())*
//     (sqr(inpart.mass())-sqr(mass));
//   cout << "testing matrix element for " << inpart.PDGName() << " -> " 
//        << decay[0]->PDGName() << " " << decay[1]->PDGName() << " " 
//        << me << " " << (me-test)/(me+test) << endl;
  return 0.5*ME()->contract(_rho).real();
}


void PScalarLeptonNeutrinoDecayer::dataBaseOutput(ofstream & output,
						  bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    if(ix<_initsize) {
      output << "newdef " << name() << ":Incoming   " << ix << " "
	     << _incoming[ix]   << "\n";
      output << "newdef " << name() << ":Leptons    " << ix << " "
	     << _leptons[ix]   << "\n";
      output << "newdef " << name() << ":MaxWeightElectron " << ix << " "
	     << _maxweighte[ix]   << "\n";
      output << "newdef " << name() << ":MaxWeightMuon "     << ix << " "
	     << _maxweightmu[ix]   << "\n";
      output << "newdef " << name() << ":MaxWeightTau "      << ix << " "
	     << _maxweighttau[ix]   << "\n";
      output << "newdef " << name() << ":DecayConstant "     << ix << " "
	     << _decayconstant[ix]/MeV  << "\n";
    }
    else {
      output << "insert " << name() << ":Incoming   " << ix << " "
	     << _incoming[ix]   << "\n";
      output << "insert " << name() << ":Leptons    " << ix << " "
	     << _leptons[ix]   << "\n";
      output << "insert " << name() << ":MaxWeightElectron " << ix << " "
	     << _maxweighte[ix]   << "\n";
      output << "insert " << name() << ":MaxWeightMuon "     << ix << " "
	     << _maxweightmu[ix]   << "\n";
      output << "insert " << name() << ":MaxWeightTau "      << ix << " "
	     << _maxweighttau[ix]   << "\n";
      output << "insert " << name() << ":DecayConstant "     << ix << " "
	     << _decayconstant[ix]/MeV  << "\n";
    }
  }
  if(header) 
    output << "\n\" where BINARY ThePEGName=\"" << fullName() << "\";" << endl;
}
#line 1 "./PScalarPScalarVectorDecayer.cc"
// -*- C++ -*-
//
// PScalarPScalarVectorDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the PScalarPScalarVectorDecayer class.
//

#include "PScalarPScalarVectorDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"
#include "Herwig/Decay/TwoBodyDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void PScalarPScalarVectorDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()) {
    for(unsigned int ix=0;ix<_incoming.size();++ix)
      if(mode(ix)) _maxweight[ix] = mode(ix)->maxWeight();
  }
}

PScalarPScalarVectorDecayer::PScalarPScalarVectorDecayer() 
  : _incoming(19), _outgoingP(19), _outgoingV(19), 
    _coupling(19), _maxweight(19) {
  // decay pi' to rho pi
  _incoming[0] =  100111; _outgoingV[0] =  213; _outgoingP[0] = -211; 
  _coupling[0] = 3.57; _maxweight[0] = 4.5; 
  _incoming[1] =  100211; _outgoingV[1] =  213; _outgoingP[1] =  111; 
  _coupling[1] = 3.57; _maxweight[1] = 4.5; 
  _incoming[2] =  100211; _outgoingV[2] =  113; _outgoingP[2] =  211; 
  _coupling[2] = 3.57; _maxweight[2] = 4.5; 
  // K' to K rho
  _incoming[3] =  100311; _outgoingP[3] =  311; _outgoingV[3] =  113; 
  _coupling[3] = 1.; _maxweight[3] = 4.; 
  _incoming[4] =  100321; _outgoingP[4] =  321; _outgoingV[4] =  113; 
  _coupling[4] = 1.; _maxweight[4] = 4.; 
  _incoming[5] =  100311; _outgoingP[5] =  321; _outgoingV[5] = -213; 
  _coupling[5] = 1.41; _maxweight[5] = 4.; 
  _incoming[6] =  100321; _outgoingP[6] =  311; _outgoingV[6] =  213; 
  _coupling[6] = 1.41; _maxweight[6] = 4.; 
  // K' to K* pi
  _incoming[7] =  100311; _outgoingV[7] =  313; _outgoingP[7] =  111; 
  _coupling[7] = 1.55; _maxweight[7] = 2.; 
  _incoming[8] =  100321; _outgoingV[8] =  323; _outgoingP[8] =  111; 
  _coupling[8] = 1.55; _maxweight[8] = 2.; 
  _incoming[9] =  100311; _outgoingV[9] =  323; _outgoingP[9] = -211; 
  _coupling[9] = 2.19; _maxweight[9] = 2.; 
  _incoming[10] =  100321; _outgoingV[10] =  313; _outgoingP[10] =  211; 
  _coupling[10] = 2.19; _maxweight[10] = 2.; 
  // eta (1475) to K* K
  _incoming[11] =  100331; _outgoingV[11] =  323; _outgoingP[11] = -321; 
  _coupling[11] = 2.92; _maxweight[11] = 3.5; 
  _incoming[12] =  100331; _outgoingV[12] =  313; _outgoingP[12] = -311; 
  _coupling[12] = 2.92; _maxweight[12] = 3.5; 
  // eta (1475) to K* K
  _incoming[13] =  9020221; _outgoingV[13] =  323; _outgoingP[13] = -321; 
  _coupling[13] = 0.956; _maxweight[13] = 4.; 
  _incoming[14] =  9020221; _outgoingV[14] =  313; _outgoingP[14] = -311; 
  _coupling[14] = 0.956; _maxweight[14] = 4.; 
  // decay f_0(1370) to a_1 pi
  _incoming[15] =  10221; _outgoingV[15] = 20213; _outgoingP[15] = -211; 
  _coupling[15] = 2.68; _maxweight[15] = 4.5; 
  _incoming[16] =  10221; _outgoingV[16] = 20113; _outgoingP[16] =  111; 
  _coupling[16] = 2.68; _maxweight[16] = 4.5; 
  // decay f_0(1500) to a_1 pi
  _incoming[17] =  9030221; _outgoingV[17] = 20213; _outgoingP[17] = -211; 
  _coupling[17] = 1.147; _maxweight[17] = 3.2; 
  _incoming[18] =  9030221; _outgoingV[18] = 20113; _outgoingP[18] =  111; 
  _coupling[18] = 1.147; _maxweight[18] = 3.2; 
  // initial size of the arrays
  _initsize=_incoming.size();
  // intermediates
  generateIntermediates(false);
}

void PScalarPScalarVectorDecayer::doinit() {
  DecayIntegrator::doinit();
  // check the parameters arew consistent
  unsigned int isize=_coupling.size();
  if(isize!=_incoming.size()  || isize!=_outgoingP.size()||
     isize!=_outgoingV.size() || isize!=_maxweight.size())
    throw InitException() << "Inconsistent parameters in PScalarPScalarVectorDecayer" 
			  << Exception::abortnow;
  // set up the integration channels
  vector<double> wgt;
  DecayPhaseSpaceModePtr mode;
  tPDVector extpart(3);
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    extpart[0] = getParticleData(_incoming[ix]);
    extpart[1] = getParticleData(_outgoingP[ix]);
    extpart[2] = getParticleData(_outgoingV[ix]);
    if(extpart[0]&&extpart[1]&&extpart[2]) 
      mode=new_ptr(DecayPhaseSpaceMode(extpart,this));
    else
      mode=DecayPhaseSpaceModePtr();
    addMode(mode,_maxweight[ix],wgt);
  }
}

int PScalarPScalarVectorDecayer::modeNumber(bool & cc,tcPDPtr parent,
					   const tPDVector & children) const {
  if(children.size()!=2) return -1;
  int id(parent->id());
  int idbar = parent->CC() ? parent->CC()->id() : id;
  int id1(children[0]->id());
  int id1bar = children[0]->CC() ? children[0]->CC()->id() : id1;
  int id2(children[1]->id());
  int id2bar = children[1]->CC() ? children[1]->CC()->id() : id2;
  int imode(-1);
  unsigned int ix(0);
  cc=false;
  do {
    if(id   ==_incoming[ix]) {
      if((id1   ==_outgoingP[ix]&&id2   ==_outgoingV[ix])||
	 (id2   ==_outgoingP[ix]&&id1   ==_outgoingV[ix])) imode=ix;
    }
    if(idbar==_incoming[ix]) {
      if((id1bar==_outgoingP[ix]&&id2bar==_outgoingV[ix])||
	 (id2bar==_outgoingP[ix]&&id1bar==_outgoingV[ix])) {
	imode=ix;
	cc=true;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  return imode;
}


void PScalarPScalarVectorDecayer::persistentOutput(PersistentOStream & os) const {
  os << _coupling << _incoming << _outgoingP << _outgoingV << _maxweight;
}

void PScalarPScalarVectorDecayer::persistentInput(PersistentIStream & is, int) {
  is >> _coupling >> _incoming >> _outgoingP >> _outgoingV >> _maxweight;
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<PScalarPScalarVectorDecayer,DecayIntegrator>
describeHerwigPScalarPScalarVectorDecayer("Herwig::PScalarPScalarVectorDecayer", "HwSMDecay.so");

void PScalarPScalarVectorDecayer::Init() {

  static ClassDocumentation<PScalarPScalarVectorDecayer> documentation
    ("The PScalarPScalarVectorDecayer class is designed for"
     " the decay of a pseduoscalar meson to two spin-1 particles.");

  static ParVector<PScalarPScalarVectorDecayer,int> interfaceIncoming
    ("Incoming",
     "The PDG code for the incoming particle",
     &PScalarPScalarVectorDecayer::_incoming,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<PScalarPScalarVectorDecayer,int> interfaceOutgoingScalar
    ("OutgoingPScalar",
     "The PDG code for the outgoing pseudoscalar meson",
     &PScalarPScalarVectorDecayer::_outgoingP,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<PScalarPScalarVectorDecayer,int> interfaceOutgoingVector
    ("OutgoingVector",
     "The PDG code for the outgoing vector meson",
     &PScalarPScalarVectorDecayer::_outgoingV,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<PScalarPScalarVectorDecayer,double> interfaceCoupling
    ("Coupling",
     "The coupling for the decay mode",
     &PScalarPScalarVectorDecayer::_coupling,
     0, 0, 0, 0., 100., false, false, true);

  static ParVector<PScalarPScalarVectorDecayer,double> interfaceMaxWeight
    ("MaxWeight",
     "The maximum weight for the decay mode",
     &PScalarPScalarVectorDecayer::_maxweight,
     0, 0, 0, 0., 100., false, false, true);

}

double PScalarPScalarVectorDecayer::me2( const int,
					 const Particle & inpart,
					 const ParticleVector & decay,
					 MEOption meopt) const {
  if(!ME())
    ME(new_ptr(TwoBodyDecayMatrixElement(PDT::Spin0,PDT::Spin0,PDT::Spin1)));
  if(meopt==Initialize) {
    ScalarWaveFunction::
      calculateWaveFunctions(_rho,const_ptr_cast<tPPtr>(&inpart),incoming);
  }
  if(meopt==Terminate) {
    // set up the spin information for the decay products
    ScalarWaveFunction::constructSpinInfo(const_ptr_cast<tPPtr>(&inpart),
					  incoming,true);
    ScalarWaveFunction::constructSpinInfo(decay[0],outgoing,true);
    VectorWaveFunction::constructSpinInfo(_vectors,decay[1],
					  outgoing,true,false);
    return 0.;
  }
  VectorWaveFunction::calculateWaveFunctions(_vectors,decay[1],
					     outgoing,false);
  // calculate the matrix element
  Lorentz5Momentum psum(inpart.momentum()+decay[0]->momentum());
  for(unsigned int ix=0;ix<3;++ix) {
    (*ME())(0,0,ix)=Complex(_coupling[imode()]/inpart.mass()*(_vectors[ix]*psum));
  }
  // test of the matrix element
//   double me=newME.contract(rhoin).real();
//   Energy pcm = Kinematics::pstarTwoBodyDecay(inpart.mass(),decay[0]->mass(),
// 					     decay[1]->mass());
//   double test = 4.*sqr(_coupling[imode()]*pcm/decay[1]->mass());
//   cerr << "testing matrix element for " << inpart.PDGName() << " -> " 
//        << decay[0]->PDGName() << " " << decay[1]->PDGName() << " "
//        << me << " " << (me-test)/(me+test) << "\n";
  // output the answer
  return ME()->contract(_rho).real();
}

// specify the 1-2 matrix element to be used in the running width calculation
bool PScalarPScalarVectorDecayer::twoBodyMEcode(const DecayMode & dm, int & mecode,
					       double & coupling) const {
  int imode(-1);
  int id(dm.parent()->id());
  int idbar = dm.parent()->CC() ? dm.parent()->CC()->id() : id;
  ParticleMSet::const_iterator pit(dm.products().begin());
  int id1((**pit).id());
  int id1bar = (**pit).CC() ? (**pit).CC()->id() : id1;
  ++pit;
  int id2((**pit).id());
  int id2bar = (**pit).CC() ? (**pit).CC()->id() : id2;
  unsigned int ix(0); bool order(true);
  do {
    if(id   ==_incoming[ix]) {
      if(id1==_outgoingP[ix]&&id2==_outgoingV[ix]) {
	imode=ix;
	order=true;
      }
      if(id2==_outgoingP[ix]&&id1==_outgoingV[ix]) {
	imode=ix;
	order=false;
      }
    }
    if(idbar==_incoming[ix]&&imode<0) {
      if(id1bar==_outgoingP[ix]&&id2bar==_outgoingV[ix]) {
	imode=ix;
	order=true;
      }
      if(id2bar==_outgoingP[ix]&&id1bar==_outgoingV[ix]) {
	imode=ix;
	order=false;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  coupling=_coupling[imode];
  mecode=10;
  return order;
}

// output the setup information for the particle database
void PScalarPScalarVectorDecayer::dataBaseOutput(ofstream & output,
						 bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  // the rest of the parameters
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    if(ix<_initsize) {
      output << "newdef " << name() << ":Incoming " << ix << " " 
	     << _incoming[ix] << "\n";
      output << "newdef " << name() << ":OutgoingPScalar " << ix << " " 
	     << _outgoingP[ix] << "\n";
      output << "newdef " << name() << ":OutgoingVector " << ix << " " 
	     << _outgoingV[ix] << "\n";
      output << "newdef " << name() << ":Coupling " << ix << " " 
	     << _coupling[ix] << "\n";
      output << "newdef " << name() << ":MaxWeight " << ix << " " 
	     << _maxweight[ix] << "\n";
    }
    else {
      output << "insert " << name() << ":Incoming " << ix << " " 
	     << _incoming[ix] << "\n";
      output << "insert " << name() << ":OutgoingPScalar " << ix << " " 
	     << _outgoingP[ix] << "\n";
      output << "insert " << name() << ":OutgoingVector " << ix << " " 
	     << _outgoingV[ix] << "\n";
      output << "insert " << name() << ":Coupling " << ix << " " 
	     << _coupling[ix] << "\n";
      output << "insert " << name() << ":MaxWeight " << ix << " " 
	     << _maxweight[ix] << "\n";
    }
  }
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}
#line 1 "./PScalarVectorFermionsDecayer.cc"
// -*- C++ -*-
//
// PScalarVectorFermionsDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the PScalarVectorFermionsDecayer class.
//

#include "PScalarVectorFermionsDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/SpinorWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/SpinorBarWaveFunction.h"
#include "ThePEG/Helicity/epsilon.h"
#include "Herwig/PDT/ThreeBodyAllOn1IntegralCalculator.h"
#include "Herwig/Decay/GeneralDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void PScalarVectorFermionsDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()) {
    for(unsigned int ix=0;ix<_incoming.size();++ix)
      _maxweight[ix]=mode(ix)->maxWeight();
  }
}

PScalarVectorFermionsDecayer::PScalarVectorFermionsDecayer() 
  : _coupling(5), _incoming(5), _outgoingV(5), _outgoingf(5), 
    _outgoinga(5),_maxweight(5), _includeVMD(5), _VMDid(5),
    _VMDmass(5), _VMDwidth(5) {
  // pi0 -> gamma e+e-
  _incoming[0] = 111;_outgoingV[0] =  22;
  _outgoingf[0] = 11;_outgoinga[0] = -11;
  _coupling[0] = 0.00761872/GeV;_maxweight[0] = 0.027;
  _includeVMD[0] = 2;_VMDid[0] = 113;
  _VMDmass[0] = 0.7758*GeV;_VMDwidth[0] = 0.1503*GeV;
  // eta -> gamma e+e-/mu+/mu-
  _incoming[1] = 221;_outgoingV[1] =  22;
  _outgoingf[1] = 11;_outgoinga[1] = -11;
  _coupling[1] = 0.007554164/GeV;_maxweight[1] = 2.8;
  _includeVMD[1] = 2;_VMDid[1] = 113;
  _VMDmass[1] = 0.7758*GeV;_VMDwidth[1] = 0.1503*GeV;
  _incoming[2] = 221;_outgoingV[2] =  22;
  _outgoingf[2] = 13;_outgoinga[2] = -13;
  _coupling[2] = 0.007554164/GeV;_maxweight[2] = 2.1;
  _includeVMD[2] = 2;_VMDid[2] = 113;
  _VMDmass[2] = 0.7758*GeV;_VMDwidth[2] = 0.1503*GeV;
  // eta' -> gamma e+e-/mu+mu-
  _incoming[3] = 331;_outgoingV[3] =  22;
  _outgoingf[3] = 11;_outgoinga[3] = -11;
  _coupling[3] = 0.0104/GeV;_maxweight[3] = 5.2;
  _includeVMD[3] = 2;_VMDid[3] = 113;
  _VMDmass[3] = 0.7758*GeV;_VMDwidth[3] = 0.1503*GeV;
  _incoming[4] = 331;_outgoingV[4] =  22;
  _outgoingf[4] = 13;_outgoinga[4] = -13;
  _coupling[4] = 0.0104/GeV;_maxweight[4] = 3.0;
  _includeVMD[4] = 2;_VMDid[4] = 113;
  _VMDmass[4] = 0.7758*GeV;_VMDwidth[4] = 0.1503*GeV;
  // initial size of the arrays
  _initsize = _incoming.size();
  // intermediates
  generateIntermediates(false);
}

void PScalarVectorFermionsDecayer::doinit() {
  DecayIntegrator::doinit();
  // check the parameters are consistent
  unsigned int isize=_coupling.size();
  if(isize!=_incoming.size()  || isize!=_outgoingV.size()|| isize!=_outgoingf.size()||
     isize!=_outgoinga.size() || isize!=_maxweight.size()|| isize!=_includeVMD.size()||
     isize!=_VMDid.size()     || isize!=_VMDmass.size()  || isize!=_VMDwidth.size())
    throw InitException() << "Inconsistent parameters in PScalarVectorFermionsDecayer"
			  << Exception::abortnow;
  // create the integration channel for each mode 
  tPDVector extpart(4);
  tPDPtr gamma(getParticleData(ParticleID::gamma));
  DecayPhaseSpaceChannelPtr newchannel;
  DecayPhaseSpaceModePtr mode;
  vector<double> wgt(1,1.);
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    extpart[0] = getParticleData(_incoming[ix]);
    extpart[1] = getParticleData(_outgoingV[ix]);
    extpart[2] = getParticleData(_outgoingf[ix]);
    extpart[3] = getParticleData(_outgoinga[ix]);
    mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,1);
    newchannel->addIntermediate(gamma     ,1,-1.1, 2,3);
    mode->addChannel(newchannel);
    addMode(mode,_maxweight[ix],wgt);
  }
  // set up the values for the VMD factor if needed (copy the default mass and width 
  //                                                 into the array)
  for(unsigned ix=0;ix<isize;++ix) {
    if(_includeVMD[ix]==1) {
      _VMDmass[ix]=getParticleData(_VMDid[ix])->mass();
      _VMDwidth[ix]=getParticleData(_VMDid[ix])->width();
    }
  }
}

int PScalarVectorFermionsDecayer::modeNumber(bool & cc,tcPDPtr parent,
					   const tPDVector & children) const {
  int imode(-1);
  // must be three outgoing particles
  if(children.size()!=3) return imode;
  // ids of the particles
  int id0(parent->id()),idf[2]={0,0},idv(0);
  unsigned int nf(0);
  tPDVector::const_iterator pit = children.begin();
  for( ;pit!=children.end();++pit) {
    if((**pit).iSpin()==PDT::Spin1) {
      idv=(**pit).id();
    }
    else {
      idf[nf]=(**pit).id();
      ++nf;
    }
  }
  // loop over the modes and see if this is one of them
  unsigned int ix=0;
  do {
    if(_incoming[ix]==id0&&_outgoingV[ix]==idv)
      {if((idf[0]==_outgoingf[ix]&&idf[1]==_outgoinga[ix])||
	  (idf[1]==_outgoingf[ix]&&idf[0]==_outgoinga[ix])){imode=ix;}}
    ++ix;
  }
  while(imode<0&&ix<_incoming.size());
  cc=false;
  return imode;
}

void PScalarVectorFermionsDecayer::persistentOutput(PersistentOStream & os) const {
  os << ounit(_coupling,1/MeV) 
     << _incoming << _outgoingV << _outgoingf << _outgoinga << _maxweight
     << _includeVMD << _VMDid 
     << ounit(_VMDmass,MeV) << ounit(_VMDwidth,MeV);
}

void PScalarVectorFermionsDecayer::persistentInput(PersistentIStream & is, int) {
  is >> iunit(_coupling,1/MeV) 
     >> _incoming >> _outgoingV >> _outgoingf >> _outgoinga >> _maxweight
     >> _includeVMD >> _VMDid 
     >> iunit(_VMDmass,MeV) >> iunit(_VMDwidth,MeV);
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<PScalarVectorFermionsDecayer,DecayIntegrator>
describeHerwigPScalarVectorFermionsDecayer("Herwig::PScalarVectorFermionsDecayer", "HwSMDecay.so");

void PScalarVectorFermionsDecayer::Init() {

  static ClassDocumentation<PScalarVectorFermionsDecayer> documentation
    ("The PScalarVectorFermionsDecayer class is designed"
     " for the decay of a pseudoscalar meson to a photon and a"
     "fermion-antifermion pair");

  static ParVector<PScalarVectorFermionsDecayer,int> interfaceIncoming
    ("Incoming",
     "The PDG code for the incoming particle",
     &PScalarVectorFermionsDecayer::_incoming,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<PScalarVectorFermionsDecayer,int> interfaceOutcomingV
    ("OutgoingVector",
     "The PDG code for the outgoing pseudoscalar",
     &PScalarVectorFermionsDecayer::_outgoingV,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<PScalarVectorFermionsDecayer,int> interfaceOutcomingF
    ("OutgoingFermion",
     "The PDG code for the outgoing fermion",
     &PScalarVectorFermionsDecayer::_outgoingf,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<PScalarVectorFermionsDecayer,int> interfaceOutcomingA
    ("OutgoingAntiFermion",
     "The PDG code for the outgoing antifermion",
     &PScalarVectorFermionsDecayer::_outgoinga,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<PScalarVectorFermionsDecayer,InvEnergy> interfaceCoupling
    ("Coupling",
     "The coupling for the decay mode",
     &PScalarVectorFermionsDecayer::_coupling,
     1/MeV, 0, ZERO, -10000000/MeV, 10000000/MeV, false, false, true);

  static ParVector<PScalarVectorFermionsDecayer,double> interfaceMaxWeight
    ("MaxWeight",
     "The maximum weight for the decay mode",
     &PScalarVectorFermionsDecayer::_maxweight,
     0, 0, 0, 0.0, 100., false, false, true);

  static ParVector<PScalarVectorFermionsDecayer,int> interfaceIncludeVMD
    ("IncludeVMD",
     "There are three options for 0 the VMD factor is not included, for 1 the factor "
     "is included using the default mass and width of the particle specified by"
     " VMDID, and for 2 the factor is included using the mass and width specified"
     " by VMDwidth and VMDmass.",
     &PScalarVectorFermionsDecayer::_includeVMD,
     0, 0, 0, -10000, 10000, false, false, true);

  static ParVector<PScalarVectorFermionsDecayer,int> interfaceVMDID
    ("VMDID",
     "The PDG code for the particle to be used for the VMD factor.",
     &PScalarVectorFermionsDecayer::_VMDid,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<PScalarVectorFermionsDecayer,Energy> interfaceVMDmass
    ("VMDmass",
     "The mass to use for the particle in the VMD factor",
     &PScalarVectorFermionsDecayer::_VMDmass,
     MeV, 0, ZERO, ZERO, 10000.*MeV, false, false, true);

  static ParVector<PScalarVectorFermionsDecayer,Energy> interfaceVMDwidth
    ("VMDwidth",
     "The width to use for the particle in the VMD factor",
     &PScalarVectorFermionsDecayer::_VMDwidth,
     MeV, 0, ZERO, ZERO, 10000.*MeV, false, false, true);

}

double PScalarVectorFermionsDecayer::me2(const int,
					 const Particle & inpart,
					 const ParticleVector & decay,
					 MEOption meopt) const {
  if(!ME())
    ME(new_ptr(GeneralDecayMatrixElement(PDT::Spin0,PDT::Spin1,PDT::Spin1Half,
					 PDT::Spin1Half)));
  // initialization
  if(meopt==Initialize) {
    ScalarWaveFunction::
      calculateWaveFunctions(_rho,const_ptr_cast<tPPtr>(&inpart),incoming);
  }
  if(meopt==Terminate) {
    // set up the spin information for the decay products
    ScalarWaveFunction::constructSpinInfo(const_ptr_cast<tPPtr>(&inpart),
					  incoming,true);
    // set up the spin information for the decay products
    VectorWaveFunction::
      constructSpinInfo(_vectors,decay[0],outgoing,true,true);
    SpinorBarWaveFunction::
      constructSpinInfo(_wavebar,decay[1],outgoing,true);
    SpinorWaveFunction::
      constructSpinInfo(_wave   ,decay[2],outgoing,true);
    return 0.;
  }
  // calculate the spinors and polarization vectors
  VectorWaveFunction::
    calculateWaveFunctions(_vectors,decay[0],outgoing,true);
  SpinorBarWaveFunction::
    calculateWaveFunctions(_wavebar,decay[1],outgoing);
  SpinorWaveFunction::
    calculateWaveFunctions(_wave   ,decay[2],outgoing);
  // now compute the matrix element
  Complex ii(0.,1.);
  Lorentz5Momentum pff(decay[1]->momentum()+decay[2]->momentum());
  pff.rescaleMass();
  Energy2 mff2(pff.mass()*pff.mass());
  // compute the prefactor
  complex<InvEnergy3> pre(_coupling[imode()]/mff2);
  // the VMD factor
  if(_includeVMD[imode()]>0) {
    Energy2 mrho2=_VMDmass[imode()]*_VMDmass[imode()];
    Energy2 mwrho=_VMDmass[imode()]*_VMDwidth[imode()];
    pre = pre*(-mrho2+ii*mwrho)/(mff2-mrho2+ii*mwrho);
  }
  LorentzVector<complex<Energy3> > eps;
  LorentzVector<complex<Energy> > fcurrent;
  // compute the matrix element
  vector<unsigned int> ispin(4);ispin[0]=0;
  for(ispin[3]=0;ispin[3]<2;++ispin[3]) {
    for(ispin[2]=0;ispin[2]<2;++ispin[2]) {
      fcurrent = _wave[ispin[3]].vectorCurrent(_wavebar[ispin[2]]);
      // compute the current for this part
      eps = epsilon(decay[0]->momentum(),pff,fcurrent);
      for(ispin[1]=0;ispin[1]<3;++ispin[1]) {
	(*ME())(ispin) = Complex(pre *_vectors[ispin[1]].dot(eps));
      }
    }	  
  }
  double me = ME()->contract(_rho).real();
//   //code to test the matrix element against the analytic result
//   Energy   m[4]={inpart.mass(),decay[0]->mass(),decay[1]->mass(),decay[2]->mass()};
//   Energy2 m2[4]={m[0]*m[0],m[1]*m[1],m[2]*m[2],m[3]*m[3]};
//   Lorentz5Momentum p12=decay[0]->momentum()+decay[1]->momentum();p12.rescaleMass();
//   Energy2 m122(p12.mass2());
//   Complex output( ((pre*conj(pre)).real()*(
// 				 -2*m122*m122*mff2 - mff2*mff2*mff2 + 
// 				 m2[1]*(2*m2[2]*m2[3] - 2*m2[3]*m2[3] + 
// 					m2[1]*(m2[2] - 2*m[2]*m[3] - m2[3])) - 
// 				 2*m[2]*(m[2]*m2[2] - 2*m2[1]*m[3] - m[2]*m2[3])*
// 				 m2[0] - (m2[2] + 2*m[2]*m[3] - m2[3])*
// 				 m2[0]*m2[0] +mff2*mff2*
// 				 (2*m2[1] + (m[2] - m[3])*(m[2] - m[3]) + 2*m2[0]) - 
// 				 mff2*(m2[1]*m2[1] + 2*m2[1]*m[2]*(m[2] - 2*m[3]) + 
// 				       2*m2[2]*m2[3] - 2*(2*m[2] - m[3])*m[3]*m2[0] + 
// 				       m2[0]*m2[0]) + 2*m122*
// 				 (-mff2*mff2 - (m2[2] - m2[3])*
// 				  (m2[1] - m2[0]) + 
// 				  mff2*(m2[1] + m2[2] + m2[3] + 
// 					m2[0])))));
//   cout << "testing the matrix element " 
//        << real(output) << " " << me << " " << test2 << endl;
  return me;
}

// method to return an object to calculate the 3 or higher body partial width
WidthCalculatorBasePtr 
PScalarVectorFermionsDecayer::threeBodyMEIntegrator(const DecayMode & dm) const {
  // workout which mode we are doing
  int imode(-1);
  // ids of the particles
  int id0(dm.parent()->id()),idf[2]={0,0},idv(0);
  unsigned int nf(0);
  ParticleMSet::const_iterator pit = dm.products().begin();
  for( ;pit!=dm.products().end();++pit) {
    if((**pit).iSpin()==PDT::Spin1){idv=(**pit).id();}
    else{idf[nf]=(**pit).id();++nf;}
  }
  // loop over the modes and see if this is one of them
  unsigned int ix=0;
  do {
    if(_incoming[ix]==id0&&_outgoingV[ix]==idv) {
      if((idf[0]==_outgoingf[ix]&&idf[1]==_outgoinga[ix])||
	 (idf[1]==_outgoingf[ix]&&idf[0]==_outgoinga[ix])) imode=ix;
    }
    ++ix;
  }
  while(imode<0&&ix<_incoming.size());
  // get the masses we need
  Energy m[3]={getParticleData(_outgoingV[imode])->mass(),
	       getParticleData(_outgoingf[imode])->mass(),
	       getParticleData(_outgoinga[imode])->mass()};
  return 
    new_ptr(ThreeBodyAllOn1IntegralCalculator<PScalarVectorFermionsDecayer>
	    (3,-1000.*MeV,-0.9*MeV,-0.9,*this,imode,m[0],m[1],m[2]));
}

InvEnergy PScalarVectorFermionsDecayer::threeBodydGammads(const int imodeb,
							  const Energy2 q2, 
							  const Energy2 mff2, 
							  const Energy m1,
							  const Energy m2,
							  const Energy m3) const {
  // the masses of the external particles
  Energy q=sqrt(q2);
  Energy2 m12=m1*m1;
  Energy2 m22=m2*m2;
  Energy2 m32=m3*m3;
  // calculate the prefactor
  Complex ii(0.,1.);
  complex<InvEnergy3> pre = _coupling[imodeb] / mff2;
  // the VMD factor
  if(_includeVMD[imodeb]>0) {
    Energy2 mrho2=_VMDmass[imodeb]*_VMDmass[imodeb];
    Energy2 mwrho=_VMDmass[imodeb]*_VMDwidth[imodeb];
    pre = pre*(-mrho2+ii*mwrho)/(mff2-mrho2+ii*mwrho);
  }
  InvEnergy6 factor=real(pre*conj(pre));
  // compute the pieces from the integration limits
  Energy mff=sqrt(mff2);
  Energy e2star = 0.5*(mff2-m32+m22)/mff;
  Energy e1star = 0.5*(q2-mff2-m12)/mff;
  Energy e1sm = sqrt(e1star*e1star-m12);
  Energy e2sm = sqrt(e2star*e2star-m22);
  Energy2 a = 2*e1star*e2star+m12+m22;
  Energy2 b = 2*e1sm*e2sm;
  // term independent of s3
  Energy8 me = 2*b*(2*(m12*(mff2*mff2 + 4*mff2*m2*m3 -(m22 - m32)*(m22 - m32)) + 
		       2*m2*(m12 +m22)*m3*(-mff2 +m22 + q2))
		    +(m12 +m22)*(m12 +m22)*(-mff2 +m22 - 2*m2*m3 - m32)
		    -(mff2 +m22 + 2*m2*m3 - m32)*(-mff2 +m22 + q2)*(-mff2 +m22 + q2));
  // linear term
  me+= 2.*a*b*(2*(-mff2*mff2 - (m22 - m32)*(m12 - q2) + 
		  mff2*(m12 + m22 + m32 + q2)));
  // quadratic term
  me+=-4.*mff2*b*(3.*a*a+b*b)/3.;

  // phase space factors
  using Constants::pi;
  return -factor * me/256./pi/pi/pi/q2/q;
}

// output the setup information for the particle database
void PScalarVectorFermionsDecayer::dataBaseOutput(ofstream & output,
						  bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    if(ix<_initsize) {
      output << "newdef " << name() << ":Incoming   " << ix << "  " 
	     << _incoming[ix]   << "\n";
      output << "newdef " << name() << ":OutgoingVector  " 
	     << ix << "  " << _outgoingV[ix]  << "\n";
      output << "newdef " << name() << ":OutgoingFermion  " 
	     << ix << "  " << _outgoingf[ix]  << "\n";
      output << "newdef " << name() << ":OutgoingAntiFermion " 
	     << ix << "  " << _outgoinga[ix]  << "\n";
      output << "newdef " << name() << ":Coupling   " << ix << "  " 
	     << _coupling[ix]*MeV   << "\n";
      output << "newdef " << name() << ":MaxWeight  " << ix << "  " 
	     << _maxweight[ix]  << "\n";
      output << "newdef " << name() << ":IncludeVMD " << ix << "  " 
	     << _includeVMD[ix] << "\n";
      output << "newdef " << name() << ":VMDID      " << ix << "  " 
	     << _VMDid[ix]      << "\n";
      output << "newdef " << name() << ":VMDmass    " << ix << "  " 
	     << _VMDmass[ix]/MeV    << "\n";
      output << "newdef " << name() << ":VMDwidth   " << ix << "  " 
	     << _VMDwidth[ix]/MeV   << "\n";
    }
    else {
      output << "insert " << name() << ":Incoming   " << ix << "  " 
	     << _incoming[ix]   << "\n";
      output << "insert " << name() << ":OutgoingVector  " 
	     << ix << "  " << _outgoingV[ix]  << "\n";
      output << "insert " << name() << ":OutgoingFermion  " 
	     << ix << "  " << _outgoingf[ix]  << "\n";
      output << "insert " << name() << ":OutgoingAntiFermion " 
	     << ix << "  " << _outgoinga[ix]  << "\n";
      output << "insert " << name() << ":Coupling   " << ix << "  " 
	     << _coupling[ix]*MeV   << "\n";
      output << "insert " << name() << ":IncludeVMD " << ix << "  " 
	     << _includeVMD[ix] << "\n";
      output << "insert " << name() << ":VMDID      " << ix << "  " 
	     << _VMDid[ix]      << "\n";
      output << "insert " << name() << ":VMDmass    " << ix << "  " 
	     << _VMDmass[ix]/MeV    << "\n";
      output << "insert " << name() << ":VMDwidth   " << ix << "  " 
	     << _VMDwidth[ix]/MeV   << "\n";
    }
  }
  if(header) output << "\n\" where BINARY ThePEGName=\"" << fullName() << "\";" << endl;
}
#line 1 "./PScalarVectorVectorDecayer.cc"
// -*- C++ -*-
//
// PScalarVectorVectorDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the PScalarVectorVectorDecayer class.
//
#include "PScalarVectorVectorDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "Herwig/Utilities/Kinematics.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"
#include "ThePEG/Helicity/epsilon.h"
#include "Herwig/Decay/TwoBodyDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void PScalarVectorVectorDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()) {
    for(unsigned int ix=0;ix<_incoming.size();++ix)
      _maxweight[ix] = mode(ix)->maxWeight();
  }
}

PScalarVectorVectorDecayer::PScalarVectorVectorDecayer() 
  : _incoming(10), _outgoing1(10), _outgoing2(10), _coupling(10), _maxweight(10) {
  // decay eta -> omega gamma
  _incoming[0] = 331; _outgoing1[0] = 223; _outgoing2[0] = 22; 
  _coupling[0] = 0.1412/GeV; _maxweight[0] = 1.2; 
  // decay pi -> gamma gamma
  _incoming[1] = 111; _outgoing1[1] = 22; _outgoing2[1] = 22; 
  _coupling[1] = 0.0178/GeV; _maxweight[1] = 1.1; 
  // decay eta -> gamma gamma
  _incoming[2] = 221; _outgoing1[2] = 22; _outgoing2[2] = 22; 
  _coupling[2] = 0.0176/GeV; _maxweight[2] = 1.1; 
  // decay eta' -> gamma gamma
  _incoming[3] = 331; _outgoing1[3] = 22; _outgoing2[3] = 22; 
  _coupling[3] = 0.0221/GeV; _maxweight[3] = 1.1; 
  // decay eta_c -> rho rho
  _incoming[4] = 441; _outgoing1[4] = 213; _outgoing2[4] = -213; 
  _coupling[4] = 0.0525/GeV; _maxweight[4] = 2.7; 
  _incoming[5] = 441; _outgoing1[5] = 113; _outgoing2[5] =  113; 
  _coupling[5] = 0.0371/GeV; _maxweight[5] = 2.7; 
  // decay eta-c -> phi phi
  _incoming[6] = 441; _outgoing1[6] = 333; _outgoing2[6] = 333; 
  _coupling[6] = 0.0267/GeV; _maxweight[6] = 9.; 
  // decay eta-c -> gamma gamma
  _incoming[7] = 441; _outgoing1[7] = 22; _outgoing2[7] = 22; 
  _coupling[7] = 0.00521/GeV; _maxweight[7] = 1.2; 
  // decay eta_c -> K* K*
  _incoming[8] = 441; _outgoing1[8] = 323; _outgoing2[8] = -323; 
  _coupling[8] = 0.0308/GeV; _maxweight[8] = 5.3; 
  _incoming[9] = 441; _outgoing1[9] = 313; _outgoing2[9] = -313; 
  _coupling[9] = 0.0308/GeV; _maxweight[9] = 5.3; 
  // initial size of the vectors
  _initsize = _incoming.size();
  // intermediates
  generateIntermediates(false);
}

void PScalarVectorVectorDecayer::doinit() {
  DecayIntegrator::doinit();
  // check the parameters arew consistent
  unsigned int isize(_coupling.size());
  if(isize!=_incoming.size()  || isize!=_outgoing1.size()||
     isize!=_outgoing2.size() || isize!=_maxweight.size())
    throw InitException() << "Inconsistent parameters in PScalarVectorVectorDecayer" 
			  << Exception::abortnow;
  // set up the integration channels
  vector<double> wgt;
  DecayPhaseSpaceModePtr mode;
  tPDVector extpart(3);
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    extpart[0] = getParticleData(_incoming[ix]);
    extpart[1] = getParticleData(_outgoing1[ix]);
    extpart[2] = getParticleData(_outgoing2[ix]);
    if(extpart[0]&&extpart[1]&&extpart[2]) 
      mode=new_ptr(DecayPhaseSpaceMode(extpart,this));
    else
      mode=DecayPhaseSpaceModePtr();
    addMode(mode,_maxweight[ix],wgt);
  }
}

int PScalarVectorVectorDecayer::modeNumber(bool & cc,tcPDPtr parent,
					   const tPDVector & children) const {
  cc = false;
  if(children.size()!=2) return -1;
  int id(parent->id());
  int id1(children[0]->id());
  int id2(children[1]->id());
  unsigned int ix(0);
  int imode(-1);
  do {
    if(_incoming[ix]==id) {
      if((id1==_outgoing1[ix]&&id2==_outgoing2[ix])||
	 (id2==_outgoing1[ix]&&id1==_outgoing2[ix])) imode=ix;
    }
    ++ix;
  }
  while(imode<0&&ix<_incoming.size());
  return imode;
}

void PScalarVectorVectorDecayer::persistentOutput(PersistentOStream & os) const {
  os << ounit(_coupling,1/GeV) << _incoming << _outgoing1 << _outgoing2 << _maxweight;
}

void PScalarVectorVectorDecayer::persistentInput(PersistentIStream & is, int) {
  is >> iunit(_coupling,1/GeV) >> _incoming >> _outgoing1 >> _outgoing2 >> _maxweight;
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<PScalarVectorVectorDecayer,DecayIntegrator>
describeHerwigPScalarVectorVectorDecayer("Herwig::PScalarVectorVectorDecayer", "HwSMDecay.so");

void PScalarVectorVectorDecayer::Init() {

  static ClassDocumentation<PScalarVectorVectorDecayer> documentation
    ("The PScalarVectorVectorDecayer class is designed for"
     " the decay of a pseduoscalar meson to two spin-1 particles.");

  static ParVector<PScalarVectorVectorDecayer,int> interfaceIncoming
    ("Incoming",
     "The PDG code for the incoming particle",
     &PScalarVectorVectorDecayer::_incoming,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<PScalarVectorVectorDecayer,int> interfaceOutcoming1
    ("FirstOutgoing",
     "The PDG code for the first outgoing particle",
     &PScalarVectorVectorDecayer::_outgoing1,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<PScalarVectorVectorDecayer,int> interfaceOutcoming2
    ("SecondOutgoing",
     "The PDG code for the second outgoing particle",
     &PScalarVectorVectorDecayer::_outgoing2,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<PScalarVectorVectorDecayer,InvEnergy> interfaceCoupling
    ("Coupling",
     "The coupling for the decay mode",
     &PScalarVectorVectorDecayer::_coupling,
     1/GeV, 0, ZERO, ZERO, 10000/GeV, false, false, true);

  static ParVector<PScalarVectorVectorDecayer,double> interfaceMaxWeight
    ("MaxWeight",
     "The maximum weight for the decay mode",
     &PScalarVectorVectorDecayer::_maxweight,
     0, 0, 0, 0., 200., false, false, true);
}

double PScalarVectorVectorDecayer::me2(const int,
				       const Particle & inpart,
				       const ParticleVector & decay,
				       MEOption meopt) const {
  if(!ME())
    ME(new_ptr(TwoBodyDecayMatrixElement(PDT::Spin0,PDT::Spin1,PDT::Spin1)));
  bool photon[2]={false,false};
  for(unsigned int ix=0;ix<2;++ix)
    photon[ix] = decay[ix]->id()==ParticleID::gamma;
  if(meopt==Initialize) {
    ScalarWaveFunction::
      calculateWaveFunctions(_rho,const_ptr_cast<tPPtr>(&inpart),incoming);
  }
  if(meopt==Terminate) {
    // set up the spin information for the decay products
    ScalarWaveFunction::constructSpinInfo(const_ptr_cast<tPPtr>(&inpart),
					  incoming,true);
    for(unsigned int ix=0;ix<2;++ix)
      VectorWaveFunction::constructSpinInfo(_vectors[ix],decay[ix],
					    outgoing,true,photon[ix]);
    return 0.;
  }
  for(unsigned int ix=0;ix<2;++ix)
    VectorWaveFunction::
      calculateWaveFunctions(_vectors[ix],decay[ix],outgoing,photon[ix]);
  // now compute the matrix element
  InvEnergy2 fact(_coupling[imode()]/inpart.mass());
  unsigned int ix,iy;
  for(ix=0;ix<3;++ix) {
    for(iy=0;iy<3;++iy) {
      (*ME())(0,ix,iy)=Complex(fact*epsilon(_vectors[0][ix],decay[1]->momentum(),
					    _vectors[1][iy])
			       *decay[0]->momentum());
    }
  }
  // test of the matrix element
//   double test = 2.*sqr(fact*inpart.mass())*
//     sqr(Kinematics::pstarTwoBodyDecay(inpart.mass(),decay[0]->mass(),decay[1]->mass()));
//   double me = newME.contract(rhoin).real();
//   cerr << "testing the matrix element for " << inpart.PDGName() << " -> " 
//        << decay[0]->PDGName() << " " << decay[1]->PDGName() << " " 
//        << me << " " << (me-test)/(me+test) << "\n";
  return ME()->contract(_rho).real();
}

// specify the 1-2 matrix element to be used in the running width calculation
bool PScalarVectorVectorDecayer::twoBodyMEcode(const DecayMode & dm, int & itype,
					       double & coupling) const {
  int imode(-1);
  int id(dm.parent()->id());
  ParticleMSet::const_iterator pit(dm.products().begin());
  int id1((**pit).id());++pit;
  int id2((**pit).id());
  unsigned int ix(0);
  do {
    if(_incoming[ix]==id) {
      if((id1==_outgoing1[ix]&&id2==_outgoing2[ix])||
	 (id2==_outgoing1[ix]&&id1==_outgoing2[ix])) imode=ix;
    }
    ++ix;
  }
  while(imode<0&&ix<_incoming.size());
  coupling=_coupling[imode]*dm.parent()->mass();
  itype = 3;
  return id1==_outgoing1[imode]&&id2==_outgoing2[imode];
}

// output the setup info for the particle database
void PScalarVectorVectorDecayer::dataBaseOutput(ofstream & output,
						bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    if(ix<_initsize) {
      output << "newdef " << name() << ":Incoming   " << ix << " "
	     << _incoming[ix]   << "\n";
      output << "newdef " << name() << ":FirstOutgoing  " << ix << " "
	     << _outgoing1[ix]  << "\n";
      output << "newdef " << name() << ":SecondOutgoing " << ix << " "
	     << _outgoing2[ix]  << "\n";
      output << "newdef " << name() << ":Coupling   " << ix << " "
	     << _coupling[ix]*GeV   << "\n";
      output << "newdef " << name() << ":MaxWeight  " << ix << " "
	     << _maxweight[ix]  << "\n";
    }
    else {
      output << "insert " << name() << ":Incoming   " << ix << " "
	     << _incoming[ix]   << "\n";
      output << "insert " << name() << ":FirstOutgoing  " << ix << " "
	     << _outgoing1[ix]  << "\n";
      output << "insert " << name() << ":SecondOutgoing " << ix << " "
	     << _outgoing2[ix]  << "\n";
      output << "insert " << name() << ":Coupling   " << ix << " "
	     << _coupling[ix]*GeV   << "\n";
      output << "insert " << name() << ":MaxWeight  " << ix << " "
	     << _maxweight[ix]  << "\n";
    }
  }
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}
#line 1 "./ScalarMesonTensorScalarDecayer.cc"
// -*- C++ -*-
//
// ScalarMesonTensorScalarDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the ScalarMesonTensorScalarDecayer class.
//

#include "ScalarMesonTensorScalarDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Helicity/WaveFunction/TensorWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "Herwig/Decay/TwoBodyDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void ScalarMesonTensorScalarDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()) {
    for(unsigned int ix=0;ix<_incoming.size();++ix) 
      if(mode(ix)) _maxweight[ix] = mode(ix)->maxWeight();
  }
}

ScalarMesonTensorScalarDecayer::ScalarMesonTensorScalarDecayer() 
  : _incoming(3), _outgoingT(3), _outgoingS(3), _coupling(3), _maxweight(3) {
  // D+ -> f_2 pi
  _incoming[0] =  411; _outgoingT[0] = 225; _outgoingS[0] =  211; 
  _coupling[0] = 8.23E-7/GeV; _maxweight[0] = 5; 
  // chi_c0 -> K*_0 K*_2
  _incoming[1] =  10441; _outgoingT[1] = 325; _outgoingS[1] =  -10321; 
  _coupling[1] = 0.0217/GeV; _maxweight[1] = 5; 
  _incoming[2] =  10441; _outgoingT[2] = 315; _outgoingS[2] =  -10311; 
  _coupling[2] = 0.0217/GeV; _maxweight[2] = 5; 
  // initial size of the arrays
  _initsize=_incoming.size();
  // intermediates
  generateIntermediates(false);
}

void ScalarMesonTensorScalarDecayer::doinit() {
  DecayIntegrator::doinit();
  // check the parameters arew consistent
  unsigned int isize=_coupling.size();
  if(isize!=_incoming.size()  || isize!=_outgoingT.size()||
     isize!=_outgoingS.size() || isize!=_maxweight.size())
    throw InitException() << "Inconsistent parameters in "
			  << "ScalarMesonTensorScalarDecayer" 
			  << Exception::abortnow;
  // set up the integration channels
  vector<double> wgt;
  DecayPhaseSpaceModePtr mode;
  tPDVector extpart(3);
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    extpart[0] = getParticleData(_incoming[ix]);
    extpart[1] = getParticleData(_outgoingT[ix]);
    extpart[2] = getParticleData(_outgoingS[ix]);
    if(extpart[0]&&extpart[1]&&extpart[2]) 
      mode=new_ptr(DecayPhaseSpaceMode(extpart,this));
    else
      mode=DecayPhaseSpaceModePtr();
    addMode(mode,_maxweight[ix],wgt);
  }
}

int ScalarMesonTensorScalarDecayer::modeNumber(bool & cc,tcPDPtr parent,
					       const tPDVector & children) const {
  if(children.size()!=2) return -1;
  int id0(parent->id());
  int id0bar = parent->CC() ? parent->CC()->id() : id0;
  int id1(children[0]->id());
  int id1bar = children[0]->CC() ? children[0]->CC()->id() : id1;
  int id2(children[1]->id());
  int id2bar = children[1]->CC() ? children[1]->CC()->id() : id2;
  unsigned int ix(0);
  int imode(-1);
  do {
    if(id0   ==_incoming[ix]) {
      if((id1   ==_outgoingT[ix]&&id2   ==_outgoingS[ix])||
	 (id2   ==_outgoingT[ix]&&id1   ==_outgoingS[ix])) {
	imode=ix;
	cc=false;
      }
    }
    if(id0bar==_incoming[ix]&&imode<0) {
      if((id1bar==_outgoingT[ix]&&id2bar==_outgoingS[ix])||
	 (id2bar==_outgoingT[ix]&&id1bar==_outgoingS[ix])) {
	imode=ix;
	cc=true;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  return imode;
}

void ScalarMesonTensorScalarDecayer::persistentOutput(PersistentOStream & os) const {
  os << ounit(_coupling,1/GeV) << _incoming << _outgoingT << _outgoingS << _maxweight;
}

void ScalarMesonTensorScalarDecayer::persistentInput(PersistentIStream & is, int) {
  is >> iunit(_coupling,1/GeV) >> _incoming >> _outgoingT >> _outgoingS >> _maxweight;
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<ScalarMesonTensorScalarDecayer,DecayIntegrator>
describeHerwigScalarMesonTensorScalarDecayer("Herwig::ScalarMesonTensorScalarDecayer", "HwSMDecay.so");

void ScalarMesonTensorScalarDecayer::Init() {

  static ClassDocumentation<ScalarMesonTensorScalarDecayer> documentation
    ("The ScalarMesonTensorScalarDecayer class is designed for"
     " the decay of a pseduoscalar meson to two spin-1 particles.");

  static ParVector<ScalarMesonTensorScalarDecayer,int> interfaceIncoming
    ("Incoming",
     "The PDG code for the incoming particle",
     &ScalarMesonTensorScalarDecayer::_incoming,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<ScalarMesonTensorScalarDecayer,int> interfaceOutcomingT
    ("OutgoingTensor",
     "The PDG code for the outgoing tensor",
     &ScalarMesonTensorScalarDecayer::_outgoingT,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<ScalarMesonTensorScalarDecayer,int> interfaceOutcomingS
    ("OutgoingScalar",
     "The PDG code for the outgoing scalar",
     &ScalarMesonTensorScalarDecayer::_outgoingS,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<ScalarMesonTensorScalarDecayer,InvEnergy> interfaceCoupling
    ("Coupling",
     "The coupling for the decay mode",
     &ScalarMesonTensorScalarDecayer::_coupling,
     1/GeV, 0, ZERO, ZERO, 100./GeV, false, false, true);

  static ParVector<ScalarMesonTensorScalarDecayer,double> interfaceMaxWeight
    ("MaxWeight",
     "The maximum weight for the decay mode",
     &ScalarMesonTensorScalarDecayer::_maxweight,
     0, 0, 0, 0., 100., false, false, true);
}

double ScalarMesonTensorScalarDecayer::me2(const int,
					   const Particle & inpart,
					   const ParticleVector & decay,
					   MEOption meopt) const {
  if(!ME())
    ME(new_ptr(TwoBodyDecayMatrixElement(PDT::Spin0,PDT::Spin2,PDT::Spin0)));
  if(meopt==Initialize) {
    ScalarWaveFunction::
      calculateWaveFunctions(_rho,const_ptr_cast<tPPtr>(&inpart),incoming);
  }
  if(meopt==Terminate) {
    // set up the spin information for the decay products
    ScalarWaveFunction::constructSpinInfo(const_ptr_cast<tPPtr>(&inpart),
					  incoming,true);
    TensorWaveFunction::constructSpinInfo(_tensors,decay[0],
					  outgoing,true,false);
    ScalarWaveFunction::constructSpinInfo(decay[1],outgoing,true);
    return 0.;
  }
  TensorWaveFunction::
    calculateWaveFunctions(_tensors,decay[0],outgoing,false);
  // calculate the matrix element
  InvEnergy2 fact(_coupling[imode()]/inpart.mass());
  LorentzPolarizationVectorE vtemp;
  for(unsigned int ix=0;ix<5;++ix) {
    vtemp = _tensors[ix]*inpart.momentum(); 
    (*ME())(0,ix,0) = Complex(fact * decay[1]->momentum().dot(vtemp));
  }
  // test of the matrix element
//   double me=newME.contract(rhoin).real();
//   Energy pcm = Kinematics::pstarTwoBodyDecay(inpart.mass(),decay[0]->mass(),
// 					     decay[1]->mass());
//   double test = 2.*pow<4,1>(pcm)*sqr(_coupling[imode()]*inpart.mass())/
//     3./pow<4,1>(decay[0]->mass());
//   cerr << "testing matrix element for " << inpart.PDGName() << " -> " 
//        << decay[0]->PDGName() << " " << decay[1]->PDGName() << " "
//        << me << " " << (me-test)/(me+test) << "\n";
  // output the answer
  return ME()->contract(_rho).real();
}

// specify the 1-2 matrix element to be used in the running width calculation
bool ScalarMesonTensorScalarDecayer::twoBodyMEcode(const DecayMode & dm, int & itype,
						   double & coupling) const {
  int id(dm.parent()->id());
  int idbar = dm.parent()->CC() ? dm.parent()->CC()->id() : id;
  ParticleMSet::const_iterator pit(dm.products().begin());
  int id1((**pit).id());
  int id1bar = (**pit).CC() ? (**pit).CC()->id() : id1;
  ++pit;
  int id2((**pit).id());
  int id2bar = (**pit).CC() ? (**pit).CC()->id() : id2;
  unsigned int ix(0); 
  bool order(false);
  int imode(-1);
  do {
    if(id   ==_incoming[ix]) {
      if(id1==_outgoingT[ix]&&id2==_outgoingS[ix]) {
	imode=ix;
	order=true;
      }
      if(id2==_outgoingT[ix]&&id1==_outgoingS[ix]) {
	imode=ix;
	order=false;
      }
    }
    if(idbar==_incoming[ix]&&imode<0) {
      if(id1bar==_outgoingT[ix]&&id2bar==_outgoingS[ix]) {
	imode=ix;
	order=true;
      }
      if(id2bar==_outgoingT[ix]&&id1bar==_outgoingS[ix]) {
	imode=ix;
	order=false;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  coupling=_coupling[imode]*dm.parent()->mass();
  itype = 11;
  return order;
}

// output the setup information for the particle database
void ScalarMesonTensorScalarDecayer::dataBaseOutput(ofstream & output,
						    bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  // the rest of the parameters
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    if(ix<_initsize) {
      output << "newdef " << name() << ":Incoming " << ix << " " 
	     << _incoming[ix] << "\n";
      output << "newdef " << name() << ":OutgoingTensor " << ix << " " 
	     << _outgoingT[ix] << "\n";
      output << "newdef " << name() << ":OutgoingScalar " << ix << " " 
	     << _outgoingS[ix] << "\n";
      output << "newdef " << name() << ":Coupling " << ix << " " 
	     << _coupling[ix]*GeV << "\n";
      output << "newdef " << name() << ":MaxWeight " << ix << " " 
	     << _maxweight[ix] << "\n";
    }
    else {
      output << "insert " << name() << ":Incoming " << ix << " " 
	     << _incoming[ix] << "\n";
      output << "insert " << name() << ":OutgoingTensor " << ix << " " 
	     << _outgoingT[ix] << "\n";
      output << "insert " << name() << ":OutgoingScalar " << ix << " " 
	     << _outgoingS[ix] << "\n";
      output << "insert " << name() << ":Coupling " << ix << " " 
	     << _coupling[ix]*GeV << "\n";
      output << "insert " << name() << ":MaxWeight " << ix << " " 
	     << _maxweight[ix] << "\n";
    }
  }
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}
#line 1 "./ScalarScalarScalarDecayer.cc"
// -*- C++ -*-
//
// ScalarScalarScalarDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the ScalarScalarScalarDecayer class.
//

#include "ScalarScalarScalarDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "Herwig/Decay/TwoBodyDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void ScalarScalarScalarDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()) {
    for(unsigned int ix=0;ix<_incoming.size();++ix)
      if(mode(ix) )_maxweight[ix] = mode(ix)->maxWeight();
  }
}

ScalarScalarScalarDecayer::ScalarScalarScalarDecayer() 
  : _incoming(78), _outgoing1(78), _outgoing2(78), 
    _coupling(78), _maxweight(78) {
  // f_0(980) to pi pi
  _incoming[0] = 9010221; _outgoing1[0] = 111; _outgoing2[0] =  111; 
  _coupling[0] = 1.66*GeV; _maxweight[0] = 1.05; 
  _incoming[1] = 9010221; _outgoing1[1] = 211; _outgoing2[1] = -211; 
  _coupling[1] = 2.35*GeV; _maxweight[1] = 1.05; 
  // f_0(980) to K K 
  _incoming[2] = 9010221; _outgoing1[2] = 321; _outgoing2[2] = -321; 
  _coupling[2] = 1.02*GeV; _maxweight[2] = 1.05; 
  _incoming[3] = 9010221; _outgoing1[3] = 311; _outgoing2[3] = -311; 
  _coupling[3] = 1.02*GeV; _maxweight[3] = 1.05; 
  // f_0(1370) to pi pi
  _incoming[4] = 10221; _outgoing1[4] = 111; _outgoing2[4] = 111; 
  _coupling[4] = 0.745*GeV; _maxweight[4] = 1.05; 
  _incoming[5] = 10221; _outgoing1[5] = 211; _outgoing2[5] = -211; 
  _coupling[5] = 1.054*GeV; _maxweight[5] = 1.05; 
  // f_0(1370) to pi' pi
  _incoming[6] = 10221; _outgoing1[6] = 100111; _outgoing2[6] = 111; 
  _coupling[6] = 5.027*GeV; _maxweight[6] = 2.1; 
  _incoming[7] = 10221; _outgoing1[7] = 100211; _outgoing2[7] = -211; 
  _coupling[7] = 5.027*GeV; _maxweight[7] = 2.1; 
  // f_0(1370) to K K 
  _incoming[8] = 10221; _outgoing1[8] = 321; _outgoing2[8] = -321; 
  _coupling[8] = 0.886*GeV; _maxweight[8] = 1.1; 
  _incoming[9] = 10221; _outgoing1[9] = 311; _outgoing2[9] = -311; 
  _coupling[9] = 0.886*GeV; _maxweight[9] = 1.1; 
  // f_0(1710) to pi pi
  _incoming[10] = 10331; _outgoing1[10] = 111; _outgoing2[10] = 111; 
  _coupling[10] = 0.503*GeV; _maxweight[10] = 1.05; 
  _incoming[11] = 10331; _outgoing1[11] = 211; _outgoing2[11] = -211; 
  _coupling[11] = 0.711*GeV; _maxweight[11] = 1.05; 
  // f_0(1710) to K K 
  _incoming[12] = 10331; _outgoing1[12] = 321; _outgoing2[12] = -321; 
  _coupling[12] = 2.096*GeV; _maxweight[12] = 1.05; 
  _incoming[13] = 10331; _outgoing1[13] = 311; _outgoing2[13] = -311; 
  _coupling[13] = 2.096*GeV; _maxweight[13] = 1.05; 
  // sigma to pi pi
  _incoming[14] = 9000221; _outgoing1[14] = 111; _outgoing2[14] = 111; 
  _coupling[14] = 3.654*GeV; _maxweight[14] = 1.05; 
  _incoming[15] = 9000221; _outgoing1[15] = 211; _outgoing2[15] = -211; 
  _coupling[15] = 5.178*GeV; _maxweight[15] = 1.05; 
  // a_0 to eta pi
  _incoming[16] =  9000111; _outgoing1[16] = 221; _outgoing2[16] =  111; 
  _coupling[16] = 3.33*GeV; _maxweight[16] = 1.1; 
  _incoming[17] =  9000211; _outgoing1[17] = 221; _outgoing2[17] =  211; 
  _coupling[17] = 3.33*GeV; _maxweight[17] = 1.1; 
  // a_0 to K K
  _incoming[18] =  9000111; _outgoing1[18] = 321; _outgoing2[18] = -321; 
  _coupling[18] = 2.54*GeV; _maxweight[18] = 1.05; 
  _incoming[19] =  9000111; _outgoing1[19] = 311; _outgoing2[19] = -311; 
  _coupling[19] = 2.54*GeV; _maxweight[19] = 1.05; 
  _incoming[20] =  9000211; _outgoing1[20] = 321; _outgoing2[20] = -311; 
  _coupling[20] = 3.59*GeV; _maxweight[20] = 1.05; 
  // a'_0 to eta pi
  _incoming[21] =  10111; _outgoing1[21] = 221; _outgoing2[21] =  111; 
  _coupling[21] = 1.357*GeV; _maxweight[21] = 1.05; 
  _incoming[22] =  10211; _outgoing1[22] = 221; _outgoing2[22] =  211; 
  _coupling[22] = 1.357*GeV; _maxweight[22] = 1.05; 
  // a'_0 to eta' pi
  _incoming[23] =  10111; _outgoing1[23] = 331; _outgoing2[23] =  111; 
  _coupling[23] = 0.995*GeV; _maxweight[23] = 1.6; 
  _incoming[24] =  10211; _outgoing1[24] = 331; _outgoing2[24] =  211; 
  _coupling[24] = 0.995*GeV; _maxweight[24] = 1.6; 
  // a'_0 to K K
  _incoming[25] =  10111; _outgoing1[25] = 321; _outgoing2[25] = -321; 
  _coupling[25] = 0.950*GeV; _maxweight[25] = 1.05; 
  _incoming[26] =  10111; _outgoing1[26] = 311; _outgoing2[26] = -311; 
  _coupling[26] = 0.950*GeV; _maxweight[26] = 1.05; 
  _incoming[27] =  10211; _outgoing1[27] = 321; _outgoing2[27] = -311; 
  _coupling[27] = 1.344*GeV; _maxweight[27] = 1.05; 
  // f_0(1370) to eta eta  
  _incoming[28] =  10221; _outgoing1[28] = 221; _outgoing2[28] = 221; 
  _coupling[28] = 0.235*GeV; _maxweight[28] = 1.1; 
  // f_0(1710) to eta eta  
  _incoming[29] =  10331; _outgoing1[29] = 221; _outgoing2[29] = 221; 
  _coupling[29] = 2.189*GeV; _maxweight[29] = 1.2; 
  // f_0(1370) to sigma sigma  
  _incoming[30] =  10221; _outgoing1[30] = 9000221; _outgoing2[30] = 9000221; 
  _coupling[30] = 21.46*GeV; _maxweight[30] = 7.; 
  // K_0* to K pi
  _incoming[31] =  10311; _outgoing1[31] =  311; _outgoing2[31] =  111; 
  _coupling[31] = 2.837*GeV; _maxweight[31] = 1.05; 
  _incoming[32] =  10311; _outgoing1[32] =  321; _outgoing2[32] = -211; 
  _coupling[32] = 4.000*GeV; _maxweight[32] = 1.05; 
  _incoming[33] =  10321; _outgoing1[33] =  321; _outgoing2[33] =  111; 
  _coupling[33] = 2.837*GeV; _maxweight[33] = 1.05; 
  _incoming[34] =  10321; _outgoing1[34] =  311; _outgoing2[34] =  211; 
  _coupling[34] = 4.000*GeV; _maxweight[34] = 1.05; 
  // D_0* to D pi
  _incoming[35] =  10411; _outgoing1[35] =  411; _outgoing2[35] =  111; 
  _coupling[35] = 5.472*GeV; _maxweight[35] = 1.05; 
  _incoming[36] =  10411; _outgoing1[36] =  421; _outgoing2[36] =  211; 
  _coupling[36] = 7.714*GeV; _maxweight[36] = 1.05; 
  _incoming[37] =  10421; _outgoing1[37] =  421; _outgoing2[37] =  111; 
  _coupling[37] = 5.447*GeV; _maxweight[37] = 1.05; 
  _incoming[38] =  10421; _outgoing1[38] =  411; _outgoing2[38] = -211; 
  _coupling[38] = 7.818*GeV; _maxweight[38] = 1.05; 
  // B_0* to B pi
  _incoming[39] =  10511; _outgoing1[39] =  511; _outgoing2[39] =  111; 
  _coupling[39] = 9.698*GeV; _maxweight[39] = 1.05; 
  _incoming[40] =  10511; _outgoing1[40] =  521; _outgoing2[40] = -211; 
  _coupling[40] = 13.71*GeV; _maxweight[40] = 1.05; 
  _incoming[41] =  10521; _outgoing1[41] =  521; _outgoing2[41] =  111; 
  _coupling[41] = 9.698*GeV; _maxweight[41] = 1.05; 
  _incoming[42] =  10521; _outgoing1[42] =  511; _outgoing2[42] =  211; 
  _coupling[42] = 13.71*GeV; _maxweight[42] = 1.05; 
  // K' to K_0* pi
  _incoming[43] =  100311; _outgoing1[43] =  10311; _outgoing2[43] =  111; 
  _coupling[43] = 6.595*GeV; _maxweight[43] = 2.; 
  _incoming[44] =  100311; _outgoing1[44] =  10321; _outgoing2[44] = -211; 
  _coupling[44] = 9.445*GeV; _maxweight[44] = 2.; 
  _incoming[45] =  100321; _outgoing1[45] =  10321; _outgoing2[45] =  111; 
  _coupling[45] = 6.595*GeV; _maxweight[45] = 2.; 
  _incoming[46] =  100321; _outgoing1[46] =  10311; _outgoing2[46] =  211; 
  _coupling[46] = 9.445*GeV; _maxweight[46] = 2.; 
  // D_s0* to D_s pi
  _incoming[47] =  10431; _outgoing1[47] =  431; _outgoing2[47] =  111; 
  _coupling[47] = 0.103*GeV; _maxweight[47] = 1.05; 
  // B_s0* to B_s pi
  _incoming[48] =  10531; _outgoing1[48] =  531; _outgoing2[48] =  111; 
  _coupling[48] = 8.314*GeV; _maxweight[48] = 1.05; 
  // eta'' to a_0 pi
  _incoming[49] = 100221; _outgoing1[49] = 9000111; _outgoing2[49] = 111; 
  _coupling[49] = 2.057*GeV; _maxweight[49] = 2.; 
  _incoming[50] = 100221; _outgoing1[50] = 9000211; _outgoing2[50] = -211; 
  _coupling[50] = 2.057*GeV; _maxweight[50] = 2.; 
  // eta''' to a_0 pi
  _incoming[51] = 9020221; _outgoing1[51] = 9000111; _outgoing2[51] = 111; 
  _coupling[51] = 1.470*GeV; _maxweight[51] = 2.; 
  _incoming[52] = 9020221; _outgoing1[52] = 9000211; _outgoing2[52] = -211; 
  _coupling[52] = 1.470*GeV; _maxweight[52] = 2.; 
  // eta''' to sigma eta
  _incoming[53] = 9020221; _outgoing1[53] = 221; _outgoing2[53] = 9000221; 
  _coupling[53] = 4.051*GeV; _maxweight[53] = 2.; 
  // eta'' to sigma eta
  _incoming[54] = 100221; _outgoing1[54] = 9000221; _outgoing2[54] = 221; 
  _coupling[54] = 4.316*GeV; _maxweight[54] = 2.; 
  // chi_0c decays to K K 
  _incoming[55] = 10441; _outgoing1[55] = 321; _outgoing2[55] = -321; 
  _coupling[55] = 0.104*GeV; _maxweight[55] = 1.05; 
  _incoming[56] = 10441; _outgoing1[56] = 311; _outgoing2[56] = -311; 
  _coupling[56] = 0.104*GeV; _maxweight[56] = 1.05; 
  // chi_0c decays to pi pi
  _incoming[57] = 10441; _outgoing1[57] = 211; _outgoing2[57] = -211; 
  _coupling[57] = 0.093*GeV; _maxweight[57] = 1.05; 
  _incoming[58] = 10441; _outgoing1[58] = 111; _outgoing2[58] = 111; 
  _coupling[58] = 0.066*GeV; _maxweight[58] = 1.05; 
  // chi_0c decays to eta eta
  _incoming[59] = 10441; _outgoing1[59] = 221; _outgoing2[59] = 221; 
  _coupling[59] = 0.064*GeV; _maxweight[59] = 1.05; 
  // f_0(1500) to pipi
  _incoming[60] = 9030221; _outgoing1[60] = 211; _outgoing2[60] = -211; 
  _coupling[60] = 1.398*GeV; _maxweight[60] = 1.05; 
  _incoming[61] = 9030221; _outgoing1[61] = 111; _outgoing2[61] = 111; 
  _coupling[61] = 0.989*GeV; _maxweight[61] = 1.05; 
  // f_0(1500) to sigma sigma
  _incoming[62] = 9030221; _outgoing1[62] = 9000221; _outgoing2[62] = 9000221; 
  _coupling[62] = 6.079*GeV; _maxweight[62] = 7.; 
  // f_0(1500) to eta eta
  _incoming[63] = 9030221; _outgoing1[63] = 221; _outgoing2[63] = 221; 
  _coupling[63] = 0.809*GeV; _maxweight[63] = 1.05; 
  // f_0(1500) to eta eta'
  _incoming[64] = 9030221; _outgoing1[64] = 221; _outgoing2[64] = 331; 
  _coupling[64] = 2.844*GeV; _maxweight[64] = 4.5; 
  // f_0(1500) to K K
  _incoming[65] = 9030221; _outgoing1[65] = 321; _outgoing2[65] = -321; 
  _coupling[65] = 0.686*GeV; _maxweight[65] = 1.05; 
  _incoming[66] = 9030221; _outgoing1[66] = 311; _outgoing2[66] = -311; 
  _coupling[66] = 0.686*GeV; _maxweight[66] = 1.05; 
  // f_0(1500) to pi' pi
  _incoming[67] = 9030221; _outgoing1[67] = 100111; _outgoing2[67] = 111; 
  _coupling[67] = 2.615*GeV; _maxweight[67] = 2.1; 
  _incoming[68] = 9030221; _outgoing1[68] = 100211;_outgoing2[68] = -211; 
  _coupling[68] = 2.615*GeV; _maxweight[68] = 2.1;
  // kappa to K pi
  _incoming[69] =  9000311; _outgoing1[69] =  311; _outgoing2[69] =  111; 
  _coupling[69] = 3.834*GeV; _maxweight[69] = 1.05; 
  _incoming[70] =  9000311; _outgoing1[70] =  321; _outgoing2[70] = -211; 
  _coupling[70] = 5.406*GeV; _maxweight[70] = 1.05; 
  _incoming[71] =  9000321; _outgoing1[71] =  321; _outgoing2[71] =  111; 
  _coupling[71] = 3.834*GeV; _maxweight[71] = 1.05; 
  _incoming[72] =  9000321; _outgoing1[72] =  311; _outgoing2[72] =  211; 
  _coupling[72] = 5.406*GeV; _maxweight[72] = 1.05; 
  // chi_0c decays to K*_0 K*_0 
  _incoming[73] = 10441; _outgoing1[73] = 10321; _outgoing2[73] = -10321; 
  _coupling[73] = 0.104*GeV; _maxweight[73] = 1.05; 
  _incoming[74] = 10441; _outgoing1[74] = 10311; _outgoing2[74] = -10311; 
  _coupling[74] = 0.104*GeV; _maxweight[74] = 1.05; 
  // B*_s0 decays
  _incoming[75] = 10531; _outgoing1[75] = 511; _outgoing2[75] = -311; 
  _coupling[75] = 12.17*GeV; _maxweight[75] = 1.05; 
  _incoming[76] = 10531; _outgoing1[76] = 521; _outgoing2[76] = -321; 
  _coupling[76] = 12.17*GeV; _maxweight[76] = 1.05;  
  // chi_0c decays to f_0 f_0
  _incoming[77] = 10441; _outgoing1[77] = 9010221; _outgoing2[77] = 9010221; 
  _coupling[77] = 0.084*GeV; _maxweight[77] = 1.05; 

  // initial size
  _initsize = _coupling.size();
  // intermediates
  generateIntermediates(false);
}

void ScalarScalarScalarDecayer::doinit() {
  DecayIntegrator::doinit();
  // check the parameters arew consistent
  unsigned int isize(_coupling.size());
  if(isize!=_incoming.size()  || isize!=_outgoing1.size()||
     isize!=_outgoing2.size() || isize!=_maxweight.size())
    throw InitException() << "Inconsistent parameters in ScalarScalarScalarDecayer" 
			  << Exception::abortnow;
  // set up the integration channels
  vector<double> wgt;
  DecayPhaseSpaceModePtr mode;
  tPDVector extpart(3);
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    extpart[0] = getParticleData(_incoming[ix]);
    extpart[1] = getParticleData(_outgoing1[ix]);
    extpart[2] = getParticleData(_outgoing2[ix]);
    if(extpart[0]&&extpart[1]&&extpart[2]) 
      mode=new_ptr(DecayPhaseSpaceMode(extpart,this));
    else
      mode=DecayPhaseSpaceModePtr();
    addMode(mode,_maxweight[ix],wgt);
  }
}

int ScalarScalarScalarDecayer::modeNumber(bool & cc,tcPDPtr parent,
					   const tPDVector & children) const {
  if(children.size()!=2) return -1;
  int id0(parent->id());
  int id0bar = parent->CC() ? parent->CC()->id() : id0;
  int id1(children[0]->id());
  int id1bar = children[0]->CC() ? children[0]->CC()->id() : id1;
  int id2(children[1]->id());
  int id2bar = children[1]->CC() ? children[1]->CC()->id() : id2;
  // loop over the modes and see if this is one of them
  unsigned int ix(0);
  int imode(-1);
  do {
    if(id0   ==_incoming[ix]) {
      if((id1   ==_outgoing1[ix]&&id2   ==_outgoing2[ix])||
	 (id2   ==_outgoing1[ix]&&id1   ==_outgoing2[ix])) {
	imode=ix;
	cc=false;
      }
    }
    if(id0bar==_incoming[ix]&&imode<0) {
      if((id1bar==_outgoing1[ix]&&id2bar==_outgoing2[ix])||
	 (id2bar==_outgoing1[ix]&&id1bar==_outgoing2[ix])) {
	imode=ix;
	cc=true;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  return imode;
}

void ScalarScalarScalarDecayer::persistentOutput(PersistentOStream & os) const {
  os << ounit(_coupling,MeV) << _incoming << _outgoing1 << _outgoing2 << _maxweight;
}

void ScalarScalarScalarDecayer::persistentInput(PersistentIStream & is, int) {
  is >> iunit(_coupling,MeV) >> _incoming >> _outgoing1 >> _outgoing2 >> _maxweight;
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<ScalarScalarScalarDecayer,DecayIntegrator>
describeHerwigScalarScalarScalarDecayer("Herwig::ScalarScalarScalarDecayer", "HwSMDecay.so");

void ScalarScalarScalarDecayer::Init() {

  static ClassDocumentation<ScalarScalarScalarDecayer> documentation
    ("The ScalarScalarScalarDecayer class is designed for the"
     " decay of a scalar meson to two scalar mesons including off-shell effects");

  static ParVector<ScalarScalarScalarDecayer,int> interfaceIncoming
    ("Incoming",
     "The PDG code for the incoming particle",
     &ScalarScalarScalarDecayer::_incoming,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<ScalarScalarScalarDecayer,int> interfaceOutcoming1
    ("FirstOutgoing",
     "The PDG code for the first outgoing particle",
     &ScalarScalarScalarDecayer::_outgoing1,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<ScalarScalarScalarDecayer,int> interfaceOutcoming2
    ("SecondOutgoing",
     "The PDG code for the second outgoing particle",
     &ScalarScalarScalarDecayer::_outgoing2,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<ScalarScalarScalarDecayer,Energy> interfaceCoupling
    ("Coupling",
     "The coupling for the decay mode",
     &ScalarScalarScalarDecayer::_coupling,
     MeV, 0, ZERO, ZERO, 1000000.*MeV, false, false, true);

  static ParVector<ScalarScalarScalarDecayer,double> interfaceMaxWeight
    ("MaxWeight",
     "The maximum weight for the decay mode",
     &ScalarScalarScalarDecayer::_maxweight,
     0, 0, 0, -10000, 10000, false, false, true);

}

double ScalarScalarScalarDecayer::me2(const int,
				      const Particle & inpart,
				      const ParticleVector & decay,
				      MEOption meopt) const {
  if(!ME())
    ME(new_ptr(TwoBodyDecayMatrixElement(PDT::Spin0,PDT::Spin0,PDT::Spin0)));
  if(meopt==Initialize) {
    ScalarWaveFunction::
      calculateWaveFunctions(_rho,const_ptr_cast<tPPtr>(&inpart),incoming);
  }
  if(meopt==Terminate) {
    // set up the spin information for the decay products
    ScalarWaveFunction::constructSpinInfo(const_ptr_cast<tPPtr>(&inpart),
					  incoming,true);
    for(unsigned int ix=0;ix<2;++ix)
    ScalarWaveFunction::constructSpinInfo(decay[ix],outgoing,true);
    return 0.;
  }
  double fact(_coupling[imode()]/inpart.mass());
  (*ME())(0,0,0) = fact;
  return sqr(fact);
}

// specify the 1-2 matrix element to be used in the running width calculation
bool ScalarScalarScalarDecayer::twoBodyMEcode(const DecayMode & dm, int & itype,
					       double & coupling) const {
  int imode(-1);
  int id(dm.parent()->id());
  int idbar = dm.parent()->CC() ? dm.parent()->CC()->id() : id;
  ParticleMSet::const_iterator pit(dm.products().begin());
  int id1((**pit).id());
  int id1bar = (**pit).CC() ? (**pit).CC()->id() : id1;
  ++pit;
  int id2((**pit).id());
  int id2bar = (**pit).CC() ? (**pit).CC()->id() : id2;
  unsigned int ix(0); bool order(true);
  do {
    if(id   ==_incoming[ix]) {
      if(id1==_outgoing1[ix]&&id2==_outgoing2[ix]) {
	imode=ix;
	order=true;
      }
      if(id2==_outgoing1[ix]&&id1==_outgoing2[ix]) {
	imode=ix;
	order=false;
      }
    }
    if(idbar==_incoming[ix]&&imode<0) {
      if(id1bar==_outgoing1[ix]&&id2bar==_outgoing2[ix]) {
	imode=ix;
	order=true;
      }
      if(id2bar==_outgoing1[ix]&&id1bar==_outgoing2[ix]) {
	imode=ix;
	order=false;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  coupling=_coupling[imode]/dm.parent()->mass();
  itype = 6;
  return order;
}

// output the setup information for the particle database
void ScalarScalarScalarDecayer::dataBaseOutput(ofstream & output,
					       bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  // the rest of the parameters
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    if(ix<_initsize) {
      output << "newdef " << name() << ":Incoming " << ix << " " 
	     << _incoming[ix] << "\n";
      output << "newdef " << name() << ":FirstOutgoing " << ix << " " 
	     << _outgoing1[ix] << "\n";
      output << "newdef " << name() << ":SecondOutgoing " << ix << " " 
	     << _outgoing2[ix] << "\n";
      output << "newdef " << name() << ":Coupling " << ix << " " 
	     << _coupling[ix]/MeV << "\n";
      output << "newdef " << name() << ":MaxWeight " << ix << " " 
	     << _maxweight[ix] << "\n";
    }
    else {
      output << "insert " << name() << ":Incoming " << ix << " " 
	     << _incoming[ix] << "\n";
      output << "insert " << name() << ":FirstOutgoing " << ix << " " 
	     << _outgoing1[ix] << "\n";
      output << "insert " << name() << ":SecondOutgoing " << ix << " " 
	     << _outgoing2[ix] << "\n";
      output << "insert " << name() << ":Coupling " << ix << " " 
	     << _coupling[ix]/MeV << "\n";
      output << "insert " << name() << ":MaxWeight " << ix << " " 
	     << _maxweight[ix] << "\n";
    }
  }
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}
#line 1 "./SemiLeptonicScalarDecayer.cc"
// -*- C++ -*-
//
// SemiLeptonicScalarDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the SemiLeptonicScalarDecayer class.
//

#include "SemiLeptonicScalarDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/StandardModel/StandardModelBase.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/Interface/Reference.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Helicity/LorentzPolarizationVector.h"
#include "ThePEG/Helicity/epsilon.h"
#include "ThePEG/Helicity/LorentzTensor.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/TensorWaveFunction.h"
#include "Herwig/Decay/GeneralDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

SemiLeptonicScalarDecayer::SemiLeptonicScalarDecayer() {
  // intermediates
  generateIntermediates(true);
}

void SemiLeptonicScalarDecayer::doinitrun() {
  _current->initrun();
  _form->initrun();
  DecayIntegrator::doinitrun();
  if(initialize()) {
    _maxwgt.clear();
    for(unsigned int ix=0;ix<numberModes();++ix) {
      _maxwgt.push_back(mode(ix)->maxWeight());
    }
  }
}

void SemiLeptonicScalarDecayer::doinit() {
  DecayIntegrator::doinit();
  // make sure the current got initialised
  _current->init();
  // and the form factors
  _form->init();
  // the channels
  tPDVector extpart,ptemp;
  _modemap.clear();
  double maxweight;
  vector<double> channelwgts(1,1.);
  int id0(0),id1(0),Wcharge(0);
  Energy min;
  DecayPhaseSpaceModePtr mode;
  DecayPhaseSpaceChannelPtr channel;
  int iq,ia; unsigned int ix,iy,iz;
  bool done;
  for(ix=0;ix<_form->numberOfFactors();++ix) {
    // get the external particles for this mode
    extpart.resize(2);
    _form->particleID(ix,id0,id1);
    extpart[0]=getParticleData(id0);
    extpart[1]=getParticleData(id1);
    _modemap.push_back(numberModes());
    if(!extpart[0]||!extpart[1]) continue;
    Wcharge =(extpart[0]->iCharge()-extpart[1]->iCharge());
    min = extpart[0]->mass()+extpart[0]->widthUpCut()
      -extpart[1]->mass()+extpart[1]->widthLoCut();
    for(iy=0;iy<_current->numberOfModes();++iy) {
      extpart.resize(2); 	
      _current->decayModeInfo(iy,iq,ia);
      ptemp=_current->particles(Wcharge,iy,iq,ia);
      for(iz=0;iz<ptemp.size();++iz) {
	extpart.push_back(ptemp[iz]);
      }
      // create the mode
      mode=new_ptr(DecayPhaseSpaceMode(extpart,this));
      // create the first piece of the channel
      channel = new_ptr(DecayPhaseSpaceChannel(mode));
      channel->addIntermediate(extpart[0],0,0.0,1,-1);
      done=_current->createMode(Wcharge,iy,mode,2,1,channel,min);
      if(done) {
	// the maximum weight
	if(_maxwgt.size()>numberModes()) maxweight=_maxwgt[numberModes()];
	else                             maxweight=2.;
	channelwgts.resize(mode->numberChannels(),
			   1./(mode->numberChannels()));
	addMode(mode,maxweight,channelwgts);
      }
    }
  }
}

bool SemiLeptonicScalarDecayer::accept(tcPDPtr parent, 
				       const tPDVector & children) const {
  // find the non-lepton
  int imes(0),idtemp,idin(parent->id());
  vector<int> idother; bool dummy;
  tPDVector::const_iterator pit  = children.begin();
  tPDVector::const_iterator pend = children.end();
  for( ; pit!=pend;++pit) {
    idtemp=(**pit).id();
    if(abs(idtemp)>16) imes=idtemp;
    else               idother.push_back(idtemp);
  }
  // check that the form factor exists
  if(_form->formFactorNumber(idin,imes,dummy)<0) return false;
  // and the current
  return _current->accept(idother);
}

int  SemiLeptonicScalarDecayer::modeNumber(bool & cc,tcPDPtr parent,
					   const tPDVector & children) const {
  // find the ids of the particles for the decay current
  tPDVector::const_iterator pit  = children.begin();
  tPDVector::const_iterator pend = children.end();
  int idtemp,imes(0),idin(parent->id());
  vector<int> idother;
  cc=false;
  for( ; pit!=pend;++pit) {
    idtemp=(**pit).id();
    if(abs(idtemp)>16) imes=idtemp;
    else               idother.push_back(idtemp);
  }
  return _modemap[_form->formFactorNumber(idin,imes,cc)]
    +_current->decayMode(idother);  
}


void SemiLeptonicScalarDecayer::persistentOutput(PersistentOStream & os) const {
  os << _current << _form << _maxwgt << _modemap;
}

void SemiLeptonicScalarDecayer::persistentInput(PersistentIStream & is, int) {
  is >> _current >> _form >> _maxwgt >> _modemap;
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<SemiLeptonicScalarDecayer,DecayIntegrator>
describeHerwigSemiLeptonicScalarDecayer("Herwig::SemiLeptonicScalarDecayer", "HwSMDecay.so");

void SemiLeptonicScalarDecayer::Init() {

  static ClassDocumentation<SemiLeptonicScalarDecayer> documentation
    ("The SemiLeptonicScalarDecayer class is designed for the"
    "semi-leptonic decay of a (pseudo)-scalar meson.");

  static Reference<SemiLeptonicScalarDecayer,LeptonNeutrinoCurrent> interfaceCurrent
    ("Current",
     "The current for the leptons produced in the decay.",
     &SemiLeptonicScalarDecayer::_current, true, true, true, false, false);

  static Reference<SemiLeptonicScalarDecayer,ScalarFormFactor> interfaceFormFactor
    ("FormFactor",
     "The form factor",
     &SemiLeptonicScalarDecayer::_form, true, true, true, false, false);

  static ParVector<SemiLeptonicScalarDecayer,double> interfaceMaximumWeight
    ("MaximumWeight",
     "The maximum weights for the decays",
     &SemiLeptonicScalarDecayer::_maxwgt,
     0, 0, 0, 0, 100., false, false, true);

}

// combine the currents and form-factors to give the matrix element
double SemiLeptonicScalarDecayer::me2(const int ichan,
				      const Particle & inpart,
				      const ParticleVector & decay,
				      MEOption meopt) const {
  // get the information on the form-factor
  int jspin(0),id0(inpart.id()),id1(decay[0]->id());
  bool cc(false);
  unsigned int iloc(_form->formFactorNumber(id0,id1,cc));
  int spect,iq,ia;
  _form->formFactorInfo(iloc,jspin,spect,iq,ia);
  // extract leptons for the lepton current
  Energy scale;
  ParticleVector leptons;
  leptons.push_back(decay[decay.size()-2]);
  leptons.push_back(decay[decay.size()-1]);
  int mode=(abs(decay[1]->id())-11)/2;
  if(!ME()) {
    if(jspin==0)
      ME(new_ptr(GeneralDecayMatrixElement(PDT::Spin0,PDT::Spin0,PDT::Spin1Half,PDT::Spin1Half)));
    else if(jspin==1)       
      ME(new_ptr(GeneralDecayMatrixElement(PDT::Spin0,PDT::Spin1,PDT::Spin1Half,PDT::Spin1Half)));
    else if(jspin==2)       
      ME(new_ptr(GeneralDecayMatrixElement(PDT::Spin0,PDT::Spin2,PDT::Spin1Half,PDT::Spin1Half)));
  }
  // initialisation
  if(meopt==Initialize) {
    ScalarWaveFunction::
      calculateWaveFunctions(_rho,const_ptr_cast<tPPtr>(&inpart),incoming);
    // work out the mapping for the lepton vector
    _constants.resize(decay.size()+1);
    _ispin.resize(decay.size());
    _imes=0;
    unsigned int itemp(1);
    for(int ix=int(decay.size()-1);ix>=0;--ix) {
      _ispin[ix]=decay[ix]->data().iSpin();
      if(abs(decay[ix]->id())<=16) {
	itemp*=_ispin[ix];
	_constants[ix]=itemp;
      }
      else _imes=ix;
    }
    _constants[decay.size()]=1;
    _constants[_imes]=_constants[_imes+1];
  }
  if(meopt==Terminate) {
    // set up the spin information for the decay products
    ScalarWaveFunction::constructSpinInfo(const_ptr_cast<tPPtr>(&inpart),
					  incoming,true);
    if(jspin==0)
      ScalarWaveFunction::
	constructSpinInfo(decay[0],outgoing,true);
    else if(jspin==1)
      VectorWaveFunction::
	constructSpinInfo(_vectors,decay[0],outgoing,true,false);
    else if(jspin==2)
      TensorWaveFunction::
	constructSpinInfo(_tensors,decay[0],outgoing,true,false);
    _current->current(mode,ichan,scale,leptons,meopt);
    return 0.;
  }
  // get the wavefunctions of the decay products
  switch(decay[0]->dataPtr()->iSpin()) {
  case 1:
    break;
  case 3:
    VectorWaveFunction::
      calculateWaveFunctions(_vectors,decay[0],outgoing,false);
    break;
  case 5:
    TensorWaveFunction::
      calculateWaveFunctions(_tensors,decay[0],outgoing,false);
    break;
  default:
    assert(false);
  }
  // work out the value of q and calculate the form factors
  Lorentz5Momentum q(inpart.momentum()-decay[0]->momentum());
  q.rescaleMass();
  Energy2 q2(q.mass2());
  Lorentz5Momentum sum(inpart.momentum()+decay[0]->momentum());
  // calculate the hadronic current for the decay
  Complex ii(0.,1.);
  vector<LorentzPolarizationVectorE> hadron;
  if(jspin==0) {
    Complex fp,f0;
    _form->ScalarScalarFormFactor(q2,iloc,id0,id1,inpart.mass(),decay[0]->mass(),
				  f0,fp);
    Complex pre((sqr(inpart.mass())-sqr(decay[0]->mass()))/q2*(f0-fp));
    hadron.push_back(fp*sum+(pre*q));
  }
  else if(jspin==1) {
    Complex A0,A1,A2,A3,V;
    complex<Energy> dot;
    Energy MP(inpart.mass()),MV(decay[0]->mass()),msum(MP+MV),mdiff(MP-MV);
    _form->ScalarVectorFormFactor(q2,iloc,id0,id1,MP,MV,A0,A1,A2,V);
    A3 = Complex(0.5/MV*(msum*A1-mdiff*A2));
    if(cc) V*=-1.;
    // compute the hadron currents
    for(unsigned int ix=0;ix<3;++ix) {
      // dot product
      dot = _vectors[ix]*inpart.momentum();
      // current
      hadron.push_back(-ii*msum*A1*_vectors[ix]
		       +ii*A2/msum*dot*sum
		       +2.*ii*MV/q2*(A3-A0)*dot*q
		       +2.*V/msum*Helicity::epsilon(_vectors[ix],inpart.momentum(),
						    decay[0]->momentum()));
    }
  }
  else if(jspin==2) {
    complex<InvEnergy2> h,bp,bm;
    complex<double> k;
    complex<Energy2> dot;
    _form->ScalarTensorFormFactor(q2,iloc,id0,id1,inpart.mass(),decay[0]->mass(),
				  h,k,bp,bm);
    if(!cc) h*=-1.;
    LorentzPolarizationVectorE dotv;
    // compute the hadron currents
    for(unsigned int ix=0;ix<5;++ix) {
      dotv = _tensors[ix]*inpart.momentum();
      dot = dotv*inpart.momentum();
      hadron.push_back(ii*h*Helicity::epsilon(dotv,sum,q)
		       -k*dotv-bp*dot*sum-bm*dot*q);
    }
  }
  // construct the lepton current
  vector<LorentzPolarizationVectorE> 
    lepton(_current->current(mode,ichan,scale,leptons,meopt));
  // compute the matrix element
  vector<unsigned int> ihel(decay.size()+1);
  for(unsigned int mhel=0;mhel<hadron.size();++mhel) {
    for(unsigned int lhel=0;lhel<lepton.size();++lhel) {
      // map the index for the leptons to a helicity state
      for(unsigned int ix=decay.size();ix>0;--ix) {
	if(ix-1!=_imes) ihel[ix]=(lhel%_constants[ix-1])/_constants[ix];
      }
      // helicities of mesons
      ihel[0]=0;
      ihel[_imes+1]=mhel;
      (*ME())(ihel) = Complex(lepton[lhel].dot(hadron[mhel])*SM().fermiConstant());
    }
  }
  // store the matrix element
  double ckm(1.);
  if(iq<=6) {
    if(iq%2==0) ckm = SM().CKM(abs(iq)/2-1,(abs(ia)-1)/2);
    else        ckm = SM().CKM(abs(ia)/2-1,(abs(iq)-1)/2);
  }
  // return the answer
  return 0.5*(ME()->contract(_rho)).real()*ckm; 
}
 
// output the setup information for the particle database
void SemiLeptonicScalarDecayer::dataBaseOutput(ofstream & output,
					       bool header) const {
  if(header) output << "update decayers set parameters=\"";
  DecayIntegrator::dataBaseOutput(output,false);
  for(unsigned int ix=0;ix<_maxwgt.size();++ix) {
    output << "insert " << name() << ":MaximumWeight " << ix << " " 
	   << _maxwgt[ix] << "\n";
  }
  _current->dataBaseOutput(output,false,true);
  output << "newdef " << name() << ":Current " << _current->name() << " \n";
  _form->dataBaseOutput(output,false,true);
  output << "newdef " << name() << ":FormFactor " << _form->name() << " \n";
  if(header) output << "\n\" where BINARY ThePEGName=\"" << fullName() << "\";" << endl;
}
#line 1 "./ScalarMesonFactorizedDecayer.cc"
// -*- C++ -*-
//
// ScalarMesonFactorizedDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the ScalarMesonFactorizedDecayer class.
//

#include "ScalarMesonFactorizedDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/RefVector.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Interface/Reference.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"
#include "ThePEG/Helicity/epsilon.h"
#include "ThePEG/Helicity/WaveFunction/TensorWaveFunction.h"
#include "Herwig/Decay/GeneralDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;


ScalarMesonFactorizedDecayer::ScalarMesonFactorizedDecayer() 
// default values of the couplings (taken from ZPC34, 103)
  : _a1b(1.10), _a2b(-0.24), _a1c(1.30), _a2c(-0.55) { 
  // intermediates
  generateIntermediates(true);
}

void ScalarMesonFactorizedDecayer::rebind(const TranslationMap & trans)
  {
  _ckm = trans.translate(_ckm);
  DecayIntegrator::rebind(trans);
}

IVector ScalarMesonFactorizedDecayer::getReferences() {
  IVector ret = DecayIntegrator::getReferences();
  ret.push_back(_ckm);
  return ret;
}

void ScalarMesonFactorizedDecayer::doinit() {
  DecayIntegrator::doinit();
  // get the ckm object
  _ckm=dynamic_ptr_cast<Ptr<StandardCKM>::pointer>(SM().CKM());
  if(!_ckm) throw InitException() << "ScalarMesonFactorizedDecayer::doinit() "
				  << "the CKM object must be the Herwig one"
				  << Exception::runerror;
  unsigned int ix,iy,iz,icurr,iform;
  // get the CKM matrix (unsquared for interference)
  Complex ckmmat[3][3];
  vector< vector<Complex > > CKM(_ckm->getUnsquaredMatrix(SM().families()));
  for(ix=0;ix<3;++ix){for(iy=0;iy<3;++iy){ckmmat[ix][iy]=CKM[ix][iy];}}
  int id0,id1,Wcharge,iq,ia,jspin,spect,inq,outq;
  // make sure the currents and form factors got initialised
  for(ix=0;ix<_current.size();++ix) _current[ix]->init();
  for(ix=0;ix<_form.size();++ix)    _form[ix]->init();
  // find all the possible modes
  vector<unsigned int> tformmap[2],tcurrmap[2];
  vector<int> inquark,outquark,currq,curra;
  vector<tPDVector> particles;
  tPDVector extpart,ptemp;
  Energy min,minb;
  // loop over the modes in the form factors and currents
  for(iform=0;iform<_form.size();++iform) {
    for(ix=0;ix<_form[iform]->numberOfFactors();++ix) {
      // particles from the form-factor
      extpart.resize(2);
      _form[iform]->particleID(ix,id0,id1);
      _form[iform]->formFactorInfo(ix,jspin,spect,inq,outq);
      // particles from the form factor
      extpart[0]=getParticleData(id0);
      extpart[1]=getParticleData(id1);
      // charge of the decay products
      Wcharge =extpart[0]->iCharge()-extpart[1]->iCharge();
      // max mass for the particles in the current
      min = extpart[0]->massMax()-extpart[1]->massMin();
      for(icurr=0;icurr<_current.size();++icurr) {
	for(iy=0;iy<_current[icurr]->numberOfModes();++iy) {
	  extpart.resize(2);
	  // get the particles from the current
	  _current[icurr]->decayModeInfo(iy,iq,ia);
	  ptemp=_current[icurr]->particles(Wcharge,iy,iq,ia);
	  minb=ZERO;
	  for(iz=0;iz<ptemp.size();++iz) {
	    extpart.push_back(ptemp[iz]);
	    minb+=ptemp[iz]->massMin();
	  }
	  // add this mode to the list
	  if(extpart.size()>2&&minb<min&&
	     (Wcharge!=0||(Wcharge==0&&((inq>0&&inq%2!=iq%2)||
					(inq<0&&abs(inq)%2!=abs(ia)%2))))) {
	    tformmap[0].push_back(iform);tformmap[1].push_back(ix);
	    tcurrmap[0].push_back(icurr);tcurrmap[1].push_back(iy);
	    particles.push_back(extpart);
	    inquark.push_back(inq);outquark.push_back(outq);
	    currq.push_back(iq);curra.push_back(ia);
	  }
	  // if the meson in the current is neutral try the CC mode
	  if(Wcharge==0&&iq!=-ia&&((inq>0&&inq%2!=iq%2)||
				   (inq<0&&abs(inq)%2!=abs(ia)%2))) {
	    extpart.resize(2);
	    // get the particles from the current
	    ptemp=_current[icurr]->particles(Wcharge,iy,-ia,-iq);
	    minb=ZERO;
	    for(iz=0;iz<ptemp.size();++iz) {
	      extpart.push_back(ptemp[iz]);
	      minb+=ptemp[iz]->massMin();
	    }
	    if(extpart.size()>2&&minb<min) {
	      tformmap[0].push_back(iform);tformmap[1].push_back(ix);
	      tcurrmap[0].push_back(icurr);tcurrmap[1].push_back(iy);
	      particles.push_back(extpart);
	      inquark.push_back(inq);outquark.push_back(outq);
	      currq.push_back(-ia);curra.push_back(-iq);
	    }
	  }
	}
      }
    }
  }
  // loop over the modes and find the dupliciates
  vector<bool> modecc; vector<unsigned int> modeloc,tformpart,ttform[2],ttcurr[2];
  vector<Complex> tCKM; Complex ckm;
  DecayPhaseSpaceModePtr mode;
  DecayPhaseSpaceChannelPtr channel;
  bool done;
  int id,idbar;
  double maxweight;
  vector<double> channelwgts;
  unsigned int isize;double ort(sqrt(0.5));
  vector<double>::iterator start,end;
  for(ix=0;ix<particles.size();++ix) {
    while (true) {
      if(particles[ix].empty()) break;
      findModes(ix,particles,modeloc,modecc);
      // if more than three particles only allow one diagram
      if ( particles[ix].size()>3 && !modeloc.empty() ) break;
      // create the mode and set the particles as for the first instance
      mode=new_ptr(DecayPhaseSpaceMode(particles[ix],this));
      channel = new_ptr(DecayPhaseSpaceChannel(mode));
      channel->addIntermediate(particles[ix][0],0,0.0,1,-1);
      min = particles[ix][0]->massMax()-particles[ix][1]->massMin();
      Wcharge = particles[ix][0]->iCharge()-particles[ix][1]->iCharge();
      done=_current[tcurrmap[0][ix]]->createMode(Wcharge,tcurrmap[1][ix],
						 mode,2,1,channel,min);
      if(!done) throw InitException() << "Failed to construct mode in "
				      << "ScalarMesonFactorizedDecayer::doinit()." 
				      << Exception::abortnow;
      // set the parameters for the additional modes
      ttform[0].clear();ttform[1].clear();
      ttcurr[0].clear();ttcurr[1].clear();
      ttform[0].push_back(tformmap[0][ix]);ttform[1].push_back(tformmap[1][ix]);
      ttcurr[0].push_back(tcurrmap[0][ix]);ttcurr[1].push_back(tcurrmap[1][ix]);
      tformpart.clear();tformpart.push_back(0);
      id=particles[ix][1]->id();
      idbar = particles[ix][1]->CC() ? particles[ix][1]->CC()->id() : id;
      for(iy=0;iy<modeloc.size();++iy) {
	ttform[0].push_back(tformmap[0][modeloc[iy]]);
	ttform[1].push_back(tformmap[1][modeloc[iy]]);
	ttcurr[0].push_back(tcurrmap[0][modeloc[iy]]);
	ttcurr[1].push_back(tcurrmap[1][modeloc[iy]]);
	iz=1;
	do {
	  if(( modecc[iy]&&particles[modeloc[iy]][iz]->id()==idbar)||
	     (!modecc[iy]&&particles[modeloc[iy]][iz]->id()==id))
	    tformpart.push_back(iz-1);
	  ++iz;
	}
	while(tformpart.size()!=iy+2&&iz<3);
      }
      // calculate ckm factors
      tCKM.clear();
      for(iy=0;iy<ttcurr[0].size();++iy) {
	// get the quarks involved in the process
	if(iy==0) {
	  iq=currq[ix];ia=curra[ix];
	  inq=inquark[ix];outq=outquark[ix];
	}
	else {
	  if(!modecc[iy-1]) {
	    iq=currq[modeloc[iy-1]];ia=curra[modeloc[iy-1]];
	    inq=inquark[modeloc[iy-1]];outq=outquark[modeloc[iy-1]];
	  }
	  else {
	    ia=-currq[modeloc[iy-1]];iq=-curra[modeloc[iy-1]];
	    inq=-inquark[modeloc[iy-1]];outq=-outquark[modeloc[iy-1]];
	  }
	}
	_form[ttform[0][iy]]->particleID(ttform[1][iy],id0,id1);
	Wcharge = getParticleData(id0)->iCharge()-getParticleData(id1)->iCharge();
	ckm=1.;
	if(Wcharge!=0) {
	  ckm=1.;
	  if(abs(iq)%2==0)  ckm *= conj(ckmmat[abs(iq)/2-1][(abs(ia)-1)/2]);
	  else              ckm *= conj(ckmmat[abs(ia)/2-1][(abs(iq)-1)/2]);
	  if(abs(inq)%2==0) ckm *= ckmmat[abs(inq)/2-1][(abs(outq)-1)/2];
	  else              ckm *= ckmmat[abs(outq)/2-1][(abs(inq)-1)/2];
	  if(abs(inq)==5)   ckm*=_a1b;
	  else              ckm*=_a1c;
	}
	else {
	  ckm=1.;
	  if(inq>0) {
	    if(abs(inq)%2==0)  ckm *= ckmmat[abs(inq)/2-1][(abs(iq)-1)/2];
	    else               ckm *= ckmmat[abs(iq)/2-1][(abs(inq)-1)/2];
	    if(abs(outq)%2==0) ckm *= conj(ckmmat[abs(outq)/2-1][(abs(ia)-1)/2]);
	    else               ckm *= conj(ckmmat[abs(ia)/2-1][(abs(outq)-1)/2]);
	  }
	  else {
	    if(abs(inq)%2==0)  ckm *= ckmmat[abs(inq)/2-1][(abs(ia)-1)/2];
	    else               ckm *= ckmmat[abs(ia)/2-1][(abs(inq)-1)/2];
	    if(abs(outq)%2==0) ckm *= conj(ckmmat[abs(outq)/2-1][(abs(iq)-1)/2]);
	    else               ckm *= conj(ckmmat[abs(iq)/2-1][(abs(outq)-1)/2]);
	  }
	  if(abs(inq)==5) ckm*=_a2b;
	  else            ckm*=_a2c;
	}
	if((abs(inq)%2==0&&inq<0)||(abs(inq)%2!=0&&inq>0)){ckm=conj(ckm);}
	tCKM.push_back(ckm);
      }
      // special if the particles are idential add additional modes and 
      // identical particle factors
      if(particles[ix][1]->id()==particles[ix][2]->id()&&particles[ix].size()==3) {
	isize=ttcurr[0].size();
	for(iy=0;iy<isize;++iy) {
	  ttcurr[0].push_back(ttcurr[0][iy]);ttcurr[1].push_back(ttcurr[1][iy]);
	  ttform[0].push_back(ttform[0][iy]);ttform[1].push_back(ttform[1][iy]);
	  if(tformpart[iy]==0){tformpart.push_back(1);}
	  else{tformpart.push_back(0);}
	  tCKM[iy]*=ort;tCKM.push_back(tCKM[iy]);
	}
      }
      // add the parameters for the mode to the list
      _currentmapA.push_back(ttcurr[0]);_currentmapB.push_back(ttcurr[1]);
      _formmapA.push_back(ttform[0]);_formmapB.push_back(ttform[1]);
      _formpart.push_back(tformpart);
      _CKMfact.push_back(tCKM);
      // add the mode to the list
      if(_wgtmax.size()>numberModes()){maxweight=_wgtmax[numberModes()];}
      else{maxweight=0.;}
      // the weights for the channel
      if(_wgtloc.size()>numberModes()&&
	 _wgtloc[numberModes()]+mode->numberChannels()<=_weights.size()) {
	start=_weights.begin()+_wgtloc[numberModes()];
	end  = start+mode->numberChannels();
	channelwgts=vector<double>(start,end);
      }
      else {
	channelwgts.resize(mode->numberChannels(),1./(mode->numberChannels()));
      }
      // don't need channels for two body decays
      if(particles[ix].size()==3) {
	channelwgts.clear();
	mode=new_ptr(DecayPhaseSpaceMode(particles[ix],this));
      }
      addMode(mode,maxweight,channelwgts);
      // resize the duplicate modes to remove them
      for(iy=0;iy<modeloc.size();++iy) particles[modeloc[iy]] = tPDVector();
      break;
    }
  }
}

void ScalarMesonFactorizedDecayer::doinitrun() {
  unsigned int ix,iy;
  for(ix=0;ix<_current.size();++ix) _current[ix]->initrun();
  for(ix=0;ix<_form.size();++ix)    _form[ix]->initrun();
  DecayIntegrator::doinitrun();
  if(initialize()) {
    _weights.clear();
    _wgtloc.clear();
    _wgtmax.clear();
    for(ix=0;ix<numberModes();++ix) {
      _wgtmax.push_back(mode(ix)->maxWeight());
      _wgtloc.push_back(_weights.size());
      for(iy=0;iy<mode(ix)->numberChannels();++iy) {
	_weights.push_back(mode(ix)->channelWeight(iy));
      }
    }
  }
}

bool ScalarMesonFactorizedDecayer::accept(tcPDPtr parent,
					  const tPDVector & children) const {
  // N.B. this is a necessary but not sufficient test
  bool allowed(false),dummy;
  // find the ids of the particles
  tPDVector::const_iterator pit  = children.begin();
  tPDVector::const_iterator pend = children.end();
  vector<int> ids,idcurr;
  int id(parent->id());
  for( ; pit!=pend;++pit) ids.push_back((**pit).id());
  // loop over the possible particles in the formfactor
  unsigned int ipart(0),iform,icurr,ix;
  do {
    idcurr.clear();
    for(ix=0;ix<ids.size();++ix){if(ix!=ipart){idcurr.push_back(ids[ix]);}}
    iform=0;
    do {
      // check if possible from the form factor
      if(_form[iform]->formFactorNumber(id,ids[ipart],dummy)>=0) {
	// check if possible from the current
	icurr=0;
	do {
	  allowed=_current[icurr]->accept(idcurr);
	  ++icurr;
	}
	while(!allowed&&icurr<_current.size());
      }
      ++iform;
    }
    while(!allowed&&iform<_form.size());
    ++ipart;
  }
  while(!allowed&&ipart<ids.size());
  return allowed;
}

int ScalarMesonFactorizedDecayer::modeNumber(bool & cc,tcPDPtr parent,
					     const tPDVector & children) const {
  int imode(-1);
  // id's of the particles and CC
  // of the parent
  int id0(parent->id()),id0bar(id0);
  if(parent->CC()){id0bar=parent->CC()->id();}
  // of the products
  vector<int> ids,idbars;
  tPDVector::const_iterator pit  = children.begin();
  tPDVector::const_iterator pend = children.end();
  for( ;pit!=pend;++pit) {
    ids.push_back((**pit).id());
    if((**pit).CC()) idbars.push_back((**pit).CC()->id());
    else             idbars.push_back(ids.back());
  }
  // loop over the modes
  vector<bool> done(ids.size(),false);
  unsigned int nfound,ix,iy,iz;
  int idtemp;
  bool found;
  cc=false;
  ix=0;
  do {
    // particle mode
    if(id0==mode(ix)->externalParticles(0)->id()&&
       ids.size()+1==mode(ix)->numberofParticles()) {
      nfound=0;
      for(iy=0;iy<ids.size();++iy){done[iy]=false;}
      for(iy=0;iy<ids.size();++iy) {
	idtemp=mode(ix)->externalParticles(iy+1)->id();
	iz=0;found=false;
	do{if(idtemp==ids[iz]&&!done[iz]){done[iz]=true;found=true;}++iz;}
	while(iz<ids.size()&&!found);
	if(found){++nfound;}
      }
      if(nfound==ids.size()){cc=false;imode=ix;}
    }
    // CC mode
    if(id0bar==mode(ix)->externalParticles(0)->id()&&
       ids.size()+1==mode(ix)->numberofParticles()) {
      nfound=0;
      for(iy=0;iy<idbars.size();++iy) done[iy]=false;
      for(iy=0;iy<idbars.size();++iy) {
	idtemp=mode(ix)->externalParticles(iy+1)->id();
	iz=0;found=false;
	do {
	  if(idtemp==idbars[iz]&&!done[iz]) {
	    done[iz]=true;
	    found=true;
	  }
	  ++iz;
	}
	while(iz<idbars.size()&&!found);
	if(found) ++nfound;
      }
      if(nfound==idbars.size()) {
	cc=true;
	imode=ix;
      }
    }
    ++ix;
  }
  while(imode<0&&ix<numberModes());
  if(imode<0) {
    string mode = parent->PDGName() + "->";
    for(unsigned int ix=0;ix<children.size();++ix) 
      mode += children[ix]->PDGName() +",";
    throw DecayIntegratorError() << "Unable to find the mode " << mode << " in " 
				 << name() 
				 << " ScalarMesonFactorizedDecayer::decay()" 
				 << Exception::abortnow;
  }
  return imode;
}


void ScalarMesonFactorizedDecayer::persistentOutput(PersistentOStream & os) const {
  os << _current << _form << _ckm 
     << _a1b << _a2b << _a1c << _a2c 
     << _currentmapA << _currentmapB 
     << _formmapA << _formmapB << _formpart << _wgtloc 
     << _wgtmax << _weights << _CKMfact ;
}

void ScalarMesonFactorizedDecayer::persistentInput(PersistentIStream & is, int) {
  is >> _current >> _form >> _ckm 
     >> _a1b >> _a2b >> _a1c >> _a2c 
     >> _currentmapA >> _currentmapB 
     >> _formmapA >> _formmapB >> _formpart >> _wgtloc
     >> _wgtmax >> _weights >> _CKMfact;
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<ScalarMesonFactorizedDecayer,DecayIntegrator>
describeHerwigScalarMesonFactorizedDecayer("Herwig::ScalarMesonFactorizedDecayer", "HwSMDecay.so");

void ScalarMesonFactorizedDecayer::Init() {

  static ClassDocumentation<ScalarMesonFactorizedDecayer> documentation
    ("The ScalarMesonFactorizedDecayer class is designed for the weak decay of"
     " scalar mesons using the factorization approximation.");

  static RefVector<ScalarMesonFactorizedDecayer,WeakDecayCurrent> interfaceCurrents
    ("Currents",
     "A vector of references to the currents",
     &ScalarMesonFactorizedDecayer::_current, -1, false, false, true, false, false);

  static RefVector<ScalarMesonFactorizedDecayer,ScalarFormFactor> interfaceFormFactors
    ("FormFactors",
     "A vector of references to the form-factors",
     &ScalarMesonFactorizedDecayer::_form, -1, false, false, true, false, false);

  static Parameter<ScalarMesonFactorizedDecayer,double> interfacea1Bottom
    ("a1Bottom",
     "The factorization paramter a_1 for decays of bottom baryons",
     &ScalarMesonFactorizedDecayer::_a1b, 1.1, -10.0, 10.0,
     false, false, true);

  static Parameter<ScalarMesonFactorizedDecayer,double> interfacea2Bottom
    ("a2Bottom",
     "The factorization paramter a_2 for decays of bottom baryons",
     &ScalarMesonFactorizedDecayer::_a2b, -0.24, -10.0, 10.0,
     false, false, true);

  static Parameter<ScalarMesonFactorizedDecayer,double> interfacea1Charm
    ("a1Charm",
     "The factorization paramter a_1 for decays of charm baryons",
     &ScalarMesonFactorizedDecayer::_a1c, 1.3, -10.0, 10.0,
     false, false, true);

  static Parameter<ScalarMesonFactorizedDecayer,double> interfacea2Charm
    ("a2Charm",
     "The factorization paramter a_2 for decays of charm baryons",
     &ScalarMesonFactorizedDecayer::_a2c, -0.55, -10.0, 10.0,
     false, false, true);

  static ParVector<ScalarMesonFactorizedDecayer,int> interfaceWeightLocation
    ("WeightLocation",
     "The locations of the weights for a given channel in the vector",
     &ScalarMesonFactorizedDecayer::_wgtloc,
     0, 0, 0, 0, 10000, false, false, true);

  static ParVector<ScalarMesonFactorizedDecayer,double> interfaceWeightMax
    ("MaximumWeight",
     "The maximum weight for a given channel.",
     &ScalarMesonFactorizedDecayer::_wgtmax,
     0, 0, 0, 0., 100., false, false, true);

  static ParVector<ScalarMesonFactorizedDecayer,double> interfaceWeights
    ("Weights",
     "The weights for the integration.",
     &ScalarMesonFactorizedDecayer::_weights,
     0, 0, 0, 0., 1., false, false, true);
}
 
double ScalarMesonFactorizedDecayer::me2(const int ichan,
					 const Particle & part,
					 const ParticleVector & decay,
					 MEOption meopt) const {
  if(!ME()) {
    // create the matrix element
    vector<PDT::Spin> spin;
    for(unsigned int ix=0;ix<decay.size();++ix)
      spin.push_back(decay[ix]->dataPtr()->iSpin());
    ME(new_ptr(GeneralDecayMatrixElement(PDT::Spin0,spin)));
  }
  // initialisation
  if(meopt==Initialize) {
    ScalarWaveFunction::
      calculateWaveFunctions(_rho,const_ptr_cast<tPPtr>(&part),incoming);
    _vectors.resize(decay.size());
    _tensors.resize(decay.size());
  }
  if(meopt==Terminate) {
    // set up the spin information for the decay products
    ScalarWaveFunction::constructSpinInfo(const_ptr_cast<tPPtr>(&part),
					  incoming,true);
    // get the wavefunctions of the decay products
    for(unsigned int ix=0;ix<decay.size();++ix) {
      switch(decay[ix]->dataPtr()->iSpin()) {
      case 1:
	ScalarWaveFunction::constructSpinInfo(decay[ix],outgoing,true);
	break;
      case 3:
	VectorWaveFunction::constructSpinInfo(_vectors[ix],decay[ix],outgoing,
					      true,false);
	break;
      case 5:
	TensorWaveFunction::constructSpinInfo(_tensors[ix],decay[ix],outgoing,
					      true,false);
	break;
      default:
	assert(false);
      }
    }
    return 0.;
  }
  // get the wavefunctions of the decay products
  for(unsigned int ix=0;ix<decay.size();++ix) {
    switch(decay[ix]->dataPtr()->iSpin()) {
    case 1:
      break;
    case 3:
      VectorWaveFunction::
	calculateWaveFunctions(_vectors[ix],decay[ix],outgoing,false);
      break;
    case 5:
      TensorWaveFunction::
	calculateWaveFunctions(_tensors[ix],decay[ix],outgoing,false);
      break;
    default:
      assert(false);
    }
  }
  ME()->zero();
  // find the mode
  unsigned int mode(imode()),chel,fhel;
  int id0(part.id()),id1;
  Complex ii(0.,1.);
  vector<unsigned int> ihel(decay.size());
  // loop over the different diagrams
  vector<LorentzPolarizationVectorE> form;
  Complex fp,f0,A0,A1,A2,A3,V,k;
  complex<InvEnergy2> h,bp,bm;
  // complex<Energy2> dot;
  Lorentz5Momentum q,sum; 
  Energy2 q2;
  Energy MP(part.mass()),MV,msum,mdiff,scale;
  LorentzPolarizationVectorE dotv;
  double pre;
  ParticleVector cpart;
  for(unsigned int iy=0;iy<_CKMfact[mode].size();++iy) {
    MV=decay[_formpart[mode][iy]]->mass();
    id1=decay[_formpart[mode][iy]]->id();
    int id0t,id1t;
    _form[_formmapA[mode][iy]]->particleID(_formmapB[mode][iy],id0t,id1t);
    bool cc(id0t!=id0);
    // calculate the form-factor part
    form.clear();
    q   = part.momentum()-decay[_formpart[mode][iy]]->momentum();  q.rescaleMass();
    sum = part.momentum()+decay[_formpart[mode][iy]]->momentum();sum.rescaleMass();
    q2=q.mass2();
    if(decay[_formpart[mode][iy]]->dataPtr()->iSpin()==1) {
      _form[_formmapA[mode][iy]]->ScalarScalarFormFactor(q2,_formmapB[mode][iy],
							 id0,id1,MP,MV,f0,fp);
      pre=(MP*MP-MV*MV)/q2;
      form.push_back(fp*sum+pre*(f0-fp)*q);
    }
    else if(decay[_formpart[mode][iy]]->dataPtr()->iSpin()==3) {
      msum=MP+MV;mdiff=MP-MV;
      _form[_formmapA[mode][iy]]->ScalarVectorFormFactor(q2,_formmapB[mode][iy],id0,
							 id1,MP,MV,A0,A1,A2,V);
      if(cc){V=-V;}
      A3 = Complex(0.5/MV*(msum*A1-mdiff*A2));
      // compute the hadron currents
      for(unsigned int ix=0;ix<3;++ix) {
	// dot product
	complex<Energy> dot = _vectors[_formpart[mode][iy]][ix]*part.momentum();
	// current
	form.push_back(-ii*msum*A1*_vectors[_formpart[mode][iy]][ix]
		       +ii*A2/msum*dot*sum
		       +2.*ii*MV/q2*(A3-A0)*dot*q
		       +2.*V/msum*epsilon(_vectors[_formpart[mode][iy]][ix],
					  part.momentum(),
					  decay[_formpart[mode][iy]]->momentum())); 
      }
    }
    else if(decay[_formpart[mode][iy]]->dataPtr()->iSpin()==5) {
      _form[_formmapA[mode][iy]]->ScalarTensorFormFactor(q2,_formmapB[mode][iy],
							 id0,id1,MP,MV,h,k,bp,bm);
      if(cc){h=-h;}
      // compute the hadron currents
      for(unsigned int ix=0;ix<5;++ix) {
	dotv = _tensors[_formpart[mode][iy]][ix]*part.momentum();
	complex<Energy2> dot = dotv*part.momentum();
	form.push_back(ii*h*epsilon(dotv,sum,q)-k*dotv
		       -bp*dot*sum-bm*dot*q);
      }
    }
    // find the particles for the current
    cpart.clear();
    for(unsigned int ix=0;ix<decay.size();++ix)
      {if(ix!=_formpart[mode][iy]){cpart.push_back(decay[ix]);}}
    unsigned int ix=decay.size();
    vector<unsigned int> constants(decay.size()+1),ihel(decay.size()+1);
    int itemp(1);
    do {
      --ix;
      if(ix!=_formpart[mode][iy]) {
	itemp*=decay[ix]->data().iSpin();
	constants[ix]=itemp;
      }
    }
    while(ix!=0);
    constants[decay.size()]=1;
    if(_formpart[mode][iy]!=decay.size())
      constants[_formpart[mode][iy]]=constants[_formpart[mode][iy]+1];
    // calculate the current
    vector<LorentzPolarizationVectorE>
      curr=_current[_currentmapA[mode][iy]]->
      current(_currentmapB[mode][iy],ichan,scale,cpart,meopt);
    pre = (pow(part.mass()/scale,int(cpart.size()-2)));
    // loop over the helicities to calculate the matrix element
    ihel[0]=0;
    for(chel=0;chel<curr.size();++chel) {
      for(ix=decay.size();ix>0;--ix) {
	if(ix!=_formpart[mode][iy]+1)
	  ihel[ix]=(chel%constants[ix-1])/constants[ix];
      }
      for(fhel=0;fhel<form.size();++fhel) {
	ihel[_formpart[mode][iy]+1]=fhel;
	(*ME())(ihel) += Complex(pre*_CKMfact[mode][iy]*
				 form[fhel].dot(curr[chel])*SM().fermiConstant());
      }
    }
  }
  // perform the contraction
  return 0.5*(ME()->contract(_rho)).real();
}
  
void ScalarMesonFactorizedDecayer::findModes(unsigned int imode,
					     vector<tPDVector> & particles,
					     vector<unsigned int> & loc,
					     vector<bool> & cc) {
  unsigned int ix,iy,nfound,iz;
  // resize the vectors
  loc.clear();cc.clear();
  // get the id's for the mode
  vector<int> id,idbar;
  int idtemp; bool found;
  for(ix=0;ix<particles[imode].size();++ix) {
    id.push_back(particles[imode][ix]->id());
    if(particles[imode][ix]->CC()) idbar.push_back(particles[imode][ix]->CC()->id());
    else                           idbar.push_back(id[ix]);
  }
  vector<bool> done(id.size(),false);
  // loop over the modes
  for(ix=0;ix<particles.size();++ix) {
    if(ix==imode||particles[ix].empty()) continue;
    assert(!particles[ix].empty());
    assert(particles[ix][0]);
    // the particle mode
    if(particles[ix][0]->id()==id[0]&&particles[ix].size()==id.size()) {
      nfound=1;
      for(iy=0;iy<id.size();++iy){done[iy]=false;}
      for(iy=1;iy<id.size();++iy) {
	idtemp=particles[ix][iy]->id();
	iz=1;
	found=false;
	do {
	  if(idtemp==id[iz]&&!done[iz]) {
	    done[iz]=true;
	    found=true;
	  }
	  ++iz;
	}
	while(iz<id.size()&&!found);
	if(found) ++nfound;
      }
      if(nfound==id.size()) {
	cc.push_back(false);
	loc.push_back(ix);
      }
    }
    // the charge conjugate mode
    if(particles[ix][0]->id()==idbar[0]&&particles[ix].size()==idbar.size()) {
      nfound=1;
      for(iy=0;iy<idbar.size();++iy) done[iy]=false;
      for(iy=1;iy<idbar.size();++iy) {
	idtemp=particles[ix][iy]->id();
	iz=1;
	found=false;
	do {
	  if(idtemp==idbar[iz]&&!done[iz]) {
	    done[iz]=true;
	    found=true;
	  }
	  ++iz;
	}
	while(iz<idbar.size()&&!found);
	if(found){++nfound;}
      }
      if(nfound==idbar.size()){cc.push_back(false);loc.push_back(ix);}
    }
  }
}

void ScalarMesonFactorizedDecayer::dataBaseOutput(ofstream & output,
						  bool header) const {
  unsigned int ix;
  if(header) output << "update decayers set parameters=\"";
  DecayIntegrator::dataBaseOutput(output,false);
  output << "newdef " << name() << ":a1Bottom "  << _a1b << "\n";
  output << "newdef " << name() << ":a2Bottom "  << _a2b << "\n";
  output << "newdef " << name() << ":a1Charm "   << _a1c << "\n";
  output << "newdef " << name() << ":a2Charm "   << _a2c << "\n";
  for(ix=0;ix<_current.size();++ix) {
    _current[ix]->dataBaseOutput(output,false,true);
    output << "insert " << name() << ":Currents " << ix << " " 
	   << _current[ix]->name() << " \n";
  }
  for(ix=0;ix<_form.size();++ix) {
    _form[ix]->dataBaseOutput(output,false,true);
    output << "insert " << name() << ":FormFactors " << ix << " " 
	   << _form[ix]->name() << " \n";
  }
  for(ix=0;ix<_wgtloc.size();++ix) {
    output << "insert " << name() << ":WeightLocation " << ix << " " 
	   << _wgtloc[ix] << "\n";
  }
  for(ix=0;ix<_wgtmax.size();++ix) {
    output << "insert " << name() << ":MaximumWeight "  << ix << " " 
	   << _wgtmax[ix] << "\n";
  }
  for(ix=0;ix<_weights.size();++ix) {
    output << "insert " << name() << ":Weights "        << ix << " " 
	   << _weights[ix] << "\n";
  }
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}
#line 1 "./ScalarVectorVectorDecayer.cc"
// -*- C++ -*-
//
// ScalarVectorVectorDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the ScalarVectorVectorDecayer class.
//

#include "ScalarVectorVectorDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "Herwig/Utilities/Kinematics.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"
#include "Herwig/Decay/TwoBodyDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void ScalarVectorVectorDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()) {
    for(unsigned int ix=0;ix<_incoming.size();++ix)
      if(mode(ix)) _maxweight[ix] = mode(ix)->maxWeight();
  }
}

ScalarVectorVectorDecayer::ScalarVectorVectorDecayer() 
  : _incoming(13), _outgoing1(13), _outgoing2(13), _coupling(13), 
    _maxweight(13) {
  // f_0(1370) to rho rho
  _incoming[0] = 10221; _outgoing1[0] = 113; _outgoing2[0] = 113; 
  _coupling[0] = 11.26/GeV; _maxweight[0] = 20.; 
  _incoming[1] = 10221; _outgoing1[1] = 213; _outgoing2[1] = -213; 
  _coupling[1] = 15.92/GeV; _maxweight[1] = 20.; 
  // f_0(1500) to rho rho
  _incoming[2] = 9030221; _outgoing1[2] = 113; _outgoing2[2] = 113; 
  _coupling[2] = 1.691/GeV; _maxweight[2] = 20.; 
  _incoming[3] = 9030221; _outgoing1[3] = 213; _outgoing2[3] = -213; 
  _coupling[3] = 2.391/GeV; _maxweight[3] = 20.; 
  // chi_c0 decays
  _incoming[4] = 10441; _outgoing1[4] = 443; _outgoing2[4] = 22; 
  _coupling[4] = 0.251/GeV; _maxweight[4] = 1.; 
  _incoming[5] = 10441; _outgoing1[5] = 323; _outgoing2[5] = -323; 
  _coupling[5] = 0.0088/GeV; _maxweight[5] = 1.; 
  _incoming[6] = 10441; _outgoing1[6] = 313; _outgoing2[6] = -313; 
  _coupling[6] = 0.0088/GeV; _maxweight[6] = 1.; 
  _incoming[7] = 10441; _outgoing1[7] = 333; _outgoing2[7] = 333; 
  _coupling[7] = 0.0067/GeV; _maxweight[7] = 1.; 
  _incoming[8] = 10441; _outgoing1[8] = 22; _outgoing2[8] = 22; 
  _coupling[8] = 0.0027/GeV; _maxweight[8] = 1.; 
  _incoming[12] = 10441; _outgoing1[12] = 223; _outgoing2[12] = 223; 
  _coupling[12] = 0.0093/GeV; _maxweight[12] = 1.; 
  // a'_0 -> omega rho
  _incoming[9] = 10111; _outgoing1[9] = 113; _outgoing2[9] = 223; 
  _coupling[9] = 27.09/GeV; _maxweight[9] = 20.;
  _incoming[10] = 10211; _outgoing1[10] = 213; _outgoing2[10] = 223; 
  _coupling[10] = 27.09/GeV; _maxweight[10] = 20.;
  _incoming[11] =-10211; _outgoing1[11] =-213; _outgoing2[11] = 223; 
  _coupling[11] = 27.09/GeV; _maxweight[11] = 20.; 
  // size of arrays
  _initsize = _incoming.size();
  // intermediates
  generateIntermediates(false);
}

void ScalarVectorVectorDecayer::doinit() {
  DecayIntegrator::doinit();
  // check the parameters arew consistent
  unsigned int isize(_coupling.size());
  if(isize!=_incoming.size()  || isize!=_outgoing1.size()||
     isize!=_outgoing2.size() || isize!=_maxweight.size())
    throw InitException() << "Inconsistent parameters in " 
			  << "ScalarVectorVectorDecayerDecayer" 
			  << Exception::abortnow;
  // set up the integration channels
  vector<double> wgt;
  DecayPhaseSpaceModePtr mode;
  tPDVector extpart(3);
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    extpart[0] = getParticleData(_incoming[ix]);
    extpart[1] = getParticleData(_outgoing1[ix]);
    extpart[2] = getParticleData(_outgoing2[ix]);
    if(extpart[0]&&extpart[1]&&extpart[2]) 
      mode=new_ptr(DecayPhaseSpaceMode(extpart,this));
    else
      mode=DecayPhaseSpaceModePtr();
    addMode(mode,_maxweight[ix],wgt);
  }
}

int ScalarVectorVectorDecayer::modeNumber(bool & cc,tcPDPtr parent,
					  const tPDVector & children) const {
  cc = false;
  // check that at least some modes exist
  // must be two outgoing particles
  if(_incoming.size()==0||children.size()!=2) return -1;
  // ids of the particles
  int id0(parent->id());
  int id1(children[0]->id());
  int id2(children[1]->id());
  // loop over the modes and see if this is one of them
  unsigned int ix=0;
  int imode(-1);
  do {
    if(_incoming[ix]==id0) {
      if((_outgoing1[ix]==id1&&_outgoing2[ix]==id2)||
	 (_outgoing1[ix]==id2&&_outgoing2[ix]==id1)) imode=ix;
    }
    ++ix;
  }
  while(imode<0&&ix<_incoming.size());
  return imode;
}

void ScalarVectorVectorDecayer::persistentOutput(PersistentOStream & os) const {
  os << ounit(_coupling,1/GeV) << _incoming << _outgoing1 << _outgoing2 << _maxweight;
}

void ScalarVectorVectorDecayer::persistentInput(PersistentIStream & is, int) {
  is >> iunit(_coupling,1/GeV) >> _incoming >> _outgoing1 >> _outgoing2 >> _maxweight;
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<ScalarVectorVectorDecayer,DecayIntegrator>
describeHerwigScalarVectorVectorDecayer("Herwig::ScalarVectorVectorDecayer", "HwSMDecay.so");

void ScalarVectorVectorDecayer::Init() {

  static ClassDocumentation<ScalarVectorVectorDecayer> documentation
    ("The ScalarVectorVectorDecayer class is designed for"
     " the decay of a pseduoscalar meson to two spin-1 particles.");

  static ParVector<ScalarVectorVectorDecayer,int> interfaceIncoming
    ("Incoming",
     "The PDG code for the incoming particle",
     &ScalarVectorVectorDecayer::_incoming,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<ScalarVectorVectorDecayer,int> interfaceOutcoming1
    ("FirstOutgoing",
     "The PDG code for the first outgoing particle",
     &ScalarVectorVectorDecayer::_outgoing1,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<ScalarVectorVectorDecayer,int> interfaceOutcoming2
    ("SecondOutgoing",
     "The PDG code for the second outgoing particle",
     &ScalarVectorVectorDecayer::_outgoing2,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<ScalarVectorVectorDecayer,InvEnergy> interfaceCoupling
    ("Coupling",
     "The coupling for the decay mode",
     &ScalarVectorVectorDecayer::_coupling,
     1/GeV, 0, ZERO, ZERO, 10000/GeV, false, false, true);

  static ParVector<ScalarVectorVectorDecayer,double> interfaceMaxWeight
    ("MaxWeight",
     "The maximum weight for the decay mode",
     &ScalarVectorVectorDecayer::_maxweight,
     0, 0, 0, 0., 500000., false, false, true);

}

double ScalarVectorVectorDecayer::me2(const int,
				      const Particle & inpart,
				      const ParticleVector & decay,
				      MEOption meopt) const {
  if(!ME())
    ME(new_ptr(TwoBodyDecayMatrixElement(PDT::Spin0,PDT::Spin1,PDT::Spin1)));
  bool photon[2]={false,false};
  for(unsigned int ix=0;ix<2;++ix)
    photon[ix] = decay[ix]->id()==ParticleID::gamma;
  if(meopt==Initialize) {
    ScalarWaveFunction::
      calculateWaveFunctions(_rho,const_ptr_cast<tPPtr>(&inpart),incoming);
  }
  if(meopt==Terminate) {
    // set up the spin information for the decay products
    ScalarWaveFunction::constructSpinInfo(const_ptr_cast<tPPtr>(&inpart),
					  incoming,true);
    for(unsigned int ix=0;ix<2;++ix)
      VectorWaveFunction::constructSpinInfo(_vectors[ix],decay[ix],
					    outgoing,true,photon[ix]);
    return 0.;
  }
  for(unsigned int ix=0;ix<2;++ix)
    VectorWaveFunction::
      calculateWaveFunctions(_vectors[ix],decay[ix],outgoing,photon[ix]);
  // now compute the matrix element
  InvEnergy2 fact(_coupling[imode()]/inpart.mass());
  Energy2 p1p2(decay[0]->momentum()*decay[1]->momentum());
  unsigned int ix,iy;
  for(ix=0;ix<3;++ix) {
    for(iy=0;iy<3;++iy) {
      (*ME())(0,ix,iy)=Complex(fact*(p1p2*_vectors[0][ix].dot(_vectors[1][iy])-
				     (_vectors[1][iy]*decay[0]->momentum())*
				     (_vectors[0][ix]*decay[1]->momentum())));
    }
  }
  // test of the matrix element
  //   double me = newME.contract(rhoin).real();
  //   Energy pcm=Kinematics::pstarTwoBodyDecay(inpart.mass(),decay[0]->mass(),
  // 					   decay[1]->mass());
  //   double test = sqr(_coupling[imode()]/inpart.mass())*
  //     (2.*sqr(pcm*inpart.mass())+3.*sqr(decay[0]->mass()*decay[1]->mass()));
  //   cerr << "testing matrix element for " << inpart.PDGName() << " -> " 
  //        << decay[0]->PDGName() << " " << decay[1]->PDGName() << " "
  //        << me << " " << test << " " << (me-test)/(me+test) << "\n";
  return ME()->contract(_rho).real();
}

// output the setup info for the particle database
void ScalarVectorVectorDecayer::dataBaseOutput(ofstream & output,
					       bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class  
  DecayIntegrator::dataBaseOutput(output,false);
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    if(ix<_initsize) {
      output << "newdef " << name() << ":Incoming   " << ix << " "
	     << _incoming[ix]   << "\n";
      output << "newdef " << name() << ":FirstOutgoing  " << ix << " "
	     << _outgoing1[ix]  << "\n";
      output << "newdef " << name() << ":SecondOutgoing " << ix << " "
	     << _outgoing2[ix]  << "\n";
      output << "newdef " << name() << ":Coupling   " << ix << " "
	     << _coupling[ix]*GeV   << "\n";
      output << "newdef " << name() << ":MaxWeight  " << ix << " "
	     << _maxweight[ix]  << "\n";
    }
    else {
      output << "insert " << name() << ":Incoming   " << ix << " "
	     << _incoming[ix]   << "\n";
      output << "insert " << name() << ":FirstOutgoing  " << ix << " "
	     << _outgoing1[ix]  << "\n";
      output << "insert " << name() << ":SecondOutgoing " << ix << " "
	     << _outgoing2[ix]  << "\n";
      output << "insert " << name() << ":Coupling   " << ix << " "
	     << _coupling[ix]*GeV   << "\n";
      output << "insert " << name() << ":MaxWeight  " << ix << " "
	     << _maxweight[ix]  << "\n";
    }
  }
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}

// specify the 1-2 matrix element to be used in the running width calculation
bool ScalarVectorVectorDecayer::twoBodyMEcode(const DecayMode & dm, int & itype,
					      double & coupling) const {
  int imode(-1);
  int id(dm.parent()->id());
  ParticleMSet::const_iterator pit(dm.products().begin());
  int id1((**pit).id());++pit;
  int id2((**pit).id());
  unsigned int ix(0);
  do {
    if(_incoming[ix]==id) {
      if((id1==_outgoing1[ix]&&id2==_outgoing2[ix])||
	 (id2==_outgoing1[ix]&&id1==_outgoing2[ix])) imode=ix;
    }
    ++ix;
  }
  while(imode<0&&ix<_incoming.size());
  coupling=_coupling[imode]*dm.parent()->mass();
  itype = 12;
  return id1==_outgoing1[imode]&&id2==_outgoing2[imode];
}
#line 1 "./DtoKPiPiCLEO.cc"
// -*- C++ -*-
//
// DtoKPiPiCLEO.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the DtoKPiPiCLEO class.
//

#include "DtoKPiPiCLEO.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/Interface/Switch.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "Herwig/Decay/GeneralDecayMatrixElement.h"

using namespace Herwig;

using ThePEG::Helicity::ScalarWaveFunction;
using ThePEG::Helicity::incoming;
using ThePEG::Helicity::outgoing;

DtoKPiPiCLEO::DtoKPiPiCLEO() : _c1NR(), _c1rho(), _c1Kstarm(), _c1Kstar0(), 
			       _c1K1430m(), _c1K14300(), _c1rho1700(), _c1K1680(), 
			       _c2Kstarp(), _c2rho(), _c2omega(), _c2Kstarm(),
			       _c2f980(), _c2f2(), _c2f1370(), _c2K14300(),
			       _c2K14302(), _c2K1680(), _c2NR(), _rD0(), _rres() {
  // use local values for masses and widths
  _localparameters=true;
  // masses and widths
  _momega   =  782.57*MeV; _womega   =   8.44*MeV;
  _mf2      = 1275.4 *MeV; _wf2      = 185.1 *MeV;
  _mf1370   = 1310   *MeV; _wf1370   = 272.0 *MeV;
  _mK14300  = 1412   *MeV; _wK14300  = 294   *MeV;
  _mK14302  = 1425.6 *MeV; _wK14302  =  98.5 *MeV;
  _mK1680   = 1717   *MeV; _wK1680   = 322   *MeV;
  _mrho1700 = 1700   *MeV; _wrho1700 = 240   *MeV;
  _mK8920   =  896.1 *MeV; _wK8920   =  50.5 *MeV;
  _mK892A   =  891.5 *MeV; _wK892A   =  50   *MeV;
  _mK892B   =  891.66*MeV; _wK892B   =  50.8 *MeV;
  _mrhoA    =  770   *MeV; _wrhoA    = 150.7 *MeV;
  _mrhoB    =  769.3 *MeV; _wrhoB    = 150.2 *MeV;
  _mf980    =  977.00*MeV; _wf980    =  50.  *MeV; 
  _f0opt=false; _gpi=0.09; _gK=0.02; 
  // amplitudes and phases for D0 -> K-pi+pi0
  _a1NR      = 1.75     ; _phi1NR      =  31.2;
  _a1rho     = 1.00     ; _phi1rho     =   0. ;
  _a1Kstarm  = 0.44     ; _phi1Kstarm  = 163  ;
  _a1Kstar0  = 0.39     ; _phi1Kstar0  =  -0.2;
  _a1K1430m  = 0.77*GeV2; _phi1K1430m  =  55.5;
  _a1K14300  = 0.85*GeV2; _phi1K14300  = 166  ;
  _a1rho1700 = 2.50     ; _phi1rho1700 = 171  ;
  _a1K1680   = 2.50     ; _phi1K1680   = 103  ;
  // amplitudes and phases for D0 -> K0pi+pi-
  _a2Kstarp   = 0.11     ; _phi2Kstarp = 321;
  _a2rho      = 1.00     ; _phi2rho    =   0;
  _a2omega    = 0.037    ; _phi2omega  = 114;
  _a2Kstarm   = 1.56     ; _phi2Kstarm = 150;
  _a2f980     = 0.34*GeV2; _phi2f980   = 188;
  _a2f2       = 0.7/GeV2 ; _phi2f2     = 308;
  _a2f1370    = 1.8*GeV2 ; _phi2f1370  =  85;
  _a2K14300   = 2.0*GeV2 ; _phi2K14300 =   3;
  _a2K14302   = 1.0/GeV2 ; _phi2K14302 = 335;
  _a2K1680    = 5.6      ; _phi2K1680  = 174;
  _a2NR       = 1.1      ; _phi2NR     = 340;
  // radial sizes
  _rD0  = 5.0/GeV;
  _rres = 1.5/GeV;
  // zero masses
  _mpi=ZERO;
  _mkp=ZERO;
  _mk0=ZERO;
  // intermediates
  generateIntermediates(true);
}

void DtoKPiPiCLEO::doinit() {
  DecayIntegrator::doinit();
  // complex amplitudes for K-pi+pi0
  double fact = Constants::pi/180.;
  _c1NR      = _a1NR     *Complex(cos(_phi1NR     *fact),sin(_phi1NR     *fact));
  _c1rho     = _a1rho    *Complex(cos(_phi1rho    *fact),sin(_phi1rho    *fact));
  _c1Kstarm  = _a1Kstarm *Complex(cos(_phi1Kstarm *fact),sin(_phi1Kstarm *fact));
  _c1Kstar0  = _a1Kstar0 *Complex(cos(_phi1Kstar0 *fact),sin(_phi1Kstar0 *fact));
  _c1K1430m  = _a1K1430m *Complex(cos(_phi1K1430m *fact),sin(_phi1K1430m *fact));
  _c1K14300  = _a1K14300 *Complex(cos(_phi1K14300 *fact),sin(_phi1K14300 *fact));
  _c1rho1700 = _a1rho1700*Complex(cos(_phi1rho1700*fact),sin(_phi1rho1700*fact));
  _c1K1680   = _a1K1680  *Complex(cos(_phi1K1680  *fact),sin(_phi1K1680  *fact));
  // complex amplitudes for D0 -> K0pi+pi-
  _c2Kstarp  = _a2Kstarp*Complex(cos(_phi2Kstarp*fact),sin(_phi2Kstarp*fact));
  _c2rho     = _a2rho   *Complex(cos(_phi2rho   *fact),sin(_phi2rho   *fact));
  _c2omega   = _a2omega *Complex(cos(_phi2omega *fact),sin(_phi2omega *fact));
  _c2Kstarm  = _a2Kstarm*Complex(cos(_phi2Kstarm*fact),sin(_phi2Kstarm*fact));
  _c2f980    = _a2f980  *Complex(cos(_phi2f980  *fact),sin(_phi2f980  *fact));
  _c2f2      = _a2f2    *Complex(cos(_phi2f2    *fact),sin(_phi2f2    *fact));
  _c2f1370   = _a2f1370 *Complex(cos(_phi2f1370 *fact),sin(_phi2f1370 *fact));
  _c2K14300  = _a2K14300*Complex(cos(_phi2K14300*fact),sin(_phi2K14300*fact));
  _c2K14302  = _a2K14302*Complex(cos(_phi2K14302*fact),sin(_phi2K14302*fact));
  _c2K1680   = _a2K1680 *Complex(cos(_phi2K1680 *fact),sin(_phi2K1680 *fact));
  _c2NR      = _a2NR    *Complex(cos(_phi2NR    *fact),sin(_phi2NR    *fact));
  // pion and kaon masses
  _mpi = getParticleData(ParticleID::piplus)->mass();
  _mkp = getParticleData(ParticleID::Kplus )->mass();
  _mk0 = getParticleData(ParticleID::K0    )->mass();
  // resonances for the channels 
  tPDPtr k892m   = getParticleData(ParticleID::Kstarminus);
  tPDPtr k892p   = getParticleData(ParticleID::Kstarplus);
  tPDPtr k8920   = getParticleData(ParticleID::Kstarbar0);
  tPDPtr rho770  = getParticleData(ParticleID::rhoplus);
  tPDPtr rho0    = getParticleData(ParticleID::rho0);
  tPDPtr rho1700 = getParticleData(30213);
  tPDPtr k1680m  = getParticleData(-30323);
  tPDPtr k1430m0 = getParticleData(ParticleID::Kstar_0minus);
  tPDPtr k1430m2 = getParticleData(ParticleID::Kstar_2minus);
  tPDPtr k143000 = getParticleData(ParticleID::Kstar_0bar0 );
  tPDPtr omega   = getParticleData(ParticleID::omega);
  tPDPtr f980    = getParticleData(9010221);
  tPDPtr f1370   = getParticleData(10221);
  tPDPtr f2      = getParticleData(ParticleID::f_2);
  DecayPhaseSpaceChannelPtr newchannel;
  // D0 -> K- pi+ pi0
  tPDVector extpart(4);
  extpart[0]=getParticleData(ParticleID::D0);
  extpart[1]=getParticleData(ParticleID::Kminus);
  extpart[2]=getParticleData(ParticleID::piplus);
  extpart[3]=getParticleData(ParticleID::pi0);
  DecayPhaseSpaceModePtr mode1 = new_ptr(DecayPhaseSpaceMode(extpart,this));
  int ix=0;
  if(rho770) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode1));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,1);
    newchannel->addIntermediate(rho770,0,0., 2,3);
    mode1->addChannel(newchannel);
    ++ix;
  }
  if(k892m) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode1));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,2);
    newchannel->addIntermediate(k892m,0,0., 1,3);
    mode1->addChannel(newchannel);
    ++ix;
  }
  if(k8920) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode1));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,3);
    newchannel->addIntermediate(k8920,0,0., 1,2);
    mode1->addChannel(newchannel);
    ++ix;
  }
  if(k1430m0) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode1));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,2);
    newchannel->addIntermediate(k1430m0,0,0., 1,3);
    mode1->addChannel(newchannel);
    ++ix;
  }
  if(k143000) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode1));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,3);
    newchannel->addIntermediate(k143000,0,0., 1,2);
    mode1->addChannel(newchannel);
    ++ix;
  }
  if(rho1700) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode1));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,1);
    newchannel->addIntermediate(rho1700,0,0., 2,3);
    mode1->addChannel(newchannel);
    ++ix;
  }
  if(k1680m) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode1));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,2);
    newchannel->addIntermediate(k1680m,0,0., 1,3);
    mode1->addChannel(newchannel);
    ++ix;
  }
  // add the mode
  vector<double> wtemp;
  if(ix<=int(_weights.size())) {
    vector<double>::const_iterator wit=_weights.begin();
    wtemp=vector<double>(wit,wit+ix);
  }
  else {
    wtemp=vector<double>(ix,1./double(ix));
  }
  if(_maxwgt.empty()) _maxwgt.push_back(1.);
  addMode(mode1,_maxwgt[0],wtemp);
  // D0 -> Kbar0 pi+ pi-
  extpart[0]=getParticleData(ParticleID::D0);
  extpart[1]=getParticleData(ParticleID::Kbar0);
  extpart[2]=getParticleData(ParticleID::piplus);
  extpart[3]=getParticleData(ParticleID::piminus);
  DecayPhaseSpaceModePtr mode2 = new_ptr(DecayPhaseSpaceMode(extpart,this));
  int iy=ix;
  if(k892p) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode2));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,3);
    newchannel->addIntermediate(k892p,0,0., 1,2);
    mode2->addChannel(newchannel);
    ++iy;
  }
  if(rho0) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode2));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,1);
    newchannel->addIntermediate(rho0,0,0., 2,3);
    mode2->addChannel(newchannel);
    ++iy;
  }
  if(omega) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode2));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,1);
    newchannel->addIntermediate(omega,0,0., 2,3);
    mode2->addChannel(newchannel);
    ++iy;
  }
  if(k892m) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode2));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,2);
    newchannel->addIntermediate(k892m,0,0., 1,3);
    mode2->addChannel(newchannel);
    ++iy;
  }
  if(f980) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode2));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,1);
    newchannel->addIntermediate(f980,0,0., 2,3);
    mode2->addChannel(newchannel);
    ++iy;
  }
  if(f2) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode2));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,1);
    newchannel->addIntermediate(f2,0,0., 2,3);
    mode2->addChannel(newchannel);
    ++iy;
  }
  if(f1370) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode2));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,1);
    newchannel->addIntermediate(f1370,0,0., 2,3);
    mode2->addChannel(newchannel);
    ++iy;
  }
  if(k1430m0) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode2));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,2);
    newchannel->addIntermediate(k1430m0,0,0., 1,3);
    mode2->addChannel(newchannel);
    ++iy;
  }
  if(k1430m2) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode2));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,2);
    newchannel->addIntermediate(k1430m2,0,0., 1,3);
    mode2->addChannel(newchannel);
    ++iy;
  }
  if(k1680m) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode2));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,2);
    newchannel->addIntermediate(k1680m,0,0., 1,3);
    mode2->addChannel(newchannel);
    ++iy;
  }
  // add the mode
  if(iy<=int(_weights.size())) {
    vector<double>::const_iterator wit=_weights.begin();
    wtemp=vector<double>(wit+ix,wit+iy);
  }
  else {
    wtemp=vector<double>(iy-ix,1./double(iy-ix));
  }
  if(_maxwgt.size()<2) _maxwgt.push_back(1.);
  addMode(mode2,_maxwgt[1],wtemp);
  if(!_localparameters) {
    _momega   = omega  ->mass();
    _mf980    = f980   ->mass();
    _mf2      = f2     ->mass();
    _mf1370   = f1370  ->mass();
    _mK14300  = k1430m0->mass();
    _mK14302  = k1430m2->mass();
    _mK1680   = k1680m ->mass();
    _mrho1700 = rho1700->mass();
    _mK8920   = k8920  ->mass();
    _mK892A   = k892p  ->mass();
    _mK892B   = k892p  ->mass();
    _mrhoA    = rho770 ->mass();
    _mrhoB    = rho0   ->mass();
    _womega   = omega  ->width();
    _wf980    = f980   ->width();
    _wf2      = f2     ->width();
    _wf1370   = f1370  ->width();
    _wK14300  = k1430m0->width();
    _wK14302  = k1430m2->width();
    _wK1680   = k1680m ->width();
    _wrho1700 = rho1700->width();
    _wK8920   = k8920  ->width();
    _wK892A   = k892p  ->width();
    _wK892B   = k892p  ->width();
    _wrhoA    = rho770 ->width();
    _wrhoB    = rho0   ->width();
  }
  else {
    mode1->resetIntermediate(rho770 ,_mrhoA   ,_wrhoA   );
    mode1->resetIntermediate(k892m  ,_mK892A  ,_wK892A  );
    mode1->resetIntermediate(k8920  ,_mK8920  ,_wK8920  );
    mode1->resetIntermediate(k1430m0,_mK14300 ,_wK14300 );
    mode1->resetIntermediate(k143000,_mK14300 ,_wK14300 );
    mode1->resetIntermediate(rho1700,_mrho1700,_wrho1700);
    mode1->resetIntermediate(k1680m ,_mK1680  ,_wK1680  );
    mode2->resetIntermediate(k892p  ,_mK892B  ,_wK892B  );
    mode2->resetIntermediate(rho0   ,_mrhoB   ,_wrhoB   );
    mode2->resetIntermediate(omega  ,_momega  ,_womega  );
    mode2->resetIntermediate(k892m  ,_mK892B  ,_wK892B  );
    mode2->resetIntermediate(f980   ,_mf980   ,_wf980   );
    mode2->resetIntermediate(f2     ,_mf2     ,_wf2     );
    mode2->resetIntermediate(f1370  ,_mf1370  ,_wf1370  );
    mode2->resetIntermediate(k1430m0,_mK14300 ,_wK14300 );
    mode2->resetIntermediate(k1430m2,_mK14302 ,_wK14302 );
    mode2->resetIntermediate(k1680m ,_mK1680  ,_wK1680  );
  }
}

void DtoKPiPiCLEO::persistentOutput(PersistentOStream & os) const {
  os << ounit(_momega,GeV) << ounit(_womega,GeV) << ounit(_mf980,GeV) 
     << _gpi << _gK  << ounit(_mf2,GeV) << ounit(_wf2,GeV) << ounit(_mf1370,GeV) 
     << ounit(_wf1370,GeV) << ounit(_mK14300,GeV) << ounit(_wK14300,GeV) 
     << ounit(_mK14302,GeV) << ounit(_wK14302,GeV) << ounit(_mK1680,GeV) 
     << ounit(_wK1680,GeV) << ounit(_mrho1700,GeV) << ounit(_wrho1700,GeV) 
     << ounit(_mK8920,GeV) << ounit(_wK8920,GeV) << ounit(_mK892A,GeV) 
     << ounit(_wK892A,GeV) << ounit(_mK892B,GeV) << ounit(_wK892B,GeV) 
     << ounit(_mrhoA,GeV) << ounit(_wrhoA,GeV) << ounit(_mrhoB,GeV) 
     << ounit(_wrhoB,GeV) << _a1NR << _phi1NR << _a1rho << _phi1rho 
     << _a1Kstarm << _phi1Kstarm << _a1Kstar0 << _phi1Kstar0 << ounit(_a1K1430m,GeV2) 
     << _phi1K1430m << ounit(_a1K14300,GeV2) << _phi1K14300 << _a1rho1700 
     << _phi1rho1700 << _a1K1680 << _phi1K1680 << _c1NR << _c1rho << _c1Kstarm 
     << _c1Kstar0 << ounit(_c1K1430m,GeV2) << ounit(_c1K14300,GeV2) 
     << _c1rho1700 << _c1K1680 << _a2Kstarp << _phi2Kstarp << _a2rho 
     << _phi2rho << _a2omega << _phi2omega << _a2Kstarm << _phi2Kstarm 
     << ounit(_a2f980,GeV2) << _phi2f980 << ounit(_a2f2,1./GeV2) << _phi2f2 
     << ounit(_a2f1370,GeV2) << _phi2f1370  << ounit(_a2K14300,GeV2) << _phi2K14300 
     << ounit(_a2K14302,1./GeV2) << _phi2K14302 << _a2K1680 << _phi2K1680 << _a2NR 
     << _phi2NR << _c2Kstarp << _c2rho << _c2omega << _c2Kstarm << ounit(_c2f980,GeV2)
     << ounit(_c2f2,1./GeV2) << ounit(_c2f1370,GeV2) << ounit(_c2K14300,GeV2) 
     << ounit(_c2K14302,1./GeV2) << _c2K1680 << _c2NR << _maxwgt << _weights 
     << ounit(_rD0,1./GeV) << ounit(_rres,1./GeV) << ounit(_mpi,GeV) 
     << ounit(_mkp,GeV) << ounit(_mk0,GeV) << ounit(_wf980,GeV) << _f0opt 
     << _localparameters;
}

void DtoKPiPiCLEO::persistentInput(PersistentIStream & is, int) {
  is >> iunit(_momega,GeV) >> iunit(_womega,GeV) >> iunit(_mf980,GeV) 
     >> _gpi >> _gK  >> iunit(_mf2,GeV) >> iunit(_wf2,GeV) >> iunit(_mf1370,GeV) 
     >> iunit(_wf1370,GeV) >> iunit(_mK14300,GeV) >> iunit(_wK14300,GeV) 
     >> iunit(_mK14302,GeV) >> iunit(_wK14302,GeV) >> iunit(_mK1680,GeV) 
     >> iunit(_wK1680,GeV) >> iunit(_mrho1700,GeV) >> iunit(_wrho1700,GeV) 
     >> iunit(_mK8920,GeV) >> iunit(_wK8920,GeV) >> iunit(_mK892A,GeV) 
     >> iunit(_wK892A,GeV) >> iunit(_mK892B,GeV) >> iunit(_wK892B,GeV) 
     >> iunit(_mrhoA,GeV) >> iunit(_wrhoA,GeV) >> iunit(_mrhoB,GeV) 
     >> iunit(_wrhoB,GeV) >> _a1NR >> _phi1NR >> _a1rho >> _phi1rho 
     >> _a1Kstarm >> _phi1Kstarm >> _a1Kstar0 >> _phi1Kstar0 >> iunit(_a1K1430m,GeV2) 
     >> _phi1K1430m >> iunit(_a1K14300,GeV2) >> _phi1K14300 >> _a1rho1700 
     >> _phi1rho1700 >> _a1K1680 >> _phi1K1680 >> _c1NR >> _c1rho >> _c1Kstarm 
     >> _c1Kstar0 >> iunit(_c1K1430m,GeV2) >> iunit(_c1K14300,GeV2) 
     >> _c1rho1700 >> _c1K1680 >> _a2Kstarp >> _phi2Kstarp >> _a2rho 
     >> _phi2rho >> _a2omega >> _phi2omega >> _a2Kstarm >> _phi2Kstarm 
     >> iunit(_a2f980,GeV2) >> _phi2f980 >> iunit(_a2f2,1./GeV2) >> _phi2f2 
     >> iunit(_a2f1370,GeV2) >> _phi2f1370  >> iunit(_a2K14300,GeV2) >> _phi2K14300 
     >> iunit(_a2K14302,1./GeV2) >> _phi2K14302 >> _a2K1680 >> _phi2K1680 >> _a2NR 
     >> _phi2NR >> _c2Kstarp >> _c2rho >> _c2omega >> _c2Kstarm >> iunit(_c2f980,GeV2)
     >> iunit(_c2f2,1./GeV2) >> iunit(_c2f1370,GeV2) >> iunit(_c2K14300,GeV2) 
     >> iunit(_c2K14302,1./GeV2) >> _c2K1680 >> _c2NR >> _maxwgt >> _weights 
     >> iunit(_rD0,1./GeV) >> iunit(_rres,1./GeV) >> iunit(_mpi,GeV) 
     >> iunit(_mkp,GeV) >> iunit(_mk0,GeV) >> iunit(_wf980,GeV) >> _f0opt 
     >> _localparameters;
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<DtoKPiPiCLEO,DecayIntegrator>
describeHerwigDtoKPiPiCLEO("Herwig::DtoKPiPiCLEO", "HwSMDecay.so");

void DtoKPiPiCLEO::Init() {

  static ClassDocumentation<DtoKPiPiCLEO> documentation
    ("The DtoKPiPiCLEO class implements the models of CLEO for"
     " D0 -> Kbar0 pi+pi- and D0 -> K- pi+ pi0",
     "The CLEO fits of \\cite{Muramatsu:2002jp} and \\cite{Kopp:2000gv} were"
     " used for the decays $D^0\\to\\bar{K}^0\\pi^+\\pi^-$ and"
     " $D^0\\to K^-\\pi^+\\pi^0$.",
     "\\bibitem{Muramatsu:2002jp} H.~Muramatsu {\\it et al.}  "
     "[CLEO Collaboration],Phys.\\ Rev.\\ Lett.\\  {\\bf 89} (2002) 251802"
     "[Erratum-ibid.\\  {\\bf 90} (2003) 059901] [arXiv:hep-ex/0207067].\n"
     "\\bibitem{Kopp:2000gv} S.~Kopp {\\it et al.}  [CLEO Collaboration], "
     "Phys.\\ Rev.\\  D {\\bf 63} (2001) 092001 [arXiv:hep-ex/0011065]."
     );

  static Switch<DtoKPiPiCLEO,bool> interfaceLocalParameters
    ("LocalParameters",
     "Whether to use local values for the masses and widths or"
     " those from the ParticleData objects",
     &DtoKPiPiCLEO::_localparameters, true, false, false);
  static SwitchOption interfaceLocalParametersLocal
    (interfaceLocalParameters,
     "Local",
     "Use local values",
     true);
  static SwitchOption interfaceLocalParametersParticleData
    (interfaceLocalParameters,
     "ParticleData",
     "Use the values from the ParticleData objects",
     false);

  static Parameter<DtoKPiPiCLEO,Energy> interfaceOmegaMass
    ("OmegaMass",
     "The mass of the omega meson",
     &DtoKPiPiCLEO::_momega, MeV, 782.57*MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfacef980Mass
    ("f980Mass",
     "The mass of the f_0(980) meson",
     &DtoKPiPiCLEO::_mf980, MeV, 977.00*MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfacef_2Mass
    ("f_2Mass",
     "The mass of the f_2 meson",
     &DtoKPiPiCLEO::_mf2, MeV, 1275.4 *MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfacef1370Mass
    ("f1370Mass",
     "The mass of the f_0(1370) meson",
     &DtoKPiPiCLEO::_mf1370, MeV, 1310   *MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfaceK_01430Mass
    ("K_01430Mass",
     "The mass of the K_0(1430) meson",
     &DtoKPiPiCLEO::_mK14300, MeV, 1412   *MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfaceK_21430Mass
    ("K_21430Mass",
     "The mass of the K_2(1430) meson",
     &DtoKPiPiCLEO::_mK14302, MeV, 1425.6 *MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfaceKstar1680Mass
    ("Kstar1680Mass",
     "The mass of the K*(1680) meson",
     &DtoKPiPiCLEO::_mK1680, MeV, 1717   *MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfacerho1700Mass
    ("rho1700Mass",
     "The mass of the rho(1700) meson",
     &DtoKPiPiCLEO::_mrho1700, MeV, 1700   *MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfaceKstar0892Mass
    ("Kstar0892Mass",
     "The mass of the K*0(892) meson",
     &DtoKPiPiCLEO::_mK8920, MeV, 896.1 *MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfaceKstarPlus892AMass
    ("KstarPlus892AMass",
     "The mass of the K*+(892) meson in D0 -> K-pi+pi0",
     &DtoKPiPiCLEO::_mK892A, MeV, 891.5 *MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfaceKstarPlus892BMass
    ("KstarPlus892BMass",
     "The mass of the K*+(892) meson in D0 -> K0pi+pi-",
     &DtoKPiPiCLEO::_mK892B, MeV, 891.66*MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfacerhoPlusMass
    ("RhoPlusMass",
     "The mass of the rho+ meson in D0 -> K-pi+pi0",
     &DtoKPiPiCLEO::_mrhoA, MeV, 770   *MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfacerho0Mass
    ("Rho0Mass",
     "The mass of the rho+ meson in D0 -> K0pi+pi-",
     &DtoKPiPiCLEO::_mrhoB, MeV, 769.3 *MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfaceOmegaWidth
    ("OmegaWidth",
     "The width of the omega meson",
     &DtoKPiPiCLEO::_womega, MeV, 8.44*MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfacef980Width
    ("f980Width",
     "The width of the f_0(980) meson",
     &DtoKPiPiCLEO::_wf980, MeV,  50.  *MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfacef_2Width
    ("f_2Width",
     "The width of the f_2 meson",
     &DtoKPiPiCLEO::_wf2, MeV, 185.1 *MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfacef1370Width
    ("f1370Width",
     "The width of the f_0(1370) meson",
     &DtoKPiPiCLEO::_wf1370, MeV, 272.0 *MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfaceK_01430Width
    ("K_01430Width",
     "The width of the K_0(1430) meson",
     &DtoKPiPiCLEO::_wK14300, MeV, 294   *MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfaceK_21430Width
    ("K_21430Width",
     "The width of the K_2(1430) meson",
     &DtoKPiPiCLEO::_wK14302, MeV,  98.5 *MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfaceKstar1680Width
    ("Kstar1680Width",
     "The width of the K*(1680) meson",
     &DtoKPiPiCLEO::_wK1680, MeV, 322   *MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfacerho1700Width
    ("rho1700Width",
     "The width of the rho(1700) meson",
     &DtoKPiPiCLEO::_wrho1700, MeV, 240   *MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfaceKstar0892Width
    ("Kstar0892Width",
     "The width of the K*0(892) meson",
     &DtoKPiPiCLEO::_wK8920, MeV, 50.5 *MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfaceKstarPlus892AWidth
    ("KstarPlus892AWidth",
     "The width of the K*+(892) meson in D0 -> K-pi+pi0",
     &DtoKPiPiCLEO::_wK892A, MeV,  50   *MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfaceKstarPlus892BWidth
    ("KstarPlus892BWidth",
     "The width of the K*+(892) meson in D0 -> K0pi+pi-",
     &DtoKPiPiCLEO::_wK892B, MeV, 50.8 *MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfacerhoPlusWidth
    ("RhoPlusWidth",
     "The width of the rho+ meson in D0 -> K-pi+pi0",
     &DtoKPiPiCLEO::_wrhoA, MeV, 150.7 *MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy> interfacerho0Width
    ("Rho0Width",
     "The width of the rho+ meson in D0 -> K0pi+pi-",
     &DtoKPiPiCLEO::_wrhoB, MeV, 150.2 *MeV, ZERO, 10000.0*MeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfacegPi
    ("gPi",
     "The g_pi coupling for the f_0(980) width",
     &DtoKPiPiCLEO::_gpi, 0.09, 0.0, 1.,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfacegK
    ("gK",
     "The g_K coupling for the f_0(980) width",
     &DtoKPiPiCLEO::_gK, 0.02, 0.0, 1.,
     false, false, Interface::limited);

  static Switch<DtoKPiPiCLEO,bool> interfacef0Option
    ("f0Option",
     "Option for the treatment of the f_0(980) width",
     &DtoKPiPiCLEO::_f0opt, true, false, false);
  static SwitchOption interfacef0OptionCoupled
    (interfacef0Option,
     "Coupled",
     "Use the coupling pion and kaon channels",
     true);
  static SwitchOption interfacef0OptionSWave
    (interfacef0Option,
     "SWave",
     "Use a simple s-wave running width",
     false);

  static Parameter<DtoKPiPiCLEO,double> interfaceChargedNonResonantAmplitude
    ("ChargedNonResonantAmplitude",
     "Amplitude for the non-resonant component for D0 -> K- pi+ pi0",
     &DtoKPiPiCLEO::_a1NR, 1.75, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceChargedNonResonantPhase
    ("ChargedNonResonantPhase",
     "Phase for the non-resonant component for D0 -> K- pi+ pi0",
     &DtoKPiPiCLEO::_phi1NR, 31.2, -180.0, 180.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceChargedRhoAmplitude
    ("ChargedRhoAmplitude",
     "Amplitude for the rho+ component for D0 -> K- pi+ pi0",
     &DtoKPiPiCLEO::_a1rho, 1.0, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceChargedRhoPhase
    ("ChargedRhoPhase",
     "Phase for the rho+ component for D0 -> K- pi+ pi0",
     &DtoKPiPiCLEO::_phi1rho, 0.0, -180.0, 180.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceChargedKStarMinusAmplitude
    ("ChargedKStarMinusAmplitude",
     "Amplitude for the K*(892)- component for D0 -> K- pi+ pi0",
     &DtoKPiPiCLEO::_a1Kstarm, 0.44, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceChargedKStarMinusPhase
    ("ChargedKStarMinusPhase",
     "Phase for the K*(892)- component for D0 -> K- pi+ pi0",
     &DtoKPiPiCLEO::_phi1Kstarm, 163, -180.0, 180.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceChargedKStar0Amplitude
    ("ChargedKStar0Amplitude",
     "Amplitude for the K*(892)0 component for D0 -> K- pi+ pi0",
     &DtoKPiPiCLEO::_a1Kstar0, 0.39, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceChargedKStar0Phase
    ("ChargedKStar0Phase",
     "Phase for the K*(892)0 component for D0 -> K- pi+ pi0",
     &DtoKPiPiCLEO::_phi1Kstar0, -0.2, -180.0, 180.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy2> interfaceChargedK_0MinusAmplitude
    ("ChargedK_0MinusAmplitude",
     "Amplitude for the K_0(1430)- component for D0 -> K- pi+ pi0",
     &DtoKPiPiCLEO::_a1K1430m, GeV2, 0.77*GeV2, ZERO, 10.0*GeV2,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceChargedK_0MinusPhase
    ("ChargedK_0MinusPhase",
     "Phase for the K_0(1430)- component for D0 -> K- pi+ pi0",
     &DtoKPiPiCLEO::_phi1K1430m, 55.5, -180.0, 180.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy2> interfaceChargedK_00Amplitude
    ("ChargedK_00Amplitude",
     "Amplitude for the K_0(1430)0 component for D0 -> K- pi+ pi0",
     &DtoKPiPiCLEO::_a1K14300, GeV2, 0.85*GeV2, ZERO, 10.0*GeV2,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceChargedK_00Phase
    ("ChargedK_00Phase",
     "Phase for the K_0(1430)0 component for D0 -> K- pi+ pi0",
     &DtoKPiPiCLEO::_phi1K14300, 166, -180.0, 180.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceChargedRho1700Amplitude
    ("ChargedRho1700Amplitude",
     "Amplitude for the rho1700+ component for D0 -> K- pi+ pi0",
     &DtoKPiPiCLEO::_a1rho1700, 2.5, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceChargedRho1700Phase
    ("ChargedRho1700Phase",
     "Phase for the rho1700+ component for D0 -> K- pi+ pi0",
     &DtoKPiPiCLEO::_phi1rho1700, 171., -180.0, 180.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceChargedK1680MinusAmplitude
    ("ChargedK1680MinusAmplitude",
     "Amplitude for the K*(1680)- component for D0 -> K- pi+ pi0",
     &DtoKPiPiCLEO::_a1K1680, 2.5, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceChargedK1680MinusPhase
    ("ChargedK1680MinusPhase",
     "Phase for the K*(1680)- component for D0 -> K- pi+ pi0",
     &DtoKPiPiCLEO::_phi1K1680, 103, -180.0, 180.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceNeutralKStarPlusAmplitude
    ("NeutralKStarPlusAmplitude",
     "Amplitude for the K*(892)+ component for D0 -> Kbar0 pi+ pi-",
     &DtoKPiPiCLEO::_a2Kstarp, 0.11, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceNeutralKStarPlusPhase
    ("NeutralKStarPlusPhase",
     "Phase for the K*(892)+ component for D0 -> Kbar0 pi+ pi-",
     &DtoKPiPiCLEO::_phi2Kstarp, 321., 0.0, 360.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceNeutralRhoAmplitude
    ("NeutralRhoAmplitude",
     "Amplitude for the rho0 component for D0 -> Kbar0 pi+ pi-",
     &DtoKPiPiCLEO::_a2rho, 1.0, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceNeutralRhoPhase
    ("NeutralRhoPhase",
     "Phase for the rho0 component for D0 -> Kbar0 pi+ pi-",
     &DtoKPiPiCLEO::_phi2rho, 0.0, 0.0, 360.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceNeutralOmegaAmplitude
    ("NeutralOmegaAmplitude",
     "Amplitude for the omega component for D0 -> Kbar0 pi+ pi-",
     &DtoKPiPiCLEO::_a2omega, 0.037, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceNeutralOmegaPhase
    ("NeutralOmegaPhase",
     "Phase for the omega component for D0 -> Kbar0 pi+ pi-",
     &DtoKPiPiCLEO::_phi2omega, 114., 0.0, 360.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceNeutralKStarMinusAmplitude
    ("NeutralKStarMinusAmplitude",
     "Amplitude for the K*(892)- component for D0 -> Kbar0 pi+ pi-",
     &DtoKPiPiCLEO::_a2Kstarm, 1.56, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceNeutralKStarMinusPhase
    ("NeutralKStarMinusPhase",
     "Phase for the K*(892)- component for D0 -> Kbar0 pi+ pi-",
     &DtoKPiPiCLEO::_phi2Kstarm, 150., 0.0, 360.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy2> interfaceNeutralf980Amplitude
    ("Neutralf980Amplitude",
     "Amplitude for the f_0(980) component for D0 -> Kbar0 pi+ pi-",
     &DtoKPiPiCLEO::_a2f980, GeV2, 0.34*GeV2, ZERO, 10.0*GeV2,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceNeutralf980Phase
    ("Neutralf980Phase",
     "Phase for the f_0(980) component for D0 -> Kbar0 pi+ pi-",
     &DtoKPiPiCLEO::_phi2f980, 188., 0.0, 360.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,InvEnergy2> interfaceNeutralf2Amplitude
    ("Neutralf2Amplitude",
     "Amplitude for the f_2 component for D0 -> Kbar0 pi+ pi-",
     &DtoKPiPiCLEO::_a2f2, 1./GeV2, 0.7/GeV2, ZERO, 10.0/GeV2,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceNeutralf2Phase
    ("Neutralf2Phase",
     "Phase for the f_0(2) component for D0 -> Kbar0 pi+ pi-",
     &DtoKPiPiCLEO::_phi2f2, 308., 0.0, 360.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy2> interfaceNeutralf1370Amplitude
    ("Neutralf1370Amplitude",
     "Amplitude for the f_0(1370) component for D0 -> Kbar0 pi+ pi-",
     &DtoKPiPiCLEO::_a2f1370, GeV2, 1.8*GeV2, ZERO, 10.0*GeV2,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceNeutralf1370Phase
    ("Neutralf1370Phase",
     "Phase for the f_0(1370) component for D0 -> Kbar0 pi+ pi-",
     &DtoKPiPiCLEO::_phi2f1370, 85., 0.0, 360.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,Energy2> interfaceNeutralKK_0MinusAmplitude
    ("NeutralKK_0MinusAmplitude",
     "Amplitude for the K_0(1430)- component for D0 -> Kbar0 pi+ pi-",
     &DtoKPiPiCLEO::_a2K14300, GeV2, 2.0*GeV2, ZERO, 10.0*GeV2,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceNeutralKK_0MinusPhase
    ("NeutralKK_0MinusPhase",
     "Phase for the K_0(1430)- component for D0 -> Kbar0 pi+ pi-",
     &DtoKPiPiCLEO::_phi2K14300, 3, 0.0, 360.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,InvEnergy2> interfaceNeutralKK_2MinusAmplitude
    ("NeutralKK_2MinusAmplitude",
     "Amplitude for the K_2(1430)- component for D0 -> Kbar0 pi+ pi-",
     &DtoKPiPiCLEO::_a2K14302, 1./GeV2, 1.0/GeV2, ZERO, 10.0/GeV2,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceNeutralKK_2MinusPhase
    ("NeutralKK_2MinusPhase",
     "Phase for the K_2(1430)- component for D0 -> Kbar0 pi+ pi-",
     &DtoKPiPiCLEO::_phi2K14302, 335, 0.0, 360.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceNeutralK1680MinusAmplitude
    ("NeutralK1680MinusAmplitude",
     "Amplitude for the K*(892)- component for D0 -> Kbar0 pi+ pi-",
     &DtoKPiPiCLEO::_a2K1680, 5.6, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceNeutralK1680MinusPhase
    ("NeutralK1680MinusPhase",
     "Phase for the K*(892)- component for D0 -> Kbar0 pi+ pi-",
     &DtoKPiPiCLEO::_phi2K1680, 174, 0.0, 360.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceNeutralNonResonantAmplitude
    ("NeutralNonResonantAmplitude",
     "Amplitude for the non-resonant component for D0 -> Kbar0 pi+ pi-",
     &DtoKPiPiCLEO::_a2NR, 1.1, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,double> interfaceNeutralNonResonantPhase
    ("NeutralNonResonantPhase",
     "Phase for the non-resonant component for D0 -> Kbar0 pi+ pi-",
     &DtoKPiPiCLEO::_phi2NR, 340, 0.0, 360.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,InvEnergy> interfaceDRadius
    ("DRadius",
     "The radius parameter for the Blatt-Weisskopf form-factor for the D",
     &DtoKPiPiCLEO::_rD0, 1./GeV, 5./GeV, ZERO, 10./GeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiCLEO,InvEnergy> interfaceResonanceRadius
    ("ResonanceRadius",
     "The radius parameter for the Blatt-Weisskopf form-factor for the"
     "intermediate resonances",
     &DtoKPiPiCLEO::_rres, 1./GeV, 1.5/GeV, ZERO, 10./GeV,
     false, false, Interface::limited);

  static ParVector<DtoKPiPiCLEO,double> interfaceMaximumWeights
    ("MaximumWeights",
     "The maximum weights for the unweighting of the decays",
     &DtoKPiPiCLEO::_maxwgt, -1, 1.0, 0.0, 1.0e11,
     false, false, Interface::limited);

  static ParVector<DtoKPiPiCLEO,double> interfaceWeights
    ("Weights",
     "The weights for the different channels for the phase-space integration",
     &DtoKPiPiCLEO::_weights, -1, 1.0, 0.0, 1.0,
     false, false, Interface::limited);
}

int DtoKPiPiCLEO::modeNumber(bool & cc,tcPDPtr parent,
			     const tPDVector & children) const {
  int id0(parent->id());
  // incoming particle must be D0
  if(abs(id0)!=ParticleID::D0) return -1;
  cc = id0==ParticleID::Dbar0;
  // must be three decay products
  if(children.size()!=3) return -1;
  tPDVector::const_iterator pit = children.begin();
  unsigned int npip(0),npim(0),nkm(0),nk0(0),npi0(0);
  for( ;pit!=children.end();++pit) {
    id0=(**pit).id();
    if(id0          ==ParticleID::piplus)  ++npip;
    else if(id0     ==ParticleID::pi0)     ++npi0;
    else if(id0     ==ParticleID::piminus) ++npim;
    else if(abs(id0)==ParticleID::K0)      ++nk0;
    else if(id0     ==ParticleID::K_L0)    ++nk0;
    else if(id0     ==ParticleID::K_S0)    ++nk0;
    else if(abs(id0)==ParticleID::Kplus)   ++nkm;
  }
  if(npim==1&&npip==1&&nk0==1) return  1;
  else if(nkm==1&&(npip+npim)==1&&npi0==1) return 0;
  else                        return -1;
}

double DtoKPiPiCLEO::me2(const int ichan,
			 const Particle & inpart,
			 const ParticleVector & decay,
			 MEOption meopt) const {
  if(!ME())
    ME(new_ptr(GeneralDecayMatrixElement(PDT::Spin0,PDT::Spin0,PDT::Spin0,PDT::Spin0)));
  useMe();
  if(meopt==Initialize) {
    ScalarWaveFunction::
      calculateWaveFunctions(_rho,const_ptr_cast<tPPtr>(&inpart),incoming);
  }
  if(meopt==Terminate) {
    // set up the spin information for the decay products
    ScalarWaveFunction::constructSpinInfo(const_ptr_cast<tPPtr>(&inpart),
					  incoming,true);
    for(unsigned int ix=0;ix<3;++ix)
    ScalarWaveFunction::constructSpinInfo(decay[ix],outgoing,true);
    return 0.;
  }
  // compute the invariant masses needed to calulate the amplitudes
  Energy mD  = inpart.mass();
  Energy mA  = decay[0]->mass();
  Energy mB  = decay[1]->mass();
  Energy mC  = decay[2]->mass();
  Energy mAB = (decay[0]->momentum()+decay[1]->momentum()).m();
  Energy mAC = (decay[0]->momentum()+decay[2]->momentum()).m();
  Energy mBC = (decay[1]->momentum()+decay[2]->momentum()).m();
  // compute the amplitudes for the resonaces present in both models
  Complex amp(0);
  // calculate the matrix element
  if(imode()==0) {
    if(ichan<0) {
      amp = _c1NR
	+_c1rho        *amplitude(1,false,mD,mB,mC,mA,mBC,mAB,mAC,_mrhoA   ,_wrhoA   )
	+_c1rho1700    *amplitude(1,false,mD,mB,mC,mA,mBC,mAB,mAC,_mrho1700,_wrho1700)
	+_c1Kstarm     *amplitude(1,false,mD,mA,mC,mB,mAC,mAB,mBC,_mK892A  ,_wK892A  )
	+_c1K1430m/GeV2*amplitude(0,false,mD,mA,mC,mB,mAC,mAB,mBC,_mK14300 ,_wK14300 )
	+_c1K1680      *amplitude(1,false,mD,mA,mC,mB,mAC,mAB,mBC,_mK1680  ,_wK1680)
	+_c1Kstar0     *amplitude(1,false,mD,mA,mB,mC,mAB,mAC,mBC,_mK8920  ,_wK8920  )
	+_c1K14300/GeV2*amplitude(0,false,mD,mA,mB,mC,mAB,mAC,mBC,_mK14300 ,_wK14300 );
    }
    else if(ichan==0) {
      amp= _c1rho       *amplitude(1,false,mD,mB,mC,mA,mBC,mAB,mAC,_mrhoA   ,_wrhoA   );
    }
    else if(ichan==1) {
      amp=_c1Kstarm     *amplitude(1,false,mD,mA,mC,mB,mAC,mAB,mBC,_mK892A  ,_wK892A  ) ;
    }
    else if(ichan==2) {
      amp=_c1Kstar0     *amplitude(1,false,mD,mA,mB,mC,mAB,mAC,mBC,_mK8920  ,_wK8920  ) ;
    }
    else if(ichan==3) {
      amp=_c1K1430m/GeV2*amplitude(0,false,mD,mA,mC,mB,mAC,mAB,mBC,_mK14300 ,_wK14300 );
    }
    else if(ichan==4) {
      amp=_c1K14300/GeV2*amplitude(0,false,mD,mA,mB,mC,mAB,mAC,mBC,_mK14300 ,_wK14300 );
    }
    else if(ichan==5) {
      amp=_c1rho1700    *amplitude(1,false,mD,mB,mC,mA,mBC,mAB,mAC,_mrho1700,_wrho1700);
    }
    else if(ichan==6) {
      amp=_c1K1680      *amplitude(1,false,mD,mA,mC,mB,mAC,mAB,mBC,_mK1680  ,_wK1680);
    }
  }
  else {
    if(ichan<0) {
      amp = _c2NR
	+_c2Kstarm     *amplitude(1,false ,mD,mA,mC,mB,mAC,mAB,mBC,_mK892B ,_wK892B )
	+Complex(_c2K14300/GeV2)*amplitude(0,false ,mD,mA,mC,mB,mAC,mAB,mBC,_mK14300,_wK14300)
	+Complex(_c2K14302*GeV2)*amplitude(2,false ,mD,mA,mC,mB,mAC,mAB,mBC,_mK14302,_wK14302)
	+_c2K1680      *amplitude(1,false ,mD,mA,mC,mB,mAC,mAB,mBC,_mK1680 ,_wK1680 )
	+_c2Kstarp     *amplitude(1,false ,mD,mA,mB,mC,mAB,mAC,mBC,_mK892B ,_wK892B )
	+_c2rho        *amplitude(1,false ,mD,mB,mC,mA,mBC,mAB,mAC,_mrhoB  ,_wrhoB  )
	+_c2omega      *amplitude(1,false ,mD,mB,mC,mA,mBC,mAB,mAC,_momega ,_womega )
	+Complex(_c2f980/GeV2  )*amplitude(0,_f0opt,mD,mB,mC,mA,mBC,mAB,mAC,_mf980  ,_wf980  )
	+Complex(_c2f1370/GeV2 )*amplitude(0,false ,mD,mB,mC,mA,mBC,mAB,mAC,_mf1370 ,_wf1370 )
	+Complex(_c2f2*GeV2    )*amplitude(2,false ,mD,mB,mC,mA,mBC,mAB,mAC,_mf2    ,_wf2    );
    }
    else if(ichan==0) {
      amp=_c2Kstarp     *amplitude(1,false ,mD,mA,mB,mC,mAB,mAC,mBC,_mK892B ,_wK892B );
    }
    else if(ichan==1) {
      amp=_c2rho        *amplitude(1,false ,mD,mB,mC,mA,mBC,mAB,mAC,_mrhoB  ,_wrhoB  );
    }
    else if(ichan==2) {
      amp=_c2omega      *amplitude(1,false ,mD,mB,mC,mA,mBC,mAB,mAC,_momega ,_womega );
    }
    else if(ichan==3) {
      amp=_c2Kstarm     *amplitude(1,false ,mD,mA,mC,mB,mAC,mAB,mBC,_mK892B ,_wK892B );
    }
    else if(ichan==4) {
      amp=_c2f980/GeV2  *amplitude(0,_f0opt,mD,mB,mC,mA,mBC,mAB,mAC,_mf980  ,_wf980  );
    }
    else if(ichan==5) {
      amp=Complex(_c2f2*GeV2    *amplitude(2,false ,mD,mB,mC,mA,mBC,mAB,mAC,_mf2    ,_wf2    ));
    }
    else if(ichan==6) {
      amp=_c2f1370/GeV2 *amplitude(0,false ,mD,mB,mC,mA,mBC,mAB,mAC,_mf1370 ,_wf1370 );
    }
    else if(ichan==7) {
      amp=_c2K14300/GeV2*amplitude(0,false ,mD,mA,mC,mB,mAC,mAB,mBC,_mK14300,_wK14300);
    }
    else if(ichan==8) {
      amp=Complex(_c2K14302*GeV2*amplitude(2,false ,mD,mA,mC,mB,mAC,mAB,mBC,_mK14302,_wK14302));
    }
    else if(ichan==9) {
      amp=_c2K1680      *amplitude(1,false ,mD,mA,mC,mB,mAC,mAB,mBC,_mK1680 ,_wK1680 );
    }
  }
  // now compute the matrix element
  (*ME())(0,0,0,0)=amp;
  return norm(amp);
}

void DtoKPiPiCLEO::dataBaseOutput(ofstream & output, bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  // parameters
  output << "newdef " << name() << ":LocalParameters " << _localparameters << "\n";
  output << "newdef " << name() << ":OmegaMass "          << _momega/MeV   << "\n";
  output << "newdef " << name() << ":f980Mass "           << _mf980/MeV    << "\n";
  output << "newdef " << name() << ":f_2Mass "            << _mf2/MeV      << "\n";
  output << "newdef " << name() << ":f1370Mass "          << _mf1370/MeV   << "\n";
  output << "newdef " << name() << ":K_01430Mass "        << _mK14300/MeV  << "\n";
  output << "newdef " << name() << ":K_21430Mass "        << _mK14302/MeV  << "\n";
  output << "newdef " << name() << ":Kstar1680Mass "      << _mK1680/MeV   << "\n";
  output << "newdef " << name() << ":rho1700Mass "        << _mrho1700/MeV << "\n";
  output << "newdef " << name() << ":Kstar0892Mass "      << _mK8920/MeV   << "\n";
  output << "newdef " << name() << ":KstarPlus892AMass "  << _mK892A/MeV   << "\n";
  output << "newdef " << name() << ":KstarPlus892BMass "  << _mK892B/MeV   << "\n";
  output << "newdef " << name() << ":RhoPlusMass "        << _mrhoA/MeV    << "\n";
  output << "newdef " << name() << ":Rho0Mass "           << _mrhoB/MeV    << "\n";
  output << "newdef " << name() << ":OmegaWidth "         << _womega/MeV   << "\n";
  output << "newdef " << name() << ":f980Width "          << _wf980/MeV    << "\n";
  output << "newdef " << name() << ":f_2Width "           << _wf2/MeV      << "\n";
  output << "newdef " << name() << ":f1370Width "         << _wf1370/MeV   << "\n";
  output << "newdef " << name() << ":K_01430Width "       << _wK14300/MeV  << "\n";
  output << "newdef " << name() << ":K_21430Width "       << _wK14302/MeV  << "\n";
  output << "newdef " << name() << ":Kstar1680Width "     << _wK1680/MeV   << "\n";
  output << "newdef " << name() << ":rho1700Width "       << _wrho1700/MeV << "\n";
  output << "newdef " << name() << ":Kstar0892Width "     << _wK8920/MeV   << "\n";
  output << "newdef " << name() << ":KstarPlus892AWidth " << _wK892A/MeV   << "\n";
  output << "newdef " << name() << ":KstarPlus892BWidth " << _wK892B/MeV   << "\n";
  output << "newdef " << name() << ":RhoPlusWidth "       << _wrhoA/MeV    << "\n";
  output << "newdef " << name() << ":Rho0Width "          << _wrhoB/MeV    << "\n";
  output << "newdef " << name() << ":gPi " << _gpi << "\n";
  output << "newdef " << name() << ":gK " << _gK << "\n";
  output << "newdef " << name() << ":f0Option " << _f0opt << "\n";
  output << "newdef " << name() << ":ChargedNonResonantAmplitude " << _a1NR << "\n";
  output << "newdef " << name() << ":ChargedNonResonantPhase " << _phi1NR<< "\n";
  output << "newdef " << name() << ":ChargedRhoAmplitude " << _a1rho<< "\n";
  output << "newdef " << name() << ":ChargedRhoPhase " << _phi1rho<< "\n";
  output << "newdef " << name() << ":ChargedKStarMinusAmplitude " << _a1Kstarm<< "\n";
  output << "newdef " << name() << ":ChargedKStarMinusPhase " << _phi1Kstarm<< "\n";
  output << "newdef " << name() << ":ChargedKStar0Amplitude " << _a1Kstar0<< "\n";
  output << "newdef " << name() << ":ChargedKStar0Phase " << _phi1Kstar0<< "\n";
  output << "newdef " << name() << ":ChargedK_0MinusAmplitude " 
	 << _a1K1430m/GeV2 << "\n";
  output << "newdef " << name() << ":ChargedK_0MinusPhase " << _phi1K1430m<< "\n";
  output << "newdef " << name() << ":ChargedK_00Amplitude " 
	 << _a1K14300/GeV2 << "\n";
  output << "newdef " << name() << ":ChargedK_00Phase " << _phi1K14300<< "\n";
  output << "newdef " << name() << ":ChargedRho1700Amplitude " << _a1rho1700<< "\n";
  output << "newdef " << name() << ":ChargedRho1700Phase " << _phi1rho1700<< "\n";
  output << "newdef " << name() << ":ChargedK1680MinusAmplitude " << _a1K1680<< "\n";
  output << "newdef " << name() << ":ChargedK1680MinusPhase " << _phi1K1680<< "\n";
  output << "newdef " << name() << ":NeutralKStarPlusAmplitude " << _a2Kstarp<< "\n";
  output << "newdef " << name() << ":NeutralKStarPlusPhase " << _phi2Kstarp<< "\n";
  output << "newdef " << name() << ":NeutralRhoAmplitude " << _a2rho<< "\n";
  output << "newdef " << name() << ":NeutralRhoPhase " << _phi2rho<< "\n";
  output << "newdef " << name() << ":NeutralOmegaAmplitude " << _a2omega<< "\n";
  output << "newdef " << name() << ":NeutralOmegaPhase " <<_phi2omega << "\n";
  output << "newdef " << name() << ":NeutralKStarMinusAmplitude " << _a2Kstarm<< "\n";
  output << "newdef " << name() << ":NeutralKStarMinusPhase " << _phi2Kstarm<< "\n";
  output << "newdef " << name() << ":Neutralf980Amplitude " << _a2f980/GeV2<< "\n";
  output << "newdef " << name() << ":Neutralf980Phase " << _phi2f980<< "\n";
  output << "newdef " << name() << ":Neutralf2Amplitude " << _a2f2*GeV2<< "\n";
  output << "newdef " << name() << ":Neutralf2Phase " << _phi2f2<< "\n";
  output << "newdef " << name() << ":Neutralf1370Amplitude " << _a2f1370/GeV2<< "\n";
  output << "newdef " << name() << ":Neutralf1370Phase " << _phi2f1370<< "\n";
  output << "newdef " << name() << ":NeutralKK_0MinusAmplitude " 
	 << _a2K14300/GeV2 << "\n";
  output << "newdef " << name() << ":NeutralKK_0MinusPhase " << _phi2K14300 << "\n";
  output << "newdef " << name() << ":NeutralKK_2MinusAmplitude " 
	 << _a2K14302*GeV2<< "\n";
  output << "newdef " << name() << ":NeutralKK_2MinusPhase " << _phi2K14302 << "\n";
  output << "newdef " << name() << ":NeutralK1680MinusAmplitude " << _a2K1680<< "\n";
  output << "newdef " << name() << ":NeutralK1680MinusPhase " << _phi2K1680<< "\n";
  output << "newdef " << name() << ":NeutralNonResonantAmplitude " << _a2NR<< "\n";
  output << "newdef " << name() << ":NeutralNonResonantPhase " << _phi2NR << "\n";
  output << "newdef " << name() << ":DRadius " << _rD0*GeV << "\n";
  output << "newdef " << name() << ":ResonanceRadius " << _rres*GeV << "\n";
  for(unsigned int ix=0;ix<_maxwgt.size();++ix) {
    output << "insert " << name() << ":MaximumWeights " 
	   << ix << " " << _maxwgt[ix] << "\n";
  }
  for(unsigned int ix=0;ix<_weights.size();++ix) {
    output << "insert " << name() << ":Weights " 
	   << ix << " " << _weights[ix] << "\n";
  }
  if(header) {
    output << "\n\" where BINARY ThePEGName=\"" << fullName() << "\";" << endl;
  }
}

void DtoKPiPiCLEO::doinitrun() {
  DecayIntegrator::doinitrun();
  _weights.resize(mode(0)->numberChannels()+mode(1)->numberChannels());
  _maxwgt.resize(2);
  unsigned int iy=0;
  for(unsigned int ix=0;ix<2;++ix) {
    _maxwgt[ix]=mode(ix)->maxWeight();
    for(unsigned int iz=0;iz<mode(ix)->numberChannels();++iz) {
      _weights[iy]=mode(ix)->channelWeight(iz);
      ++iy;
    }
  }
}

Complex DtoKPiPiCLEO::amplitude(int ispin,bool f0, Energy mD,
					Energy mA , Energy mB , Energy mC ,
					Energy mAB, Energy mAC, Energy mBC,
					Energy mres, Energy wres) const{
  // compute the production momenta
  Energy pDR  = Kinematics::pstarTwoBodyDecay(mD,mres,mC);
  Energy pDAB = Kinematics::pstarTwoBodyDecay(mD,mAB ,mC);
  // and the decay momenta
  Energy pAB = Kinematics::pstarTwoBodyDecay(mAB ,mA,mB);
  Energy pR  = Kinematics::pstarTwoBodyDecay(mres,mA,mB);
  double Fd(1.),Fr(1.),s(1.);
  switch(ispin) {
  case 0:
    // default values of parameters are correct
    break;
  case 1:
    Fr = sqrt((1.+sqr(_rres*pR ))/(1.+sqr(_rres*pAB )));
    Fd = sqrt((1.+sqr(_rD0 *pDR))/(1.+sqr(_rD0 *pDAB)));
    s = ((mAC-mBC)*(mAC+mBC)+(mD-mC)*(mD+mC)*(mB-mA)*(mB+mA)/sqr(mres))/GeV2;
    break;
  case 2:
    Fr = sqrt((9.+3.*sqr(_rres*pR  )+Math::powi(_rres*pR  ,4))/
	      (9.+3.*sqr(_rres*pAB )+Math::powi(_rres*pAB ,4)));
    Fd = sqrt((9.+3.*sqr(_rD0 *pDR )+Math::powi(_rD0 *pDR ,4))/
	      (9.+3.*sqr(_rD0 *pDAB)+Math::powi(_rD0 *pDAB,4)));
    s = sqr(((mBC-mAC)*(mBC+mAC)+(mD-mC)*(mD+mC)*(mA-mB)*(mA+mB)/sqr(mres))/GeV2)
      -(mAB*mAB-2.*mD*mD-2.*mC*mC+sqr((mD-mC)*(mD+mC))/sqr(mres))*
       (mAB*mAB-2.*mA*mA-2.*mB*mB+sqr((mA-mB)*(mA+mB))/sqr(mres))/3./GeV2/GeV2;
    break;
  default:
    throw Exception() << "D0toKPiPiCLEO::amplitude spin is too high ispin = " 
		      << ispin << Exception::runerror;
  }
  // calculate the width term
  complex<Energy2> bw;
  if(!f0) {
    Energy2 mwid=wres*Math::powi(pAB/pR,2*ispin+1)*sqr(Fr*mres)/mAB;
    bw = sqr(mres)-sqr(mAB)-complex<Energy2>(ZERO,mwid);
  }
  else {
    Energy Gamma_pi = _gpi*sqrt(0.25*sqr(mAB)-sqr(_mpi));
    Energy Gamma_K  = 0.5*_gK *(sqrt(0.25*sqr(mAB)-sqr(_mkp))+
				sqrt(0.25*sqr(mAB)-sqr(_mk0)));
    bw = sqr(mres)-sqr(mAB)-complex<Energy2>(ZERO,mres*(Gamma_pi+Gamma_K));
  }
  return s*Fr*Fd*GeV2/bw;
}
#line 1 "./DtoKPiPiE691.cc"
// -*- C++ -*-
//
// DtoKPiPiE691.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the DtoKPiPiE691 class.
//

#include "DtoKPiPiE691.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Interface/Switch.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "Herwig/Decay/GeneralDecayMatrixElement.h"

using namespace Herwig;

using ThePEG::Helicity::ScalarWaveFunction;
using ThePEG::Helicity::incoming;
using ThePEG::Helicity::outgoing;

DtoKPiPiE691::DtoKPiPiE691() {
  // amplitudes and phases for D+ -> K-pi+pi+
  _a1NR    = 1.  ; _phi1NR    =    0.;
  _a1K892  = 0.78; _phi1K892  = - 60.;
  _a1K1430 = 0.53; _phi1K1430 =  132.;
  _a1K1680 = 0.47; _phi1K1680 = - 51.;
  // amplitudes and phases for D0 -> K-pi+pi0
  _a2NR    = 1.00; _phi2NR    =    0.;
  _a2K8920 = 3.19; _phi2K8920 =  167.;
  _a2K892m = 2.96; _phi2K892m = -112.;
  _a2rho   = 8.56; _phi2rho   =   40.;
  // amplitudes and phases for D0 -> Kbar0 pi+pi-
  _a3NR    = 1.00; _phi3NR    =    0.;
  _a3K892  = 2.31; _phi3K892  =  109.;
  _a3rho   = 1.59; _phi3rho   = -123.;
  // masses and widths
  _localparameters=true;
  _mK8920 = 0.8961 *GeV; _wK8920 = 0.0505*GeV;
  _mK892m = 0.89159*GeV; _wK892m = 0.0498*GeV;
  _mK1680 = 1.714  *GeV; _wK1680 = 0.323 *GeV;
  _mK1430 = 1.429  *GeV; _wK1430 = 0.287 *GeV;
  _mrho0  = 0.7681 *GeV; _wrho0  = 0.1515*GeV;
  _mrhop  = 0.7681 *GeV; _wrhop  = 0.1515*GeV;
  // generate the intermediate particles
  generateIntermediates(true);
}

void DtoKPiPiE691::doinit() {
  DecayIntegrator::doinit();
  // complex amplitudes calculated from magnitudes and phases
  double fact = Constants::pi/180.;
  // D+ -> K-pi+pi+
  _c1NR   = _a1NR   *Complex(cos(_phi1NR   *fact),sin(_phi1NR   *fact)); 
  _c1K892 = _a1K892 *Complex(cos(_phi1K892 *fact),sin(_phi1K892 *fact));
  _c1K1430= _a1K1430*Complex(cos(_phi1K1430*fact),sin(_phi1K1430*fact));
  _c1K1680= _a1K1680*Complex(cos(_phi1K1680*fact),sin(_phi1K1680*fact));
  // D0 -> K-pi+pi0
  _c2NR   = _a2NR   *Complex(cos(_phi2NR   *fact),sin(_phi2NR   *fact));
  _c2K8920= _a2K8920*Complex(cos(_phi2K8920*fact),sin(_phi2K8920*fact));
  _c2K892m= _a2K892m*Complex(cos(_phi2K892m*fact),sin(_phi2K892m*fact));
  _c2rho  = _a2rho  *Complex(cos(_phi2rho  *fact),sin(_phi2rho  *fact));
  // D0 -> Kbar0pi+pi-
  _c3NR   = _a3NR   *Complex(cos(_phi3NR   *fact),sin(_phi3NR   *fact));
  _c3K892 = _a3K892 *Complex(cos(_phi3K892 *fact),sin(_phi3K892 *fact));
  _c3rho  = _a3rho  *Complex(cos(_phi3rho  *fact),sin(_phi3rho  *fact));
  // intermediate resonances
  tPDPtr k8920 = getParticleData(ParticleID::Kstarbar0 );
  tPDPtr k892m = getParticleData(ParticleID::Kstarminus);
  tPDPtr k1430 = getParticleData(ParticleID::Kstar_0bar0);
  tPDPtr k1680 = getParticleData(-30313);
  tPDPtr rho0  = getParticleData(ParticleID::rho0);
  tPDPtr rhop  = getParticleData(ParticleID::rhoplus);
  // D+ -> K-pi+pi+
  tPDVector extpart(4);
  extpart[0]=getParticleData(ParticleID::Dplus);
  extpart[1]=getParticleData(ParticleID::Kminus);
  extpart[2]=getParticleData(ParticleID::piplus);
  extpart[3]=getParticleData(ParticleID::piplus);
  DecayPhaseSpaceChannelPtr newchannel;
  DecayPhaseSpaceModePtr mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
  int ix=0;
  if(k8920) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,3);
    newchannel->addIntermediate(k8920,0,0., 1,2);
    mode->addChannel(newchannel);
    ++ix;
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,2);
    newchannel->addIntermediate(k8920,0,0., 1,3);
    mode->addChannel(newchannel);
    ++ix;
  }
  if(k1430) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,3);
    newchannel->addIntermediate(k1430,0,0., 1,2);
    mode->addChannel(newchannel);
    ++ix;
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,2);
    newchannel->addIntermediate(k1430,0,0., 1,3);
    mode->addChannel(newchannel);
    ++ix;
  }
  if(k1680) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,3);
    newchannel->addIntermediate(k1680,0,0., 1,2);
    mode->addChannel(newchannel);
    ++ix;
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,2);
    newchannel->addIntermediate(k1680,0,0., 1,3);
    mode->addChannel(newchannel);
    ++ix;
  }
  // add the mode
  vector<double> wtemp;
  if(ix<=int(_weights.size())) {
    vector<double>::const_iterator wit=_weights.begin();
    wtemp=vector<double>(wit,wit+ix);
  }
  else {
    wtemp=vector<double>(ix,1./double(ix));
  }
  if(_maxwgt.empty()) _maxwgt.push_back(1.);
  addMode(mode,_maxwgt[0],wtemp);
  // D0 -> K-pi+pi0
  extpart[0]=getParticleData(ParticleID::D0);
  extpart[1]=getParticleData(ParticleID::Kminus);
  extpart[2]=getParticleData(ParticleID::piplus);
  extpart[3]=getParticleData(ParticleID::pi0);
  mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
  int iy=ix;
  if(k8920) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,3);
    newchannel->addIntermediate(k8920,0,0., 1,2);
    mode->addChannel(newchannel);
    ++iy;
  }
  if(k892m) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,2);
    newchannel->addIntermediate(k892m,0,0., 1,3);
    mode->addChannel(newchannel);
    ++iy;
  }
  if(rhop) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,1);
    newchannel->addIntermediate(rhop,0,0., 2,3);
    mode->addChannel(newchannel);
    ++iy;
  }
  // add the mode
  if(iy<=int(_weights.size())) {
    vector<double>::const_iterator wit=_weights.begin();
    wtemp=vector<double>(wit+ix,wit+iy);
  }
  else {
    wtemp=vector<double>(iy-ix,1./double(iy-ix));
  }
  if(_maxwgt.size()<2) _maxwgt.push_back(1.);
  addMode(mode,_maxwgt[1],wtemp);
  // D0 -> Kbar0 pi+ pi-
  extpart[0]=getParticleData(ParticleID::D0);
  extpart[1]=getParticleData(ParticleID::Kbar0);
  extpart[2]=getParticleData(ParticleID::piplus);
  extpart[3]=getParticleData(ParticleID::piminus);
  mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
  ix=iy;
  if(rho0) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,1);
    newchannel->addIntermediate(rho0,0,0., 2,3);
    mode->addChannel(newchannel);
    ++iy;
  }
  if(k892m) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,2);
    newchannel->addIntermediate(k892m,0,0., 1,3);
    mode->addChannel(newchannel);
    ++iy;
  }
  // add the mode
  if(iy<=int(_weights.size())) {
    vector<double>::const_iterator wit=_weights.begin();
    wtemp=vector<double>(wit+ix,wit+iy);
  }
  else {
    wtemp=vector<double>(iy-ix,1./double(iy-ix));
  }
  if(_maxwgt.size()<3) _maxwgt.push_back(1.);
  addMode(mode,_maxwgt[2],wtemp);
  // reset the resonance parameters in the integration if needed
  if(_localparameters) {
    resetIntermediate(k8920,_mK8920,_wK8920);
    resetIntermediate(k892m,_mK892m,_wK892m);
    resetIntermediate(k1430,_mK1680,_wK1680);
    resetIntermediate(k1680,_mK1430,_wK1430);
    resetIntermediate(rho0 ,_mrho0 ,_wrho0 );
    resetIntermediate(rhop ,_mrhop ,_wrhop );
  }
  // get values from the ParticleData objects if needed
  else {
    _mK8920 = k8920->mass();
    _mK892m = k892m->mass();
    _mK1680 = k1430->mass();
    _mK1430 = k1680->mass();
    _mrho0  = rho0 ->mass();
    _mrhop  = rhop ->mass();
    _wK8920 = k8920->width();
    _wK892m = k892m->width();
    _wK1680 = k1430->width();
    _wK1430 = k1680->width();
    _wrho0  = rho0 ->width();
    _wrhop  = rhop ->width();
  }
}

void DtoKPiPiE691::persistentOutput(PersistentOStream & os) const {
  os << _a1NR << _phi1NR << _a1K892 << _phi1K892 << _a1K1430 << _phi1K1430 
     << _a1K1680 << _phi1K1680 << _a2NR << _phi2NR << _a2K8920 << _phi2K8920 
     << _a2K892m << _phi2K892m << _a2rho << _phi2rho << _a3NR << _phi3NR 
     << _a3K892 << _phi3K892 << _a3rho << _phi3rho << _c1NR << _c1K892 << _c1K1430 
     << _c1K1680 << _c2NR << _c2K8920 << _c2K892m << _c2rho << _c3NR << _c3K892 
     << _c3rho << _localparameters << ounit(_mK8920,GeV) << ounit(_wK8920,GeV) 
     << ounit(_mK892m,GeV) << ounit(_wK892m,GeV) << ounit(_mK1680,GeV) 
     << ounit(_wK1680,GeV) << ounit(_mK1430,GeV) << ounit(_wK1430,GeV) 
     << ounit(_mrho0 ,GeV) << ounit(_wrho0 ,GeV) << ounit(_mrhop ,GeV) 
     << ounit(_wrhop ,GeV) << _maxwgt << _weights;
}

void DtoKPiPiE691::persistentInput(PersistentIStream & is, int) {
  is >> _a1NR >> _phi1NR >> _a1K892 >> _phi1K892 >> _a1K1430 >> _phi1K1430 
     >> _a1K1680 >> _phi1K1680 >> _a2NR >> _phi2NR >> _a2K8920 >> _phi2K8920 
     >> _a2K892m >> _phi2K892m >> _a2rho >> _phi2rho >> _a3NR >> _phi3NR 
     >> _a3K892 >> _phi3K892 >> _a3rho >> _phi3rho >> _c1NR >> _c1K892 >> _c1K1430 
     >> _c1K1680 >> _c2NR >> _c2K8920 >> _c2K892m >> _c2rho >> _c3NR >> _c3K892 
     >> _c3rho >> _localparameters >> iunit(_mK8920,GeV) >> iunit(_wK8920,GeV) 
     >> iunit(_mK892m,GeV) >> iunit(_wK892m,GeV) >> iunit(_mK1680,GeV) 
     >> iunit(_wK1680,GeV) >> iunit(_mK1430,GeV) >> iunit(_wK1430,GeV) 
     >> iunit(_mrho0 ,GeV) >> iunit(_wrho0 ,GeV) >> iunit(_mrhop ,GeV) 
     >> iunit(_wrhop ,GeV) >> _maxwgt >> _weights;
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<DtoKPiPiE691,DecayIntegrator>
describeHerwigDtoKPiPiE691("Herwig::DtoKPiPiE691", "HwSMDecay.so");

void DtoKPiPiE691::Init() {

  static ClassDocumentation<DtoKPiPiE691> documentation
    ("The DtoKPiPiE691 class implements the model of E691 for D -> K pi pi"
     "Dalitz decays",
     "The model of \\cite{Anjos:1992kb} for $D\\to K\\pi\\pi$ decays was used",
     "\\bibitem{Anjos:1992kb} J.~C.~Anjos {\\it et al.}  [E691 Collaboration],"
     "Phys.\\ Rev.\\  D {\\bf 48} (1993) 56.");

  static Parameter<DtoKPiPiE691,double> interfaceKmPipPipNonResonantMagnitude
    ("KmPipPipNonResonantMagnitude",
     "The magnitude of the non-resonant component for D+ -> K- pi+ pi+",
     &DtoKPiPiE691::_a1NR, 1.0, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,double> interfaceKmPipPipNonResonantPhase
    ("KmPipPipNonResonantPhase",
     "The phase of the non-resonant component for D+ -> K- pi+ pi+",
     &DtoKPiPiE691::_phi1NR, 0., -180., 180.,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,double> interfaceKmPipPipK892Magnitude
    ("KmPipPipK892Magnitude",
     "The magnitude of the K*(892) component for D+ -> K- pi+ pi+",
     &DtoKPiPiE691::_a1K892, 0.78, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,double> interfaceKmPipPipK892Phase
    ("KmPipPipK892Phase",
     "The phase of the K*(892) component for D+ -> K- pi+ pi+",
     &DtoKPiPiE691::_phi1K892, -60., -180., 180.,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,double> interfaceKmPipPipK1430Magnitude
    ("KmPipPipK1430Magnitude",
     "The magnitude of the K*0(1430) component for D+ -> K- pi+ pi+",
     &DtoKPiPiE691::_a1K1430, 0.53, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,double> interfaceKmPipPipK1430Phase
    ("KmPipPipK1430Phase",
     "The phase of the K*0(1430) component for D+ -> K- pi+ pi+",
     &DtoKPiPiE691::_phi1K1430, 132., -180., 180.,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,double> interfaceKmPipPipK1680Magnitude
    ("KmPipPipK1680Magnitude",
     "The magnitude of the K*(1680) component for D+ -> K- pi+ pi+",
     &DtoKPiPiE691::_a1K1680, 0.47, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,double> interfaceKmPipPipK1680Phase
    ("KmPipPipK1680Phase",
     "The phase of the K*(1680) component for D+ -> K- pi+ pi+",
     &DtoKPiPiE691::_phi1K1680, - 51., -180., 180.,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,double> interfaceKmPipPi0NonResonantMagnitude
    ("KmPipPi0NonResonantMagnitude",
     "The magnitude of the non-resonant component for D0 -> K- pi+ pi0",
     &DtoKPiPiE691::_a2NR, 1.0, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,double> interfaceKmPipPi0NonResonantPhase
    ("KmPipPi0NonResonantPhase",
     "The phase of the non-resonant component for D0 -> K- pi+ pi0",
     &DtoKPiPiE691::_phi2NR, 0., -180., 180.,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,double> interfaceKmPipPi0K8920Magnitude
    ("KmPipPi0K8920Magnitude",
     "The magnitude of the K*(892)0 component for D0 -> K- pi+ pi0",
     &DtoKPiPiE691::_a2K8920, 3.19, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,double> interfaceKmPipPi0K8920Phase
    ("KmPipPi0K8920Phase",
     "The phase of the K*(892)0 component for D0 -> K- pi+ pi0",
     &DtoKPiPiE691::_phi2K8920, 167., -180., 180.,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,double> interfaceKmPipPi0K892mMagnitude
    ("KmPipPi0K892mMagnitude",
     "The magnitude of the K*(892)- component for D0 -> K- pi+ pi0",
     &DtoKPiPiE691::_a2K892m, 2.96, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,double> interfaceKmPipPi0K892mPhase
    ("KmPipPi0K892mPhase",
     "The phase of the K*(892)- component for D0 -> K- pi+ pi0",
     &DtoKPiPiE691::_phi2K892m, -112., -180., 180.,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,double> interfaceKmPipPi0RhoMagnitude
    ("KmPipPi0RhoMagnitude",
     "The magnitude of the rho component for D0 -> K- pi+ pi0",
     &DtoKPiPiE691::_a2rho, 8.56, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,double> interfaceKmPipPi0RhoPhase
    ("KmPipPi0RhoPhase",
     "The phase of the rho component for D0 -> K- pi+ pi0",
     &DtoKPiPiE691::_phi2rho, 40., -180., 180.,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,double> interfaceK0PipPimNonResonantMagnitude
    ("K0PipPimNonResonantMagnitude",
     "The magnitude of the non-resonant component for D0 -> Kbar0 pi+pi-",
     &DtoKPiPiE691::_a3NR, 1.0, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,double> interfaceK0PipPimNonResonantPhase
    ("K0PipPimNonResonantPhase",
     "The phase of the non-resonant component for D0 -> Kbar0 pi+pi-",
     &DtoKPiPiE691::_phi3NR, 0., -180., 180.,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,double> interfaceK0PipPimK892Magnitude
    ("K0PipPimK892Magnitude",
     "The magnitude of the K*(892) component for D0 -> Kbar0 pi+pi-",
     &DtoKPiPiE691::_a3K892, 2.31, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,double> interfaceK0PipPimK892Phase
    ("K0PipPimK892Phase",
     "The phase of the K*(892)0 component for D0 -> Kbar0 pi+pi-",
     &DtoKPiPiE691::_phi3K892, 109., -180., 180.,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,double> interfaceK0PipPimRhoMagnitude
    ("K0PipPimRhoMagnitude",
     "The magnitude of the rho component for D0 -> Kbar0 pi+pi-",
     &DtoKPiPiE691::_a3rho, 1.59, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,double> interfaceK0PipPimRhoPhase
    ("K0PipPimRhoPhase",
     "The phase of the rho component for D0 -> Kbar0 pi+pi-",
     &DtoKPiPiE691::_phi3rho, -123., -180., 180.,
     false, false, Interface::limited);

  static Switch<DtoKPiPiE691,bool> interfaceLocalParameters
    ("LocalParameters",
     "Whether to use local values for the masses and widths or"
     " those from the ParticleData objects",
     &DtoKPiPiE691::_localparameters, true, false, false);
  static SwitchOption interfaceLocalParametersLocal
    (interfaceLocalParameters,
     "Local",
     "Use local values",
     true);
  static SwitchOption interfaceLocalParametersParticleData
    (interfaceLocalParameters,
     "ParticleData",
     "Use the values from the ParticleData objects",
     false);

  static Parameter<DtoKPiPiE691,Energy> interfaceK8920Mass
    ("K8920Mass",
     "The mass of the K*(892)0",
     &DtoKPiPiE691::_mK8920, GeV, 0.8961 *GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,Energy> interfaceK8920Width
    ("K8920Width",
     "The width of the K*(892)0",
     &DtoKPiPiE691::_wK8920, GeV, 0.0505*GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,Energy> interfaceK892MinusMass
    ("K892MinusMass",
     "The mass of the K*(892)-",
     &DtoKPiPiE691::_mK892m, GeV, 0.89159*GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,Energy> interfaceK892MinusWidth
    ("K892MinusWidth",
     "The width of the K*(892)-",
     &DtoKPiPiE691::_wK892m, GeV, 0.0498*GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,Energy> interfaceK1680Mass
    ("K1680Mass",
     "The mass of the K*(1680)",
     &DtoKPiPiE691::_mK1680, GeV, 1.714  *GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,Energy> interfaceK1680Width
    ("K1680Width",
     "The width of the K*(1680)",
     &DtoKPiPiE691::_wK1680, GeV, 0.323 *GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,Energy> interfaceK1430Mass
    ("K1430Mass",
     "The mass of the K*(1430)",
     &DtoKPiPiE691::_mK1430, GeV, 1.429  *GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,Energy> interfaceK1430Width
    ("K1430Width",
     "The width of the K*(1430)",
     &DtoKPiPiE691::_wK1430, GeV, 0.287 *GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,Energy> interfaceRho0Mass
    ("Rho0Mass",
     "The mass of the rho0",
     &DtoKPiPiE691::_mrho0, GeV, 0.7681 *GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,Energy> interfaceRho0Width
    ("Rho0Width",
     "The width of the rho0",
     &DtoKPiPiE691::_wrho0, GeV, 0.1515*GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,Energy> interfaceRhoPlusMass
    ("RhoPlusMass",
     "The mass of the rho+",
     &DtoKPiPiE691::_mrhop, GeV, 0.7681 *GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiE691,Energy> interfaceRhoPlusWidth
    ("RhoPlusWidth",
     "The width of the rho+",
     &DtoKPiPiE691::_wrhop, GeV, 0.1515*GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static ParVector<DtoKPiPiE691,double> interfaceMaximumWeights
    ("MaximumWeights",
     "The maximum weights for the unweighting of the decays",
     &DtoKPiPiE691::_maxwgt, -1, 1.0, 0.0, 1.0e11,
     false, false, Interface::limited);

  static ParVector<DtoKPiPiE691,double> interfaceWeights
    ("Weights",
     "The weights for the different channels for the phase-space integration",
     &DtoKPiPiE691::_weights, -1, 1.0, 0.0, 1.0,
     false, false, Interface::limited);
}

int DtoKPiPiE691::modeNumber(bool & cc,tcPDPtr parent,
			     const tPDVector & children) const {
  int id0(parent->id());
  // incoming particle must be D0 or D+
  if(abs(id0)!=ParticleID::D0&&abs(id0)!=ParticleID::Dplus) return -1;
  cc = id0<0;
  // must be three decay products
  if(children.size()!=3) return -1;
  tPDVector::const_iterator pit = children.begin();
  unsigned int npip(0),npim(0),nkm(0),nk0(0),npi0(0);
  int id;
  for( ;pit!=children.end();++pit) {
    id=(**pit).id();
    if(id          ==ParticleID::piplus)  ++npip;
    else if(id     ==ParticleID::pi0)     ++npi0;
    else if(id     ==ParticleID::piminus) ++npim;
    else if(abs(id)==ParticleID::K0)      ++nk0;
    else if(id     ==ParticleID::K_L0)    ++nk0;
    else if(id     ==ParticleID::K_S0)    ++nk0;
    else if(abs(id)==ParticleID::Kplus)   ++nkm;
  }
  if(abs(id0)==ParticleID::Dplus) {
    if((id0==ParticleID::Dplus &&(nkm==1&&npip==2))||
       (id0==ParticleID::Dminus&&(nkm==1&&npim==2))) return 0;
    else return -1;
  }
  else {
    if(npim==1&&npip==1&&nk0==1) return  2;
    else if(nkm==1&&(npip+npim)==1&&npi0==1) return 1;
    else                        return -1;
  }
}

double DtoKPiPiE691::me2(const int ichan,
			 const Particle & inpart,
			 const ParticleVector & decay,
			 MEOption meopt) const {
  if(!ME())
    ME(new_ptr(GeneralDecayMatrixElement(PDT::Spin0,PDT::Spin0,PDT::Spin0,PDT::Spin0)));
  useMe();
  if(meopt==Initialize) {
    ScalarWaveFunction::
      calculateWaveFunctions(_rho,const_ptr_cast<tPPtr>(&inpart),incoming);
  }
  if(meopt==Terminate) {
    // set up the spin information for the decay products
    ScalarWaveFunction::constructSpinInfo(const_ptr_cast<tPPtr>(&inpart),
					  incoming,true);
    for(unsigned int ix=0;ix<3;++ix)
    ScalarWaveFunction::constructSpinInfo(decay[ix],outgoing,true);
    return 0.;
  }
  Complex amp;
  // D+ -> K-pi+pi+
  if(imode()==0) {
    Lorentz5Momentum pres1=decay[0]->momentum()+decay[1]->momentum();
    pres1.rescaleMass();
    double ct1 =-decayAngle(inpart.momentum(),pres1,decay[0]->momentum());
    Lorentz5Momentum pres2=decay[0]->momentum()+decay[2]->momentum();
    pres2.rescaleMass();
    double ct2 =-decayAngle(inpart.momentum(),pres2,decay[0]->momentum());
    if(ichan<0) {
      amp = _c1NR*sqrt(2.)
	+_c1K892 *amplitude(1,ct1,pres1.mass(),_wK8920,_mK8920)
	+_c1K892 *amplitude(1,ct2,pres2.mass(),_wK8920,_mK8920)
	+_c1K1430*amplitude(0,ct1,pres1.mass(),_wK1430,_mK1430)
	+_c1K1430*amplitude(0,ct2,pres2.mass(),_wK1430,_mK1430)
	+_c1K1680*amplitude(1,ct1,pres1.mass(),_wK1680,_mK1680)
	+_c1K1680*amplitude(1,ct2,pres2.mass(),_wK1680,_mK1680);
    }
    else if(ichan==0) {
      amp=_c1K892 *amplitude(1,ct1,pres1.mass(),_wK8920,_mK8920);
    }
    else if(ichan==1) {
      amp=_c1K892 *amplitude(1,ct2,pres2.mass(),_wK8920,_mK8920);
    }
    else if(ichan==2) {
      amp=_c1K1430*amplitude(1,ct1,pres1.mass(),_wK1430,_mK1430);
    }
    else if(ichan==3) {
      amp=_c1K1430*amplitude(1,ct2,pres2.mass(),_wK1430,_mK1430);
    }
    else if(ichan==4) {
      amp=_c1K1680*amplitude(1,ct1,pres1.mass(),_wK1680,_mK1680);
    }
    else if(ichan==5) {
      amp=_c1K1680*amplitude(1,ct2,pres2.mass(),_wK1680,_mK1680);
    }
  }
  // D0 -> K-pi+pi0
  else if(imode()==1) {
    Lorentz5Momentum pres1=decay[0]->momentum()+decay[1]->momentum();
    pres1.rescaleMass();
    double ct1 = decayAngle(inpart.momentum(),pres1,decay[0]->momentum());
    Lorentz5Momentum pres2=decay[0]->momentum()+decay[2]->momentum();
    pres2.rescaleMass();
    double ct2 = decayAngle(inpart.momentum(),pres2,decay[2]->momentum());
    Lorentz5Momentum pres3=decay[1]->momentum()+decay[2]->momentum();
    pres3.rescaleMass();
    double ct3 = decayAngle(inpart.momentum(),pres3,decay[2]->momentum());
    if(ichan<0) {
      amp = _c2NR
	+_c2K8920*amplitude(1,ct1,pres1.mass(),_wK8920,_mK8920)
	+_c2K892m*amplitude(1,ct2,pres2.mass(),_wK892m,_mK892m)
	+_c2rho  *amplitude(1,ct3,pres3.mass(),_wrhop ,_mrhop );
    }
    else if(ichan==0) {
      amp = _c2K8920*amplitude(1,ct1,pres1.mass(),_wK8920,_mK8920);
    }
    else if(ichan==1) {
      amp = _c2K892m*amplitude(1,ct2,pres2.mass(),_wK892m,_mK892m);
    }
    else if(ichan==2) {
      amp = _c2rho  *amplitude(1,ct3,pres3.mass(),_wrhop ,_mrhop);
    }
  }
  // D0 -> Kbar0pi+pi-
  else if(imode()==2) {
    Lorentz5Momentum pres1=decay[0]->momentum()+decay[2]->momentum();
    pres1.rescaleMass();
    double ct1 = decayAngle(inpart.momentum(),pres1,decay[0]->momentum());
    Lorentz5Momentum pres2=decay[1]->momentum()+decay[2]->momentum();
    pres2.rescaleMass();
    double ct2 = decayAngle(inpart.momentum(),pres2,decay[1]->momentum());
    if(ichan<0) {
      amp = _c3NR
	+_c3K892*amplitude(1,ct1,pres1.mass(),_wK892m,_mK892m)
	+_c3rho *amplitude(1,ct2,pres2.mass(),_wrho0 ,_mrho0 );
    }
    else if(ichan==0) {
      amp = _c3rho *amplitude(1,ct2,pres2.mass(),_wrho0 ,_mrho0 );
    }
    else if(ichan==1) {
      amp = _c3K892*amplitude(1,ct1,pres1.mass(),_wK892m,_mK892m);
    }
  }
  // now compute the matrix element
  (*ME())(0,0,0,0)=amp;
  return norm(amp);
}

void DtoKPiPiE691::dataBaseOutput(ofstream & output, bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  // parameters
  output << "newdef " << name() << ":KmPipPipNonResonantMagnitude " 
	 << _a1NR      << "\n";
  output << "newdef " << name() << ":KmPipPipNonResonantPhase     " 
	 << _phi1NR    << "\n";
  output << "newdef " << name() << ":KmPipPipK892Magnitude        " 
	 << _a1K892    << "\n";
  output << "newdef " << name() << ":KmPipPipK892Phase            " 
	 << _phi1K892  << "\n";
  output << "newdef " << name() << ":KmPipPipK1430Magnitude       " 
	 << _a1K1430   << "\n";
  output << "newdef " << name() << ":KmPipPipK1430Phase           " 
	 << _phi1K1430 << "\n";
  output << "newdef " << name() << ":KmPipPipK1680Magnitude       " 
	 << _a1K1680   << "\n";
  output << "newdef " << name() << ":KmPipPipK1680Phase           " 
	 << _phi1K1680 << "\n";
  output << "newdef " << name() << ":KmPipPi0NonResonantMagnitude " 
	 << _a2NR      << "\n";
  output << "newdef " << name() << ":KmPipPi0NonResonantPhase     " 
	 << _phi2NR    << "\n";
  output << "newdef " << name() << ":KmPipPi0K8920Magnitude       " 
	 << _a2K8920   << "\n";
  output << "newdef " << name() << ":KmPipPi0K8920Phase           " 
	 << _phi2K8920 << "\n";
  output << "newdef " << name() << ":KmPipPi0K892mMagnitude       " 
	 << _a2K892m   << "\n";
  output << "newdef " << name() << ":KmPipPi0K892mPhase           " 
	 << _phi2K892m << "\n";
  output << "newdef " << name() << ":KmPipPi0RhoMagnitude         " 
	 << _a2rho     << "\n";
  output << "newdef " << name() << ":KmPipPi0RhoPhase             " 
	 << _phi2rho   << "\n";
  output << "newdef " << name() << ":K0PipPimNonResonantMagnitude " 
	 << _a3NR      << "\n";
  output << "newdef " << name() << ":K0PipPimNonResonantPhase     " 
	 << _phi3NR    << "\n";
  output << "newdef " << name() << ":K0PipPimK892Magnitude        " 
	 << _a3K892    << "\n";
  output << "newdef " << name() << ":K0PipPimK892Phase            " 
	 << _phi3K892  << "\n";
  output << "newdef " << name() << ":K0PipPimRhoMagnitude         " 
	 << _a3rho     << "\n";
  output << "newdef " << name() << ":K0PipPimRhoPhase             " 
	 << _phi3rho   << "\n";
  output << "newdef " << name() << ":LocalParameters " << _localparameters << "\n";
  output << "newdef " << name() << ":K8920Mass      " << _mK8920/GeV << "\n";
  output << "newdef " << name() << ":K8920Width     " << _wK8920/GeV << "\n";
  output << "newdef " << name() << ":K892MinusMass  " << _mK892m/GeV << "\n";
  output << "newdef " << name() << ":K892MinusWidth " << _wK892m/GeV << "\n";
  output << "newdef " << name() << ":K1680Mass      " << _mK1680/GeV << "\n";
  output << "newdef " << name() << ":K1680Width     " << _wK1680/GeV << "\n";
  output << "newdef " << name() << ":K1430Mass      " << _mK1430/GeV << "\n";
  output << "newdef " << name() << ":K1430Width     " << _wK1430/GeV << "\n";
  output << "newdef " << name() << ":Rho0Mass       " << _mrho0 /GeV << "\n";
  output << "newdef " << name() << ":Rho0Width      " << _wrho0 /GeV << "\n";
  output << "newdef " << name() << ":RhoPlusMass    " << _mrhop /GeV << "\n";
  output << "newdef " << name() << ":RhoPlusWidth   " << _wrhop /GeV << "\n";
  for(unsigned int ix=0;ix<_maxwgt.size();++ix) {
    output << "insert " << name() << ":MaximumWeights " 
	   << ix << " " << _maxwgt[ix] << "\n";
  }
  for(unsigned int ix=0;ix<_weights.size();++ix) {
    output << "insert " << name() << ":Weights " 
	   << ix << " " << _weights[ix] << "\n";
  }
  if(header) {
    output << "\n\" where BINARY ThePEGName=\"" << fullName() << "\";" << endl;
  }
}

void DtoKPiPiE691::doinitrun() {
  DecayIntegrator::doinitrun();
  _weights.resize(mode(0)->numberChannels()+mode(1)->numberChannels()+
		  mode(2)->numberChannels());
  _maxwgt.resize(3);
  unsigned int iy=0;
  for(unsigned int ix=0;ix<3;++ix) {
    _maxwgt[ix]=mode(ix)->maxWeight();
    for(unsigned int iz=0;iz<mode(ix)->numberChannels();++iz) {
      _weights[iy]=mode(ix)->channelWeight(iz);
      ++iy;
    }
  }
}
#line 1 "./DtoKPiPiMarkIII.cc"
// -*- C++ -*-
//
// DtoKPiPiMarkIII.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the DtoKPiPiMarkIII class.
//

#include "DtoKPiPiMarkIII.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/Switch.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "Herwig/Decay/GeneralDecayMatrixElement.h"

using namespace Herwig;

using ThePEG::Helicity::ScalarWaveFunction;
using ThePEG::Helicity::incoming;
using ThePEG::Helicity::outgoing;

Complex DtoKPiPiMarkIII::amplitude(bool rho, Energy mD, 
					  Energy mA , Energy mB , Energy mC ,
					  Energy mAB, Energy mAC, Energy mBC,
					  Energy mres, Energy wres) const {
  InvEnergy radius = rho ? _rrho : _rKstar;
  Energy  pAB  = Kinematics::pstarTwoBodyDecay(mAB ,mA,mB);
  Energy  pR   = Kinematics::pstarTwoBodyDecay(mres,mA,mB);
  Energy2 mgam = wres*sqr(mres)/mAB*Math::powi(pAB/pR,3)*
    (1.+sqr(radius*pR))/(1.+sqr(radius*pAB));
  Energy2 s = (sqr(mAC)-sqr(mBC)-(sqr(mD)-sqr(mC))*(sqr(mA)-sqr(mB))/sqr(mres))*
    sqrt((1.+sqr(radius*pR))/(1.+sqr(radius*pAB))); 
  Complex output=s/((sqr(mres)- sqr(mAB)-complex<Energy2>(ZERO,mgam)));
  return output;
}

DtoKPiPiMarkIII::DtoKPiPiMarkIII() {
  // Amplitudes and phases for D0 -> K- pi+ pi0
  _a1rho    = 1.0000; _phi1rho    =   0.;
  _a1Kstarm = 0.4018; _phi1Kstarm = 154.;
  _a1Kstar0 = 0.4244; _phi1Kstar0 =   7.;
  _a1NR     = 2.0693; _phi1NR     =  52.;
  // Amplitudes and phases for D0 -> Kbar0 pi+ pi-
  _a2rho    = 0.0975; _phi2rho    =  93.;
  _a2Kstar  = 0.2225; _phi2Kstar  =   0.;
  _a2NR     = 1.0000; _phi2NR     =   0.;
  // Amplitudes and phases for D+ -> Kbar0 pi+ pi0
  _a3rho    = 1.0000; _phi3rho    =   0.;
  _a3Kstar  = 0.5617; _phi3Kstar  =  43.;
  _a3NR     = 2.7250; _phi3NR     = 250.;
  // Amplitudes and phases for D+ -> K- pi+ pi+
  _a4Kstar  = 0.04749; _phi4Kstar  = 105.;
  _a4NR     = 1.00000; _phi4NR     =   0.;
  // masses and widths of the resonances
  _localparameters = true;
  _mrhop   = 0.770*GeV; _wrhop   = 0.1533*GeV;
  _mrho0   = 0.770*GeV; _wrho0   = 0.1533*GeV;
  _mKstarm = 0.8921*GeV; _wKstarm = 0.0511*GeV;
  _mKstar0 = 0.8695*GeV; _wKstar0 = 0.0502*GeV;
  // radii of the mesons
  _rrho   = 5.*mm*1e-12/hbarc; 
  _rKstar = 2.*mm*1e-12/hbarc;
  // intermediates
  generateIntermediates(true);
}


void DtoKPiPiMarkIII::persistentOutput(PersistentOStream & os) const {
  os << _a1rho << _phi1rho << _a1Kstarm << _phi1Kstarm << _a1Kstar0 << _phi1Kstar0 
     << _a1NR << _phi1NR << _c1rho << _c1Kstarm << _c1Kstar0 << _c1NR << _a2rho 
     << _phi2rho << _a2Kstar << _phi2Kstar << _a2NR << _phi2NR << _c2rho << _c2Kstar 
     << _c2NR << _a3rho << _phi3rho << _a3Kstar << _phi3Kstar << _a3NR << _phi3NR 
     << _c3rho << _c3Kstar << _c3NR << _a4Kstar << _phi4Kstar << _a4NR << _phi4NR 
     << _c4Kstar << _c4NR << _localparameters << ounit(_mrhop,GeV) << ounit(_wrhop,GeV) 
     << ounit(_mrho0,GeV) << ounit(_wrho0,GeV) << ounit(_mKstarm,GeV) 
     << ounit(_wKstarm,GeV) << ounit(_mKstar0,GeV) << ounit(_wKstar0,GeV) 
     << ounit(_rrho,1./GeV) << ounit(_rKstar,1./GeV) << _maxwgt << _weights;
}

void DtoKPiPiMarkIII::persistentInput(PersistentIStream & is, int) {
  is >> _a1rho >> _phi1rho >> _a1Kstarm >> _phi1Kstarm >> _a1Kstar0 >> _phi1Kstar0 
     >> _a1NR >> _phi1NR >> _c1rho >> _c1Kstarm >> _c1Kstar0 >> _c1NR >> _a2rho 
     >> _phi2rho >> _a2Kstar >> _phi2Kstar >> _a2NR >> _phi2NR >> _c2rho >> _c2Kstar 
     >> _c2NR >> _a3rho >> _phi3rho >> _a3Kstar >> _phi3Kstar >> _a3NR >> _phi3NR 
     >> _c3rho >> _c3Kstar >> _c3NR >> _a4Kstar >> _phi4Kstar >> _a4NR >> _phi4NR 
     >> _c4Kstar >> _c4NR >> _localparameters >> iunit(_mrhop,GeV) >> iunit(_wrhop,GeV) 
     >> iunit(_mrho0,GeV) >> iunit(_wrho0,GeV) >> iunit(_mKstarm,GeV) 
     >> iunit(_wKstarm,GeV) >> iunit(_mKstar0,GeV) >> iunit(_wKstar0,GeV) 
     >> iunit(_rrho,1./GeV) >> iunit(_rKstar,1./GeV) >> _maxwgt >> _weights;
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<DtoKPiPiMarkIII,DecayIntegrator>
describeHerwigDtoKPiPiMarkIII("Herwig::DtoKPiPiMarkIII", "HwSMDecay.so");

void DtoKPiPiMarkIII::Init() {

  static ClassDocumentation<DtoKPiPiMarkIII> documentation
    ("The DtoKPiPiMarkIII class performs the D -> K pi pi decays using the fit"
     "of the MarkIII collaboration",
     "The fit of the Mark III collaboration \\cite{Adler:1987sd} was used for "
     "$D\\to K\\pi\\pi$ decays",
     "\\bibitem{Adler:1987sd} J.~Adler {\\it et al.} [MARK-III Collaboration], "
     "Phys.\\ Lett.\\  B {\\bf 196} (1987) 107.");

  static Parameter<DtoKPiPiMarkIII,double> interfaceKmPipPi0RhoMagnitude
    ("KmPipPi0RhoMagnitude",
     "The magnitude of the rho component for D0 -> K- pi+ pi0",
     &DtoKPiPiMarkIII::_a1rho, 1.00, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,double> interfaceKmPipPi0RhoPhase
    ("KmPipPi0RhoPhase",
     "The phase of the rho component for D0 -> K- pi+ pi0",
     &DtoKPiPiMarkIII::_phi1rho,  0.,0., 360.,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,double> interfaceKmPipPi0KstarmMagnitude
    ("KmPipPi0KstarmMagnitude",
     "The magnitude of the K*(892)- component for D0 -> K- pi+ pi0",
     &DtoKPiPiMarkIII::_a1Kstarm, 0.371, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,double> interfaceKmPipPi0KstarmPhase
    ("KmPipPi0KstarmPhase",
     "The phase of the K*(892)- component for D0 -> K- pi+ pi0",
     &DtoKPiPiMarkIII::_phi1Kstarm, 154.,0., 360.,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,double> interfaceKmPipPi0Kstar0Magnitude
    ("KmPipPi0Kstar0Magnitude",
     "The magnitude of the K*(892)0 component for D0 -> K- pi+ pi0",
     &DtoKPiPiMarkIII::_a1Kstar0, 0.391, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,double> interfaceKmPipPi0Kstar0Phase
    ("KmPipPi0Kstar0Phase",
     "The phase of the K*(892)0 component for D0 -> K- pi+ pi0",
     &DtoKPiPiMarkIII::_phi1Kstar0, 7.,0., 360.,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,double> interfaceKmPipPi0NonResonantMagnitude
    ("KmPipPi0NonResonantMagnitude",
     "The magnitude of the non-resonant component for D0 -> K- pi+ pi0",
     &DtoKPiPiMarkIII::_a1NR, 1.889, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,double> interfaceKmPipPi0NonResonantPhase
    ("KmPipPi0NonResonantPhase",
     "The phase of the non-resonant component for D0 -> K- pi+ pi0",
     &DtoKPiPiMarkIII::_phi1NR, 52.,0., 360.,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,double> interfaceK0PipPimRhoMagnitude
    ("K0PipPimRhoMagnitude",
     "The magnitude of the rho component for D0 -> Kbar0 pi+pi-",
     &DtoKPiPiMarkIII::_a2rho, 1.000, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,double> interfaceK0PipPimRhoPhase
    ("K0PipPimRhoPhase",
     "The phase of the rho component for D0 -> Kbar0 pi+pi-",
     &DtoKPiPiMarkIII::_phi2rho,  93.,0., 360.,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,double> interfaceK0PipPimKstarMagnitude
    ("K0PipPimKstarMagnitude",
     "The magnitude of the K*(892) component for D0 -> Kbar0 pi+pi-",
     &DtoKPiPiMarkIII::_a2Kstar, 2.106, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,double> interfaceK0PipPimKstarPhase
    ("K0PipPimKstarPhase",
     "The phase of the K*(892)0 component for D0 -> Kbar0 pi+pi-",
     &DtoKPiPiMarkIII::_phi2Kstar, 0.,0., 360.,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,double> interfaceK0PipPimNonResonantMagnitude
    ("K0PipPimNonResonantMagnitude",
     "The magnitude of the non-resonant component for D0 -> Kbar0 pi+pi-",
     &DtoKPiPiMarkIII::_a2NR, 9.379, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,double> interfaceK0PipPimNonResonantPhase
    ("K0PipPimNonResonantPhase",
     "The phase of the non-resonant component for D0 -> Kbar0 pi+pi-",
     &DtoKPiPiMarkIII::_phi2NR, 0.,0., 360.,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,double> interfaceK0PipPi0RhoMagnitude
    ("K0PipPi0RhoMagnitude",
     "The magnitude of the rho component for D+ -> Kbar0 pi+ pi0",
     &DtoKPiPiMarkIII::_a3rho, 1.0, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,double> interfaceK0PipPi0RhoPhase
    ("K0PipPi0RhoPhase",
     "The phase of the rho component for D+ -> Kbar0 pi+ pi0",
     &DtoKPiPiMarkIII::_phi3rho, 0.0, 0.0, 360.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,double> interfaceK0PipPi0KstarMagnitude
    ("K0PipPi0KstarMagnitude",
     "The magnitude of the K* component for D+ -> Kbar0 pi+ pi0",
     &DtoKPiPiMarkIII::_a3Kstar, 0.517, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,double> interfaceK0PipPi0KstarPhase
    ("K0PipPi0KstarPhase",
     "The phase of the K* component for D+ -> Kbar0 pi+ pi0",
     &DtoKPiPiMarkIII::_phi3Kstar, 43.0, 0.0, 360.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,double> interfaceK0PipPi0NonResonantMagnitude
    ("K0PipPi0NonResonantMagnitude",
     "The magnitude of the non-resonant component for D0 -> Kbar0 pi+pi-",
     &DtoKPiPiMarkIII::_a3NR, 2.490, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,double> interfaceK0PipPi0NonResonantPhase
    ("K0PipPi0NonResonantPhase",
     "The phase of the non-resonant component for D0 -> Kbar0 pi+pi-",
     &DtoKPiPiMarkIII::_phi3NR, 250.,0., 360.,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,double> interfaceKmPipPipNonResonantMagnitude
    ("KmPipPipNonResonantMagnitude",
     "The magnitude of the non-resonant component for D+ -> K- pi+ pi+",
     &DtoKPiPiMarkIII::_a4NR, 1.00, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,double> interfaceKmPipPipNonResonantPhase
    ("KmPipPipNonResonantPhase",
     "The phase of the non-resonant component for D+ -> K- pi+ pi+",
     &DtoKPiPiMarkIII::_phi4NR, 0., 0., 360.,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,double> interfaceKmPipPipKstarMagnitude
    ("KmPipPipKstarMagnitude",
     "The magnitude of the K*(892) component for D+ -> K- pi+ pi+",
     &DtoKPiPiMarkIII::_a4Kstar, 0.049, 0.0, 10.0,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,double> interfaceKmPipPipKstarPhase
    ("KmPipPipKstarPhase",
     "The phase of the K*(892) component for D+ -> K- pi+ pi+",
     &DtoKPiPiMarkIII::_phi4Kstar, 105.,0., 360.,
     false, false, Interface::limited);

  static Switch<DtoKPiPiMarkIII,bool> interfaceLocalParameters
    ("LocalParameters",
     "Whether to use local values for the masses and widths or"
     " those from the ParticleData objects",
     &DtoKPiPiMarkIII::_localparameters, true, false, false);
  static SwitchOption interfaceLocalParametersLocal
    (interfaceLocalParameters,
     "Local",
     "Use local values",
     true);
  static SwitchOption interfaceLocalParametersParticleData
    (interfaceLocalParameters,
     "ParticleData",
     "Use the values from the ParticleData objects",
     false);

  static Parameter<DtoKPiPiMarkIII,Energy> interfaceKstar0Mass
    ("Kstar0Mass",
     "The mass of the K*(892)0",
     &DtoKPiPiMarkIII::_mKstar0, GeV, 0.8965 *GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,Energy> interfaceKstar0Width
    ("Kstar0Width",
     "The width of the K*(892)0",
     &DtoKPiPiMarkIII::_wKstar0, GeV, 0.0502*GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,Energy> interfaceKstarMinusMass
    ("KstarMinusMass",
     "The mass of the K*(892)-",
     &DtoKPiPiMarkIII::_mKstarm, GeV, 0.8921*GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,Energy> interfaceKstarMinusWidth
    ("KstarMinusWidth",
     "The width of the K*(892)-",
     &DtoKPiPiMarkIII::_wKstarm, GeV, 0.0511*GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,Energy> interfaceRho0Mass
    ("Rho0Mass",
     "The mass of the rho0",
     &DtoKPiPiMarkIII::_mrho0, GeV, 0.770 *GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,Energy> interfaceRho0Width
    ("Rho0Width",
     "The width of the rho0",
     &DtoKPiPiMarkIII::_wrho0, GeV, 0.1533*GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,Energy> interfaceRhoPlusMass
    ("RhoPlusMass",
     "The mass of the rho+",
     &DtoKPiPiMarkIII::_mrhop, GeV, 0.770 *GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,Energy> interfaceRhoPlusWidth
    ("RhoPlusWidth",
     "The width of the rho+",
     &DtoKPiPiMarkIII::_wrhop, GeV, 0.1533*GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,InvEnergy> interfaceRhoRadius
    ("RhoRadius",
     "The radius of the rho for the Blatt-Weisskopf factor",
     &DtoKPiPiMarkIII::_rrho, mm*1e-12/hbarc, 5.0*mm*1e-12/hbarc, 0.0*mm*1e-12/hbarc, 10.0*mm*1e-12/hbarc,
     true, false, Interface::limited);

  static Parameter<DtoKPiPiMarkIII,InvEnergy> interfaceKstarRadius
    ("KstarRadius",
     "The radius of the K* for the Blatt-Weisskopf factor",
     &DtoKPiPiMarkIII::_rKstar, mm*1e-12/hbarc, 2.0*mm*1e-12/hbarc, 0.0*mm*1e-12/hbarc, 10.0*mm*1e-12/hbarc,
     true, false, Interface::limited);

  static ParVector<DtoKPiPiMarkIII,double> interfaceMaximumWeights
    ("MaximumWeights",
     "The maximum weights for the unweighting of the decays",
     &DtoKPiPiMarkIII::_maxwgt, -1, 1.0, 0.0, 1.0e11,
     false, false, Interface::limited);

  static ParVector<DtoKPiPiMarkIII,double> interfaceWeights
    ("Weights",
     "The weights for the different channels for the phase-space integration",
     &DtoKPiPiMarkIII::_weights, -1, 1.0, 0.0, 1.0,
     false, false, Interface::limited);
}

void DtoKPiPiMarkIII::doinit() {
  DecayIntegrator::doinit();
  double fact = Constants::pi/180.;
  // amplitudes for D0 -> K- pi+ pi0
  _c1rho    = _a1rho   *Complex(cos(_phi1rho   *fact),sin(_phi1rho   *fact));
  _c1Kstarm = _a1Kstarm*Complex(cos(_phi1Kstarm*fact),sin(_phi1Kstarm*fact));
  _c1Kstar0 = _a1Kstar0*Complex(cos(_phi1Kstar0*fact),sin(_phi1Kstar0*fact));
  _c1NR     = _a1NR    *Complex(cos(_phi1NR    *fact),sin(_phi1NR    *fact));
  // amplitudes for D0 -> Kbar0 pi+ pi-
  _c2rho    = _a2rho   *Complex(cos(_phi2rho   *fact),sin(_phi2rho   *fact));
  _c2Kstar  = _a2Kstar *Complex(cos(_phi2Kstar *fact),sin(_phi2Kstar *fact));
  _c2NR     = _a2NR    *Complex(cos(_phi2NR    *fact),sin(_phi2NR    *fact));
  // amplitudes for D+ -> Kbar0 pi+ pi0
  _c3rho    = _a3rho   *Complex(cos(_phi3rho   *fact),sin(_phi3rho   *fact));
  _c3Kstar  = _a3Kstar *Complex(cos(_phi3Kstar *fact),sin(_phi3Kstar *fact));
  _c3NR     = _a3NR    *Complex(cos(_phi3NR    *fact),sin(_phi3NR    *fact));
  // amplitudes for D+ -> K- pi+ pi+
  _c4Kstar  = _a4Kstar *Complex(cos(_phi4Kstar *fact),sin(_phi4Kstar *fact));
  _c4NR     = _a4NR    *Complex(cos(_phi4NR    *fact),sin(_phi4NR    *fact));
  // intermediate resonances
  tPDPtr k8920 = getParticleData(ParticleID::Kstarbar0 );
  tPDPtr k892m = getParticleData(ParticleID::Kstarminus);
  tPDPtr rho0  = getParticleData(ParticleID::rho0      );
  tPDPtr rhop  = getParticleData(ParticleID::rhoplus   );
  // D0 -> K- pi+ pi0
  tPDVector extpart(4);
  DecayPhaseSpaceChannelPtr newchannel;
  DecayPhaseSpaceModePtr mode;
  extpart[0]=getParticleData(ParticleID::D0);
  extpart[1]=getParticleData(ParticleID::Kminus);
  extpart[2]=getParticleData(ParticleID::piplus);
  extpart[3]=getParticleData(ParticleID::pi0);
  mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
  unsigned int ix=0;
  if(rhop) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,1);
    newchannel->addIntermediate(rhop,0,0., 2,3);
    mode->addChannel(newchannel);
    ++ix;
  }
  if(k892m) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,2);
    newchannel->addIntermediate(k892m,0,0., 1,3);
    mode->addChannel(newchannel);
    ++ix;
  }
  if(k8920) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,3);
    newchannel->addIntermediate(k8920,0,0., 1,2);
    mode->addChannel(newchannel);
    ++ix;
  }
  // add the mode
  vector<double> wtemp;
  if(ix<=_weights.size()) {
    vector<double>::const_iterator wit=_weights.begin();
    wtemp=vector<double>(wit,wit+ix);
  }
  else {
    wtemp=vector<double>(ix,1./double(ix));
  }
  if(_maxwgt.empty()) _maxwgt.push_back(1.);
  addMode(mode,_maxwgt[0],wtemp);
  // D0 -> Kbar0 pi+ pi-
  extpart[0]=getParticleData(ParticleID::D0);
  extpart[1]=getParticleData(ParticleID::Kbar0);
  extpart[2]=getParticleData(ParticleID::piplus);
  extpart[3]=getParticleData(ParticleID::piminus);
  mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
  unsigned int iy=ix;
  if(rho0) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,1);
    newchannel->addIntermediate(rho0,0,0., 2,3);
    mode->addChannel(newchannel);
    ++iy;
  }
  if(k892m) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,2);
    newchannel->addIntermediate(k892m,0,0., 1,3);
    mode->addChannel(newchannel);
    ++iy;
  }
  // add the mode
  if(iy<=_weights.size()) {
    vector<double>::const_iterator wit=_weights.begin();
    wtemp=vector<double>(wit+ix,wit+iy);
  }
  else {
    wtemp=vector<double>(iy-ix,1./double(iy-ix));
  }
  if(_maxwgt.size()<2) _maxwgt.push_back(1.);
  addMode(mode,_maxwgt[1],wtemp);
  // D+ -> Kbar0pi+pi0
  ix=iy;
  extpart[0]=getParticleData(ParticleID::Dplus);
  extpart[1]=getParticleData(ParticleID::Kbar0);
  extpart[2]=getParticleData(ParticleID::piplus);
  extpart[3]=getParticleData(ParticleID::pi0);
  mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
  if(rhop) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,1);
    newchannel->addIntermediate(rhop,0,0., 2,3);
    mode->addChannel(newchannel);
    ++iy;
  }
  if(k8920) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,2);
    newchannel->addIntermediate(k8920,0,0., 1,3);
    mode->addChannel(newchannel);
    ++iy;
  }
  // add the mode
  if(iy<=_weights.size()) {
    vector<double>::const_iterator wit=_weights.begin();
    wtemp=vector<double>(wit+ix,wit+iy);
  }
  else {
    wtemp=vector<double>(iy-ix,1./double(iy-ix));
  }
  if(_maxwgt.size()<3) _maxwgt.push_back(1.);
  addMode(mode,_maxwgt[2],wtemp);
  // D+ -> K-pi+pi+
  ix=iy;
  extpart[0]=getParticleData(ParticleID::Dplus);
  extpart[1]=getParticleData(ParticleID::Kminus);
  extpart[2]=getParticleData(ParticleID::piplus);
  extpart[3]=getParticleData(ParticleID::piplus);
  mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
  if(k8920) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,3);
    newchannel->addIntermediate(k8920,0,0., 1,2);
    mode->addChannel(newchannel);
    ++iy;
    newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,2);
    newchannel->addIntermediate(k8920,0,0., 1,3);
    mode->addChannel(newchannel);
    ++iy;
  }
  // add the mode
  if(iy<=_weights.size()) {
    vector<double>::const_iterator wit=_weights.begin();
    wtemp=vector<double>(wit+ix,wit+iy);
  }
  else {
    wtemp=vector<double>(iy-ix,1./double(iy-ix));
  }
  if(_maxwgt.size()<4) _maxwgt.push_back(1.);
  addMode(mode,_maxwgt[3],wtemp);
  // reset the resonance parameters in the integration if needed
  if(_localparameters) {
    resetIntermediate(k8920,_mKstar0,_wKstar0);
    resetIntermediate(k892m,_mKstarm,_wKstarm);
    resetIntermediate(rho0 ,_mrho0 ,_wrho0 );
    resetIntermediate(rhop ,_mrhop ,_wrhop );
  }
  else {
    _mKstar0 = k8920->mass();
    _mKstarm = k892m->mass();
    _mrho0   = rho0 ->mass();
    _mrhop   = rhop ->mass();
    _wKstar0 = k8920->width();
    _wKstarm = k892m->width();
    _wrho0   = rho0 ->width();
    _wrhop   = rhop ->width();
  }
}

int DtoKPiPiMarkIII::modeNumber(bool & cc, tcPDPtr parent, 
				const tPDVector & children) const {
  int id0(parent->id());
  // incoming particle must be D0 or D+
  if(abs(id0)!=ParticleID::D0&&abs(id0)!=ParticleID::Dplus) return -1;
  cc = id0<0;
  // must be three decay products
  if(children.size()!=3) return -1;
  tPDVector::const_iterator pit = children.begin();
  unsigned int npip(0),npim(0),nkm(0),nk0(0),npi0(0);
  int id;
  for( ;pit!=children.end();++pit) {
    id=(**pit).id();
    if(id          ==ParticleID::piplus)  ++npip;
    else if(id     ==ParticleID::pi0)     ++npi0;
    else if(id     ==ParticleID::piminus) ++npim;
    else if(abs(id)==ParticleID::K0)      ++nk0;
    else if(id     ==ParticleID::K_L0)    ++nk0;
    else if(id     ==ParticleID::K_S0)    ++nk0;
    else if(abs(id)==ParticleID::Kplus)   ++nkm;
  }
  if(abs(id0)==ParticleID::Dplus) {
    if((id0==ParticleID::Dplus &&(nkm==1&&npip==2))||
       (id0==ParticleID::Dminus&&(nkm==1&&npim==2))) 
      return 3;
    else if((id0==ParticleID::Dplus &&nk0==1&&npip==1&&npi0==1)||
	    (id0==ParticleID::Dminus&&nk0==1&&npim==1&&npi0==1))
      return 2;
    else 
      return -1;
  }
  else {
    if(npim==1&&npip==1&&nk0==1) return  1;
    else if(nkm==1&&(npip+npim)==1&&npi0==1) return 0;
    else                        return -1;
  }
}

double DtoKPiPiMarkIII::me2( const int ichan,const Particle & inpart,
			    const ParticleVector & decay,
			     MEOption meopt) const {
  if(!ME())
    ME(new_ptr(GeneralDecayMatrixElement(PDT::Spin0,PDT::Spin0,PDT::Spin0,PDT::Spin0)));
  useMe();
  if(meopt==Initialize) {
    ScalarWaveFunction::
      calculateWaveFunctions(_rho,const_ptr_cast<tPPtr>(&inpart),incoming);
  }
  if(meopt==Terminate) {
    // set up the spin information for the decay products
    ScalarWaveFunction::constructSpinInfo(const_ptr_cast<tPPtr>(&inpart),
					  incoming,true);
    for(unsigned int ix=0;ix<3;++ix)
    ScalarWaveFunction::constructSpinInfo(decay[ix],outgoing,true);
    return 0.;
  }
  // compute the invariant masses needed to calulate the amplitudes
  Energy mA  = decay[0]->mass();
  Energy mB  = decay[1]->mass();
  Energy mC  = decay[2]->mass();
  Energy mD  = inpart.mass();
  Energy mAB = (decay[0]->momentum()+decay[1]->momentum()).m();
  Energy mAC = (decay[0]->momentum()+decay[2]->momentum()).m();
  Energy mBC = (decay[1]->momentum()+decay[2]->momentum()).m();
  // compute the amplitudes for the resonaces present in both models
  Complex amp;
  // D0 -> K- pi+ pi0
  if(imode()==0) {
    if(ichan<0) {
      amp = _c1NR
	-_c1rho   *amplitude(true ,mD,mB,mC,mA,mBC,mAB,mAC,_mrhop  ,_wrhop  )
 	-_c1Kstarm*amplitude(false,mD,mA,mC,mB,mAC,mAB,mBC,_mKstarm,_wKstarm)
 	-_c1Kstar0*amplitude(false,mD,mA,mB,mC,mAB,mAC,mBC,_mKstar0,_wKstar0);
    }
    else if(ichan==0) {
      amp = _c1rho   *amplitude(true ,mD,mB,mC,mA,mBC,mAB,mAC,_mrhop  ,_wrhop  );
    }
    else if(ichan==1) {
      amp = _c1Kstarm*amplitude(false,mD,mA,mC,mB,mAC,mAB,mBC,_mKstarm,_wKstarm);
    }
    else {
      amp = _c1Kstar0*amplitude(false,mD,mA,mB,mC,mAB,mAC,mBC,_mKstar0,_wKstar0);
    }
  }
  // D0 -> Kbar0 pi+pi-
  else if(imode()==1) {
    if(ichan<0) {
      amp = _c2NR*(-1.+2.*UseRandom::rndbool())
	-_c2rho  *amplitude(true ,mD,mB,mC,mA,mBC,mAB,mAC,_mrho0  ,_wrho0  )
 	+_c2Kstar*amplitude(false,mD,mA,mC,mB,mAC,mAB,mBC,_mKstarm,_wKstarm);
    }
    else if(ichan==0) {
      amp =_c2rho  *amplitude(true ,mD,mB,mC,mA,mBC,mAB,mAC,_mrho0  ,_wrho0  );
    }
    else {
      amp =_c2Kstar*amplitude(false,mD,mA,mC,mB,mAC,mAB,mBC,_mKstarm,_wKstarm);
    }
  }
  // D+ -> Kbar0 pi+ pi0
  else if(imode()==2) {
    if(ichan<0) {
      amp = _c3NR
 	-_c3rho  *amplitude(true ,mD,mB,mC,mA,mBC,mAB,mAC,_mrhop  ,_wrhop  )
  	-_c3Kstar*amplitude(false,mD,mA,mC,mB,mAC,mAB,mBC,_mKstar0,_wKstar0);
    }
    else if(ichan==0) {
      amp = _c3rho  *amplitude(true ,mD,mB,mC,mA,mBC,mAB,mAC,_mrhop,_wrhop);
    }
    else {
      amp = _c3Kstar*amplitude(false,mD,mA,mC,mB,mAC,mAB,mBC,_mKstar0,_wKstar0);
    }
  }
  // D+ -> K- pi+ pi+
  else {
    if(ichan<0) {
      amp = _c4NR
	+_c4Kstar*amplitude(false,mD,mA,mB,mC,mAB,mAC,mBC,_mKstar0,_wKstar0)
 	+_c4Kstar*amplitude(false,mD,mA,mC,mB,mAC,mAB,mBC,_mKstar0,_wKstar0);
    }
    else if(ichan==0) {
      amp =_c4Kstar*amplitude(false,mD,mA,mB,mC,mAB,mAC,mBC,_mKstar0,_wKstar0);
    }
    else {
      amp =_c4Kstar*amplitude(false,mD,mA,mC,mB,mAC,mAB,mBC,_mKstar0,_wKstar0);
    }
  }
  // now compute the matrix element
  (*ME())(0,0,0,0)=amp;
  return norm(amp);
}

void DtoKPiPiMarkIII::dataBaseOutput(ofstream & output, bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  // parameters
  output << "newdef " << name() << ":KmPipPi0RhoMagnitude " << _a1rho << "\n";
  output << "newdef " << name() << ":KmPipPi0RhoPhase " << _phi1rho << "\n";
  output << "newdef " << name() << ":KmPipPi0KstarmMagnitude " << _a1Kstarm << "\n";
  output << "newdef " << name() << ":KmPipPi0KstarmPhase " << _phi1Kstarm << "\n";
  output << "newdef " << name() << ":KmPipPi0Kstar0Magnitude " << _a1Kstar0 << "\n";
  output << "newdef " << name() << ":KmPipPi0Kstar0Phase " << _phi1Kstar0 << "\n";
  output << "newdef " << name() << ":KmPipPi0NonResonantMagnitude " << _a1NR << "\n";
  output << "newdef " << name() << ":KmPipPi0NonResonantPhase " << _phi1NR << "\n";
  output << "newdef " << name() << ":K0PipPimRhoMagnitude " << _a2rho << "\n";
  output << "newdef " << name() << ":K0PipPimRhoPhase " << _phi2rho << "\n";
  output << "newdef " << name() << ":K0PipPimKstarMagnitude " << _a2Kstar << "\n";
  output << "newdef " << name() << ":K0PipPimKstarPhase " << _phi2Kstar << "\n";
  output << "newdef " << name() << ":K0PipPimNonResonantMagnitude " << _a2NR << "\n";
  output << "newdef " << name() << ":K0PipPimNonResonantPhase " << _phi2NR << "\n";
  output << "newdef " << name() << ":K0PipPi0RhoMagnitude " << _a3rho << "\n";
  output << "newdef " << name() << ":K0PipPi0RhoPhase " << _phi3rho << "\n";
  output << "newdef " << name() << ":K0PipPi0KstarMagnitude " << _a3Kstar << "\n";
  output << "newdef " << name() << ":K0PipPi0KstarPhase " << _phi3Kstar << "\n";
  output << "newdef " << name() << ":K0PipPi0NonResonantMagnitude " << _a3NR << "\n";
  output << "newdef " << name() << ":K0PipPi0NonResonantPhase " << _phi3NR << "\n";
  output << "newdef " << name() << ":KmPipPipNonResonantMagnitude " << _a4NR << "\n";
  output << "newdef " << name() << ":KmPipPipNonResonantPhase " << _phi4NR << "\n";
  output << "newdef " << name() << ":KmPipPipKstarMagnitude " << _a4Kstar << "\n";
  output << "newdef " << name() << ":KmPipPipKstarPhase " << _phi4Kstar << "\n";
  output << "newdef " << name() << ":LocalParameters " << _localparameters << "\n";
  output << "newdef " << name() << ":Kstar0Mass " << _mKstar0/GeV << "\n";
  output << "newdef " << name() << ":Kstar0Width " << _wKstar0/GeV << "\n";
  output << "newdef " << name() << ":KstarMinusMass " << _mKstarm/GeV << "\n";
  output << "newdef " << name() << ":KstarMinusWidth " << _wKstarm/GeV << "\n";
  output << "newdef " << name() << ":Rho0Mass " << _mrho0/GeV << "\n";
  output << "newdef " << name() << ":Rho0Width " << _wrho0/GeV << "\n";
  output << "newdef " << name() << ":RhoPlusMass " << _mrhop/GeV << "\n";
  output << "newdef " << name() << ":RhoPlusWidth " << _wrhop/GeV << "\n";
  output << "newdef " << name() << ":RhoRadius " << _rrho/(mm*1e-12/hbarc) << "\n";
  output << "newdef " << name() << ":KstarRadius " << _rKstar/(mm*1e-12/hbarc) << "\n";
  for(unsigned int ix=0;ix<_maxwgt.size();++ix) {
    output << "insert " << name() << ":MaximumWeights " 
	   << ix << " " << _maxwgt[ix] << "\n";
  }
  for(unsigned int ix=0;ix<_weights.size();++ix) {
    output << "insert " << name() << ":Weights " 
	   << ix << " " << _weights[ix] << "\n";
  }
  if(header) {
    output << "\n\" where BINARY ThePEGName=\"" << fullName() << "\";" << endl;
  }
}

void DtoKPiPiMarkIII::doinitrun() {
  DecayIntegrator::doinitrun();
  _weights.resize(mode(0)->numberChannels()+mode(1)->numberChannels()+
		  mode(2)->numberChannels()+mode(3)->numberChannels());
  _maxwgt.resize(4);
  unsigned int iy=0;
  for(unsigned int ix=0;ix<4;++ix) {
    _maxwgt[ix]=mode(ix)->maxWeight();
    for(unsigned int iz=0;iz<mode(ix)->numberChannels();++iz) {
      _weights[iy]=mode(ix)->channelWeight(iz);
      ++iy;
    }
  }
}

